@@ -19,6 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // cl_ents.c -- entity parsing and management
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "client.h"
 
 clentity_t *active_clentities, *free_clentities;
@@ -196,6 +200,7 @@ void CL_DeltaEntity (frame_t *frame, int newnum, entity_state_t *old, int bits)
 	if (ent->serverframe != cl.frame.serverframe - 1)
 	{	// wasn't in last update, so initialize some things
 		ent->trailcount = 1024;		// for diminishing rocket / grenade trails
+		ent->pr = NULL;             // for chained particle trails
 		// duplicate the current state so lerping doesn't hurt anything
 		ent->prev = *state;
 		if (state->event == EV_OTHER_TELEPORT)
@@ -230,7 +235,7 @@ void CL_ParsePacketEntities (frame_t *oldframe, frame_t *newframe)
 {
 	int			newnum;
 	int			bits;
-	entity_state_t	*oldstate;
+	entity_state_t	*oldstate = NULL;
 	int			oldindex, oldnum;
 
 	newframe->parse_entities = cl.parse_entities;
@@ -253,7 +258,7 @@ void CL_ParsePacketEntities (frame_t *oldframe, frame_t *newframe)
 
 	while (1)
 	{
-		newnum = CL_ParseEntityBits (&bits);
+		newnum = CL_ParseEntityBits ( (unsigned *)&bits );
 		if (newnum >= MAX_EDICTS)
 			Com_Error (ERR_DROP,"CL_ParsePacketEntities: bad number:%i", newnum);
 
@@ -378,9 +383,9 @@ void CL_ParsePlayerstate (frame_t *oldframe, frame_t *newframe)
 
 	if (flags & PS_M_ORIGIN)
 	{
-		state->pmove.origin[0] = MSG_ReadShort (&net_message);
-		state->pmove.origin[1] = MSG_ReadShort (&net_message);
-		state->pmove.origin[2] = MSG_ReadShort (&net_message);
+		state->pmove.origin[0] = MSG_ReadSizeInt (&net_message, coord_bytes);
+		state->pmove.origin[1] = MSG_ReadSizeInt (&net_message, coord_bytes);
+		state->pmove.origin[2] = MSG_ReadSizeInt (&net_message, coord_bytes);
 	}
 
 	if (flags & PS_M_VELOCITY)
@@ -506,8 +511,12 @@ void CL_ParseFrame (void)
 	int			cmd;
 	int			len;
 	frame_t		*old;
+	byte		new_areabits[MAX_MAP_AREAS/8];
 
+	//HACK: reduce the number of areabit changes detected
+	memcpy (new_areabits, cl.frame.areabits, MAX_MAP_AREAS/8);
 	memset (&cl.frame, 0, sizeof(cl.frame));
+	memcpy (cl.frame.areabits, new_areabits, MAX_MAP_AREAS/8);
 
 	cl.frame.serverframe = MSG_ReadLong (&net_message);
 	cl.frame.deltaframe = MSG_ReadLong (&net_message);
@@ -559,7 +568,12 @@ void CL_ParseFrame (void)
 
 	// read areabits
 	len = MSG_ReadByte (&net_message);
-	MSG_ReadData (&net_message, &cl.frame.areabits, len);
+	MSG_ReadData (&net_message, &new_areabits, len);
+	if (memcmp (cl.frame.areabits, new_areabits, len))
+	{
+		memcpy (cl.frame.areabits, new_areabits, len);
+		cl.refdef.areabits_changed = true;
+	}
 
 	// read playerinfo
 	cmd = MSG_ReadByte (&net_message);
@@ -647,6 +661,8 @@ void CL_AddPacketEntities (frame_t *frame)
 		s1 = &cl_parse_entities[(frame->parse_entities+pnum)&(MAX_PARSE_ENTITIES-1)];
 
 		cent = &cl_entities[s1->number];
+		
+		ent.number = s1->number;
 
 		playermodel = false;
 		
@@ -683,6 +699,14 @@ void CL_AddPacketEntities (frame_t *frame)
 		ent.oldframe = cent->prev.frame;
 		ent.backlerp = 1.0 - cl.lerpfrac;
 
+		//animation timestamps are set here
+		if(cent->prevframe != s1->frame) {
+			cent->prevframe = s1->frame;
+			cent->frametime = Sys_Milliseconds();
+		}
+		ent.prevframe = cent->prevframe;
+		ent.frametime = cent->frametime;
+
 		// create a new entity
 
 		ent.lod1 = NULL;
@@ -708,6 +732,8 @@ void CL_AddPacketEntities (frame_t *frame)
 			}
 			playermodel = true;
 
+			strcpy(ent.name, ci->name);
+
 		}
 		else
 		{
@@ -784,9 +810,12 @@ void CL_AddPacketEntities (frame_t *frame)
 
 		}
 
+		// Sigh, this is the reason for the RF_NODRAW workaround. 
+#if 1 // TODO: replace this with an IFDEF for non Alien Arena CRX games
 		// if set to invisible, skip
 		if (!s1->modelindex)
 			continue;
+#endif
 
 		if (effects & EF_PLASMA)
 		{
@@ -794,31 +823,36 @@ void CL_AddPacketEntities (frame_t *frame)
 			ent.alpha = 0.6;
 		}
 
-		if (effects & EF_FLIES)
+		if (effects & EF_BUBBLES)
 		{
 			CL_PoweredEffects (ent.origin);
 		}
 
 		//cool new ctf flag effects
-		if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/flags/flag1.md2")) {
+		if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/flags/flag1.md2")) 
+		{
 			CL_FlagEffects(ent.origin, 0);
 			ent.team = 1;
 		}
-		else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/flags/flag2.md2")) {
+		else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/flags/flag2.md2")) 
+		{
 			CL_FlagEffects(ent.origin, 1);
 			ent.team = 2;
 		}
 		
-		// add to refresh list
-		V_AddEntity (&ent);
-
-		// color shells generate a seperate entity for the main model
-		if ((effects & EF_COLOR_SHELL) && !(s1->number == cl.playernum+1))
+		if (s1->modelindex != 0 && !(renderfx & RF_NODRAW))
 		{
+			// add to refresh list
+			V_AddEntity (&ent);
+
+			// color shells generate a seperate entity for the main model
+			if ((effects & EF_COLOR_SHELL) && !(s1->number == cl.playernum+1))
+			{
 		
-			ent.flags = renderfx | RF_TRANSLUCENT;
-			ent.alpha = 0.30;
-			V_AddViewEntity (&ent);			
+				ent.flags = renderfx | RF_TRANSLUCENT;
+				ent.alpha = 0.30;
+				V_AddViewEntity (&ent);			
+			}
 		}
 
 		ent.skin = NULL;		// never use a custom skin on others
@@ -831,30 +865,42 @@ void CL_AddPacketEntities (frame_t *frame)
 
 		ci = &cl.clientinfo[s1->skinnum & 0xff];
 
-		//give health an "aura"
-		if(cl_healthaura->value) {
-			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/healing/small/tris.md2"))
-				CL_SmallHealthParticles(ent.origin);
-			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/healing/medium/tris.md2"))
-				CL_MedHealthParticles(ent.origin);
-			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/healing/large/tris.md2"))
-				CL_LargeHealthParticles(ent.origin);
-		}
+		if (s1->modelindex != 0 && !(renderfx & RF_NODRAW))
+		{
+			//give health an "aura"
+			if(cl_healthaura->value && !cl_simpleitems->value) 
+			{
+				if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/healing/small/tris.md2"))
+					CL_SmallHealthParticles(ent.origin);
+				if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/healing/medium/tris.md2"))
+					CL_MedHealthParticles(ent.origin);
+				if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/healing/large/tris.md2"))
+					CL_LargeHealthParticles(ent.origin);
+			}
 
-		if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/weapons/g_rocket/tris.md2")) {
-			//add clear cover
-			ent.model = R_RegisterModel("models/weapons/g_rocket/cover.md2");
-			ent.flags |= RF_TRANSLUCENT;
-			ent.alpha = 0.30;
-			V_AddEntity (&ent);
-		}	
-		if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/weapons/g_hyperb/tris.md2")) {
-			//add clear cover
-			ent.model = R_RegisterModel("models/weapons/g_hyperb/cover.md2");
-			ent.flags |= RF_TRANSLUCENT;
-			ent.alpha = 0.30;
-			V_AddEntity (&ent);
-		}		
+			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/weapons/g_rocket/tris.md2")) 
+			{
+				//add clear cover
+				if (!cl_simpleitems->integer)
+				{
+					ent.model = R_RegisterModel("models/weapons/g_rocket/cover.md2");
+					ent.flags |= RF_TRANSLUCENT;
+					ent.alpha = 0.30;
+					V_AddEntity (&ent);
+				}
+			}	
+			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/weapons/g_hyperb/tris.md2")) 
+			{
+				//add clear cover
+				if (!cl_simpleitems->integer)
+				{
+					ent.model = R_RegisterModel("models/weapons/g_hyperb/cover.md2");
+					ent.flags |= RF_TRANSLUCENT;
+					ent.alpha = 0.30;
+					V_AddEntity (&ent);
+				}
+			}
+		}
 
 		if (s1->modelindex2)
 		{
@@ -865,37 +911,39 @@ void CL_AddPacketEntities (frame_t *frame)
 				if (!cl_vwep->value || i > MAX_CLIENTWEAPONMODELS - 1)
 					i = 0;
 				ent.model = ci->weaponmodel[i];
-				if (!ent.model) {
+				if (!ent.model) 
+				{
 					if (i != 0)
 						ent.model = ci->weaponmodel[0];
 					if (!ent.model)
 						ent.model = cl.baseclientinfo.weaponmodel[0];
 				}
 			}
-			else {
-
+			else 
+			{
 				ent.model = cl.model_draw[s1->modelindex2];
 			}
-
-			
+						
 			//here is where we will set the alpha for certain model parts - would like to eventually
 			//do something a little less uh, hardcoded.
 			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/items/healing/globe/tris.md2"))
 			{
+				if(cl_simpleitems->value)
+					continue;
 				ent.alpha = 0.4;
 				ent.flags = RF_TRANSLUCENT;
 			}
 			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/items/quaddama/unit.md2"))
 			{
+				if(cl_simpleitems->value)
+					continue;
 				ent.alpha = 0.4;
 				ent.flags = RF_TRANSLUCENT;
 			}
 
-			if (s1->number == cl.playernum+1) {
+			if (s1->number == cl.playernum+1) 
 				ent.flags |= RF_VIEWERMODEL;
 
-			}
-
 			V_AddEntity (&ent);
 
 			//PGM - make sure these get reset.
@@ -929,18 +977,20 @@ void CL_AddPacketEntities (frame_t *frame)
 			ent.flags = 0;
 			ent.alpha = 1.0;
 
-			//cool new ctf flag effects
-			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag1.md2")) {
+			//new ctf flag effects
+			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag1.md2")) 
+			{
 				CL_FlagEffects(ent.origin, 0);
 				ent.model = 0;
 			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag2.md2")) {
+			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag2.md2")) 
+			{
 				CL_FlagEffects(ent.origin, 1);
 				ent.model = 0;
 			}
-			else {
+			else 
+			{
 				ent.model = cl.model_draw[s1->modelindex4];
-
 				V_AddEntity (&ent);
 			}
 		}
@@ -956,7 +1006,6 @@ void CL_AddPacketEntities (frame_t *frame)
 			if (effects & EF_SHIPEXHAUST)
 			{
 				CL_ShipExhaust (cent->lerp_origin, ent.origin, cent);
-				V_AddLight (ent.origin, 50, 1, 1, 0);
 			}
 			if (effects & EF_ROCKETEXHAUST)
 			{
@@ -1019,7 +1068,7 @@ void CL_AddPacketEntities (frame_t *frame)
 			else if (effects & EF_BLASTER)
 			{
 				CL_BlasterTrail(cent->lerp_origin, ent.origin, cent);
-				V_AddLight (ent.origin, 50, 0, .5, 1);
+				V_AddLight (ent.origin, 200, 0, .5, .1);
 			}
 		}
 end:
@@ -1072,6 +1121,44 @@ void CL_BrassShells(vec3_t org, vec3_t dir, int count)
 	}
 }
 
+void CL_GlassShards(vec3_t org, vec3_t dir, int count)
+{
+	int i, j;
+	clentity_t *le;
+	float d;
+
+	if (!count)
+		return;
+
+	for (i = 0; i < count; i++) {
+
+		if (!free_clentities)
+			return;
+
+		le = free_clentities;
+		free_clentities = le->next;
+		le->next = active_clentities;
+		active_clentities = le;
+		le->time = cl.time;
+		d = (192 + rand()) & 13;
+		VectorClear(le->accel);
+		VectorClear(le->vel);
+		le->accel[0] = le->accel[1] = 0;
+		le->accel[2] = -3 * PARTICLE_GRAVITY;
+		le->alpha = 1.0;
+		le->alphavel = -0.01;
+		le->flags = CLM_BOUNCE | CLM_FRICTION | CLM_ROTATE | CLM_NOSHADOW | CLM_GLASS;
+		le->model = R_RegisterModel("models/objects/debris1/tris.md2");
+		le->ang = crand() * 360;
+		le->avel = crand() * 500;
+
+		for (j = 0; j < 3; j++) {
+			le->lastOrg[j] = le->org[j] = org[j];
+			le->vel[j] = crand() * 12 + d * dir[j];
+		}
+	}
+}
+
 /*
 ==============
 CL_AddViewWeapon
@@ -1096,6 +1183,30 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 	if (!gun.model)
 		return;
 
+	//if a vehicle console, we want to render as a 2D HUD now, if a player choses to
+	if(cl_vehicle_huds->value)
+	{
+		if(!(strcmp("vehicles/bomber/v_wep.md2", gun.model->name))) 
+		{
+			vehicle_hud = 1;
+			return;
+		}
+		else if(!(strcmp("vehicles/strafer/v_wep.md2", gun.model->name))) 
+		{
+			vehicle_hud = 2;
+			return;
+		}
+		else if(!(strcmp("vehicles/hover/v_wep.md2", gun.model->name))) 
+		{
+			vehicle_hud = 3;
+			return;
+		}
+		else
+			vehicle_hud = 0;		
+	}
+	else
+		vehicle_hud = 0;
+
 	// nudge gun down and right if in wide angle view
 	if(cl.refdef.fov_x > 90)
 	{
@@ -1128,11 +1239,13 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 	VectorCopy (gun.origin, gun.oldorigin);	// don't lerp at all
 
 	//add an attached muzzleflash for chaingun
-	if(!(strcmp("models/weapons/v_shotg2/tris.md2", gun.model->name))) {
+	if(!(strcmp("models/weapons/v_shotg2/tris.md2", gun.model->name))) 
+	{
 		if(gun.frame > 4 && gun.frame < 14)
 			CL_MuzzleFlashParticle(gun.origin, gun.angles, true);
 	}	
-	else if(!(strcmp("models/weapons/v_hyperb/tris.md2", gun.model->name))) {
+	else if(!(strcmp("models/weapons/v_hyperb/tris.md2", gun.model->name))) 
+	{
 		if(gun.frame > 5 && gun.frame < 7) {
 			CL_PlasmaFlashParticle(gun.origin, gun.angles, true);
 			useFX = true;
@@ -1165,7 +1278,7 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 					gun.flags |= RF_TRANSLUCENT;
 					gun.alpha = 0.30;
 
-					if (s1->effects & EF_COLOR_SHELL && gun.flags & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN) || useFX)
+					if ( ( s1->effects & EF_COLOR_SHELL && gun.flags & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN) ) || useFX)
 					{
 						V_AddViewEntity (&gun);
 					}
@@ -1173,6 +1286,7 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 					{
 						gun.flags = oldeffects | RF_TRANSLUCENT | RF_SHELL_RED;
 
+
 						V_AddViewEntity (&gun);
 					}
 					if (s1->effects & EF_QUAD && cl_gun->value)
@@ -1189,18 +1303,34 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 
 	//add glass pieces
 	
-	if(!(strcmp("models/weapons/v_rocket/tris.md2", gun.model->name))) {
+	if(!(strcmp("models/weapons/v_rocket/tris.md2", gun.model->name))) 
+	{
 		gun.model = R_RegisterModel("models/weapons/v_rocket/cover.md2");
 		gun.flags |= RF_TRANSLUCENT;
 		gun.alpha = 0.30;
 		V_AddViewEntity (&gun);
 	}
-	if(!(strcmp("models/weapons/v_hyperb/tris.md2", gun.model->name))) {
+	if(!(strcmp("models/weapons/v_hyperb/tris.md2", gun.model->name))) 
+	{
 		gun.model = R_RegisterModel("models/weapons/v_hyperb/cover.md2");
 		gun.flags |= RF_TRANSLUCENT;
 		gun.alpha = 0.30;
 		V_AddViewEntity (&gun);
 	}
+	if(!(strcmp("models/weapons/v_blast/tris.md2", gun.model->name))) 
+	{
+		gun.model = R_RegisterModel("models/weapons/v_blast/cover.md2");
+		gun.flags |= RF_TRANSLUCENT;
+		gun.alpha = 0.30;
+		V_AddViewEntity (&gun);
+	}
+	if(!(strcmp("models/weapons/v_alienblast/tris.md2", gun.model->name))) 
+	{
+		gun.model = R_RegisterModel("models/weapons/v_alienblast/cover.md2");
+		gun.flags |= RF_TRANSLUCENT;
+		gun.alpha = 0.30;
+		V_AddViewEntity (&gun);
+	}
 }
 
 
@@ -1465,7 +1595,6 @@ void CL_AddClEntities()
 
 	memset(&ent, 0, sizeof(ent));
 
-
 	for (le = active_clentities; le; le = next) {
 		next = le->next;
 
@@ -1585,6 +1714,12 @@ void CL_AddClEntities()
 					Com_sprintf(soundname, sizeof(soundname), "weapons/clink0%i.wav", (rand() % 2) + 1);
 					S_StartSound (le->org, 0, CHAN_WEAPON, S_RegisterSound(soundname), 1.0, ATTN_NORM, 0);
 				}
+
+				//play a sound if glass
+				if ( (le->flags & (CLM_GLASS))  &&  (le->flags & (CLM_BOUNCE)) ) {
+					Com_sprintf(soundname, sizeof(soundname), "weapons/clink0%i.wav", (rand() % 2) + 1); //to do - get glass sound
+					S_StartSound (le->org, 0, CHAN_WEAPON, S_RegisterSound(soundname), 1.0, ATTN_NORM, 0);
+				}
 			}
 		}
 		// Save current origin if needed
@@ -1616,6 +1751,9 @@ void CL_AddClEntities()
 			((le->flags & CLM_ROTATE) ? (time * le->avel) : 0);
 
 		ent.frame = ent.oldframe = 0;
+
+		if(le->flags & CLM_GLASS)
+			ent.flags |= RF_TRANSLUCENT;
 			
 		V_AddEntity(&ent);
 		
@@ -1631,7 +1769,7 @@ void CL_ClearClEntities()
 	free_clentities = &clentities[0];
 	active_clentities = NULL;
 
-	for (i = 0; i < MAX_CLENTITIES; i++)
+	for (i = 0; i < MAX_CLENTITIES-1; i++)
 		clentities[i].next = &clentities[i + 1];
 	clentities[MAX_CLENTITIES - 1].next = NULL;
 }
