@@ -249,7 +249,7 @@ void CL_PredictMovement (void)
 	{	// just set angles
 		for (i=0 ; i<3 ; i++)
 		{
-			cl.predicted_angles[i] = cl.viewangles[i] + SHORT2ANGLE(cl.frame.playerstate.pmove.delta_angles[i]);
+			cl.last_predicted_angles[i] = cl.predicted_angles[i] = cl.viewangles[i] + SHORT2ANGLE(cl.frame.playerstate.pmove.delta_angles[i]);
 		}
 		return;
 	}
@@ -305,8 +305,13 @@ void CL_PredictMovement (void)
 	cl.predicted_origin[0] = pm.s.origin[0]*0.125;
 	cl.predicted_origin[1] = pm.s.origin[1]*0.125;
 	cl.predicted_origin[2] = pm.s.origin[2]*0.125;
+	
+	cl.predicted_velocity[0] = pm.s.velocity[0]*0.125;
+	cl.predicted_velocity[1] = pm.s.velocity[1]*0.125;
+	cl.predicted_velocity[2] = pm.s.velocity[2]*0.125;
 
 	VectorCopy (pm.viewangles, cl.predicted_angles);
+	VectorCopy (pm.viewangles, cl.last_predicted_angles);
 }
 
 /*
