@@ -65,6 +65,8 @@ cvar_t		*scr_consize;
 
 cvar_t		*cl_drawfps;
 cvar_t		*cl_drawtimer;
+cvar_t		*cl_drawbandwidth;
+cvar_t		*cl_drawframesover;
 
 cvar_t		*cl_drawtime;
 
@@ -75,6 +77,34 @@ void SCR_TimeRefresh_f (void);
 void SCR_Loading_f (void);
 
 extern void R_VCFreeFrame(void);
+extern cvar_t *rs_hasflag;
+extern cvar_t *rs_team;
+
+/*
+ * map performance counters
+ *
+ * Used to measure the complexity of map areas.
+ * Modified for display in performance counter array
+ * Replace general "r_speed" cvar.
+ *
+ * Note: c_visible_textures seems to be always 0 and
+ *   c_visible_lightmaps is not used
+ */
+// extern cvar_t *r_speeds;
+cvar_t *rspeed_wpolys;
+cvar_t *rspeed_epolys;
+cvar_t *rspeed_flares;
+cvar_t *rspeed_grasses;
+cvar_t *rspeed_beams;
+cvar_t *rspeed_vbobatches;
+cvar_t *rspeed_particles;
+
+extern int c_brush_polys;  /* "wpoly"   polygons from brushes */
+extern int c_alias_polys;  /* "epoly"   polygons from .md2 meshes */
+extern int c_flares;       /* "flares"  lens flares */
+extern int c_grasses;      /* "grasses" vegetation instances */
+extern int c_beams;        /* "beams"   light beams (?) */
+extern int c_vbo_batches;  /* "vbobatches" batched draw commands */
 
 /*
 ===============================================================================
@@ -103,10 +133,10 @@ void CL_AddNetgraph (void)
 		return;
 
 	for (i=0 ; i<cls.netchan.dropped ; i++)
-		SCR_DebugGraph (30, 0x40);
+		SCR_DebugGraph (30, RGBA8(167,59,49,255));
 
 	for (i=0 ; i<cl.surpressCount ; i++)
-		SCR_DebugGraph (30, 0xdf);
+		SCR_DebugGraph (30, RGBA8(255,191,15,255));
 
 	// see what the latency was on this packet
 	in = cls.netchan.incoming_acknowledged & (CMD_BACKUP-1);
@@ -114,28 +144,35 @@ void CL_AddNetgraph (void)
 	ping /= 30;
 	if (ping > 30)
 		ping = 30;
-	SCR_DebugGraph (ping, 0xd0);
+	SCR_DebugGraph (ping, RGBA(0,1,0,1));
 }
 
 
 typedef struct
 {
 	float	value;
-	int		color;
+	float	color[4];
 } graphsamp_t;
 
+#define	DEBUGGRAPH_VALUES	4096
 static	int			current;
-static	graphsamp_t	values[1024];
+static	graphsamp_t	values[DEBUGGRAPH_VALUES];
 
 /*
 ==============
 SCR_DebugGraph
 ==============
 */
-void SCR_DebugGraph (float value, int color)
+void SCR_DebugGraph (float value, const float color[])
 {
-	values[current&1023].value = value;
-	values[current&1023].color = color;
+	int i;
+	int limit;
+	limit = scr_vrect.width;
+	if (limit > DEBUGGRAPH_VALUES)
+		limit = DEBUGGRAPH_VALUES;
+	values[current%limit].value = value;
+	for (i = 0; i < 4; i++)
+		values[current%limit].color[i] = color[i];
 	current++;
 }
 
@@ -146,23 +183,26 @@ SCR_DrawDebugGraph
 */
 void SCR_DrawDebugGraph (void)
 {
-	int		a, x, y, w, i, h;
-	float	v;
-	int		color;
+	int			a, x, y, w, i, h;
+	float		v;
+	const float	*color;
 
 	//
 	// draw the graph
 	//
+	
 	w = scr_vrect.width;
+	if (w > DEBUGGRAPH_VALUES)
+		w = DEBUGGRAPH_VALUES;
 
 	x = scr_vrect.x;
 	y = scr_vrect.y+scr_vrect.height;
 	Draw_Fill (x, y-scr_graphheight->value,
-		w, scr_graphheight->value, 8);
+		w, scr_graphheight->value, RGBA8(123,123,123,255));
 
 	for (a=0 ; a<w ; a++)
 	{
-		i = (current-1-a+1024) & 1023;
+		i = (current-1-a+w) % w;
 		v = values[i].value;
 		color = values[i].color;
 		v = v*scr_graphscale->value + scr_graphshift->value;
@@ -170,7 +210,7 @@ void SCR_DrawDebugGraph (void)
 		if (v < 0)
 			v += scr_graphheight->value * (1+(int)(-v/scr_graphheight->value));
 		h = (int)v % scr_graphheight->integer;
-		Draw_Fill (x+w-1-a, y - h, 1,	h, color);
+		Draw_Fill (x+w-1-a, y - h, 1, h, color);
 	}
 }
 
@@ -380,11 +420,9 @@ static void SCR_CalcVrect (void)
 {
 
 	scr_vrect.width = viddef.width;
-	scr_vrect.width &= ~7;
-
 	scr_vrect.height = viddef.height;
-	scr_vrect.height &= ~1;
 
+	// calculate left and top margins-- for now always 0
 	scr_vrect.x = (viddef.width - scr_vrect.width)/2;
 	scr_vrect.y = (viddef.height - scr_vrect.height)/2;
 }
@@ -438,7 +476,8 @@ void SCR_Init (void)
 	scr_conspeed = Cvar_Get ("scr_conspeed", "3", CVAR_ARCHIVE);
 	scr_consize = Cvar_Get ("scr_consize", "0.5", CVAR_ARCHIVE);
 	scr_showturtle = Cvar_Get ("scr_showturtle", "0", 0);
-	scr_showpause = Cvar_Get ("scr_showpause", "1", 0);
+	// default back to 1 if we ever actually make a pause icon:
+	scr_showpause = Cvar_Get ("scr_showpause", "0", 0); 
 	scr_centertime = Cvar_Get ("scr_centertime", "2.5", CVAR_ARCHIVE);
 	scr_printspeed = Cvar_Get ("scr_printspeed", "8", CVAR_ARCHIVE);
 	scr_netgraph = Cvar_Get ("netgraph", "0", 0);
@@ -452,6 +491,18 @@ void SCR_Init (void)
 
 	cl_drawfps = Cvar_Get ("cl_drawfps", "0", CVAR_ARCHIVE);
 	cl_drawtimer = Cvar_Get("cl_drawtimer", "0", CVAR_ARCHIVE);
+	cl_drawbandwidth = Cvar_Get ("cl_drawbandwidth", "0", CVAR_ARCHIVE);
+	cl_drawframesover = Cvar_Get ("cl_drawframesover", "0", CVAR_ARCHIVE);
+
+	rspeed_wpolys   = Cvar_Get("rspeed_wpolys",  "0", 0);
+	rspeed_epolys   = Cvar_Get("rspeed_epolys",  "0", 0);
+	rspeed_flares   = Cvar_Get("rspeed_flares",  "0", 0);
+	rspeed_grasses  = Cvar_Get("rspeed_grasses", "0", 0);
+	rspeed_beams    = Cvar_Get("rspeed_beams",   "0", 0);
+	rspeed_vbobatches = Cvar_Get("rspeed_vbobatches", "0", 0);
+	rspeed_particles = Cvar_Get("rspeed_particles", "0", 0);
+	
+	memset (perftests, 0, sizeof(perftests));
 
 //
 // register our commands
@@ -602,10 +653,10 @@ void SCR_DrawLoadingBar (int percent, int scale)
 	{
 		Draw_Fill (
 			viddef.width/2-scale*15 + 1,viddef.height/2 + scale*5+1,
-			scale*30-2, scale*2-2, 3);
+			scale*30-2, scale*2-2, RGBA8(47,47,47,255));
 		Draw_Fill (
 			viddef.width/2-scale*15 + 1,viddef.height/2 + scale*5+1,
-			(scale*30-2)*percent/100, scale*2-2, 7);
+			(scale*30-2)*percent/100, scale*2-2, RGBA8(107,107,107,255));
 	}
 }
 
@@ -637,13 +688,13 @@ void SCR_DrawLoading (void)
 		if(map_pic_loaded)
 			Draw_StretchPic (0, 0, viddef.width, viddef.height, va("/levelshots/%s.pcx", mapfile));
 		else
-			Draw_Fill (0, 0, viddef.width, viddef.height, 0);
+			Draw_Fill (0, 0, viddef.width, viddef.height, RGBA(0,0,0,1));
 
 		isMap = true;
 
 	}
 	else
-		Draw_Fill (0, 0, viddef.width, viddef.height, 0);
+		Draw_Fill (0, 0, viddef.width, viddef.height, RGBA(0,0,0,1));
 
 #if 0
 	// no m_background pic, but a pic here over-writes the levelshot
@@ -841,7 +892,7 @@ void SCR_DrawConsole (void)
 	if (cls.state != ca_active || !cl.refresh_prepped)
 	{	// connected, but can't render
 		CON_DrawConsole( scr_consize->value );
-		Draw_Fill (0, viddef.height/2, viddef.width, viddef.height/2, 0);
+		Draw_Fill (0, viddef.height/2, viddef.width, viddef.height/2, RGBA(0,0,0,1));
 		return;
 	}
 
@@ -939,11 +990,13 @@ int entitycmpfnc( const entity_t *a, const entity_t *b )
  * frame flush and page flipping are not done. Modified 2011-02 with
  * additional info in result. Use 'viewpos' (also modified) command
  * for repeatable, consistent positioning.
+ *
+ * Modified 2012-12
  */
 void SCR_TimeRefresh_f( void )
 {
-	int   start, stop;
-	float time;
+	int   t_start, t_stop;
+	float d_time;
 	float frames_per_sec;
 	float msec_per_frame;
 	float save_yaw;
@@ -951,62 +1004,69 @@ void SCR_TimeRefresh_f( void )
 	float num_frames;
 	float yaw_increment;
 	float frame_yaw;
+	char  *cmd_arg;
 
 	if ( cls.state != ca_active )
 		return;
 
-	save_yaw = cl.refdef.viewangles[YAW];
-
-	num_frames = scr_timerefcount->value;
+	save_yaw    = cl.refdef.viewangles[YAW];
+	num_frames  = scr_timerefcount->value;
 	frame_count = scr_timerefcount->integer;
-	if ( frame_count < 1 || frame_count > 720 )
+	if ( frame_count < 1 || frame_count > 1440 )
 	{ // silently protect against unreasonable setting
 		num_frames = 128.0f;
 		frame_count = 128;
 	}
+	cmd_arg = Cmd_Argv(1);
 
-	frame_yaw = save_yaw; // start at original yaw
-	yaw_increment = 360.0f / num_frames;
-	start = Sys_Milliseconds();
-	if ( Cmd_Argc() == 2 )
-	{ // run without page flipping.
-		R_BeginFrame( 0 );
-		while ( --frame_count )
-		{
-			cl.refdef.viewangles[YAW] = frame_yaw;
-			R_RenderFrame( &cl.refdef );
-			frame_yaw += yaw_increment;
-		}
-		// last frame restores original yaw
-		cl.refdef.viewangles[YAW] = save_yaw;
-		R_RenderFrame( &cl.refdef );
-		R_EndFrame();
-
-	}
-	else
+	switch ( *cmd_arg )
 	{
-		while ( --frame_count )
-		{
-			cl.refdef.viewangles[YAW] = frame_yaw;
-			R_BeginFrame( 0 );
-			R_RenderFrame( &cl.refdef );
-			R_EndFrame();
-			frame_yaw += yaw_increment;
-		}
-		// last frame restores original yaw
-		cl.refdef.viewangles[YAW] = save_yaw;
+	case '1': // rotate
+		yaw_increment = 360.0f / num_frames;
+		break;
+
+	case '2': // no rotate
+		yaw_increment = 0.0f;
+		break;
+
+	default:
+		Com_Printf("usage: timerefresh <arg>, 1=rotate,2=fixed\n");
+		return;
+	}
+
+	// show position
+	Com_Printf ("x:%#1.0f y:%#1.0f z:%#1.0f yaw:%#1.0f pitch:%#1.0f\n",
+		cl.refdef.vieworg[0], cl.refdef.vieworg[1], cl.refdef.vieworg[2],
+		cl.refdef.viewangles[YAW], cl.refdef.viewangles[PITCH] );
+
+	// test loop
+	frame_yaw = save_yaw;
+	t_start = Sys_Milliseconds();
+	while ( frame_count-- )
+	{
+		cl.refdef.viewangles[YAW] = frame_yaw;
 		R_BeginFrame( 0 );
 		R_RenderFrame( &cl.refdef );
 		R_EndFrame();
+		frame_yaw += yaw_increment;
 	}
-	stop = Sys_Milliseconds();
-	time = (float)(stop - start);
+	t_stop = Sys_Milliseconds();
+
+	// restore original yaw (not included in test frame count)
+	cl.refdef.viewangles[YAW] = save_yaw;
+	R_BeginFrame( 0 );
+	R_RenderFrame( &cl.refdef );
+	R_EndFrame();
+
+	// report
+	d_time = (float)(t_stop - t_start);
+	d_time += 0.5f;
+	msec_per_frame = d_time / num_frames;
+	d_time /= 1000.0f;
+	frames_per_sec = num_frames / d_time;
 
-	msec_per_frame = time / num_frames;
-	time /= 1000.0f;
-	frames_per_sec = num_frames / time;
 	Com_Printf( "%1.0f frames, %1.3f sec, %1.3f msec/frame, %1.1f FPS\n",
-			num_frames, time, msec_per_frame, frames_per_sec );
+			num_frames, d_time, msec_per_frame, frames_per_sec );
 
 }
 
@@ -1068,47 +1128,6 @@ void SizeHUDString (char *string, int *w, int *h)
 	*h = lines * charscale;
 }
 
-void DrawHUDString (char *string, int x, int y, int centerwidth, int xor)
-{
-	int		margin;
-	char	line[1024];
-	int		width;
-	int		i;
-	int		charscale;
-
-	charscale = (float)(viddef.height)*8/600;
-	if(charscale < 8)
-		charscale = 8;
-
-	margin = x;
-
-	while (*string)
-	{
-		// scan out one line of text from the string
-		width = 0;
-		while (*string && *string != '\n')
-			line[width++] = *string++;
-		line[width] = 0;
-
-		if (centerwidth)
-			x = margin + (centerwidth - width*charscale)/2;
-		else
-			x = margin;
-		for (i=0 ; i<width ; i++)
-		{
-			Draw_Char (x, y, line[i]^xor);
-			x += charscale;
-		}
-		if (*string)
-		{
-			string++;	// skip the \n
-			x = margin;
-			y += charscale;
-		}
-	}
-}
-
-
 /*
 ==============
 SCR_DrawField
@@ -1187,12 +1206,13 @@ void SCR_ExecuteLayoutString (char *s)
 	int			x, y, ny=0; //ny is coordinates used for new sb layout client tags only
 	int			value;
 	char *			token;
-	int			width;
+	int			numwidth = 3; //amount of digits of number being drawn
 	int			index;
 	clientinfo_t *		ci;
 	int			charscale;
 	float			scale;
 	qboolean		newSBlayout = false;
+	int				left, top, right, bottom, width, height, midx, midy;
 
 	if (cls.state != ca_active || !cl.refresh_prepped)
 		return;
@@ -1200,12 +1220,34 @@ void SCR_ExecuteLayoutString (char *s)
 	if (!s[0])
 		return;
 
-	x = 0;
-	y = 0;
-	width = 3;
 	font = FNT_AutoGet( CL_gameFont );
 	charscale = font->size;
 	scale = charscale * 0.125;
+	
+	width = cl.refdef.width;
+	height = cl.refdef.height;
+	
+	x = left = cl.refdef.x;
+	y = ny = top = cl.refdef.y;
+	right = left+width;
+	bottom = top+height;
+	midx = left+width/2;
+	midy = top+height/2;
+	
+	if (width < 1024 || height < 768)
+	{
+		// Below 1024x768, start scaling things down to make sure everything
+		// fits. We scale x and y by the same amount to keep things square,
+		// but we figure out the ratios separately and use the ratio which
+		// results in the smallest scale.
+		float xscale, yscale;
+		xscale = (float)width/1024.0f;
+		yscale = (float)height/768.0f;
+		if (xscale < yscale)
+			scale *= xscale;
+		else
+			scale *= yscale;
+	}
 
 	while (s)
 	{
@@ -1213,39 +1255,39 @@ void SCR_ExecuteLayoutString (char *s)
 		if (!strcmp(token, "xl"))
 		{
 			token = COM_Parse (&s);
-			x = atoi(token)*scale;
+			x = left + atoi(token)*scale;
 			continue;
 		}
 		if (!strcmp(token, "xr"))
 		{
 			token = COM_Parse (&s);
-			x = viddef.width + atoi(token)*scale;
+			x = right + atoi(token)*scale;
 			continue;
 		}
 		if (!strcmp(token, "xv"))
 		{
 			token = COM_Parse (&s);
-			x = viddef.width/2 - 160*scale + atoi(token)*scale;
+			x = midx + (atoi(token) - 160)*scale;
 			continue;
 		}
 
 		if (!strcmp(token, "yt"))
 		{
 			token = COM_Parse (&s);
-			y = atoi(token)*scale;
+			y = top + atoi(token)*scale;
 			continue;
 		}
 		if (!strcmp(token, "yb"))
 		{
 			token = COM_Parse (&s);
-			y = viddef.height + atoi(token)*scale;
+			y = bottom + atoi(token)*scale;
 			continue;
 		}
 		if (!strcmp(token, "yv"))
 		{
 			token = COM_Parse (&s);
-			y = viddef.height/2 - 100*scale + atoi(token)*scale;
-			ny = viddef.height/2 - 100*scale + atoi(token)*scale/2;
+			y = midy + (atoi(token) - 100)*scale;
+			ny = midy + (atoi(token)/2 - 100)*scale;
 			continue;
 		}
 
@@ -1259,6 +1301,28 @@ void SCR_ExecuteLayoutString (char *s)
 				if( cl.configstrings[CS_IMAGES+value][0] )
 				{
 					Draw_ScaledPic (x, y, scale, cl.configstrings[CS_IMAGES+value]);
+					if(!strcmp(cl.configstrings[CS_IMAGES+value], "i_team1") || 
+						!strcmp(cl.configstrings[CS_IMAGES+value], "i_team2"))
+					{
+						r_gotFlag = false;
+						r_lostFlag = true;
+						Cvar_Set("rs_hasflag", "0");
+						if(!strcmp(cl.configstrings[CS_IMAGES+value], "i_team1"))
+							Cvar_Set("rs_team", "blue");
+						else
+							Cvar_Set("rs_team", "red");
+					}
+					else if(!strcmp(cl.configstrings[CS_IMAGES+value], "i_flag1") || 
+						!strcmp(cl.configstrings[CS_IMAGES+value], "i_flag2"))
+					{
+						r_gotFlag = true;
+						r_lostFlag = false;
+						Cvar_Set("rs_hasflag", "1");
+						if(!strcmp(cl.configstrings[CS_IMAGES+value], "i_flag1"))
+							Cvar_Set("rs_team", "blue");
+						else
+							Cvar_Set("rs_team", "red");
+					}
 				}
 			}
 
@@ -1268,8 +1332,8 @@ void SCR_ExecuteLayoutString (char *s)
 		if (!strcmp(token, "newsb"))
 		{
 			//print header here
-			x = viddef.width/2 - 160*scale;
-			y = viddef.height/2 - 64*scale;
+			x = midx - 160*scale;
+			y = midy - 64*scale;
 
 			box.x = x + 4 * scale , box.y = y;
 			FNT_RawPrint( font , "Player" , 6 , false , box.x , box.y , FNT_colors[ 7 ] );
@@ -1287,10 +1351,10 @@ void SCR_ExecuteLayoutString (char *s)
 		if (!strcmp(token, "newctfsb"))
 		{
 			//print header here
-			x = viddef.width/2 - 256*scale;
-			y = viddef.height/2 - 72*scale;
+			x = midx - 256*scale;
+			y = midy - 72*scale;
 
-			while ( x <= viddef.width/2 ) {
+			while ( x <= midx ) {
 				box.x = x + 14 * scale , box.y = y;
 				FNT_RawPrint( font , "Player" , 6 , false , box.x , box.y , FNT_colors[ 7 ] );
 				box.x += 118 * scale , box.width = 56 * scale , box.height = 0;
@@ -1312,12 +1376,12 @@ void SCR_ExecuteLayoutString (char *s)
 			int		timeColor;
 
 			token = COM_Parse (&s);
-			x = viddef.width/2 - 160*scale + atoi(token)*scale;
+			x = midx + (atoi(token) - 160)*scale;
 			token = COM_Parse (&s);
 			if(newSBlayout)
-				y = viddef.height/2 - 100*scale + atoi(token)*scale/2;
+				y = midy + (atoi(token)/2 - 100)*scale;
 			else
-				y = viddef.height/2 - 100*scale + atoi(token)*scale;
+				y = midy + (atoi(token) - 100)*scale;
 
 			token = COM_Parse (&s);
 			value = atoi(token);
@@ -1396,13 +1460,13 @@ void SCR_ExecuteLayoutString (char *s)
 			int		team;
 
 			token = COM_Parse (&s);
-			x = viddef.width/2 - 160*scale + atoi(token)*scale;
+			x = midx + (atoi(token) - 160)*scale;
 			if(atoi(token) < 0)
 				team = 0;
 			else
 				team = 1;
 			token = COM_Parse (&s);
-			y = viddef.height/2 - 100*scale + atoi(token)*scale;
+			y = midy + (atoi(token) - 100)*scale;
 
 			token = COM_Parse (&s);
 			value = atoi(token);
@@ -1458,10 +1522,10 @@ void SCR_ExecuteLayoutString (char *s)
 		{	// draw a number
 			if(strcmp(cl_hudimage1->string, "none")) {
 				token = COM_Parse (&s);
-				width = atoi(token);
+				numwidth = atoi(token);
 				token = COM_Parse (&s);
 				value = cl.frame.playerstate.stats[atoi(token)];
-				SCR_DrawField (x, y, 0, width, value, scale);
+				SCR_DrawField (x, y, 0, numwidth, value, scale);
 			}
 
 			continue;
@@ -1472,7 +1536,7 @@ void SCR_ExecuteLayoutString (char *s)
 			if(strcmp(cl_hudimage1->string, "none")) {
 				int		color;
 
-				width = 3;
+				numwidth = 3;
 				value = cl.frame.playerstate.stats[STAT_HEALTH];
 				if (value > 25)
 					color = 0;	// green
@@ -1481,7 +1545,7 @@ void SCR_ExecuteLayoutString (char *s)
 				else
 					color = 1;
 
-				SCR_DrawField (x, y, color, width, value, scale);
+				SCR_DrawField (x, y, color, numwidth, value, scale);
 			}
 
 			//draw the zoom scope pic if we are using the zoom alt-fire of disruptor
@@ -1489,7 +1553,7 @@ void SCR_ExecuteLayoutString (char *s)
 			{
 				char zoompic[32];
 				sprintf(zoompic, "zoomscope%i", cl.frame.playerstate.stats[STAT_ZOOMED]);
-				Draw_StretchPic (0, 0, viddef.width, viddef.height, zoompic);
+				Draw_StretchPic (left, top, width, height, zoompic);
 			}
 
 			continue;
@@ -1500,7 +1564,7 @@ void SCR_ExecuteLayoutString (char *s)
 			if(strcmp(cl_hudimage1->string, "none")) {
 				int		color;
 
-				width = 3;
+				numwidth = 3;
 				value = cl.frame.playerstate.stats[STAT_AMMO];
 				if (value > 5)
 					color = 0;	// green
@@ -1509,7 +1573,7 @@ void SCR_ExecuteLayoutString (char *s)
 				else
 					continue;	// negative number = don't show
 
-				SCR_DrawField (x, y, color, width, value, scale);
+				SCR_DrawField (x, y, color, numwidth, value, scale);
 			}
 
 			continue;
@@ -1520,14 +1584,14 @@ void SCR_ExecuteLayoutString (char *s)
 			if(strcmp(cl_hudimage1->string, "none")) {
 				int		color;
 
-				width = 3;
+				numwidth = 3;
 				value = cl.frame.playerstate.stats[STAT_ARMOR];
 				if (value < 1)
 					continue;
 
 				color = 0;	// green
 
-				SCR_DrawField (x, y, color, width, value, scale);
+				SCR_DrawField (x, y, color, numwidth, value, scale);
 			}
 
 			continue;
@@ -1552,7 +1616,11 @@ void SCR_ExecuteLayoutString (char *s)
 		if (!strcmp(token, "cstring"))
 		{
 			token = COM_Parse (&s);
-			DrawHUDString (token, x, y, 320, 0);
+			box.width = 2*(x>midx?(right-x):x); 
+			box.x = x-box.width/2;
+			box.y = y;
+			box.height = 0;
+			FNT_BoundedPrint( font , token , FNT_CMODE_NONE , FNT_ALIGN_CENTER , &box , FNT_colors[ 7 ] );
 			continue;
 		}
 
@@ -1561,7 +1629,7 @@ void SCR_ExecuteLayoutString (char *s)
 			token = COM_Parse (&s);
 			//this line is an Alien Arena specific hack of sorts, remove if needed
 			if(!strcmp(token, "Vote"))
-				Draw_ScaledPic (viddef.width/2 - 85*scale, y-8*scale, scale, "votebox");
+				Draw_ScaledPic (midx - 85*scale, y-8*scale, scale, "votebox");
 			FNT_RawPrint( font , token , strlen( token ) , false , x , y , FNT_colors[ 7 ] );
 			continue;
 		}
@@ -1569,7 +1637,11 @@ void SCR_ExecuteLayoutString (char *s)
 		if (!strcmp(token, "cstring2"))
 		{
 			token = COM_Parse (&s);
-			DrawHUDString (token, x, y, 320,0x80);
+			box.width = 2*(x>midx?(right-x):x); 
+			box.x = x-box.width/2;
+			box.y = y;
+			box.height = 0;
+			FNT_BoundedPrint( font , token , FNT_CMODE_NONE , FNT_ALIGN_CENTER , &box , FNT_colors[ 3 ] );
 			continue;
 		}
 
@@ -1713,99 +1785,236 @@ void SCR_showTimer(void)
 /*
 ================
 
-SCR_showFPS
+SCR_showPerfTest
 
 ================
 */
 
-int		fpscounter;
+perftest_t *get_perftest(char *name) {
+    int         i; 
+    perftest_t  *test = NULL;
+    
+    for (i = 0; i < MAX_PERFTESTS; i++) {
+        if (!perftests[i].in_use) {
+            test = &perftests[i];
+            break;
+        }
+        if (!strncmp(perftests[i].name, name, sizeof(perftests[i].name)))
+            return &perftests[i];
+    }
+    
+    if (!test) 
+        return NULL; //remember to handle this in your code!
+    
+    memset(test, 0, sizeof(perftest_t));
+    
+    test->in_use = true;
+    strncpy (test->name, name, sizeof(test->name));
+    
+    return test;
+}    
+
+//slotnum is used for the vertical offset on the screen
+void SCR_showPerfTest (perftest_t *test, int slotnum) {
+	FNT_font_t		font;
+	int			    end_msec;
+	float			time_sec;
+	float			rate;
+	int				height;
+	
+	if (!test->in_use)
+	    return;
+	
+	if (cls.key_dest == key_menu || cls.key_dest == key_console || !test->cvar->integer) {
+		test->restart = true;
+		return;
+	}
+	
+	if (test->restart) {
+	    test->start_msec = Sys_Milliseconds();
+	    test->counter = 0.0f;
+	    test->framecounter = 0.0f;
+	    test->update_trigger = 10.0f; //initial delay to update
+	    test->text[0] = 0; //blank the text buffer
+	    test->restart = false;
+	    return;
+	}
+	
+	test->framecounter += 1.0f;
+	if (test->framecounter >= test->update_trigger) {
+	    end_msec = Sys_Milliseconds();
+        time_sec = ((float)(end_msec - test->start_msec)) / 1000.0f;
+	    if (test->is_timerate) {
+	        //calculate rate to display
+	        rate = test->scale * test->counter / time_sec;
+	    } else {
+	        rate = test->scale * test->counter;
+	    }
+	    
+	    //update text buffer for display
+	    Com_sprintf( test->text, sizeof(test->text), test->format, rate );
+	    
+	    test->start_msec = end_msec;
+	    if (test->is_timerate)
+	        //setup for next update - 0.25 sec update interval
+	        test->update_trigger = test->framecounter / (4.0 * time_sec); 
+	    else
+	        //0.05 sec update interval
+	        test->update_trigger = test->framecounter / (20.0 * time_sec); 
+	    test->framecounter = 0.0f;
+	    test->counter = 0.0f;
+	} 
+	
+	font = FNT_AutoGet( CL_gameFont );
+
+	height = viddef.height - 2.0 * font->size * (slotnum + 1);
+	if (cl_drawtimer->integer)
+		height -= 5.25 * font->size;
+	FNT_RawPrint( font , test->text , strlen( test->text ) , false ,
+		viddef.width - 8 * font->size , height, FNT_colors[ 2 ] );
+}
+
+void SCR_showAllPerfTests (void) {
+    int i, testsDrawn;
+    for (i = 0, testsDrawn = 0; i < MAX_PERFTESTS; i++) {
+        if (perftests[i].in_use && perftests[i].cvar->integer) {
+            SCR_showPerfTest (&perftests[i], testsDrawn);
+            testsDrawn++;
+        }
+    }
+}
 
 void SCR_showFPS(void)
 {
-	static qboolean		restart		= true;
-	static float		update_trigger	= 0.0f;
-	static float		framecounter	= 0.0f;
-	static int		start_msec;
-	static char		fps_text[32];
-
-	FNT_font_t		font;
-	int			end_msec;
-	float			time_sec;
-	float			framerate;
-
-	if (cls.key_dest == key_menu || cls.key_dest == key_console) {
-		restart = true;
-		return;
-	}
-
-	if( restart ) {
-		start_msec = Sys_Milliseconds();
-		framecounter = 0.0f;
-		update_trigger = 10.0f; // initial delay to update
-		fps_text[0] = 0; // blank the text buffer
-		restart = false;
-		return;
-	}
-
-	framecounter += 1.0f;
-	if( framecounter >= update_trigger ) {
-		// calculate frames-per-second
-		end_msec = Sys_Milliseconds();
-		time_sec = ((float)(end_msec - start_msec)) / 1000.0f;
-		framerate = framecounter / time_sec ;
-
-		// update text buffer for display
-		if ( cl_drawfps->integer == 2 )
-		{ // for developers
-			Com_sprintf( fps_text, sizeof(fps_text), "%3.1ffps", framerate );
-		}
-		else
-		{
-			Com_sprintf( fps_text, sizeof(fps_text), "%3.0ffps", framerate );
-		}
-
-		// setup for next update
-		framecounter = 0.0f;
-		start_msec = end_msec;
-		update_trigger = framerate / 2.0 ; // for .5 sec update interval
-	}
-
-	font = FNT_AutoGet( CL_gameFont );
-
-	FNT_RawPrint( font , fps_text , strlen( fps_text ) , false ,
-		viddef.width - 8 * font->size , viddef.height - 2.0 * font->size , FNT_colors[ 2 ] );
+    static perftest_t *test = NULL;
+    if (!test) {
+        test = get_perftest("fps");
+        if (!test)
+            return; //couldn't acquire test
+        test->cvar = cl_drawfps;
+        test->is_timerate = true;
+        strcpy (test->format, "%3.0ffps");
+        test->scale = 1.0f;
+    }
+    
+    test->counter += 1.0f;
+    
 }
 
-/*
- * SCR_ShowRSpeeds()
- *   display the cvar r_speeds activated performance counters
+// Shows kilobytes per second from the server
+extern int c_incoming_bytes;
+void SCR_showBandwidth (void) {
+    static perftest_t *test = NULL;
+    if (!test) {
+        test = get_perftest("bandwidth");
+        if (!test)
+            return; //couldn't acquire test
+        test->is_timerate = true;
+        test->cvar = cl_drawbandwidth;
+        strcpy (test->format, "%2.2fkibps");
+        test->scale = 1.0f/1024.0f; //the SI is wrong, it's 1024
+    }
+    
+    test->counter += c_incoming_bytes;
+    c_incoming_bytes = 0;
+}
+
+// Shows number of frames per second which took longer than a configurable
+// amount of milliseconds. This is a useful measure of jitter, which isn't
+// usually reflected in average FPS rates but which can still be visually
+// annoying. 
+void SCR_showFramesOver (void) {
+    static perftest_t *test = NULL;
+    static int old_milliseconds = 0;
+    int new_milliseconds;
+    if (!test) {
+        test = get_perftest("framesover");
+        if (!test)
+            return; //couldn't acquire test
+        test->is_timerate = true;
+        test->cvar = cl_drawframesover;
+        strcpy (test->format, "%3.0fjfps"); //jfps = jitter frames per second
+        test->scale = 1.0f;
+        old_milliseconds = Sys_Milliseconds();
+    }
+    
+    new_milliseconds = Sys_Milliseconds();
+    if ((new_milliseconds - old_milliseconds) > cl_drawframesover->integer)
+    	test->counter++;
+    old_milliseconds = new_milliseconds;
+}
+
+/**
+ *
+ * @brief Construct map-maker's "r_speed" performance counter displays
+ *
+ * the traditional r_speeds cvar is replaced by individual counter cvars,
+ *  named "rspeed_*"
  */
+static void show_rspeed_helper(
+	cvar_t*     test_cvar,
+	perftest_t* test_obj,
+	char*       test_name,
+	char*       test_format,
+	int         test_counter
+	)
+{
+	if ( test_cvar && test_cvar->integer )
+	{
+		if ( test_obj == NULL )
+		{ /* constructor */
+			test_obj = get_perftest( test_name );
+			if ( test_obj != NULL )
+			{ /* slot allocated, construct */
+				test_obj->is_timerate = false;
+				test_obj->cvar = test_cvar;
+				strcpy( test_obj->format, test_format );
+				test_obj->scale = 1.0f;
+				test_obj->counter = test_counter;
+			}
+			else
+			{ /* no slot, force disable */
+				(void)Cvar_ForceSet( test_cvar->name, "0" );
+			}
+		}
+		else
+		{ /* running, update counter from refresh counter */
+			test_obj->counter = test_counter;
+		}
+	}
+}
+
 void SCR_showRSpeeds( void )
 {
- 	extern int		c_brush_polys;
- 	extern int		c_alias_polys;
+	static perftest_t *test_wpoly    = NULL;
+	static perftest_t *test_epoly    = NULL;
+	static perftest_t *test_flares   = NULL;
+	static perftest_t *test_grasses  = NULL;
+	static perftest_t *test_beams    = NULL;
+	static perftest_t *test_vbobatches = NULL;
+	static perftest_t *test_particles = NULL;
 
-	FNT_font_t		font;
-	struct FNT_window_s	box;
-	char			prtstring[32];
- 	float			scale;
+	show_rspeed_helper( rspeed_wpolys, test_wpoly,
+			"wpolys", "%5.0f wpolys", c_brush_polys );
 
- 	if (cls.key_dest == key_menu || cls.key_dest == key_console)
- 		return;
+	show_rspeed_helper( rspeed_epolys, test_epoly,
+			"epolys", "%5.0f epolys", c_alias_polys );
 
- 	scale = (float)(viddef.height)/600;
- 	if(scale < 1)
- 		scale = 1;
+	show_rspeed_helper( rspeed_flares, test_flares,
+			"flares", "%5.0f flares", c_flares );
 
- 	Com_sprintf( prtstring, sizeof( prtstring ),
- 		"%5.5i wpoly %5.5i epoly",  c_brush_polys, c_alias_polys );
+	show_rspeed_helper( rspeed_grasses, test_grasses,
+			"grasses", "%5.0f grasses", c_grasses );
+
+	show_rspeed_helper( rspeed_beams, test_beams,
+			"beams", "%5.0f beams", c_beams );
+	
+	show_rspeed_helper( rspeed_vbobatches, test_vbobatches,
+			"vbobatches", "%5.0f vbobatches", c_vbo_batches );
+	
+	show_rspeed_helper( rspeed_particles, test_particles,
+			"particles", "%5.0f particles", cl.refdef.num_particles );
 
-	font = FNT_AutoGet( CL_consoleFont );
-	box.x = 0;
-	box.y = viddef.height - 6 * font->size;
-	box.width = viddef.width;
-	box.height = 0;
-	FNT_BoundedPrint( font , prtstring , FNT_CMODE_NONE , FNT_ALIGN_CENTER , &box , FNT_colors[ 7 ] );
 }
 
 /*
@@ -1896,7 +2105,7 @@ void SCR_UpdateScreen (void)
 			SCR_CheckDrawIRCString();
 
 			if (scr_timegraph->value)
-				SCR_DebugGraph (cls.frametime*300, 0);
+				SCR_DebugGraph (cls.frametime*600, RGBA(0,0,0,1));
 
 			if (scr_debuggraph->value || scr_timegraph->value || scr_netgraph->value)
 				SCR_DrawDebugGraph ();
@@ -1912,20 +2121,29 @@ void SCR_UpdateScreen (void)
 
 			SCR_DrawLoading ();
 
-			if(cl_drawfps->value)
+			if(cl_drawfps->integer)
 			{
 				SCR_showFPS();
 			}
+			
+			if(cl_drawbandwidth->integer)
+			{
+				SCR_showBandwidth();
+			}
+			
+			if(cl_drawframesover->integer)
+			{
+				SCR_showFramesOver();
+			}
 
-			if(cl_drawtimer->value)
+			SCR_showRSpeeds(); /* map-maker's performance counters */
+			
+			SCR_showAllPerfTests();
+
+			if(cl_drawtimer->integer)
 			{
 				SCR_showTimer();
 			}
-			{
-				extern cvar_t* r_speeds;
-				if( r_speeds->value == 2 )
-					SCR_showRSpeeds();
-			}
 			SCR_DrawPlayerIcon();
 		}
 	}
