@@ -126,6 +126,8 @@ void CL_ParseSteam (void)
 	{
 		s->id = 25; //unused for now
 		s->count = MSG_ReadByte (&net_message);
+		if (!s->count)
+			s->count = 32;
 
 		MSG_ReadPos (&net_message, s->org);
 		MSG_ReadDir (&net_message, s->dir);
@@ -152,7 +154,7 @@ void CL_ParseSteam (void)
 		s->magnitude = 30;
 		s->endtime = cl.time + 10000000;
 		s->think = CL_ParticleSteamEffect;
-		s->thinkinterval = 1;
+		s->thinkinterval = 512/s->count; //~ 60 fps
 		s->nextthink = cl.time;
 	}
 	else
@@ -200,7 +202,7 @@ void CL_ParseFire (void)
 			s->magnitude = 150;//MSG_ReadShort (&net_message);
 			s->endtime = cl.time + 10000000;//MSG_ReadLong (&net_message);
 			s->think = CL_ParticleFireEffect2;
-			s->thinkinterval = 1;
+			s->thinkinterval = 16; //~ 60 fps
 			s->nextthink = cl.time;
 		}
 		else
@@ -252,7 +254,7 @@ void CL_ParseSmoke (void)
 			s->magnitude = 400;//MSG_ReadShort (&net_message);
 			s->endtime = cl.time + 10000000;//MSG_ReadLong (&net_message);
 			s->think = CL_ParticleSmokeEffect2;
-			s->thinkinterval = 1;
+			s->thinkinterval = 16; //~ 60 fps
 			s->nextthink = cl.time;
 		}
 		else
@@ -560,7 +562,7 @@ void CL_ProcessSustain ()
 
 	for (i=0, s=cl_sustains; i< MAX_SUSTAINS; i++, s++)
 	{
-		if (s->id)
+		if (s->id && CM_inPVS (cl.refdef.vieworg, s->org))
 		{
 			if ((s->endtime >= cl.time) && (cl.time >= s->nextthink))
 			{
