@@ -33,9 +33,6 @@ int			gun_frame;
 struct model_s	*gun_model;
 //=============
 
-int r_numflares;
-flare_t r_flares[MAX_FLARES];
-
 int r_numgrasses;
 grass_t r_grasses[MAX_GRASSES];
 
@@ -44,7 +41,7 @@ beam_t	r_beams[MAX_BEAMS];
 
 extern cvar_t *cl_showPlayerNames;
 extern cvar_t *name;
-extern char map_music[128];
+extern char map_music[260];
 extern cvar_t *background_music;
 extern qboolean IsVisible(vec3_t org1,vec3_t org2);
 extern void R_VCFreeFrame(void);
@@ -139,10 +136,13 @@ V_AddLight
 
 =====================
 */
+extern	cvar_t	*gl_dynamic;
 void V_AddLight (vec3_t org, float intensity, float r, float g, float b)
 {
 	dlight_t	*dl;
 
+	if (gl_dynamic->integer == 0)
+		return;
 	if (r_numdlights >= MAX_DLIGHTS)
 		return;
 	dl = &r_dlights[r_numdlights++];
@@ -151,27 +151,6 @@ void V_AddLight (vec3_t org, float intensity, float r, float g, float b)
 	dl->color[0] = r;
 	dl->color[1] = g;
 	dl->color[2] = b;
-	dl->team = 0;  //added for team lights
-}
-/*
-=====================
-V_AddTeamLight
-
-=====================
-*/
-void V_AddTeamLight (vec3_t org, float intensity, float r, float g, float b, int team)
-{
-	dlight_t	*dl;
-
-	if (r_numdlights >= MAX_DLIGHTS)
-		return;
-	dl = &r_dlights[r_numdlights++];
-	VectorCopy (org, dl->origin);
-	dl->intensity = intensity;
-	dl->color[0] = r;
-	dl->color[1] = g;
-	dl->color[2] = b;
-	dl->team = team;
 }
 
 /*
@@ -201,30 +180,13 @@ V_TestParticles
 If cl_testparticles is set, create 4096 particles in the view
 ================
 */
+void CL_LogoutEffect (vec3_t org, int type);
 void V_TestParticles (void)
 {
-#if 0
-//TODO: this will probably need moving to cl_fx.c and to use new_particle
-	particle_t	*p;
-	int			i, j;
-	float		d, r, u;
-
-	r_numparticles = MAX_PARTICLES;
-	for (i=0 ; i<r_numparticles ; i++)
-	{
-		d = i*0.25;
-		r = 4*((i&7)-3.5);
-		u = 4*(((i>>3)&7)-3.5);
-		p = &r_particles[i];
-
-		for (j=0 ; j<3 ; j++)
-			p->current_origin[j] = cl.refdef.vieworg[j] + cl.v_forward[j]*d +
-			cl.v_right[j]*r + cl.v_up[j]*u;
-
-		p->current_color = 8;
-		p->current_alpha = cl_testparticles->value;
-	}
-#endif
+    vec3_t  org;
+    
+    VectorMA (cl.refdef.vieworg, 128, cl.v_forward, org);
+    CL_LogoutEffect (org, MZ_LOGOUT);
 }
 
 /*
@@ -595,8 +557,11 @@ float CalcFov (float fov_x, float width, float height)
 SCR_DrawCrosshair
 =================
 */
-void SCR_DrawCrosshair (void)
+extern cvar_t *hand;
+void SCR_DrawCrosshair (refdef_t *fd)
 {
+	int x, y, crosshairposition;
+	
 	if (!strcmp(crosshair->string, "none"))
 		return;
 
@@ -609,8 +574,44 @@ void SCR_DrawCrosshair (void)
 	if (!crosshair_pic[0])
 		return;
 
-	Draw_Pic (scr_vrect.x + ((scr_vrect.width - crosshair_width)>>1)
-	, scr_vrect.y + ((scr_vrect.height - crosshair_height)>>1), crosshair_pic);
+	x = fd->x + ((fd->width - crosshair_width)>>1);
+	y = fd->y + ((fd->height - crosshair_height)>>1);
+	
+	// get rid of the old crosshair adjustment built into the texture
+	x -= 4;
+	y -= 4;
+	
+	// add a new crosshair adjustment offset
+	crosshairposition = cl.frame.playerstate.stats[STAT_FLAGS] & STAT_FLAGS_CROSSHAIRPOSITION;
+	if (crosshairposition != STAT_FLAGS_CROSSHAIRCENTER)
+	{
+		double x_offs, y_offs;
+		switch (crosshairposition)
+		{
+			default: // other crosshair positions reserved for future use
+			case STAT_FLAGS_CROSSHAIRPOS1: // use the original default
+				x_offs = y_offs = 4.0;
+				break;
+			case STAT_FLAGS_CROSSHAIRPOS2:
+				x_offs = 2.0;
+				y_offs = 3.0;
+				break;
+		}
+		y_offs *= (double)fd->height/480.0;
+		x_offs *= (double)fd->width/640.0;
+		if (y_offs-(int)y_offs >= 0.5)
+			y_offs = (int)y_offs + 1;
+		if (x_offs-(int)x_offs >= 0.5)
+			x_offs = (int)x_offs + 1;
+		if (hand->integer == 1)
+			x_offs = -x_offs;
+		else if (hand->integer == 2)
+			x_offs = 0;
+		x += x_offs;
+		y += y_offs;
+	}
+	
+	Draw_Pic (x, y, crosshair_pic);
 }
 
 qboolean InFront (vec3_t target)
@@ -802,6 +803,10 @@ V_RenderView
 
 ==================
 */
+extern cvar_t		*scr_netgraph;
+extern cvar_t		*scr_timegraph;
+extern cvar_t		*scr_debuggraph;
+extern cvar_t		*scr_graphheight;
 void V_RenderView( float stereo_separation )
 {
 	extern int entitycmpfnc( const entity_t *, const entity_t * );
@@ -812,11 +817,17 @@ void V_RenderView( float stereo_separation )
 	if (!cl.refresh_prepped)
 		return;			// still loading
 
-	if (cl_timedemo->value)
+	if ( cl_timedemo && cl_timedemo->integer )
 	{
-		if (!cl.timedemo_start)
+		if ( cl.timedemo_start > 0 )
+		{ /* frame counter increment for timedemo benchmark */
+			cl.timedemo_frames++;
+		}
+		else
+		{ /* time demo start trigger */
 			cl.timedemo_start = Sys_Milliseconds ();
-		cl.timedemo_frames++;
+			cl.timedemo_frames = 0;
+		}
 	}
 
 	// an invalid frame will just use the exact previous refdef
@@ -864,6 +875,10 @@ void V_RenderView( float stereo_separation )
 		cl.refdef.y = scr_vrect.y;
 		cl.refdef.width = scr_vrect.width;
 		cl.refdef.height = scr_vrect.height;
+		
+		if (scr_debuggraph->integer || scr_timegraph->integer || scr_netgraph->integer)
+			cl.refdef.height -= scr_graphheight->integer;
+		
 		cl.refdef.fov_y = CalcFov (cl.refdef.fov_x, cl.refdef.width, cl.refdef.height);
 		cl.refdef.time = cl.time*0.001;
 
@@ -891,8 +906,6 @@ void V_RenderView( float stereo_separation )
 		cl.refdef.num_dlights = r_numdlights;
 		cl.refdef.dlights = r_dlights;
 		cl.refdef.lightstyles = r_lightstyles;
-		cl.refdef.num_flares = r_numflares;
-        cl.refdef.flares = r_flares;
 		cl.refdef.num_grasses = r_numgrasses;
 		cl.refdef.grasses = r_grasses;
 		cl.refdef.num_beams = r_numbeams;
@@ -918,7 +931,7 @@ void V_RenderView( float stereo_separation )
 		fprintf( log_stats_file, "%i,%i,%i,",r_numentities, r_numdlights, r_numparticles);
 
 
-	SCR_DrawCrosshair ();
+	SCR_DrawCrosshair (&cl.refdef);
 
 	if(cl_showPlayerNames->integer) {
 		if(cl_showPlayerNames->integer == 2)
