@@ -17,6 +17,11 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "client.h"
 
 /*
@@ -91,6 +96,8 @@ keyname_t keynames[] =
 	{"MOUSE5", K_MOUSE5},
 	{"MOUSE6", K_MOUSE6},
 	{"MOUSE7", K_MOUSE7},
+	{"MOUSE8", K_MOUSE8},
+	{"MOUSE9", K_MOUSE9},
 
 	{"JOY1", K_JOY1},
 	{"JOY2", K_JOY2},
@@ -164,6 +171,13 @@ keyname_t keynames[] =
 ==============================================================================
 */
 
+
+void Key_ClearTyping (void)
+{
+	key_lines[edit_line][1] = 0;	// clear any typing
+	key_linelen = key_linepos = 1;
+}
+
 qboolean        Cmd_IsComplete(char *cmd);
 extern void		Q_strncpyz( char *dest, const char *src, size_t size );
 void CompleteCommand (void)
@@ -366,7 +380,7 @@ void Key_Console (int key)
 		if (key_linepos < key_linelen) {
 			if ( keydown[K_CTRL] )
 			{
-				qboolean found_char = false, printable;
+				qboolean found_char = false, printable = false;
 				while (key_linepos < key_linelen && (! found_char || printable))
 				{
 					key_linepos ++;
@@ -422,22 +436,24 @@ void Key_Console (int key)
 
 	if (key == K_PGUP || key == K_KP_PGUP || key == K_MWHEELUP )
 	{
-		con.display -= 2;
+		CON_console.displayOffset += 64;
 		return;
 	}
 
 	if (key == K_PGDN || key == K_KP_PGDN || key == K_MWHEELDOWN )
 	{
-		con.display += 2;
-		if (con.display > con.current)
-			con.display = con.current;
+		if ( CON_console.displayOffset >= 64 ) {
+			CON_console.displayOffset -= 64;
+		} else {
+			CON_console.displayOffset = 0;
+		}
 		return;
 	}
 
 	if (key == K_HOME || key == K_KP_HOME )
 	{
 		if ( keydown[K_CTRL] )
-			con.display = con.current - con.totallines + 10;
+			CON_console.displayOffset = 0x7fffffff;
 		else
 			key_linepos = 1;
 		return;
@@ -446,7 +462,7 @@ void Key_Console (int key)
 	if (key == K_END || key == K_KP_END )
 	{
 		if ( keydown[K_CTRL] )
-			con.display = con.current;
+			CON_console.displayOffset = 0;
 		else
 			key_linepos = key_linelen;
 		return;
@@ -890,7 +906,7 @@ void Key_Event (int key, qboolean down, unsigned time)
 			return;	// ignore most autorepeats
 
 		if (key >= 200 && !keybindings[key])
-			Com_Printf ("%s is unbound, hit F4 to set.\n", Key_KeynumToString (key) );
+			Com_Printf ("%s is not bound to a command.\n", Key_KeynumToString (key) );
 	}
 	else
 	{
@@ -910,7 +926,7 @@ void Key_Event (int key, qboolean down, unsigned time)
 			return;
 		}
 
-		Con_ToggleConsole_f ();
+		CON_ToggleConsole( );
 		return;
 	}
 
