@@ -45,6 +45,13 @@ typedef struct
 	int dataofs; // chunk starts this many bytes from file start
 } pcminfo_t;
 
+/* return 0 for little endian, 1 for big endian */
+int  bigendian( void )
+{
+	long one = 1;
+	return !(*((char*)(&one)));
+}
+
 /*
  ==
  .WAV file parsing
@@ -120,7 +127,7 @@ qboolean ReadWavFile( const char *name, byte *wav, int wavlength,
 	int format;
 	int samples;
 
-	memset( info, 0, sizeof( info ) );
+	memset( info, 0, sizeof(pcminfo_t) );
 
 	if( !wav )
 		return false;
@@ -181,6 +188,15 @@ qboolean ReadWavFile( const char *name, byte *wav, int wavlength,
 	info->samples = samples;
 	info->dataofs = data_p - wav;
 
+	if ( (info->width == 2) && bigendian() )
+	{ /* for big-endian byte swap little-endian 16-bit samples */
+		short *psample = (short*)data_p;
+		while ( samples-- )
+		{
+			*psample++ = GetLittleShort(); /* note: increments data_p */
+		}
+	}
+
 	return true;
 }
 
@@ -268,6 +284,7 @@ long ovbfr_tell( void *datasource )
 // function pointers for Ogg Vorbis file-like io
 ov_callbacks ovbfr_callbacks = { ovbfr_read, ovbfr_seek, NULL, ovbfr_tell };
 
+
 /*
  ============
  ReadVorbisFile
@@ -287,10 +304,11 @@ qboolean ReadVorbisFile( const char *name, byte *pdata, int filelength,
 	size_t bytes_remaining;
 	long read_result;
 	int bitstream;
+	int bigendianp;
 
 	Com_DPrintf("ReadVorbisFile: %s\n", name );
 
-	memset( info, 0, sizeof( info ) ); // clear return data
+	memset( info, 0, sizeof(pcminfo_t) ); // clear return data
 	*pcmbfr = NULL;
 
 	info->width = 2; // always 16-bit
@@ -329,9 +347,10 @@ qboolean ReadVorbisFile( const char *name, byte *pdata, int filelength,
 		return false;
 	}
 
+	bigendianp = bigendian();
 	do
-	{ // decode Ogg Vorbis data to little-endian, 16-bit, signed PCM data
-		read_result = ov_read( &vf, pdest, bytes_remaining, 0, 2, 1, &bitstream );
+	{ // decode Ogg Vorbis data to host endian, 16-bit, signed PCM data
+		read_result = ov_read( &vf, pdest, bytes_remaining, bigendianp, 2, 1, &bitstream );
 		if ( read_result > 0 )
 		{
 			bytes_remaining -= (size_t)read_result;
