@@ -58,7 +58,8 @@ with this program; if not, write to the Free Software Foundation, Inc.,
  * Sound CVARS
  */
 #define MAX_SRC 128
-#define MIN_SRC_DEFAULT 96
+#define MAX_SRC_DEFAULT 96
+#define MIN_SRC_DEFAULT 64
 /*--- previous system ---*/
 // These 2 are defined in cl_main.c
 //cvar_t *background_music; //     enable/disable music
@@ -240,7 +241,7 @@ const struct source_default_s
 	ALfloat cone_outer_gain;
 	ALfloat cone_inner_angle;
 	ALfloat cone_outer_angle;
-	int stop_timer_delay;
+	// int stop_timer_delay;
 } source_default =
 {
 		(ALfloat)1.0f, //    AL_PITCH
@@ -1812,7 +1813,7 @@ void S_Shutdown( void )
 
  ==
  */
-sfx_t *S_FindName( char *name, qboolean create )
+sfx_t *S_FindName( const char *name, qboolean create )
 {
 	sfxlink_t *sfxlink;
 	sfx_t *sfx = NULL;
@@ -1998,6 +1999,38 @@ void calModelFilename( entity_state_t *ent, const char *basename,
 	}
 }
 
+/*
+ ==
+ S_RegisterSoundsForPlayer()
+
+ Register the model-specific sounds (pain, jumping, etc.) that should be
+ present for each player model.
+ 
+ This function is called from the model loading code. It relies on the fact
+ that the server precaches sounds before models, so the generic (fallback)
+ version of each model-specific sound is already in the precache list at this
+ point.
+ ==
+ */
+void S_RegisterSoundsForPlayer (char *playername)
+{
+	int		i;
+	char	soundpath[MAX_QPATH];
+	
+	return; // FIXME: figure out why Windows doesn't like this.
+	
+	for (i=1 ; i<MAX_SOUNDS ; i++)
+	{
+		if (!cl.configstrings[CS_SOUNDS+i][0])
+			break;
+		if (cl.configstrings[CS_SOUNDS+i][0] != '*')
+			continue;
+		cl.sound_precache[i] = S_RegisterSound (cl.configstrings[CS_SOUNDS+i]);
+		Com_sprintf (soundpath, MAX_QPATH, "#players/%s/%s", playername, &cl.configstrings[CS_SOUNDS+i][1]);
+		S_RegisterSound (soundpath);
+	}
+}
+
 /*
  ==
 
@@ -2035,7 +2068,7 @@ void S_StartSound( vec3_t origin, int entnum, int entchannel, sfx_t *arg_sfx,
 	{ // background music from speakers is obsolete
 		return;
 	}
-
+	
 	// cull sound by distance
 	attn_class = (int)(attenuation + 0.5f);
 	if( attn_class != ATTN_NONE )
@@ -2254,7 +2287,7 @@ void S_StartMenuMusic( void )
 	S_StartMusic( "music/menumusic.wav" );
 }
 
-extern char map_music[128]; // declared and set in r_model.c
+extern char map_music[260]; // declared and set in r_model.c
 
 void S_StartMapMusic( void )
 {
@@ -2271,7 +2304,7 @@ void S_StartMapMusic( void )
 
  ==
  */
-void S_StartLocalSound( char *qfilename )
+void S_StartLocalSound( const char *qfilename )
 {
 	sfx_t *sfx;
 	src_t *src;
@@ -2865,7 +2898,7 @@ void sndCvarInit( void )
 	s_volume = Cvar_Get( "s_volume", "0.1", CVAR_ARCHIVE );
 	// OpenAL CVARs
 	s_doppler = Cvar_Get( "s_doppler", "0.0", CVAR_ARCHIVE );
-	Com_sprintf(cvarset, sizeof(cvarset),"%i", MAX_SRC );
+	Com_sprintf(cvarset, sizeof(cvarset),"%i", MAX_SRC_DEFAULT );
 	s_maxsources = Cvar_Get( "s_maxsources", cvarset , CVAR_ARCHIVE );
 	Com_sprintf(cvarset, sizeof(cvarset),"%i", MIN_SRC_DEFAULT );
 	s_minsources = Cvar_Get( "s_minsources", cvarset, CVAR_ARCHIVE );
