@@ -26,7 +26,7 @@ typedef struct vrect_s
 
 typedef struct
 {
-	unsigned		width, height;			// coordinates from main game
+	int		width, height;			// coordinates from main game
 } viddef_t;
 
 extern	viddef_t	viddef;				// global video state
@@ -37,9 +37,6 @@ void	VID_Shutdown (void);
 void	VID_CheckChanges (void);
 
 qboolean	VID_GetModeInfo( int *width, int *height, int mode );
+void		R_Register (void); //defined in ref_gl
 void		VID_MenuInit( void );
 void		VID_NewWindow( int width, int height );
-
-void	VID_MenuInit( void );
-void	VID_MenuDraw( void );
-const char *VID_MenuKey( int );
