@@ -157,6 +157,9 @@
 #define ITEMLIST_DEATHBALL			38
 #define ITEMLIST_HOVER				39
 
+//mind eraser
+#define ITEMLIST_MINDERASER			40
+
 // Special CONTENT mask for acebots, so they can detect and climb ladders
 // replaces MASK_OPAQUE defined in game/q_shared.h
 #define BOTMASK_OPAQUE (CONTENTS_SOLID|CONTENTS_SLIME|CONTENTS_LAVA|CONTENTS_LADDER)
