@@ -203,33 +203,58 @@ void ACEAI_PickLongRangeGoal(edict_t *self)
 	int current_node,goal_node=0;
 	edict_t *goal_ent=NULL, *ent;
 	float cost;
-#ifdef CTFNODES
 	gitem_t *flag1_item, *flag2_item;
 	qboolean hasFlag = false;
 
 	// look for a target
-
+	
+	flag1_item = FindItemByClassname("item_flag_red");
+	flag2_item = FindItemByClassname("item_flag_blue");
+	
 	//if flag in possession, try to find base nodes
 	if(ctf->value)
 	{
-		flag1_item = FindItemByClassname("item_flag_red");
-		flag2_item = FindItemByClassname("item_flag_blue");
-
 		if (self->client->pers.inventory[ITEM_INDEX(flag1_item)])
 		{
 			current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_BLUEBASE);
-			hasFlag = true;
+			if(current_node == -1)
+				current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
+			else
+				hasFlag = true;
 		}
 		else if (self->client->pers.inventory[ITEM_INDEX(flag2_item)])
 		{
 			current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_REDBASE);
-			hasFlag = true;
+			if(current_node == -1)
+				current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
+			else
+				hasFlag = true;
+		}
+		else
+			current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
+	}
+	else if(g_tactical->value) //when a base's laser barriers shut off, go into a more direct attack route
+	{
+		if (self->ctype == 1 && (!tacticalScore.alienComputer || !tacticalScore.alienPowerSource))
+		{
+			current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_BLUEBASE);
+			if(current_node == -1)
+				current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
+			else
+				hasFlag = true; //we can use this from CTF - bots will ignore most anything and attack base components
+		}
+		else if (self->ctype == 0 && (!tacticalScore.humanComputer || !tacticalScore.humanPowerSource))
+		{
+			current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_REDBASE);
+			if(current_node == -1)
+				current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
+			else
+				hasFlag = true;
 		}
 		else
 			current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
 	}
 	else
-#endif
 		current_node = ACEND_FindClosestReachableNode(self,NODE_DENSITY,NODE_ALL);
 
 	self->current_node = current_node;
@@ -241,10 +266,9 @@ void ACEAI_PickLongRangeGoal(edict_t *self)
 		self->goal_node = -1;
 		return;
 	}
-#ifdef CTFNODES
+
 	if(!hasFlag)
 	{
-#endif
 		///////////////////////////////////////////////////////
 		// Items
 		///////////////////////////////////////////////////////
@@ -276,7 +300,7 @@ void ACEAI_PickLongRangeGoal(edict_t *self)
 		///////////////////////////////////////////////////////
 		// This should be its own function and is for now just
 		// finds a player to set as the goal.
-		for(i=0;i<game.maxclients;i++)
+		for(i = 0; i < game.maxclients; i++)
 		{
 			ent = g_edicts + i + 1;
 			if(ent == self || !ent->inuse || (ent->client->invis_framenum > level.framenum))
@@ -301,14 +325,44 @@ void ACEAI_PickLongRangeGoal(edict_t *self)
 				goal_ent = ent;
 			}
 		}
-#ifdef CTFNODES
 	}
 	else
 	{
-		goal_node = current_node;
+		qboolean hadCTFnode = false;
+		//we need to get node at the end of the path
+		//We can walk the node table and get the last linked node of this type further down the list than this node
+		for(i = 0;i < bot_numnodes; i++)
+		{
+			if (self->client->pers.inventory[ITEM_INDEX(flag1_item)] || (g_tactical->value && self->ctype == 1))
+			{
+				if(nodes[i].type == NODE_BLUEBASE)
+				{
+					goal_node = i; 
+					hadCTFnode = true;
+				}
+			}
+
+			if (self->client->pers.inventory[ITEM_INDEX(flag2_item)] || (g_tactical->value && self->ctype == 0))
+			{
+				if(nodes[i].type == NODE_REDBASE)
+				{
+					goal_node = i;
+					hadCTFnode = true;
+				}
+			}
+		}
+
+		if(!hadCTFnode)
+		{
+			self->state = STATE_WANDER;
+			self->wander_timeout = level.time + 1.0;
+			self->goal_node = -1;
+			return;
+		}
+
 		best_weight = 1.0;
 	}
-#endif
+
 	// If do not find a goal, go wandering....
 	if(best_weight == 0.0 || goal_node == INVALID)
 	{			
@@ -377,10 +431,11 @@ void ACEAI_PickShortRangeGoal(edict_t *self)
 
 		// Missle avoidance code
 		// Set our movetarget to be the rocket or grenade fired at us.
-		if(strcmp(target->classname,"rocket")==0 || strcmp(target->classname,"grenade") ==0)
+		if(strcmp(target->classname,"rocket") == 0 || strcmp(target->classname,"grenade") == 0
+			|| strcmp(target->classname,"seeker") == 0)
 		{
 			if(debug_mode)
-				debug_printf("ROCKET ALERT!\n");
+				debug_printf("PROJECTILE ALERT!\n");
 
 			self->movetarget = target;
 			return;
@@ -437,12 +492,14 @@ qboolean ACEAI_infront (edict_t *self, edict_t *other)
 
 	vehicle = FindItemByClassname("item_bomber");
 
-	if (self->client->pers.inventory[ITEM_INDEX(vehicle)]) {
+	if (self->client->pers.inventory[ITEM_INDEX(vehicle)]) 
+	{
 		return true;	//do this so that they aren't getting lost and just flying off
 	}
 	vehicle = FindItemByClassname("item_strafer");
 
-	if (self->client->pers.inventory[ITEM_INDEX(vehicle)]) {
+	if (self->client->pers.inventory[ITEM_INDEX(vehicle)]) 
+	{
 		return true;	//do this so that they aren't getting lost and just flying off
 	}
 
@@ -468,30 +525,35 @@ qboolean ACEAI_FindEnemy(edict_t *self)
 	edict_t *target;
 	edict_t	*ent;
 
-	if(ctf->value) {
+	if(ctf->value) 
+	{
 		flag1_item = FindItemByClassname("item_flag_red");
 		flag2_item = FindItemByClassname("item_flag_blue");
 	}
 
-	if(self->in_deathball && (self->health > 25)) { //cannot, or should not, fire at players when in a deathball
+	if(self->in_deathball && (self->health > 25)) 
+	{ 
+		//cannot, or should not, fire at players when in a deathball
 		//look for goal - if health is too low, drop the ball and fight back
 		target = findradius(NULL, self->s.origin, 200);
 		self->enemy = NULL;
 		while(target)
 		{
-			if(target->classname == NULL) {
+			if(target->classname == NULL) 
+			{
 				self->enemy = NULL;
 				return false;
 			}
-			if(self->dmteam == RED_TEAM && (strcmp(target->classname, "item_blue_dbtarget") == 0))
+			if(self->dmteam == RED_TEAM && target->classname == "item_blue_dbtarget")
 				self->enemy = target;
-			else if(self->dmteam == BLUE_TEAM && (strcmp(target->classname, "item_red_dbtarget") == 0))
+			else if(self->dmteam == BLUE_TEAM && target->classname == "item_red_dbtarget")
 				self->enemy = target;
-			else if(self->dmteam == NO_TEAM && (strcmp(target->classname, "item_dbtarget") == 0))
+			else if(self->dmteam == NO_TEAM && target->classname == "item_dbtarget")
 				self->enemy = target;
 			target = findradius(target, self->s.origin, 200);
 		}
-		if(self->enemy) {
+		if(self->enemy) 
+		{
 			//safe_bprintf(PRINT_MEDIUM, "Target Aquired!\n");
 			self->movetarget = self->enemy;
 			self->goalentity= self->enemy; //face it, and fire
@@ -500,26 +562,31 @@ qboolean ACEAI_FindEnemy(edict_t *self)
 		else
 			return false;
 	}
+
 	//only look for these if your team's spider is vulnerable
 	if(tca->value && ((self->dmteam == RED_TEAM && red_team_score < 2) || (self->dmteam == BLUE_TEAM && blue_team_score < 2))) {
 		target = findradius(NULL, self->s.origin, 300);
 		self->enemy = NULL;
 		while(target)
 		{
-			if(target->classname == NULL) {
+			if(target->classname == NULL) 
+			{
 				self->enemy = NULL;
 				return false;
 			}
-			if(self->dmteam == RED_TEAM) {
-				if(strcmp(target->classname, "misc_bluespidernode") == 0)
+			if(self->dmteam == RED_TEAM) 
+			{
+				if(target->classname == "misc_bluespidernode")
 					self->enemy = target;
 			}
-			if(self->dmteam == BLUE_TEAM) {
-				if(strcmp(target->classname, "misc_redspidernode") == 0)
+			else if(self->dmteam == BLUE_TEAM) 
+			{
+				if(target->classname == "misc_redspidernode")
 					self->enemy = target;
 			}
 			target = findradius(target, self->s.origin, 300);
-			if(self->enemy) {
+			if(self->enemy) 
+			{
 				//safe_bprintf(PRINT_MEDIUM, "Target Aquired!\n");
 				self->movetarget = self->enemy;
 				self->goalentity= self->enemy; //face it, and fire
@@ -530,6 +597,53 @@ qboolean ACEAI_FindEnemy(edict_t *self)
 		}
 	}
 
+	if(g_tactical->value) 
+	{
+		target = findradius(NULL, self->s.origin, 200);
+		self->enemy = NULL;
+		while(target)
+		{
+			if(target->classname == NULL) 
+			{
+				self->enemy = NULL;
+				return false;
+			}
+			if(self->ctype == 1) 
+			{
+				if(target->classname == "hbomb")
+					return false; //prevents them from accidently destorying a planted bomb
+				else if(target->classname == "alien computer")
+					self->enemy = target;
+				else if(target->classname == "alien powersrc")
+					self->enemy = target;
+				else if(target->classname == "alien ammodepot")
+					self->enemy = target;
+				else if(target->classname == "alien backupgen")
+					self->enemy = target;
+			}
+			else if(self->ctype == 0)
+			{
+				if(target->classname == "abomb")
+					return false;
+				else if(target->classname == "human computer")
+					self->enemy = target;
+				else if(target->classname == "human powersrc")
+					self->enemy = target;
+				else if(target->classname == "human ammodepot")
+					self->enemy = target;
+				else if(target->classname == "human backupgen")
+					self->enemy = target;
+			}		
+			target = findradius(target, self->s.origin, 200);
+			if(self->enemy) 
+			{
+				self->movetarget = self->enemy;
+				self->goalentity= self->enemy; //face it, and fire
+				return true;
+			}
+		}
+	}
+
 	if(self->oldenemy != NULL) //(was shot from behind)
 	{
 		if(!OnSameTeam(self, self->oldenemy))
@@ -569,7 +683,7 @@ qboolean ACEAI_FindEnemy(edict_t *self)
 
 		self->enemy = bestenemy;
 		//if using a blaster, and it's far away, don't fire, it's pointless
-		if((self->client->pers.weapon == FindItem("blaster")) && (bestweight > 1500)) {
+		if((self->client->pers.weapon == FindItem("Blaster") || self->client->pers.weapon == FindItem("Alien Blaster")) && (bestweight > 1500)) {
 			self->enemy = NULL;
 			return false;
 		}
@@ -598,8 +712,9 @@ qboolean ACEAI_CheckShot(edict_t *self)
 
 	tr = gi.trace (self->s.origin, tv(-8,-8,-8), tv(8,8,8), self->enemy->s.origin, self, MASK_SOLID);
 
-	// Blocked, do not shoot
-	if ( tr.fraction < 1.0f )
+	// Blocked, do not shoot - JKD 4/25/2013 - changed the threshold from 1.0 to 0.6 - this seemed to work much better and solved problems where bots 
+	// would not fire at items even though they had an obviously clear shot.
+	if ( tr.fraction < 0.6f )
 		return false;
 
 	return true;
@@ -817,9 +932,44 @@ void ACEAI_ChooseWeapon(edict_t *self)
 		return;
 	}
 
+	//drop bombs if they have one, and the target is something that we want to plant a bomb near
+	if(g_tactical->integer) 
+	{
+		if(self->has_bomb && self->ctype == 0 
+			&& (self->enemy->classname == "human computer" || self->enemy->classname == "human powersrc" 
+			|| self->enemy->classname == "human ammodepot"))
+		{		
+			if(range < 300.0f)
+			{
+				if(ACEIT_ChangeWeapon( self, FindItem( "Alien Bomb" )))
+					return;
+			}
+		}
+		else if(self->has_bomb && self->ctype == 1 
+			&& (self->enemy->classname == "alien computer" || self->enemy->classname == "alien powersrc" 
+			|| self->enemy->classname == "alien ammodepot"))
+		{
+			if(range < 300.0f)
+			{
+				if(ACEIT_ChangeWeapon( self, FindItem( "Human Bomb" )))
+					return;
+			}
+		}
+	}
+
 	// what is the bot's favorite weapon?
-	// The bot will always check for it's favorite weapon first,
-	// which is set in the bot's config file.
+	// The bot will always check for it's favorite weapon first, 
+	// which is set in the bot's config file, 
+	// unless it has a minderaser
+	if ( clear_shot )
+	{
+		if ( ACEIT_ChangeWeapon( self, FindItem( "Minderaser" ) ) )
+		{
+			//keep same accurace as if firing a rocket
+			self->accuracy = self->weapacc[ACCURACY_ROCKETLAUNCHER]; 
+			return;
+		}
+	}
 	if ( !strcmp( self->faveweap, "Alien Vaporizer" ) && self->skill > 1 )
 	{
 		if ( ACEIT_ChangeWeapon( self, FindItem( self->faveweap ) ) )
@@ -896,6 +1046,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 
 	// now go through normal weapon favoring routine
 	//  always favor the Vaporizor, unless close, then use the violator
+#ifndef TACTICAL
 	if ( range < 200.0f && self->skill > 0 )
 	{
 		selected = ACEIT_ChangeWeapon( self, FindItem( "Violator" ));
@@ -903,6 +1054,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 		self->accuracy = 1.0f;
 		return;
 	}
+#endif
 	if ( self->skill > 1 )
 	{
 		if ( ACEIT_ChangeWeapon( self, FindItem( "Alien Vaporizer" )))
@@ -952,7 +1104,10 @@ void ACEAI_ChooseWeapon(edict_t *self)
 		return;
 	}
 
-	selected = ACEIT_ChangeWeapon( self, FindItem( "Blaster" ) );
+	if(g_tactical->integer && self->ctype == 0)
+		selected = ACEIT_ChangeWeapon( self, FindItem( "Alien Blaster" ) );
+	else
+		selected = ACEIT_ChangeWeapon( self, FindItem( "Blaster" ) );
 	assert( selected );
 	self->accuracy = self->weapacc[ACCURACY_BLASTER];
 
