@@ -1,3 +1,26 @@
+/*
+Copyright (C) 1997-2001 Id Software, Inc.
+Copyright (C) 1998 Steve Yeager
+Copyright (C) 2010 COR Entertainment, LLC.
+
+See below for Steve Yeager's original copyright notice.
+Modified to GPL in 2002.
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+
+See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 ///////////////////////////////////////////////////////////////////////
 //
 //  ACE - Quake II Bot Base Code
@@ -106,9 +129,9 @@ qboolean ACEMV_CanMove(edict_t *self, int direction)
 	VectorSet(offset, 36, 0, -400);
 	G_ProjectSource (self->s.origin, offset, forward, right, end);
 
-	tr = gi.trace(start, NULL, NULL, end, self, (CONTENTS_SOLID|CONTENTS_SLIME|CONTENTS_LAVA|CONTENTS_WINDOW));
-
-	if(tr.fraction > 0.3 || tr.contents & (CONTENTS_LAVA|CONTENTS_SLIME|CONTENTS_MIST))
+	tr = gi.trace(start, NULL, NULL, end, self,
+			(CONTENTS_SOLID|CONTENTS_SLIME|CONTENTS_MIST|CONTENTS_LAVA|CONTENTS_WINDOW));
+	if(tr.fraction > 0.3 || (tr.contents & (CONTENTS_LAVA|CONTENTS_SLIME|CONTENTS_MIST)) )
 	{
 		if(debug_mode)
 			debug_printf("%s: move blocked\n",self->client->pers.netname);
@@ -131,6 +154,12 @@ qboolean ACEMV_SpecialMove(edict_t *self, usercmd_t *ucmd)
 	vec3_t dir,forward,right,start,end,offset;
 	vec3_t top;
 	trace_t tr;
+	short mSpeed;
+
+	if(g_tactical->integer)
+		mSpeed = 200;
+	else 
+		mSpeed = 400;
 
 	// Get current direction
 	VectorCopy(self->client->ps.viewangles,dir);
@@ -162,7 +191,9 @@ qboolean ACEMV_SpecialMove(edict_t *self, usercmd_t *ucmd)
 		if(!tr.allsolid)
 		{
 			if(ACEMV_CanMove(self, MOVE_FORWARD))
-				ucmd->forwardmove = 400;
+				ucmd->forwardmove = mSpeed;
+			else if(ACEMV_CanMove(self, MOVE_BACK))
+				ucmd->forwardmove = -mSpeed;
 			ucmd->upmove = -400;
 			return true;
 		}
@@ -175,7 +206,9 @@ qboolean ACEMV_SpecialMove(edict_t *self, usercmd_t *ucmd)
 		if(!tr.allsolid)
 		{
 			if(ACEMV_CanMove(self, MOVE_FORWARD))
-				ucmd->forwardmove = 400;
+				ucmd->forwardmove = mSpeed;
+			else if(ACEMV_CanMove(self, MOVE_BACK))
+				ucmd->forwardmove = -mSpeed;
 			ucmd->upmove = 400;
 			return true;
 		}
@@ -199,10 +232,22 @@ qboolean ACEMV_CheckEyes(edict_t *self, usercmd_t *ucmd)
 	vec3_t  leftstart, rightstart,focalpoint;
 	vec3_t  upstart,upend;
 	vec3_t  dir,offset;
-
+	short mSpeed;
 	trace_t traceRight,traceLeft,traceUp, traceFront; // for eyesight
-
+	
+	if(g_tactical->integer)
+		mSpeed = 200;
+	else 
+		mSpeed = 400;
+	
 	// Get current angle and set up "eyes"
+
+/* make sure bot's "eyes" are straight ahead
+ * something is going wrong after rocket jump, and "eyes" are always down
+ * and bot runs in circle.
+ */
+	self->s.angles[PITCH] = 0.0f;
+
 	VectorCopy(self->s.angles,dir);
 	AngleVectors (dir, forward, right, NULL);
 
@@ -224,15 +269,15 @@ qboolean ACEMV_CheckEyes(edict_t *self, usercmd_t *ucmd)
 	{
 		ucmd->upmove = 400;
 		if(ACEMV_CanMove(self, MOVE_FORWARD))
-			ucmd->forwardmove = 400;
+			ucmd->forwardmove = mSpeed;
 		return true;
 	}
 
 	// If this check fails we need to continue on with more detailed checks
-	if(traceFront.fraction == 1)
+	if ( traceFront.fraction >= 1.0f )
 	{
-		if(ACEMV_CanMove(self, MOVE_FORWARD))
-			ucmd->forwardmove = 400;
+		if ( ACEMV_CanMove( self, MOVE_FORWARD ) )
+			ucmd->forwardmove = mSpeed; //only try forward, bot should be looking to move in direction of eyes
 		return true;
 	}
 
@@ -265,14 +310,14 @@ qboolean ACEMV_CheckEyes(edict_t *self, usercmd_t *ucmd)
 		traceUp = gi.trace(upstart, NULL, NULL, upend, self, BOTMASK_OPAQUE);
 
 		// If the upper trace is not open, we need to turn.
-		if(traceUp.fraction != 1)
+		if(traceUp.fraction < 1.0f )
 		{
 			if(traceRight.fraction > traceLeft.fraction)
 				self->s.angles[YAW] += (1.0 - traceLeft.fraction) * 45.0;
 			else
 				self->s.angles[YAW] += -(1.0 - traceRight.fraction) * 45.0;
 			if(ACEMV_CanMove(self, MOVE_FORWARD))
-				ucmd->forwardmove = 400;
+				ucmd->forwardmove = mSpeed;
 			return true;
 		}
 	}
@@ -314,15 +359,15 @@ void ACEMV_ChangeBotAngle (edict_t *ent)
 		speed = ent->yaw_speed;
 		if (ideal_yaw > current_yaw)
 		{
-			if (move >= 180)
-				move = move - 360;
+			if (move >= 180.0f)
+				move = move - 360.0f;
 		}
 		else
 		{
-			if (move <= -180)
-				move = move + 360;
+			if (move <= -180.0f)
+				move = move + 360.0f;
 		}
-		if (move > 0)
+		if (move > 0.0f)
 		{
 			if (move > speed)
 				move = speed;
@@ -342,15 +387,15 @@ void ACEMV_ChangeBotAngle (edict_t *ent)
 		speed = ent->yaw_speed;
 		if (ideal_pitch > current_pitch)
 		{
-			if (move >= 180)
-				move = move - 360;
+			if (move >= 180.0f)
+				move = move - 360.0f;
 		}
 		else
 		{
-			if (move <= -180)
-				move = move + 360;
+			if (move <= -180.0f)
+				move = move + 360.0f;
 		}
-		if (move > 0)
+		if (move > 0.0f)
 		{
 			if (move > speed)
 				move = speed;
@@ -369,23 +414,43 @@ void ACEMV_ChangeBotAngle (edict_t *ent)
 ///////////////////////////////////////////////////////////////////////
 void ACEMV_MoveToGoal(edict_t *self, usercmd_t *ucmd)
 {
+	short mSpeed;
 
+	if(g_tactical->integer)
+		mSpeed = 200;
+	else 
+		mSpeed = 400;
 
 	// If a rocket or grenade is around deal with it
 	// Simple, but effective (could be rewritten to be more accurate)
-	if(strcmp(self->movetarget->classname,"rocket")==0 ||
-	   strcmp(self->movetarget->classname,"grenade")==0)
-	{
-		VectorSubtract (self->movetarget->s.origin, self->s.origin, self->move_vector);
-		ACEMV_ChangeBotAngle(self);
+	if(strcmp(self->movetarget->classname,"rocket") == 0 ||
+	   strcmp(self->movetarget->classname,"grenade") == 0 ||
+	   strcmp(self->movetarget->classname,"seeker") == 0 )
+	{		
 		if(debug_mode)
 			debug_printf("%s: Oh crap a rocket!\n",self->client->pers.netname);
 
-		// strafe left/right
-		if(rand()%1 && ACEMV_CanMove(self, MOVE_LEFT))
-				ucmd->sidemove = -400;
-		else if(ACEMV_CanMove(self, MOVE_RIGHT))
-				ucmd->sidemove = 400;
+		if(strcmp(self->movetarget->classname,"seeker") == 0)
+		{
+			VectorSubtract (self->s.origin, self->movetarget->s.origin, self->move_vector);
+			ACEMV_ChangeBotAngle(self);
+
+			//turn and run like hell away from it
+			//to do - needs improvement - bot should run when getting hit by seeker as well
+			if(ACEMV_CanMove(self, MOVE_FORWARD))
+				ucmd->forwardmove = mSpeed;
+		}
+		else
+		{
+			VectorSubtract (self->movetarget->s.origin, self->s.origin, self->move_vector);
+			ACEMV_ChangeBotAngle(self);
+
+			// strafe left/right
+			if(rand()%1 && ACEMV_CanMove(self, MOVE_LEFT))
+					ucmd->sidemove = -mSpeed;
+			else if(ACEMV_CanMove(self, MOVE_RIGHT))
+					ucmd->sidemove = mSpeed;
+		}
 		return;
 
 	}
@@ -394,8 +459,16 @@ void ACEMV_MoveToGoal(edict_t *self, usercmd_t *ucmd)
 		// Set bot's movement direction
 		VectorSubtract (self->movetarget->s.origin, self->s.origin, self->move_vector);
 		ACEMV_ChangeBotAngle(self);
+
+		//try moving forward, if blocked, strafe around it if possible.
 		if(ACEMV_CanMove(self, MOVE_FORWARD))
-			ucmd->forwardmove = 400;
+			ucmd->forwardmove = mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_BACK))
+				ucmd->forwardmove = -mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_RIGHT))
+				ucmd->sidemove = mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_LEFT))
+				ucmd->sidemove = -mSpeed;
 		return;
 	}
 }
@@ -410,7 +483,12 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 	int next_node_type=-1;
 	int i;
 	float c;
+	short mSpeed;
 
+	if(g_tactical->integer)
+		mSpeed = 200;
+	else 
+		mSpeed = 400;
 
 	// Get current and next node back from nav code.
 	if(!ACEND_FollowPath(self))
@@ -432,8 +510,6 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 	if (self->movetarget)
 		ACEMV_MoveToGoal(self,ucmd);
 
-
-
 	////////////////////////////////////////////////////////
 	// Platforms
 	///////////////////////////////////////////////////////
@@ -464,13 +540,13 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 	  (current_node_type == NODE_JUMP && next_node_type != NODE_ITEM && nodes[self->next_node].origin[2] > self->s.origin[2]))
 	{
 		// Set up a jump move
-		ucmd->forwardmove = 400;
+		ucmd->forwardmove = mSpeed;
 		ucmd->upmove = 400;
 
 		ACEMV_ChangeBotAngle(self);
 
-		VectorCopy(self->move_vector,dist);
-		VectorScale(dist,440,self->velocity);
+		VectorCopy(self->move_vector, dist);
+		VectorScale(dist, 440, self->velocity);
 
 		return;
 	}
@@ -481,7 +557,7 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 	if(next_node_type == NODE_LADDER && nodes[self->next_node].origin[2] > self->s.origin[2])
 	{
 		// Otherwise move as fast as we can
-		ucmd->forwardmove = 400;
+		ucmd->forwardmove = mSpeed;
 		self->velocity[2] = 320;
 
 		ACEMV_ChangeBotAngle(self);
@@ -493,7 +569,7 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 	if(current_node_type == NODE_LADDER && next_node_type != NODE_LADDER &&
 	   nodes[self->next_node].origin[2] > self->s.origin[2])
 	{
-		ucmd->forwardmove = 400;
+		ucmd->forwardmove = mSpeed;
 		ucmd->upmove = 200;
 		self->velocity[2] = 200;
 		ACEMV_ChangeBotAngle(self);
@@ -513,8 +589,8 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 			ucmd->upmove = 400;
 
 		ucmd->forwardmove = 300;
-		return;
 
+		return;
 	}
 
 	// Falling off ledge?
@@ -537,25 +613,31 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 			return;
 
 		self->s.angles[YAW] += random() * 180 - 90;
+		// Try moving foward, if not, try to strafe around obstacle
 		if(ACEMV_CanMove(self, MOVE_FORWARD))
-			ucmd->forwardmove = 400;
+			ucmd->forwardmove = mSpeed;
 		else if(ACEMV_CanMove(self, MOVE_BACK))
-			ucmd->forwardmove = -400;
-
+			ucmd->forwardmove = -mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_RIGHT))
+				ucmd->sidemove = mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_LEFT))
+				ucmd->sidemove = -mSpeed;
 		return;
 	}
 
 	// Otherwise move as fast as we can
 	if(ACEMV_CanMove(self, MOVE_FORWARD))
-		ucmd->forwardmove = 400;
-
-	if(self->skill == 3) {//ultra skill level(will be 3)
+		ucmd->forwardmove = mSpeed;
 
+	if(self->skill == 3)
+	{	//ultra skill level(will be 3)
 		c = random();
 
-		if(!self->in_deathball && grapple->value && c <= .7) {	//use the grapple once in awhile to pull itself around
+		if(!self->in_deathball && grapple->value && c <= .7)
+		{	//use the grapple once in awhile to pull itself around
 
-			if(self->client->ctf_grapplestate == CTF_GRAPPLE_STATE_HANG) {
+			if(self->client->ctf_grapplestate == CTF_GRAPPLE_STATE_HANG)
+			{
 				CTFPlayerResetGrapple(self);
 				ACEMV_ChangeBotAngle(self);
 				return;
@@ -568,29 +650,39 @@ void ACEMV_Move(edict_t *self, usercmd_t *ucmd)
 			return;
 		}
 		else if(self->client->ctf_grapplestate != CTF_GRAPPLE_STATE_PULL &&
-			self->client->ctf_grapplestate != CTF_GRAPPLE_STATE_HANG){ //don't interrupt a pull
+			self->client->ctf_grapplestate != CTF_GRAPPLE_STATE_HANG)
+		{	//don't interrupt a pull
+			float weight;
+			int strafeJump = false;
 
-			if(c > .7)
+			//Strafejumping should only occur now if a bot is far enough from a node
+			//and not wandering.  We really don't want them to jump around when wandering
+			//as it seems to hinder locating a goal.
 
-				ucmd->upmove = 400; //jump around the level
+			VectorSubtract(self->s.origin, nodes[self->current_node].origin, dist);
+			weight = VectorLength( dist );
+			if(weight > 300)
+				strafeJump = true;
 
-			if(c > 0.9 && ACEMV_CanMove(self, MOVE_LEFT)) //strafejump left
+			if(strafeJump)
+			{
+				if(c > .7)
+					ucmd->upmove = 400; //jump around the level
 
-				ucmd->sidemove = -400;
+				if(c > 0.9 && ACEMV_CanMove(self, MOVE_LEFT))
+					ucmd->sidemove = -200; //strafejump left(was -400)
 
-			else if(c > 0.8 && ACEMV_CanMove(self, MOVE_RIGHT)) //strafejump right
-
-				ucmd->sidemove = 400;
+				else if(c > 0.8 && ACEMV_CanMove(self, MOVE_RIGHT))
+					ucmd->sidemove = 200; //strafejump right(was 400)
+			}
 		}
 
 		//Now if we have the Alien Smartgun, drop some prox mines :)
 		if (self->client->pers.weapon == FindItem("alien smartgun") && c < 0.2)
 			ucmd->buttons = BUTTON_ATTACK2;
-
 	}
 
 	ACEMV_ChangeBotAngle(self);
-
 }
 
 
@@ -601,6 +693,12 @@ void ACEMV_Wander(edict_t *self, usercmd_t *ucmd)
 {
 	vec3_t  temp;
 	float c;
+	short mSpeed;
+
+	if(g_tactical->integer)
+		mSpeed = 200;
+	else 
+		mSpeed = 400;
 
 	// Do not move
 	if(self->next_move_time > level.time)
@@ -652,7 +750,7 @@ void ACEMV_Wander(edict_t *self, usercmd_t *ucmd)
 	{
 		//	safe_bprintf(PRINT_MEDIUM,"lava jump\n");
 		self->s.angles[YAW] += random() * 360 - 180;
-		ucmd->forwardmove = 400;
+		ucmd->forwardmove = mSpeed;
 		ucmd->upmove = 400;
 		return;
 	}
@@ -663,50 +761,120 @@ void ACEMV_Wander(edict_t *self, usercmd_t *ucmd)
 	// Check for special movement if we have a normal move (have to test)
  	if(VectorLength(self->velocity) < 37)
 	{
-		//if(random() > 0.1 && ACEMV_SpecialMove(self,ucmd))
-		//	return; //removed this because when wandering, the last thing you want is bots jumping
+		/*if(random() > 0.1 && ACEMV_SpecialMove(self,ucmd))
+			return;*/ //removed this because when wandering, the last thing you want is bots jumping
 		//over things and going off ledges.  It's better for them to just bounce around the map.
 
 		self->s.angles[YAW] += random() * 180 - 90;
 
+		// Try to move forward, if blocked, try to strafe around obstacle
 		if(ACEMV_CanMove(self, MOVE_FORWARD))
-			ucmd->forwardmove = 400;
+			ucmd->forwardmove = mSpeed;
 		else if(ACEMV_CanMove(self, MOVE_BACK))
-			ucmd->forwardmove = -400;
+			ucmd->forwardmove = -mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_RIGHT))
+				ucmd->sidemove = mSpeed;
+		else if(ACEMV_CanMove(self, MOVE_LEFT))
+				ucmd->sidemove = -mSpeed;
 
 		if(!M_CheckBottom(self) || !self->groundentity) // if there is ground continue, otherwise wait for next move.
 		{
 			if(ACEMV_CanMove(self, MOVE_FORWARD))
-				ucmd->forwardmove = 400;
+				ucmd->forwardmove = mSpeed;
 		}
 
 		return;
 	}
 
 	if(ACEMV_CanMove(self, MOVE_FORWARD))
-		ucmd->forwardmove = 400;
-
-	if(self->skill == 3) {//ultra skill level(will be 3)
+		ucmd->forwardmove = mSpeed;
 
+	if(self->skill == 3)
+	{ //ultra skill level(will be 3)
 		c = random();
 
-		if(c > .7)
-
-			ucmd->upmove = 400; //jump around the level
-
-		if(c > 0.9 && ACEMV_CanMove(self, MOVE_LEFT)) //strafejump left
-
-			ucmd->sidemove = -400;
-
-		else if(c > 0.8 && ACEMV_CanMove(self, MOVE_RIGHT)) //strafejump right
-
-			ucmd->sidemove = 400;
-
-		//Now if we have the Alien Smartgun, drop some prox mines :)
+		//If we have the Alien Smartgun, drop some prox mines :)
 		if (self->client->pers.weapon == FindItem("alien smartgun") && c < 0.2)
 			ucmd->buttons = BUTTON_ATTACK2;
-
 	}
+}
+
+/**
+ * @brief  Apply some imprecision to an attacking bot's aim
+ *
+ * @detail Uses a randomly oriented X,Y vector with a random length
+ *         plus an offset, which leaves a hole in the center of the
+ *         target area.
+ *
+ * @param self entity for the bot
+ * @param pdx  x for targeted enemy's origin
+ * @param pdy  y for targeted enemy's origin
+ *
+ * @return  *pdx and *pdy altered
+ */
+/*
+ * constants for target fuzzification
+ */
+static const float ktgt_scale = 250.0f; //scaling factor
+static const float ktgt_ofs   = 20.0f;  //radius of hole
+/* scaling for weap skill for bot cfg skill 0, 1, 2 & 3 */
+static const float ktgt_skill[4] =
+	{ 0.7f, 0.8f, 0.9f, 1.0f }; //accuracy scaling by skill
+//accuracy factor = ktgt_skill[0] * self->accuracy minimum clamp
+static const float ktgt_acc = 0.35;
+
+static void fuzzy_target( edict_t *self, float *pdx, float *pdy )
+{
+	float accuracy;
+	float random_r;
+	float radius;
+	float angle;
+	float ca,sa;
+	float dx,dy;
+
+	/*
+	 * self->accuracy is weapon specific accuracy from bot .cfg file
+	 */
+	accuracy = self->accuracy;
+	if ( accuracy < 0.5f )
+		accuracy = 0.5f;
+	else if ( accuracy > 1.0f )
+		accuracy = 1.0f;
+
+	//calc weap accuracy factor with scaling on bot skill
+	switch ( self->skill )
+	{
+	case 0:
+		accuracy *= ktgt_skill[0];
+		break;
+	case 1:
+	default:
+		accuracy *= ktgt_skill[1];
+		break;
+	case 2:
+		accuracy *= ktgt_skill[2];
+		break;
+	case 3:
+		//override 3->2 for CTF
+		accuracy *= (ctf->integer) ? ktgt_skill[2] : ktgt_skill[3];
+		break;
+	}
+
+	//radius calc
+	random_r = ktgt_scale * crandom();
+	radius =  (random_r * ( ktgt_acc / accuracy)) + ktgt_ofs;
+	//angle calc
+	angle = random() * 2.0f * M_PI;
+	fast_sincosf( angle, &sa, &ca );
+	//apply delta to target
+	*pdx += dx = ca * radius;
+	*pdy += dy = sa * radius;
+
+//	if ( debug_mode )
+//	{
+//		gi.dprintf("{\t%0.2f\t%0.2f\tacc%0.1f\t%i}\n",
+//				dx, dy, accuracy, self->skill );
+//	}
 
 }
 
@@ -718,18 +886,32 @@ void ACEMV_Attack (edict_t *self, usercmd_t *ucmd)
 	float c, d;
 	vec3_t  target;
 	vec3_t  angles;
-	vec3_t	down, right;
+	vec3_t down;
 	int strafespeed;
 	float jump_thresh;
 	float crouch_thresh;
-	float aim;
 	gitem_t *vehicle;
 	float range = 0.0f;
 	vec3_t v;
+	qboolean use_fuzzy_aim;
+	short mSpeed;
+
+	if(g_tactical->integer)
+		mSpeed = 200;
+	else 
+		mSpeed = 400;
+
+	ucmd->buttons = 0;
+	use_fuzzy_aim = true; // unless overridden by special cases
+	if ( dmflags->integer & DF_BOT_FUZZYAIM )
+	{ // when bit is set it means fuzzy aim is disabled. for testing mostly.
+		use_fuzzy_aim = false;
+	}
 
 	vehicle = FindItemByClassname("item_bomber");
 
-	if (self->client->pers.inventory[ITEM_INDEX(vehicle)]) {
+	if (self->client->pers.inventory[ITEM_INDEX(vehicle)]) 
+	{
 		//if we are too low, don't shoot, and move up.  Should be fairly simple, right?
 		if(self->enemy->s.origin[2] >= self->s.origin[2] - 128) { //we want to be well above our target
 			ucmd->upmove += 400;
@@ -742,193 +924,244 @@ void ACEMV_Attack (edict_t *self, usercmd_t *ucmd)
 		}
 	}
 
-	switch(self->skill){ //set up the skill levels;
+	switch ( self->skill )
+	{ // set up the skill levels
 	case 0:
-		strafespeed = 300;
-		jump_thresh = 1.0;
-		crouch_thresh = 0.95;
-		aim = 100;
+		strafespeed   = 300;
+		jump_thresh   = 1.0f;  // never jump
+		crouch_thresh = 0.0f;  // never crouch
 		break;
+
 	case 1:
-		strafespeed = 400;
-		jump_thresh = 0.95;
-		crouch_thresh = 0.85;
-		aim = 40;
+	default:
+		strafespeed   = 400;
+		jump_thresh   = 0.95f;
+		crouch_thresh = 0.85f; // crouch much
 		break;
+
 	case 2:
 	case 3:
 		strafespeed = 800;
-		if(!joustmode->value) {
-			jump_thresh = 0.8;
-			crouch_thresh = 0.7;
+		if ( !joustmode->integer )
+		{
+			jump_thresh   = 0.8f;
+			crouch_thresh = 0.7f;
 		}
-		else {
-			jump_thresh = 0.5; //want to jump a whole lot more in joust mode
-			crouch_thresh = 0.4;
+		else
+		{
+			jump_thresh   = 0.5f; // want to jump a whole lot more in joust mode
+			crouch_thresh = 0.4f; // and crouch less
 		}
-		aim = 20;
-		break;
-	default:
-		strafespeed = 400;
-		jump_thresh = 0.95;
-		crouch_thresh = 0.85;
-		aim = 40;
 		break;
 	}
 
-	if(self->accuracy < 0.5)
-		self->accuracy = 0.5; //just a failsafe(don't want it crazily inaccurate)
-	// Randomly choose a movement direction
-	d = random();
-	c = random();
+	d = random(); // for skill 0 movement
+	c = random(); // for strafing, jumping, crouching
 
-
-	//violator attack
-
-
-	if (self->client->pers.weapon == FindItem("Violator")) {
-		if(ACEMV_CanMove(self,MOVE_FORWARD))
+	// violator attack
+	if ( self->client->pers.weapon == FindItem( "Violator" ) )
+	{
+		use_fuzzy_aim = false; // avoid potential odd melee attack behaviour
+		if ( ACEMV_CanMove( self, MOVE_FORWARD ) )
 			ucmd->forwardmove += 400; //lunge at enemy
 		goto attack;
 	}
 
-	//machinegun/blaster/beamgun strafing for level 3 bots
-	if(!joustmode->value && self->skill == 3  && (self->client->pers.weapon == FindItem("blaster") || self->client->pers.weapon == FindItem("Pulse Rifle") || self->client->pers.weapon == FindItem("Disruptor")) )
-
+	//machinegun/blaster/beamgun strafing for level 2/3 bots
+	if ( !joustmode->value
+			&& self->skill >= 2
+			&& (self->client->pers.weapon == FindItem( "Blaster" )
+					|| self->client->pers.weapon == FindItem( "Alien Blaster" )
+					|| self->client->pers.weapon == FindItem( "Pulse Rifle" )
+					|| self->client->pers.weapon == FindItem( "Disruptor" )))
 	{
 		//strafe no matter what
-
-		if(c < 0.5 && ACEMV_CanMove(self,MOVE_LEFT)) {
-		ucmd->sidemove -= 400;
-
+		if ( c < 0.5f && ACEMV_CanMove( self, MOVE_LEFT ) )
+		{
+			ucmd->sidemove -= mSpeed;
 		}
-		else if(c < 1 && ACEMV_CanMove(self,MOVE_RIGHT)) {
-			ucmd->sidemove += 400;
-
+		else if ( c < 1.0f && ACEMV_CanMove( self, MOVE_RIGHT ) )
+		{
+			ucmd->sidemove += mSpeed;
 		}
 		else
-			goto standardmove; //don't want high skill level bots just standing around
+		{ //don't want high skill level bots just standing around
+			goto standardmove;
+		}
 
 		//allow for some circle strafing
-		if(self->health < 50 && ACEMV_CanMove(self,MOVE_BACK))
-			ucmd->forwardmove -= 400;
-		else if(c < 0.6 && ACEMV_CanMove(self,MOVE_FORWARD)) //keep this at default, not make them TOO hard
-			ucmd->forwardmove += 400;
-		else if(c < 0.8 && ACEMV_CanMove(self,MOVE_BACK))
-			ucmd->forwardmove -= 400;
-		goto attack; //skip any jumping or crouching
-
+		if ( self->health < 50 && ACEMV_CanMove( self, MOVE_BACK ) )
+		{
+			ucmd->forwardmove -= mSpeed;
+		}
+		else if ( c < 0.6f && ACEMV_CanMove( self, MOVE_FORWARD ) )
+		{ //keep this at default, not make them TOO hard
+			ucmd->forwardmove += mSpeed;
+		}
+		else if ( c < 0.8f && ACEMV_CanMove( self, MOVE_BACK ) )
+		{
+			ucmd->forwardmove -= mSpeed;
+		}
+		goto attack;
+		//skip any jumping or crouching
 	}
 
-	if(self->skill == 0 && d < 0.9)
+	if ( self->skill == 0 && d < 0.9f )
 		goto attack; //skill 0 bots will barely move while firing
 
 standardmove:
 
-	if(c < 0.2 && ACEMV_CanMove(self,MOVE_LEFT)) {
-		ucmd->sidemove -= strafespeed; //300 for low skill 800 for hardest(3 levels?)
-
+	if ( c < 0.2f && ACEMV_CanMove( self, MOVE_LEFT ) )
+	{
+		ucmd->sidemove -= strafespeed;
+		//300 for low skill 800 for hardest(3 levels?)
 	}
-	else if(c < 0.4 && ACEMV_CanMove(self,MOVE_RIGHT)) {
+	else if ( c < 0.4f && ACEMV_CanMove( self, MOVE_RIGHT ) )
+	{
 		ucmd->sidemove += strafespeed;
-
 	}
 
-	if(self->health < 50 && ACEMV_CanMove(self,MOVE_BACK)) //run away if wounded
-		ucmd->forwardmove -= 400;
-	else if(c < 0.6 && ACEMV_CanMove(self,MOVE_FORWARD)) //keep this at default, not make them TOO hard
-		ucmd->forwardmove += 400;
-	else if(c < 0.8 && ACEMV_CanMove(self,MOVE_BACK))
-		ucmd->forwardmove -= 400;
+	if ( self->health < 50 && ACEMV_CanMove( self, MOVE_BACK ) )
+	{ //run away if wounded
+		ucmd->forwardmove -= mSpeed;
+	}
+	else if ( c < 0.6f && ACEMV_CanMove( self, MOVE_FORWARD ) )
+	{ //keep this at default, not make them TOO hard
+		ucmd->forwardmove += mSpeed;
+	}
+	else if ( c < 0.8f && ACEMV_CanMove( self, MOVE_BACK ) )
+	{
+		ucmd->forwardmove -= mSpeed;
+	}
 
 	c = random(); //really mix this up some
-	if(self->health >= 50 && c < crouch_thresh)  //raise this number to make them crouch alot(lower skills)
+	if ( self->health >= 50 && c < crouch_thresh )
+	{
 		ucmd->upmove -= 200;
-	else if(self->health >= 50 && c < jump_thresh && !self->in_vehicle && !self->in_deathball){  //lets add a rocket jump
+	}
+	else if ( c > jump_thresh )
+	{
+#ifndef ALTERIA
 		c = random();
-		if((self->skill >= 2) && (c < 0.6) && (self->health > 70) && (ACEIT_ChangeWeapon(self,FindItem("Rocket Launcher")))) {
-			self->s.angles[PITCH] = 90;
-			AngleVectors (self->s.angles, down, right, NULL);
-			fire_rocket (self, self->s.origin, down, 200, 650, 120, 120);
+		if ( self->health >= 70
+				&& self->skill >= 2
+		        && !self->in_vehicle && !self->in_deathball
+		        && ACEIT_ChangeWeapon( self, FindItem( "Rocket Launcher" ) )
+		        && c < 0.6f )
+		{ // Rocket Jump
+			self->s.angles[PITCH] = 90.0f;
+			AngleVectors( self->s.angles, down, NULL, NULL );
+			fire_rocket( self, self->s.origin, down, 200, 650, 120, 120 );
 			ucmd->upmove += 200;
-			self->s.angles[PITCH] = 0; //put it back
-			if (! ( (int)dmflags->value & DF_INFINITE_AMMO ) )
+			self->s.angles[PITCH] = 0.0f;
+			if ( (!(dmflags->integer & DF_INFINITE_AMMO))
+			        && !rocket_arena->integer && !insta_rockets->integer )
+			{
 				self->client->pers.inventory[self->client->ammo_index]--;
+			}
 			return;
 		}
 		else
-			ucmd->upmove += 200; //normal jumping
+#endif
+		{ // Normal Jump
+			ucmd->upmove += 200;
+		}
 	}
+
 attack:
 	// Set the attack
-	if(ACEAI_CheckShot(self)) {
+	if(ACEAI_CheckShot(self)) 
+	{
 		//bot is taking a shot, lose spawn protection
+		// and calculate distance to enemy
+		range = 0.0f;
 		if(self->enemy)
+		{
 			self->client->spawnprotected = false;
-		if(self->skill == 3) { //skill 3 bots can use alt-fires!
+			VectorSubtract (self->s.origin, self->enemy->s.origin, v);
+			range = VectorLength(v);
+			if ( range < 32.0f )
+			{ // point blank range, avoid potentially odd behaviour
+				use_fuzzy_aim = false;
+			}
+		}		
+
+		if(self->skill >= 2) 
+		{	//skill 2/3 bots can use alt-fires!
 
 			// Base selection on distance.
 			ucmd->buttons = BUTTON_ATTACK;
 
-			if(self->enemy) { //just in case there isn't one, though that should never happen
-				VectorSubtract (self->s.origin, self->enemy->s.origin, v);
-				range = VectorLength(v);
-			}
-
-			if (self->client->pers.weapon == FindItem("blaster")) {
+			if (self->client->pers.weapon == FindItem("Blaster") || self->client->pers.weapon == FindItem("Alien Blaster")) 
+			{
 				if( range > 500)
 					ucmd->buttons = BUTTON_ATTACK2;
 				else
 					ucmd->buttons = BUTTON_ATTACK;
 			}
 
-			if (self->client->pers.weapon == FindItem("alien disruptor")) {
+			if (self->client->pers.weapon == FindItem("Alien Disruptor")) 
+			{
 				if(range > 1000) {
 					ucmd->buttons = BUTTON_ATTACK2;
-					aim = 10; //make it more accurate, since he's sniping
-					self->accuracy = 1;
+					use_fuzzy_aim = false;
+					//make it more accurate, since he's sniping
 				}
 				else
 					ucmd->buttons = BUTTON_ATTACK;
 			}
 
-			if (self->client->pers.weapon == FindItem("flame thrower")) {
+			if (self->client->pers.weapon == FindItem("Flame Thrower")) 
+			{
 				if(range < 500)
 					ucmd->buttons = BUTTON_ATTACK;
 				else
 					ucmd->buttons = BUTTON_ATTACK2;
 			}
 
-			if (self->client->pers.weapon == FindItem("pulse rifle")) {
+			if (self->client->pers.weapon == FindItem("Pulse Rifle")) 
+			{
 				if(range < 200)
 					ucmd->buttons = BUTTON_ATTACK2;
 				else
 					ucmd->buttons = BUTTON_ATTACK;
 			}
 
-			if (self->client->pers.weapon == FindItem("disruptor")) {
-				if(range < 500)
-					ucmd->buttons = BUTTON_ATTACK2;
-				else
-					ucmd->buttons = BUTTON_ATTACK;
-			}
-
-			if (self->client->pers.weapon == FindItem("alien vaporizer")) {
+			if (self->client->pers.weapon == FindItem("Disruptor")) 
+			{
 				if(range < 300)
 					ucmd->buttons = BUTTON_ATTACK2;
 				else
 					ucmd->buttons = BUTTON_ATTACK;
 			}
 
+			if (self->client->pers.weapon == FindItem("Alien Vaporizer")) 
+			{
+				if(range < 300)
+					ucmd->buttons = BUTTON_ATTACK2;
+				else
+					ucmd->buttons = BUTTON_ATTACK;
+			}
+
+			if (self->client->pers.weapon == FindItem("Minderaser")) 
+			{
+				if(range < 400)
+					ucmd->buttons = BUTTON_ATTACK;
+				else
+					ucmd->buttons = BUTTON_ATTACK2;
+			}
+			
 			//vehicle alt-fires
 			if (self->client->pers.weapon == FindItem("bomber")
-				|| self->client->pers.weapon == FindItem("strafer")) {
+				|| self->client->pers.weapon == FindItem("strafer")) 
+			{
 				if(range > 500)
 					ucmd->buttons = BUTTON_ATTACK2;
 				else
 					ucmd->buttons = BUTTON_ATTACK;
 			}
-			if (self->client->pers.weapon == FindItem("hover")) {
+			if (self->client->pers.weapon == FindItem("hover")) 
+			{
 				if(range < 300)
 					ucmd->buttons = BUTTON_ATTACK2;
 				else
@@ -936,23 +1169,23 @@ attack:
 			}
 		}
 		else
+		{
 			ucmd->buttons = BUTTON_ATTACK;
+		}
 	}
 	else
+	{ // not taking a shot
 		ucmd->buttons = 0;
+		use_fuzzy_aim = false;
+	}
 
 	// Aim
 	VectorCopy(self->enemy->s.origin,target);
-
-	// modify attack angles based on accuracy (mess this up to make the bot's aim not so deadly)
-    // skill level determines overall accuracy.  Each weapon can be additionally tweaked.
-	if (((int)(dmflags->value) & DF_BOT_FUZZYAIM))
-		aim = 0; //only crazy people would not be running fuzzy aim!
-
-	target[0] += (random()-0.5) * (aim/self->accuracy);
-	target[1] += (random()-0.5) * (aim/self->accuracy);
-
-	// Set direction
+	if ( use_fuzzy_aim  )
+	{
+		fuzzy_target( self, &target[0], &target[1] );
+	}
+	// Set movement direction toward targeted enemy
 	VectorSubtract (target, self->s.origin, self->move_vector);
 	vectoangles (self->move_vector, angles);
 	VectorCopy(angles,self->s.angles);
