@@ -196,11 +196,9 @@ int ACEND_FindClosestReachableNode(edict_t *self, int range, int type)
 void ACEND_SetGoal(edict_t *self, int goal_node)
 {
 	int node;
-#ifdef CTFNODES
 	gitem_t *flag1_item, *flag2_item;
-#endif
 	self->goal_node = goal_node;
-#ifdef CTFNODES
+
 	//if flag in possession only use base nodes
 	if(ctf->value)
 	{
@@ -208,14 +206,38 @@ void ACEND_SetGoal(edict_t *self, int goal_node)
 		flag2_item = FindItemByClassname("item_flag_blue");
 
 		if (self->client->pers.inventory[ITEM_INDEX(flag1_item)])
+		{
 			node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_BLUEBASE);
+			if(node == -1)
+				node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_ALL);
+		}
 		else if (self->client->pers.inventory[ITEM_INDEX(flag2_item)])
+		{
 			node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_REDBASE);
+			if(node == -1)
+				node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_ALL);
+		}
+		else
+			node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_ALL);
+	}
+	else if(g_tactical->value) //when a base's laser barriers shut off, go into a more direct attack route
+	{
+		if (self->ctype == 1 && (!tacticalScore.alienComputer || !tacticalScore.alienPowerSource))
+		{
+			node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_BLUEBASE);
+			if(node == -1)
+				node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_ALL);
+		}
+		else if (self->ctype == 0 && (!tacticalScore.humanComputer || !tacticalScore.humanPowerSource))
+		{
+			node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_REDBASE);
+			if(node == -1)
+				node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_ALL);
+		}
 		else
 			node = ACEND_FindClosestReachableNode(self,NODE_DENSITY*3,NODE_ALL);
 	}
 	else
-#endif
 		node = ACEND_FindClosestReachableNode(self, NODE_DENSITY*3, NODE_ALL);
 
 	if(node == -1)
@@ -636,7 +658,7 @@ void ACEND_LoadNodes(void)
 
 	if ( !gi.FullPath( filename, sizeof(filename), relative_path ) )
 	{
-		gi.dprintf("ACEND_LoadNodes: not found: %s\n", filename );
+		gi.dprintf("ACEND_LoadNodes: not found: %s\n", relative_path );
 	}
 
 	if((pIn = fopen(filename, "rb" )) == NULL)
