@@ -197,7 +197,7 @@ static size_t loadbots_readnext( char *p_userinfo_bfr )
  * count bots and update bot information in clients
  *
  * see:
- *  p_hud.c::G_SetStats(),
+ *  p_hud.c::G_UpdateStats()
  *  sv_main.c::SV_StatusString()
  *
  */
@@ -233,6 +233,7 @@ static int client_botupdate( void )
 				pclient->client->ps.botnum = pclient->client->resp.botnum = 0;
 			}
 		}
+		return 0;
 	}
 
 	botidx = 0;
@@ -244,12 +245,14 @@ static int client_botupdate( void )
 			botname = Info_ValueForKey( pbot->client->pers.userinfo, "name" );
 			firstclient = true;
 			for ( pclient = &g_edicts[1], clients=game.maxclients ; clients--; ++pclient )
-			{ // for every client, always update first client for server
+			{ // for every active client, plus always update first client for server
 				if ( pclient->inuse || firstclient )
 				{
 					pclient->client->ps.botnum = pclient->client->resp.botnum = botnum;
 					strcpy( pclient->client->resp.bots[botidx].name, botname );
 					strcpy( pclient->client->ps.bots[botidx].name, botname );
+					pclient->client->resp.bots[botidx].score = pbot->client->resp.score;
+					pclient->client->ps.bots[botidx].score   = pbot->client->resp.score;
 					firstclient = false;
 				}
 			}
@@ -262,9 +265,9 @@ static int client_botupdate( void )
 }
 
 /**
- * \brief Update bot info in client records
+ * @brief Update bot info in client records
  *
- * \detail  Intended to be called once per frame, in RunFrame. First client
+ * @detail  Intended to be called once per frame, in RunFrame. First client
  *          record should always have current bot info, so that server status
  *          shows bots even when in intermission.
  *          In ACE debug mode, reports when bot count changes. ACE debug mode
@@ -938,7 +941,8 @@ void ACESP_PutClientInServer (edict_t *bot, qboolean respawn )
 	// find a spawn point
 	// do it before setting health back up, so farthest
 	// ranging doesn't count this client
-	SelectSpawnPoint (bot, spawn_origin, spawn_angles);
+	if(!g_tactical->integer)
+		SelectSpawnPoint (bot, spawn_origin, spawn_angles);
 
 	index = bot - g_edicts - 1;
 	client = bot->client;
@@ -1012,10 +1016,6 @@ void ACESP_PutClientInServer (edict_t *bot, qboolean respawn )
 	// clear playerstate values
 	memset (&bot->client->ps, 0, sizeof(client->ps));
 
-	client->ps.pmove.origin[0] = spawn_origin[0]*8;
-	client->ps.pmove.origin[1] = spawn_origin[1]*8;
-	client->ps.pmove.origin[2] = spawn_origin[2]*8;
-
 //ZOID
 	client->ps.pmove.pm_flags &= ~PMF_NO_PREDICTION;
 //ZOID
@@ -1074,27 +1074,57 @@ void ACESP_PutClientInServer (edict_t *bot, qboolean respawn )
 	bot->ctype = 0;
 	sprintf(modelpath, "players/%s/human", playermodel);
 	Q2_FindFile (modelpath, &file);
-	if(file) { //human
+	if(file) 
+	{ 
+		fclose(file);
+
+		//human
 		bot->ctype = 1;
-		if(classbased->value && !(rocket_arena->value || instagib->value || insta_rockets->value || excessive->value))
+		if(g_tactical->integer || (classbased->value && !(rocket_arena->integer || instagib->integer || insta_rockets->value || excessive->value)))
 		{
+			if(g_tactical->integer)
+			{
+				//read class file(tactical only)
+				//example:
+				//100-150 (health)
+				//0-3 (armor type)
+				//0-1 (has bomb)
+				//0-1 (has detonator)
+				//0-1 (has mind eraser)
+				//0-1 (has vaporizor)
+			
+				ParseClassFile(modelpath, bot); 
+				if(bot->has_bomb)
+				{
+					bot->client->pers.inventory[ITEM_INDEX(FindItem("Human Bomb"))] = 1;
+					bot->client->pers.inventory[ITEM_INDEX(FindItem("bombs"))] = 1; //tactical note - humans will use same ammo, etc, just different weapons
+				}				
+				item = FindItem("Blaster");
+			}
+			else
+			{
+				bot->health = bot->max_health = client->pers.max_health = client->pers.health = 100;
 				armor_index = ITEM_INDEX(FindItem("Jacket Armor"));
 				client->pers.inventory[armor_index] += 30;
+				
 				client->pers.inventory[ITEM_INDEX(FindItem("Rocket Launcher"))] = 1;
 				client->pers.inventory[ITEM_INDEX(FindItem("rockets"))] = 10;
-				item = FindItem("Rocket Launcher");
-				client->pers.selected_item = ITEM_INDEX(item);
-				client->pers.inventory[client->pers.selected_item] = 1;
-				client->pers.weapon = item;
+				item = FindItem("Rocket Launcher");				
+			}			
+			client->pers.selected_item = ITEM_INDEX(item);
+			client->pers.inventory[client->pers.selected_item] = 1;
+			client->pers.weapon = item;
 		}
-		fclose(file);
 	}
-	else {
+	else 
+	{
 		sprintf(modelpath, "players/%s/robot", playermodel);
 		Q2_FindFile (modelpath, &file);
-		if(file) { //robot
+		if(file && !g_tactical->integer) 
+		{ 
+			//robot - not used in tactical
 			bot->ctype = 2;
-			if(classbased->value && !(rocket_arena->value || instagib->value || insta_rockets->value || excessive->value))
+			if(classbased->value && !(rocket_arena->integer || instagib->integer || insta_rockets->value || excessive->value))
 			{
 				bot->health = bot->max_health = client->pers.max_health = client->pers.health = 85;
 				armor_index = ITEM_INDEX(FindItem("Jacket Armor"));
@@ -1102,20 +1132,53 @@ void ACESP_PutClientInServer (edict_t *bot, qboolean respawn )
 			}
 			fclose(file);
 		}
-		else { //alien
-			if(classbased->value && !(rocket_arena->value || instagib->value || insta_rockets->value || excessive->value))
+		else 
+		{ 
+			//alien
+			bot->ctype = 0;
+			if(g_tactical->integer || (classbased->value && !(rocket_arena->integer || instagib->integer || insta_rockets->value || excessive->value)))
 			{
 				bot->health = bot->max_health = client->pers.max_health = client->pers.health = 150;
-				client->pers.inventory[ITEM_INDEX(FindItem("Alien Disruptor"))] = 1;
-				client->pers.inventory[ITEM_INDEX(FindItem("cells"))] = 100;
-				item = FindItem("Alien Disruptor");
+				if(g_tactical->integer)
+				{
+					sprintf(modelpath, "players/%s/alien", playermodel);
+					Q2_FindFile (modelpath, &file);
+					if(file)
+					{
+						ParseClassFile(modelpath, bot); 		
+						if(bot->has_bomb)
+						{
+							bot->client->pers.inventory[ITEM_INDEX(FindItem("Alien Bomb"))] = 1;
+							bot->client->pers.inventory[ITEM_INDEX(FindItem("bombs"))] = 1; //tactical note - humans will use same ammo, etc, just different weapons
+						}
+					}
+					item = FindItem("Blaster");
+					client->pers.selected_item = ITEM_INDEX(item);
+					client->pers.inventory[client->pers.selected_item] = 0;					
+				
+					item = FindItem("Alien Blaster");
+				}
+				else
+				{
+					client->pers.inventory[ITEM_INDEX(FindItem("Alien Disruptor"))] = 1;
+					client->pers.inventory[ITEM_INDEX(FindItem("cells"))] = 100;
+					item = FindItem("Alien Disruptor");
+				}
 				client->pers.selected_item = ITEM_INDEX(item);
 				client->pers.inventory[client->pers.selected_item] = 1;
-				client->pers.weapon = item;
+				client->pers.weapon = item;		
 			}
 		}
 	}
 
+	//has to be done after determining the class/team - note - we don't care about spawn distances in tactical
+	if(g_tactical->integer)
+		SelectSpawnPoint (bot, spawn_origin, spawn_angles);
+	
+	client->ps.pmove.origin[0] = spawn_origin[0]*8;
+	client->ps.pmove.origin[1] = spawn_origin[1]*8;
+	client->ps.pmove.origin[2] = spawn_origin[2]*8;
+
 	bot->s.frame = 0;
 	VectorCopy (spawn_origin, bot->s.origin);
 	bot->s.origin[2] += 1;	// make sure off ground
@@ -1483,7 +1546,7 @@ qboolean ACESP_SpawnBot (char *name, char *skin, char *userinfo)
 
 	ACESP_SaveBots(); // update bots.tmp and clients bot information
 
-	if ( g_duel->value )
+	if ( g_duel->integer )
 	{
 		ClientPlaceInQueue( pbot );
 		ClientCheckQueue( pbot );
@@ -1526,7 +1589,7 @@ static void remove_bot( edict_t *bot )
 		DeadDropDeathball(bot);
 	}
 
-	if ( g_duel->value )
+	if ( g_duel->integer )
 	{// duel mode, we need to bump people down the queue if its the player in game leaving
 		MoveClientsDownQueue(bot);
 		if( !bot->client->resp.spectator )
@@ -1552,6 +1615,7 @@ static void remove_bot( edict_t *bot )
 	bot->s.angles[2] = 0;  // ?
 	bot->s.sound = 0;
 	bot->client->weapon_sound = 0;
+	bot->s.effects = 0;
 
 	// remove powerups
 	bot->client->quad_framenum = 0;
