@@ -29,14 +29,6 @@ during jumps, and using the old "look at player" angles upon dying.
 
 void UpdateChaseCam(edict_t *ent)
 {
-	vec3_t o, ownerv, goal;
-	edict_t *targ;
-	vec3_t forward, right;
-	trace_t trace;
-	int i;
-	vec3_t oldgoal;
-	vec3_t angles;
-
 	// is our chase target gone?
 	if (!ent->client->chase_target->inuse
 		|| ent->client->chase_target->client->resp.spectator) {
@@ -48,66 +40,7 @@ void UpdateChaseCam(edict_t *ent)
 			return;
 		}
 	}
-
-	targ = ent->client->chase_target;
-
-	VectorCopy(targ->s.origin, ownerv);
-	VectorCopy(ent->s.origin, oldgoal);
-
-	ownerv[2] += targ->viewheight;
-
-	VectorCopy(targ->client->v_angle, angles);
-	if (angles[PITCH] > 56)
-		angles[PITCH] = 56;
-	AngleVectors (angles, forward, right, NULL);
-	VectorNormalize(forward);
-
-	VectorMA(ownerv, -30, forward, o);
-
-	if (o[2] < targ->s.origin[2] + 20)
-			o[2] = targ->s.origin[2] + 20;
-
-	// jump animation lifts
-	if (!targ->groundentity)
-		o[2] += 16;
-
-	trace = gi.trace(ownerv, vec3_origin, vec3_origin, o, targ, MASK_SOLID);
-
-	VectorCopy(trace.endpos, goal);
-
-	VectorMA(goal, 2, forward, goal);
-
-	// pad for floors and ceilings
-	VectorCopy(goal, o);
-	o[2] += 6;
-	trace = gi.trace(goal, vec3_origin, vec3_origin, o, targ, MASK_SOLID);
-	if (trace.fraction < 1) {
-		VectorCopy(trace.endpos, goal);
-		goal[2] -= 6;
-	}
-
-	VectorCopy(goal, o);
-	o[2] -= 6;
-	trace = gi.trace(goal, vec3_origin, vec3_origin, o, targ, MASK_SOLID);
-	if (trace.fraction < 1) {
-		VectorCopy(trace.endpos, goal);
-		goal[2] += 6;
-	}
-
-	if (targ->deadflag)
-		ent->client->ps.pmove.pm_type = PM_DEAD;
-	else
-		ent->client->ps.pmove.pm_type = PM_FREEZE;
-
-	VectorCopy(goal, ent->s.origin);
-	for (i=0 ; i<3 ; i++)
-		ent->client->ps.pmove.delta_angles[i] = ANGLE2SHORT(targ->client->v_angle[i] - ent->client->resp.cmd_angles[i]);
-
-	VectorCopy(targ->client->v_angle, ent->client->ps.viewangles);
-	VectorCopy(targ->client->v_angle, ent->client->v_angle);
-
-	ent->viewheight = 0;
-	ent->client->ps.pmove.pm_flags |= PMF_NO_PREDICTION;
+	
 	gi.linkentity(ent);
 }
 
