@@ -49,6 +49,12 @@ qboolean OnSameTeam (edict_t *ent1, edict_t *ent2)
 	char	ent1Team [512];
 	char	ent2Team [512];
 
+	if(g_tactical->value)
+	{
+		if(ent1->ctype == ent2->ctype)
+			return true;
+	}
+
 	if (!((dmflags->integer & DF_SKINTEAMS) || ctf->value || tca->value || cp->value))
 		return false;
 
@@ -57,6 +63,7 @@ qboolean OnSameTeam (edict_t *ent1, edict_t *ent2)
 
 	if (strcmp(ent1Team, ent2Team) == 0)
 		return true;
+
 	return false;
 }
 
@@ -423,6 +430,10 @@ void Cmd_Use_f (edict_t *ent)
 	if (!ent->client->pers.inventory[index])
 	{
 		safe_cprintf (ent, PRINT_HIGH, "Out of item: %s\n", s);
+		if (it->flags & IT_WEAPON) {
+		    ent->client->pers.lastfailedswitch = it;
+		    ent->client->pers.failedswitch_framenum = level.framenum;
+		}
 		return;
 	}
 
@@ -683,8 +694,14 @@ int PlayerSort (void const *a, void const *b)
 	anum = *(int *)a;
 	bnum = *(int *)b;
 
-	anum = game.clients[anum].ps.stats[STAT_FRAGS];
-	bnum = game.clients[bnum].ps.stats[STAT_FRAGS];
+	if ( game.clients[anum].ps.stats[STAT_SPECTATOR] == 0 )
+		anum = game.clients[anum].ps.stats[STAT_FRAGS];
+	else
+		anum = 0;
+	if ( game.clients[bnum].ps.stats[STAT_SPECTATOR] == 0 )
+		bnum = game.clients[bnum].ps.stats[STAT_FRAGS];
+	else
+		bnum = 0;
 
 	if (anum < bnum)
 		return -1;
@@ -722,9 +739,11 @@ void Cmd_Players_f (edict_t *ent)
 
 	for (i = 0 ; i < count ; i++)
 	{
+
 		Com_sprintf (small, sizeof(small), "%3i %s\n",
 			game.clients[index[i]].ps.stats[STAT_FRAGS],
 			game.clients[index[i]].pers.netname);
+
 		if (strlen (small) + strlen(large) > sizeof(large) - 100 )
 		{	// can't print all of them in one packet
 			strcat (large, "...\n");
@@ -1091,6 +1110,13 @@ void Cmd_VoiceTaunt_f (edict_t *ent)
 
 			strcpy(name, ent->client->pers.netname);
 			info = Info_ValueForKey (ent->client->pers.userinfo, "skin");
+
+			if ( *info == '\0' )
+			{ /* could not find skin. probable programming error. */
+				gi.dprintf("Cmd_VoiceTaunt_f: skin not found in userinfo\n");
+				return;
+			}
+
 			info[96] = 0; //truncate to prevent bad people from harming the server
 
 			i = 0;
@@ -1156,11 +1182,6 @@ void ClientCommand (edict_t *ent)
 		Cmd_Score_f (ent);
 		return;
 	}
-	if (Q_strcasecmp (cmd, "help") == 0)
-	{
-		Cmd_Help_f (ent);
-		return;
-	}
 	if (Q_strcasecmp (cmd, "callvote") == 0)
 	{
 		Cmd_CallVote_f(ent);
