@@ -346,7 +346,7 @@ void T_Damage (edict_t *targ, edict_t *inflictor, edict_t *attacker, vec3_t dir,
 	if (targ->flags & FL_NO_KNOCKBACK)
 		knockback = 0;
 
-// figure momentum add
+	// figure momentum add
 	if (!(dflags & DAMAGE_NO_KNOCKBACK))
 	{
 		if ((knockback) && (targ->movetype != MOVETYPE_NONE) && (targ->movetype != MOVETYPE_BOUNCE) && (targ->movetype != MOVETYPE_PUSH) && (targ->movetype != MOVETYPE_STOP))
@@ -440,6 +440,21 @@ void T_Damage (edict_t *targ, edict_t *inflictor, edict_t *attacker, vec3_t dir,
 				if(!(strcmp(targ->classname, "misc_redspidernode")) || !(strcmp(targ->classname, "misc_bluespidernode")))
 					safe_centerprintf(attacker, "Spider health at %i percent", 100*(targ->health-take)/600);
 			}
+			if(g_tactical->value)
+			{
+				if(!strcmp(targ->classname, "alien computer"))
+					safe_centerprintf(attacker, "Alien Computer health at %i percent", 100*(targ->health-take)/1500);
+				else if(!strcmp(targ->classname, "human computer"))
+					safe_centerprintf(attacker, "Human Computer health at %i percent", 100*(targ->health-take)/1500);
+				else if(!strcmp(targ->classname, "alien powersrc"))
+					safe_centerprintf(attacker, "Alien Power Source health at %i percent", 100*(targ->health-take)/1500);
+				else if(!strcmp(targ->classname, "human powersrc"))
+					safe_centerprintf(attacker, "Human Power Source health at %i percent", 100*(targ->health-take)/1500);
+				else if(!strcmp(targ->classname, "alien ammodepot"))
+					safe_centerprintf(attacker, "Alien Ammo Depot health at %i percent", 100*(targ->health-take)/1500);
+				else if(!strcmp(targ->classname, "human ammodepot"))
+					safe_centerprintf(attacker, "Human Ammo Depot health at %i percent", 100*(targ->health-take)/1500);
+			}
 		}
 
 		targ->health = targ->health - take;
