@@ -1205,13 +1205,14 @@ void SP_func_door (edict_t *ent)
 
 	ent->moveinfo.state = STATE_BOTTOM;
 
-	if (ent->health)
-	{
-		ent->takedamage = DAMAGE_YES;
-		ent->die = door_killed;
-		ent->max_health = ent->health;
+    if (!ent->targetname && (!ent->health || ent->health > 8))
+        ent->health = 1;
+    if (ent->health) {
+	    ent->takedamage = DAMAGE_YES;
+	    ent->die = door_killed;
+	    ent->max_health = ent->health;
 	}
-	else if (ent->targetname && ent->message)
+	if (ent->targetname && ent->message)
 	{
 		gi.soundindex ("misc/talk.wav");
 		ent->touch = door_touch;
@@ -1238,10 +1239,10 @@ void SP_func_door (edict_t *ent)
 	gi.linkentity (ent);
 
 	ent->nextthink = level.time + FRAMETIME;
-	if (ent->health || ent->targetname)
-		ent->think = Think_CalcMoveSpeed;
+	if (ent->targetname)
+	    ent->think = Think_CalcMoveSpeed;
 	else
-		ent->think = Think_SpawnDoorTrigger;
+	    ent->think = Think_SpawnDoorTrigger;
 }
 
 
@@ -1980,7 +1981,7 @@ void door_secret_blocked  (edict_t *self, edict_t *other)
 		// give it a chance to go away on it's own terms (like gibs)
 		T_Damage (other, self, self, vec3_origin, other->s.origin, vec3_origin, 100000, 1, 0, MOD_CRUSH);
 		// if it's still there, nuke it
-		if (other)
+		if (other->inuse)
 			BecomeExplosion1 (other);
 		return;
 	}
