@@ -28,7 +28,11 @@ qboolean	Pickup_Weapon (edict_t *ent, edict_t *other);
 void		Use_Weapon (edict_t *ent, gitem_t *inv);
 void		Drop_Weapon (edict_t *ent, gitem_t *inv);
 
+#ifdef ALTERIA
+void Weapon_Punch ( edict_t *ent);
+#else
 void Weapon_Blaster (edict_t *ent);
+void Weapon_AlienBlaster (edict_t *ent);
 void Weapon_Violator (edict_t *ent);
 void Weapon_Smartgun (edict_t *ent);
 void Weapon_Chain (edict_t *ent);
@@ -37,10 +41,13 @@ void Weapon_Disruptor (edict_t *ent);
 void Weapon_RocketLauncher (edict_t *ent);
 void Weapon_Beamgun (edict_t *ent);
 void Weapon_Vaporizer (edict_t *ent);
+void Weapon_Minderaser (edict_t *ent);
 void Weapon_Bomber (edict_t *ent);
 void Weapon_Strafer (edict_t *ent);
 void Weapon_Deathball (edict_t *ent);
 void Weapon_Hover (edict_t *ent);
+void Weapon_TacticalBomb ( edict_t *ent);
+#endif
 
 gitem_armor_t jacketarmor_info	= { 25,  50, .30, .00, ARMOR_JACKET};
 gitem_armor_t combatarmor_info	= { 50, 100, .60, .30, ARMOR_COMBAT};
@@ -120,8 +127,84 @@ gitem_t	*FindItem (char *pickup_name)
 
 //======================================================================
 
+static void drop_temp_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
+{
+	if (other == ent->owner)
+		return;
+
+	Touch_Item (ent, other, plane, surf);
+}
+
+static void drop_make_touchable (edict_t *ent)
+{
+	ent->touch = Touch_Item;
+	if (deathmatch->integer)
+	{
+		if(!g_tactical->integer) //in tactical mode, we don't remove dropped items
+		{
+			ent->nextthink = level.time + 29;
+			ent->think = G_FreeEdict;
+		}
+	}
+}
+
+float mindEraserTime;
+void SpawnMinderaser(edict_t *ent)
+{
+	edict_t *minderaser, *cl_ent;
+	int i;
+
+	for (i = 0; i < g_maxclients->integer; i++)
+	{
+		cl_ent = g_edicts + 1 + i;
+		if (!cl_ent->inuse || cl_ent->is_bot)
+			continue;
+		safe_centerprintf(cl_ent, "A Mind Eraser has spawned!\n");
+	}
+
+	//to do - play level wide klaxxon 
+	gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/minderaser.wav" ), 1, ATTN_NONE, 0 );
+
+	minderaser = G_Spawn();
+	VectorCopy(ent->s.origin, minderaser->s.origin);
+	minderaser->spawnflags = DROPPED_PLAYER_ITEM;
+	minderaser->model = "models/weapons/g_minderaser/tris.md2";
+	minderaser->classname = "weapon_minderaser";
+	minderaser->item = FindItem ("Minderaser");
+	minderaser->s.effects = minderaser->item->world_model_flags;
+	minderaser->s.renderfx = RF_GLOW;
+	VectorSet (minderaser->mins, -15, -15, -15);
+	VectorSet (minderaser->maxs, 15, 15, 15);
+	gi.setmodel (minderaser, minderaser->item->world_model);
+	minderaser->solid = SOLID_TRIGGER;
+	minderaser->health = 100;
+	minderaser->movetype = MOVETYPE_TOSS;
+	minderaser->touch = drop_temp_touch;
+	minderaser->owner = NULL;
+
+	SetRespawn (ent, 1000000); //huge delay until ME is picked up from pad.			
+	minderaser->replaced_weapon = ent; //remember this entity
+
+	mindEraserTime = level.time;
+}
+
 void DoRespawn (edict_t *ent)
 {
+	char szTmp[64];
+	//Add mind eraser spawn here, if it's been two minutes since last respawn of it,
+	//go ahead and set the next weapon spawned to be the mind eraser
+	//need to check if first part of name is weapon
+	if(level.time > mindEraserTime + 120.0)
+	{
+		strcpy(szTmp, ent->classname);
+		szTmp[6] = 0;
+		if(!Q_strcasecmp(szTmp, "weapon"))
+		{
+			SpawnMinderaser(ent);
+			return;
+		}
+	}
+
 	if (ent->team)
 	{
 		edict_t	*master;
@@ -149,6 +232,28 @@ void DoRespawn (edict_t *ent)
 
 void SetRespawn (edict_t *ent, float delay)
 {
+
+    if (	ent->item && g_duel->integer && 
+    		ent->item->weapmodel != WEAP_MINDERASER		)
+	{
+		switch (ent->item->flags)
+		{
+			//TODO: playtest this and adjust the scaling factors.
+			case IT_WEAPON:
+				delay *= 3.0;
+				break;
+			case IT_POWERUP:
+			case IT_AMMO: //intentional fallthrough
+				delay *= 2.0;
+				break;
+			case IT_ARMOR:
+			case IT_HEALTH: //intentional fallthrough
+				delay *= 1.5;
+				break;
+			default:
+				break;
+		}
+	}
 	ent->flags |= FL_RESPAWN;
 	ent->svflags |= SVF_NOCLIENT;
 	ent->solid = SOLID_NOT;
@@ -170,11 +275,11 @@ qboolean Pickup_Powerup (edict_t *ent, edict_t *other)
 
 	other->client->pers.inventory[ITEM_INDEX(ent->item)]++;
 
-	if (deathmatch->value)
+	if (deathmatch->integer)
 	{
 		int randomSpawn;
 		//Phenax - Add random time to quad spawn
-		if(ent->item->use == Use_Quad && g_randomquad->value)
+		if(ent->item->use == Use_Quad && g_randomquad->integer)
 			randomSpawn = 10 + rand() % (30 - 10); //10 to 30 seconds randomness
 		else
 			randomSpawn = 0;
@@ -204,13 +309,13 @@ void Drop_General (edict_t *ent, gitem_t *item)
 
 qboolean Pickup_Adrenaline (edict_t *ent, edict_t *other)
 {
-	if (!deathmatch->value)
+	if (!deathmatch->integer)
 		other->max_health += 1;
 
 	if (other->health < other->max_health)
 		other->health = other->max_health;
 
-	if (!(ent->spawnflags & DROPPED_ITEM) && (deathmatch->value))
+	if (!(ent->spawnflags & DROPPED_ITEM) && (deathmatch->integer))
 		SetRespawn (ent, ent->item->quantity);
 
 	return true;
@@ -355,34 +460,51 @@ qboolean Add_Ammo (edict_t *ent, gitem_t *item, int count, qboolean weapon, qboo
 {
 	int			index;
 	int			max, base;
+	gitem_t     *failedswitch;
 
 	if (!ent->client)
 		return false;
 
-	if (item->tag == AMMO_BULLETS) {
+	if (item->tag == AMMO_BULLETS) 
+	{
 		max = ent->client->pers.max_bullets;
 		base = BASE_BULLETS;
 	}
-	else if (item->tag == AMMO_SHELLS) {
+	else if (item->tag == AMMO_SHELLS) 
+	{
 		max = ent->client->pers.max_shells;
 		base = BASE_SHELLS;
 	}
-	else if (item->tag == AMMO_ROCKETS) {
+	else if (item->tag == AMMO_ROCKETS) 
+	{
 		max = ent->client->pers.max_rockets;
 		base = BASE_ROCKETS;
 	}
-	else if (item->tag == AMMO_GRENADES) {
+	else if (item->tag == AMMO_GRENADES) 
+	{
 		max = ent->client->pers.max_grenades;
 		base = BASE_GRENADES;
 	}
-	else if (item->tag == AMMO_CELLS) {
+	else if (item->tag == AMMO_CELLS) 
+	{
 		max = ent->client->pers.max_cells;
 		base = BASE_CELLS;
 	}
-	else if (item->tag == AMMO_SLUGS) {
+	else if (item->tag == AMMO_SLUGS) 
+	{
 		max = ent->client->pers.max_slugs;
 		base = BASE_SLUGS;
 	}
+	else if (item->tag == AMMO_SEEKERS) 
+	{
+		max = ent->client->pers.max_seekers;
+		base = BASE_SEEKERS;
+	}
+	else if (item->tag == AMMO_BOMBS) 
+	{
+		max = ent->client->pers.max_bombs;
+		base = BASE_BOMBS;
+	}
 	else
 		return false;
 
@@ -402,6 +524,12 @@ qboolean Add_Ammo (edict_t *ent, gitem_t *item, int count, qboolean weapon, qboo
 
 	if (ent->client->pers.inventory[index] > max)
 		ent->client->pers.inventory[index] = max;
+    
+    failedswitch = ent->client->pers.lastfailedswitch;
+    if (failedswitch && failedswitch->ammo && 
+        (FindItem(failedswitch->ammo) == item) && 
+        (level.framenum - ent->client->pers.failedswitch_framenum) < 5)
+		ent->client->newweapon = failedswitch;
 
 	return true;
 }
@@ -427,12 +555,43 @@ qboolean Pickup_Ammo (edict_t *ent, edict_t *other)
 
 	if (weapon && !oldcount)
 	{
-		if (other->client->pers.weapon != ent->item && ( !deathmatch->value || other->client->pers.weapon == FindItem("blaster") ) )
+		if (other->client->pers.weapon != ent->item && ( !deathmatch->integer || other->client->pers.weapon == FindItem("Blaster") || other->client->pers.weapon == FindItem("Alien Blaster")) )
 			other->client->newweapon = ent->item;
 	}
 
-	if (!(ent->spawnflags & (DROPPED_ITEM | DROPPED_PLAYER_ITEM)) && (deathmatch->value))
-		SetRespawn (ent, 30);
+	if (!(ent->spawnflags & (DROPPED_ITEM | DROPPED_PLAYER_ITEM)) && (deathmatch->integer))
+	{
+		if(g_tactical->integer)
+		{
+			if(!strcmp(ent->classname, "ammo_cells") || !strcmp(ent->classname, "ammo_shells"))
+			{
+				if(!tacticalScore.alienPowerSource)
+				{
+					if(tacticalScore.alienBackupGen)
+						SetRespawn (ent, 20); //on backup power, generate ammo much slower
+					else
+						SetRespawn (ent, 40); //for the most part, dead
+				}
+				else
+					SetRespawn (ent, 5);
+			}
+			else if(!strcmp(ent->classname, "ammo_rockets") || !strcmp(ent->classname, "ammo_bullets") || !strcmp(ent->classname, "ammo_grenades"))
+			{
+				if(!tacticalScore.humanPowerSource)
+				{
+					if(tacticalScore.humanBackupGen)
+						SetRespawn (ent, 20);
+					else
+						SetRespawn (ent, 40);
+				}
+				else
+					SetRespawn (ent, 5);
+			}
+		}
+		else
+			SetRespawn (ent, 30);
+	}
+
 	return true;
 }
 
@@ -473,7 +632,7 @@ void MegaHealth_think (edict_t *self)
 		return;
 	}
 
-	if (!(self->spawnflags & DROPPED_ITEM) && (deathmatch->value))
+	if (!(self->spawnflags & DROPPED_ITEM) && (deathmatch->integer))
 		SetRespawn (self, 20);
 	else
 		G_FreeEdict (self);
@@ -516,7 +675,7 @@ qboolean Pickup_Health (edict_t *ent, edict_t *other)
 	}
 	else
 	{
-		if (!(ent->spawnflags & DROPPED_ITEM) && (deathmatch->value))
+		if (!(ent->spawnflags & DROPPED_ITEM) && (deathmatch->integer))
 			SetRespawn (ent, 30);
 	}
 
@@ -615,9 +774,9 @@ qboolean Pickup_Armor (edict_t *ent, edict_t *other)
 		}
 	}
 
-	if (!(ent->spawnflags & DROPPED_ITEM) && (deathmatch->value))
+	if (!(ent->spawnflags & DROPPED_ITEM) && (deathmatch->integer))
 		SetRespawn (ent, 20);
-
+	
 	return true;
 }
 
@@ -647,8 +806,6 @@ void Touch_Item (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf
 		other->client->bonus_alpha = 0.25;
 
 		// show icon and name on status bar
-		other->client->ps.stats[STAT_PICKUP_ICON] = gi.imageindex(ent->item->icon);
-		other->client->ps.stats[STAT_PICKUP_STRING] = CS_ITEMS+ITEM_INDEX(ent->item);
 		other->client->pickup_msg_time = level.time + 3.0;
 
 		// change selected item
@@ -681,6 +838,17 @@ void Touch_Item (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf
 	if (!taken)
 		return;
 
+	if(g_tactical->integer) //items do not respawn in tactical mode(except ammo when ammo depots are running)
+	{
+		if((!strcmp(ent->classname, "ammo_cells") || !strcmp(ent->classname, "ammo_shells")) && tacticalScore.alienAmmoDepot)
+			return;
+		else if((!strcmp(ent->classname, "ammo_rockets") || !strcmp(ent->classname, "ammo_bullets") || !strcmp(ent->classname, "ammo_grenades")) && tacticalScore.humanAmmoDepot)
+			return;
+
+		G_FreeEdict (ent); 
+		return;
+	}
+
 	if (ent->spawnflags & (DROPPED_ITEM | DROPPED_PLAYER_ITEM))
 	{
 		if (ent->flags & FL_RESPAWN)
@@ -692,24 +860,6 @@ void Touch_Item (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf
 
 //======================================================================
 
-static void drop_temp_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
-{
-	if (other == ent->owner)
-		return;
-
-	Touch_Item (ent, other, plane, surf);
-}
-
-static void drop_make_touchable (edict_t *ent)
-{
-	ent->touch = Touch_Item;
-	if (deathmatch->value)
-	{
-		ent->nextthink = level.time + 29;
-		ent->think = G_FreeEdict;
-	}
-}
-
 edict_t *Drop_Item (edict_t *ent, gitem_t *item)
 {
 	edict_t	*dropped;
@@ -945,7 +1095,7 @@ void SpawnItem (edict_t *ent, gitem_t *item)
 	}
 
 	// some items will be prevented in deathmatch
-	if (deathmatch->value)
+	if (deathmatch->integer)
 	{
 		if ( dmflags->integer & DF_NO_ARMOR )
 		{
@@ -979,7 +1129,7 @@ void SpawnItem (edict_t *ent, gitem_t *item)
 				return;
 			}
 		}
-		if(excessive->value || instagib->value || rocket_arena->value || insta_rockets->value )
+		if(excessive->integer || instagib->integer || rocket_arena->integer || insta_rockets->integer )
 		{
 			if (item->flags == IT_AMMO || (strcmp(ent->classname, "weapon_bfg") == 0) ||
 				(strcmp(ent->classname, "weapon_hyperblaster") == 0) ||
@@ -998,7 +1148,7 @@ void SpawnItem (edict_t *ent, gitem_t *item)
 
 	//CTF
 	//Don't spawn the flags unless enabled
-	if (!ctf->value &&
+	if (!ctf->integer &&
 		(strcmp(ent->classname, "item_flag_red") == 0 ||
 		strcmp(ent->classname, "item_flag_blue") == 0)) {
 		G_FreeEdict(ent);
@@ -1191,6 +1341,7 @@ gitem_t	itemlist[] =
 /* precache */ "misc/blue_scores.wav misc/blue_takes.wav misc/blue_increases.wav misc/blue_wins.wav misc/blue_picked.wav"
 	},
 
+#ifndef ALTERIA
 /*QUAKED item_bomber (1 0.2 0) (-16 -16 -24) (16 16 32)
 */
 	{
@@ -1259,6 +1410,49 @@ gitem_t	itemlist[] =
 		NULL
 	},
 
+//Tactical bombs
+	{
+		"item_alien_bomb",
+		NULL, //Pickup_alienBomb, //only owner will be able to pick this item back up
+		Use_Weapon,
+		NULL, 
+		Weapon_TacticalBomb, //fire out bomb, not too far, like prox mines 
+		NULL,
+		"models/tactical/alien_bomb.iqm", 0,
+		"vehicles/deathball/v_wep.md2", //will use db's vweap for bombs and detonators
+/* icon */		"abomb", 
+/* pickup */	"Alien Bomb",
+		0,
+		1,
+		"bombs",
+		IT_WEAPON,
+		WEAP_ABOMB,
+		NULL,
+		0,
+		NULL
+	},
+
+	{
+		"item_human_bomb",
+		NULL, //Pickup_humanBomb, //only owner will be able to pick this item back up
+		Use_Weapon,
+		NULL, 
+		Weapon_TacticalBomb, //fire out bomb, not too far, like prox mines 
+		NULL,
+		"models/tactical/human_bomb.iqm", 0,
+		"vehicles/deathball/v_wep.md2", //will use db's vweap for bombs and detonators
+/* icon */		"abomb", 
+/* pickup */	"Human Bomb",
+		0,
+		1,
+		"bombs",
+		IT_WEAPON,
+		WEAP_HBOMB,
+		NULL,
+		0,
+		NULL
+	},
+
 /*QUAKED item_deathball (1 0.2 0) (-16 -16 -24) (16 16 32)
 */
 	{
@@ -1283,7 +1477,7 @@ gitem_t	itemlist[] =
 	},
 
 	//a fake item for bots to use as a target for throwing a deathball at.
-		{
+	{
 		"item_dbtarget",
 		NULL,
 		NULL,
@@ -1343,6 +1537,7 @@ gitem_t	itemlist[] =
 		0,
 		NULL
 	},
+#endif
 	//
 	// WEAPONS
 	//
@@ -1369,6 +1564,51 @@ always owned, never in the world
 		0,
 /* precache */ "weapons/electroball.wav"
 	},
+
+#ifdef ALTERIA
+	//note some of this is clearly temporary placeholder
+	{
+		"weapon_warrior_punch",
+		NULL,
+		Use_Weapon,
+		NULL,
+		Weapon_Punch,
+		"misc/w_pkup.wav",
+		NULL, 0,
+		"models/weapons/v_warriorhands/tris.md2",
+/* icon */		"warriorpunch",
+/* pickup */	"Warriorpunch",
+		0,
+		0,
+		NULL,
+		IT_WEAPON,
+		WEAP_VIOLATOR,
+		NULL,
+		0,
+/* precache */ "weapons/viofire1.wav weapons/viofire2.wav"
+	},
+
+	{
+		"weapon_wizard_punch",
+		NULL,
+		Use_Weapon,
+		NULL,
+		Weapon_Punch,
+		"misc/w_pkup.wav",
+		NULL, 0,
+		"models/weapons/v_wizardhands/tris.md2",
+/* icon */		"wizardpunch",
+/* pickup */	"Wizardpunch",
+		0,
+		0,
+		NULL,
+		IT_WEAPON,
+		WEAP_VIOLATOR,
+		NULL,
+		0,
+/* precache */ "weapons/viofire1.wav weapons/viofire2.wav"
+	},
+#else
 /* weapon_blaster (.3 .3 1) (-16 -16 -16) (16 16 16)
 always owned, never in the world
 */
@@ -1393,6 +1633,30 @@ always owned, never in the world
 /* precache */ "weapons/blastf1a.wav misc/lasfly.wav"
 	},
 
+	/* weapon_blaster (.3 .3 1) (-16 -16 -16) (16 16 16)
+always owned, never in the world
+*/
+	{
+		"weapon_alienblaster",
+		NULL,
+		Use_Weapon,
+		NULL,
+		Weapon_AlienBlaster,
+		"misc/w_pkup.wav",
+		NULL, 0,
+		"models/weapons/v_alienblast/tris.md2",
+/* icon */		"alienblaster",
+/* pickup */	"Alien Blaster",
+		0,
+		0,
+		NULL,
+		IT_WEAPON,
+		WEAP_BLASTER,
+		NULL,
+		0,
+/* precache */ "weapons/blastf1a.wav misc/lasfly.wav" //to do tactical - change sound
+	},
+
 	{
 		"weapon_violator",
 		NULL,
@@ -1575,6 +1839,27 @@ always owned, never in the world
 /* precache */ "weapons/energyfield.wav smallmech/sight.wav weapons/vaporizer_hum.wav"
 	},
 
+	{
+		"weapon_minderaser",
+		Pickup_Weapon,
+		Use_Weapon,
+		NULL, //never drop
+		Weapon_Minderaser,
+		"misc/w_pkup.wav",
+		"models/weapons/g_minderaser/tris.md2", EF_ROTATE,
+		"models/weapons/v_minderaser/tris.md2",
+/* icon */		"minderaser",
+/* pickup */	"Minderaser",
+		0,
+		1,
+		"Seekers", 
+		IT_WEAPON,
+		WEAP_MINDERASER,
+		NULL,
+		0,
+/* precache */ "weapons/clank.wav weapons/minderaserfire.wav weapons/shotgf1b.wav weapons/smartgun_hum.wav"
+	},
+#endif
 	//
 	// AMMO ITEMS
 	//
@@ -1716,6 +2001,49 @@ always owned, never in the world
 /* precache */ ""
 	},
 
+	{
+		"ammo_seekers",
+		Pickup_Ammo, //set to null
+		NULL,
+		Drop_Ammo, //set to null
+		NULL,
+		"misc/am_pkup.wav",
+		NULL,
+		0,
+		NULL,
+/* icon */		"a_seekers",
+/* pickup */	"Seekers",
+/* width */		3,
+		1,
+		NULL,
+		IT_AMMO,
+		0,
+		NULL,
+		AMMO_SEEKERS,
+/* precache */ ""
+	},
+
+	{
+		"ammo_bombs",
+		Pickup_Ammo, //set to null
+		NULL,
+		Drop_Ammo, //set to null
+		NULL,
+		"misc/am_pkup.wav",
+		NULL,
+		0,
+		NULL,
+/* icon */		"a_bombs",
+/* pickup */	"Bombs",
+/* width */		3,
+		1,
+		NULL,
+		IT_AMMO,
+		0,
+		NULL,
+		AMMO_BOMBS,
+/* precache */ ""
+	},
 
 	//
 	// POWERUP ITEMS
@@ -1783,7 +2111,7 @@ gives +1 to maximum health
 /* width */		2,
 		60,
 		NULL,
-		0,
+		IT_HEALTH,
 		0,
 		NULL,
 		0,
@@ -1804,7 +2132,7 @@ gives +1 to maximum health
 /* width */		3,
 		0,
 		NULL,
-		0,
+		IT_HEALTH,
 		0,
 		NULL,
 		0,
@@ -1884,7 +2212,7 @@ gives +1 to maximum health
 */
 void SP_item_health (edict_t *self)
 {
-	if ( deathmatch->value && (dmflags->integer & DF_NO_HEALTH) )
+	if ( deathmatch->integer && (dmflags->integer & DF_NO_HEALTH) )
 	{
 		G_FreeEdict (self);
 		return;
@@ -1900,7 +2228,7 @@ void SP_item_health (edict_t *self)
 */
 void SP_item_health_small (edict_t *self)
 {
-	if ( deathmatch->value && (dmflags->integer & DF_NO_HEALTH) )
+	if ( deathmatch->integer && (dmflags->integer & DF_NO_HEALTH) )
 	{
 		G_FreeEdict (self);
 		return;
@@ -1918,7 +2246,7 @@ void SP_item_health_small (edict_t *self)
 */
 void SP_item_health_large (edict_t *self)
 {
-	if ( deathmatch->value && (dmflags->integer & DF_NO_HEALTH) )
+	if ( deathmatch->integer && (dmflags->integer & DF_NO_HEALTH) )
 	{
 		G_FreeEdict (self);
 		return;
@@ -1935,7 +2263,7 @@ void SP_item_health_large (edict_t *self)
 */
 void SP_item_health_mega (edict_t *self)
 {
-	if ( deathmatch->value && (dmflags->integer & DF_NO_HEALTH) )
+	if ( deathmatch->integer && (dmflags->integer & DF_NO_HEALTH) )
 	{
 		G_FreeEdict (self);
 		return;
