@@ -83,6 +83,20 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define MELEE_DISTANCE	80
 
+// Anti-camp defaults
+/** @brief Default anti-camp frames
+ *
+ * Default value for the ac_frames anti-camp CVar.
+ */
+#define G_ANTICAMP_FRAMES	30
+
+/** @brief Default anti-camp threshold
+ *
+ * Default value for the ac_threshold anti-camp CVar.
+ */
+#define G_ANTICAMP_THRESHOLD	100
+
+
 #define BODY_QUEUE_SIZE		8
 
 typedef enum
@@ -107,7 +121,9 @@ typedef enum
 	AMMO_ROCKETS,
 	AMMO_GRENADES,
 	AMMO_CELLS,
-	AMMO_SLUGS
+	AMMO_SLUGS,
+	AMMO_SEEKERS,
+	AMMO_BOMBS
 } ammo_t;
 
 //teams
@@ -129,10 +145,6 @@ typedef struct teamcensus_s
 #define TEAM_GAME ( (dmflags->integer & (DF_SKINTEAMS)) \
 		|| ctf->integer || tca->integer || cp->integer )
 
-#define RED_TEAM				0
-#define BLUE_TEAM				1
-#define NO_TEAM					2
-
 //clientinfo origins
 #define INGAME					0
 #define SPAWN					1
@@ -252,7 +264,9 @@ typedef struct
 #define IT_ARMOR		4
 #define IT_KEY			8
 #define IT_POWERUP		16
+#define IT_HEALTH		32
 
+//to do - add alteria items - will require a number of other changes, particularly in bot code as well
 // gitem_t->weapmodel for weapons indicates model index
 #define WEAP_BLASTER			1
 #define WEAP_SMARTGUN			2
@@ -264,12 +278,13 @@ typedef struct
 #define WEAP_BEAMGUN			10
 #define WEAP_VAPORIZER			11
 #define WEAP_VIOLATOR			12
-#define WEAP_BOMBER				13
-#define WEAP_STRAFER			14
-#define WEAP_DEATHBALL			15
-#define WEAP_HOVER				16
-
-
+#define WEAP_MINDERASER			13
+#define WEAP_BOMBER				14
+#define WEAP_STRAFER			15
+#define WEAP_DEATHBALL			16
+#define WEAP_HOVER				17
+#define WEAP_ABOMB				18
+#define WEAP_HBOMB				19
 
 typedef struct gitem_s
 {
@@ -300,8 +315,6 @@ typedef struct gitem_s
 	char		*precaches;		// string of all models, sounds, and images this item will use
 } gitem_t;
 
-
-
 //
 // this structure is left intact through an entire game
 // it should be initialized at dll load time, and read/written to
@@ -329,7 +342,6 @@ typedef struct
 	qboolean	autosaved;
 } game_locals_t;
 
-
 //
 // this structure is cleared as each map is entered
 // it is read/written to the level.sav file for savegames
@@ -381,7 +393,6 @@ typedef struct
 
 } level_locals_t;
 
-
 // spawn_temp_t is only used to hold entity field values that
 // can be set from the editor, but aren't actualy present
 // in edict_t during gameplay
@@ -407,7 +418,6 @@ typedef struct
 	float		maxpitch;
 } spawn_temp_t;
 
-
 typedef struct
 {
 	// fixed data
@@ -438,7 +448,6 @@ typedef struct
 	void		(*endfunc)(edict_t *);
 } moveinfo_t;
 
-
 typedef struct
 {
 	void	(*aifunc)(edict_t *self, float dist);
@@ -486,7 +495,26 @@ typedef struct
 
 } monsterinfo_t;
 
+//tactical
+typedef struct
+{
+	qboolean alienComputer;
+	int		 alienComputerHealth;
+	qboolean alienPowerSource;
+	int		 alienPowerSourceHealth;
+	qboolean alienAmmoDepot;
+	int		 alienAmmoDepotHealth;
+	qboolean alienBackupGen;
 
+	qboolean humanComputer;
+	int		 humanComputerHealth;
+	qboolean humanPowerSource;
+	int		 humanPowerSourceHealth;
+	qboolean humanAmmoDepot;
+	int		 humanAmmoDepotHealth;
+	qboolean humanBackupGen;
+
+} tactical_t;
 
 extern	game_locals_t	game;
 extern	level_locals_t	level;
@@ -494,11 +522,14 @@ extern	game_import_t	gi;
 extern	game_export_t	globals;
 extern	spawn_temp_t	st;
 extern	g_vote_t		playervote;
+extern  tactical_t		tacticalScore;
 
 extern	int	sm_meat_index;
 
 extern  int red_team_score;
 extern  int blue_team_score;
+extern	int	red_team_matches; //for tca
+extern	int	blue_team_matches; //for tca
 extern  int reddiff;
 extern	int bluediff;
 extern	int redwinning;
@@ -540,6 +571,8 @@ extern  int print3;
 #define MOD_TARGET_BLASTER	33
 #define MOD_GRAPPLE			34
 #define MOD_HEADSHOT		35
+#define MOD_MINDERASER		36
+#define MOD_SPIDER			37
 #define MOD_FRIENDLY_FIRE   0x8000000
 
 extern	int	meansOfDeath;
@@ -569,6 +602,7 @@ extern	cvar_t	*needpass;
 extern	cvar_t	*g_select_empty;
 extern	cvar_t	*g_dedicated;
 extern	cvar_t	*motdfile;
+extern	cvar_t	*motdforce;
 
 extern	cvar_t	*filterban;
 
@@ -603,6 +637,7 @@ extern  cvar_t	*sv_custombots;
 //mutators
 extern  cvar_t  *instagib;
 extern  cvar_t  *rocket_arena;
+extern  cvar_t *insta_rockets;
 extern  cvar_t  *low_grav;
 extern  cvar_t  *regeneration;
 extern  cvar_t  *vampire;
@@ -613,6 +648,9 @@ extern  cvar_t  *classbased;
 //duel mode
 extern	cvar_t	*g_duel;
 
+//tactical mode
+extern	cvar_t	*g_tactical;
+
 extern	cvar_t	*g_losehealth;
 extern	cvar_t	*g_losehealth_num;
 
@@ -628,6 +666,8 @@ extern	cvar_t	*g_maxrockets;
 extern	cvar_t	*g_maxgrenades;
 extern	cvar_t	*g_maxcells;
 extern	cvar_t	*g_maxslugs;
+extern	cvar_t	*g_maxseekers;
+extern  cvar_t	*g_maxbombs;
 
 //quick weapon change
 extern  cvar_t  *quickweap;
@@ -635,6 +675,8 @@ extern  cvar_t  *quickweap;
 //anti-camp
 extern  cvar_t  *anticamp;
 extern  cvar_t  *camptime;
+extern  cvar_t	*ac_frames;
+extern  cvar_t	*ac_threshold;
 
 extern	cvar_t  *g_randomquad;
 
@@ -662,6 +704,9 @@ extern cvar_t	*g_autobalance;
 //reward point threshold
 extern cvar_t	*g_reward;
 
+//show player lights for visibility even in non-team games
+extern cvar_t	*g_dmlights;
+
 #define world	(&g_edicts[0])
 
 // item spawnflags
@@ -707,11 +752,12 @@ typedef struct
 extern	field_t fields[];
 extern	gitem_t	itemlist[];
 
+//mind eraser globals
+extern float mindEraserTime;
 
 //
 // g_cmds.c
 //
-void Cmd_Help_f (edict_t *ent);
 void Cmd_Score_f (edict_t *ent);
 void Cmd_VoiceTaunt_f (edict_t *ent);
 
@@ -736,7 +782,7 @@ qboolean Add_Ammo (edict_t *ent, gitem_t *item, int count, qboolean weapon, qboo
 void Touch_Item (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf);
 
 //CTF
-void CTFDeadDropFlag(edict_t *self);
+void CTFDeadDropFlag(edict_t *self, edict_t *other);
 void CTFResetFlag(int team);
 void CTFEffects(edict_t *player);
 void CTFScoreboardMessage (edict_t *ent, edict_t *killer, int mapvote);
@@ -788,7 +834,7 @@ float	*tv (float x, float y, float z);
 char	*vtos (vec3_t v);
 
 float vectoyaw (vec3_t vec);
-void vectoangles (vec3_t vec, vec3_t angles);
+// void vectoangles (vec3_t vec, vec3_t angles);
 
 //
 // g_combat.c
@@ -862,25 +908,30 @@ qboolean FacingIdeal(edict_t *self);
 void ThrowDebris (edict_t *self, char *modelname, float speed, vec3_t origin);
 void ThrowDebris2 (edict_t *self, char *modelname, float speed, vec3_t origin);
 
+#ifdef ALTERIA
+void fire_punch(edict_t *self, vec3_t start, vec3_t aimdir, int damage);
+#else
 void fire_bullet (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, int hspread, int vspread, int mod);
 void fire_shotgun (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, int hspread, int vspread, int count, int mod);
-void fire_blaster (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int speed, int effect, qboolean hyper);
+void fire_blaster (edict_t *self, vec3_t start, vec3_t muzzle, vec3_t aimdir, int damage, int speed, int effect, qboolean hyper);
 void fire_rocket (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage);
 void fire_homingrocket (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage);
-void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick);
+void fire_disruptor (edict_t *self, vec3_t start, vec3_t muzzle, vec3_t aimdir, int damage, int kick);
 void fire_bomb (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage, float timer);
-void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, qboolean detonate);
-void fire_floater (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage, float timer);
-void fire_floater_alien (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage, float timer);
+void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, qboolean detonate, qboolean alien);
+void fire_smartgrenade (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage, float timer);
+void fire_smartgrenade_alien (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage, float timer);
 void fire_flamethrower(edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius);
 void fire_hover_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, qboolean detonate);
-
-//new weapon code
-void fire_energy_field (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick);
-void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, int effect, qboolean hyper);
+void fire_minderaser (edict_t *self, vec3_t start, vec3_t dir, float timer);
+void fire_spider (edict_t *self, vec3_t start, vec3_t aimdir, int speed);
+void fire_vaporizer (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick);
+void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, int effect, qboolean hyper, qboolean alien);
 void fire_prox (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage, float timer);
 void fire_fireball (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int speed, float damage_radius, int radius_damage);
 void fire_violator(edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, int alt);
+void fire_tacticalbomb (edict_t *self, vec3_t start, vec3_t aimdir, int speed);
+#endif
 
 //deathball
 void fire_deathball (edict_t *self, vec3_t start, vec3_t dir, int speed);
@@ -929,6 +980,7 @@ void MoveClientsDownQueue(edict_t *ent);
 void SaveClientData (void);
 void FetchClientEntData (edict_t *ent);
 void TeamCensus( teamcensus_t* team_census );
+void ParseClassFile( char *config_file, edict_t *ent );
 
 //unlagged - g_unlagged.c
 void G_ResetHistory( edict_t *ent );
@@ -938,7 +990,7 @@ void G_UnTimeShiftAllClients( edict_t *skip );
 void G_DoTimeShiftFor( edict_t *ent );
 void G_UndoTimeShiftFor( edict_t *ent );
 void G_UnTimeShiftClient( edict_t *ent );
-void G_PredictPlayerMove( edict_t *ent, float frametime );
+void G_AntilagProjectile( edict_t *ent );
 //unlagged - g_unlagged.c
 
 //
@@ -962,9 +1014,7 @@ void ClientEndServerFrame (edict_t *ent);
 // p_hud.c
 //
 void MoveClientToIntermission (edict_t *client);
-void G_SetStats (edict_t *ent);
-void G_SetSpectatorStats (edict_t *ent);
-void G_CheckChaseStats (edict_t *ent);
+void G_UpdateStats (edict_t *ent);
 void ValidateSelectedItem (edict_t *ent);
 void DeathmatchScoreboardMessage (edict_t *client, edict_t *killer, int mapvote);
 
@@ -986,8 +1036,8 @@ void M_ChangeYaw (edict_t *ent);
 //
 // g_phys.c
 //
-void G_RunEntity (edict_t *ent);
-
+void G_RunEntity (edict_t *ent, float timespan);
+void SV_CheckVelocity (edict_t *ent);
 
 //
 // g_chase.c
@@ -1002,6 +1052,104 @@ void GetChaseTarget(edict_t *ent);
 //
 qboolean G_NameMatch( const char* netname, const char *kickname );
 
+//
+// Spawn functions
+//
+extern void SP_item_health (edict_t *self);
+extern void SP_item_health_small (edict_t *self);
+extern void SP_item_health_large (edict_t *self);
+extern void SP_item_health_mega (edict_t *self);
+
+extern void SP_info_player_start (edict_t *ent);
+extern void SP_info_player_deathmatch (edict_t *ent);
+extern void SP_info_player_intermission (edict_t *ent);
+extern void SP_info_player_red (edict_t *ent);
+extern void SP_info_player_blue(edict_t *ent);
+
+extern void SP_func_plat (edict_t *ent);
+extern void SP_func_rotating (edict_t *ent);
+extern void SP_func_button (edict_t *ent);
+extern void SP_func_door (edict_t *ent);
+extern void SP_func_door_secret (edict_t *ent);
+extern void SP_func_door_rotating (edict_t *ent);
+extern void SP_func_water (edict_t *ent);
+extern void SP_func_train (edict_t *ent);
+extern void SP_func_conveyor (edict_t *self);
+extern void SP_func_wall (edict_t *self);
+extern void SP_func_object (edict_t *self);
+extern void SP_func_explosive (edict_t *self);
+extern void SP_func_timer (edict_t *self);
+extern void SP_func_areaportal (edict_t *ent);
+extern void SP_func_killbox (edict_t *ent);
+
+extern void SP_trigger_always (edict_t *ent);
+extern void SP_trigger_once (edict_t *ent);
+extern void SP_trigger_multiple (edict_t *ent);
+extern void SP_trigger_relay (edict_t *ent);
+extern void SP_trigger_push (edict_t *ent);
+extern void SP_trigger_hurt (edict_t *ent);
+extern void SP_trigger_key (edict_t *ent);
+extern void SP_trigger_counter (edict_t *ent);
+extern void SP_trigger_elevator (edict_t *ent);
+extern void SP_trigger_gravity (edict_t *ent);
+extern void SP_trigger_monsterjump (edict_t *ent);
+extern void SP_trigger_deathballtarget (edict_t *ent);
+extern void SP_trigger_reddeathballtarget (edict_t *ent);
+extern void SP_trigger_bluedeathballtarget (edict_t *ent);
+extern void SP_trigger_redcowtarget ( edict_t *ent);
+extern void SP_trigger_bluecowtarget (edict_t *ent);
+
+extern void SP_target_temp_entity (edict_t *ent);
+extern void SP_target_speaker (edict_t *ent);
+extern void SP_target_explosion (edict_t *ent);
+extern void SP_target_secret (edict_t *ent);
+extern void SP_target_splash (edict_t *ent);
+extern void SP_target_steam (edict_t *ent);
+extern void SP_target_spawner (edict_t *ent);
+extern void SP_target_blaster (edict_t *ent);
+extern void SP_target_laser (edict_t *self);
+extern void SP_target_lightramp (edict_t *self);
+extern void SP_target_earthquake (edict_t *ent);
+extern void SP_target_fire (edict_t *ent);
+extern void SP_target_changelevel (edict_t *ent);
+
+extern void SP_worldspawn (edict_t *ent);
+
+extern void SP_light (edict_t *self);
+extern void SP_info_null (edict_t *self);
+extern void SP_info_notnull (edict_t *self);
+extern void SP_path_corner (edict_t *self);
+extern void SP_point_combat (edict_t *self);
+
+extern void SP_misc_teleporter (edict_t *self);
+extern void SP_misc_teleporter_dest (edict_t *self);
+
+extern void SP_npc_cow (edict_t *self);
+extern void SP_npc_deathray(edict_t *self);
+
+//TCA
+extern void SP_misc_spiderpod (edict_t *self);
+extern void SP_misc_rednode (edict_t *self);
+extern void SP_misc_bluenode (edict_t *self);
+extern void SP_misc_bluespidernode (edict_t *self);
+extern void SP_misc_redspidernode (edict_t *self);
+
+//Tactical
+extern void SP_misc_aliencomputer (edict_t *self);
+extern void SP_misc_humancomputer (edict_t *self);
+extern void SP_misc_alienpowersrc (edict_t *self);
+extern void SP_misc_humanpowersrc (edict_t *self);
+extern void SP_misc_alienammodepot (edict_t *self);
+extern void SP_misc_humanammodepot (edict_t *self);
+extern void SP_misc_alienbackupgen (edict_t *self);
+extern void SP_misc_humanbackupgen (edict_t *self);
+extern void SP_misc_deathray (edict_t *self);
+extern void SP_misc_laser (edict_t *self);
+
+extern void SP_misc_mapmodel (edict_t *self);
+extern void SP_misc_watersplash (edict_t *ent);
+extern void SP_misc_electroflash (edict_t *ent);
+
 
 //============================================================================
 
@@ -1020,6 +1168,8 @@ qboolean G_NameMatch( const char* netname, const char *kickname );
 #define BASE_SLUGS		10
 #define BASE_GRENADES	50
 #define BASE_BULLETS	50
+#define BASE_SEEKERS	1
+#define BASE_BOMBS		1
 
 
 // client data that stays across multiple level loads
@@ -1050,9 +1200,13 @@ typedef struct
 	int			max_grenades;
 	int			max_cells;
 	int			max_slugs;
+	int			max_seekers;
+	int			max_bombs;
 
 	gitem_t		*weapon;
 	gitem_t		*lastweapon;
+	gitem_t     *lastfailedswitch;
+	float       failedswitch_framenum;
 
 	int			game_helpchanged;
 	int			helpchanged;
@@ -1137,8 +1291,21 @@ struct gclient_s
 	qboolean	showhelp;
 	qboolean	showhelpicon;
 
+	/** @brief "Message of the day" frames left
+	 *
+	 * Prevent centerprinting until the MOTD has been displayed
+	 * for some time. This makes sure players can actually read
+	 * the MOTD.
+	 */
+	int		motd_frames;
+
 	//stuff for bots
 	qboolean	is_bot;
+	/* --------------------------*/
+	/*
+	 * this is identical to declarations in edict_t
+	 * it appears this data in gclient_s is not used.
+	 */
 	int			skill;
 	char		faveweap[64];
 	float		weapacc[10];
@@ -1152,6 +1319,7 @@ struct gclient_s
 	char		chatmsg6[128];
 	char		chatmsg7[128];
 	char		chatmsg8[128];
+	/*----------------------------*/
 
 	int			ammo_index;
 
@@ -1257,8 +1425,10 @@ struct gclient_s
 	//movetime - used for special moves
 	float lastmovetime;
 	int lastsidemove;
+	int lastforwardmove;
 	int dodge;
 	int moved;
+	int keydown;
 	float lastdodge;
 
 	float spawnprotecttime;
@@ -1324,6 +1494,15 @@ struct edict_s
 	solid_t		solid;
 	int			clipmask;
 	edict_t		*owner;
+	
+	int			redirect_number;	//for ghost mode
+	
+	// must be accessible to server code for collision detection
+	int			dmteam;
+	int			teamset;
+#define RED_TEAM				0
+#define BLUE_TEAM				1
+#define NO_TEAM					2
 
 
 	// DO NOT MODIFY ANYTHING ABOVE THIS, THE SERVER
@@ -1444,9 +1623,8 @@ struct edict_s
 	monsterinfo_t	monsterinfo;
 
 	edict_t		*flashlight;
-
-	int			dmteam;
-	int			teamset;
+	
+	edict_t		*replaced_weapon;
 
 // ACEBOT_ADD
 	qboolean is_bot;
@@ -1501,6 +1679,12 @@ struct edict_s
 	int	 Flames;
 	int	 FlameDamage;
 	edict_t *orb;
+	
+	//Prox mines and grenades
+	int		nade_timer;
+
+	//bombs
+	int		armed;
 
 	//alt-fires
 	qboolean altfire;
@@ -1511,6 +1695,14 @@ struct edict_s
 	//class(human = 0; alien = 1; robot = 2;)
 	int ctype;
 
+	//tactical mode
+	qboolean has_bomb;
+	qboolean has_detonator;
+	qboolean has_vaporizor;
+	qboolean has_minderaser;
+	int armor_type;
+	char charModel[128];
+
 	//model specific gibs
 	int usegibs;
 	char head[64];
@@ -1518,6 +1710,38 @@ struct edict_s
 	char arm[64];
 	char body[64];
 
+	// Anti-camp velocity accumulation
+	
+	/** @brief Velocities history
+	 *
+	 * At each server frame, a client's velocity will be stored inside
+	 * this array. While the array may contain up to 100 vectors, the
+	 * ac_frames CVar will determine how many of these vectors will be
+	 * used.
+	 */
+	vec3_t old_velocities[ 100 ];
+
+	/** @brief Velocity accumulator
+	 *
+	 * This vector contains the sum of velocities for the past frames.
+	 * The amount of frames that are indeed accumulated depends on the
+	 * value of the ac_frames CVar.
+	 */
+	vec3_t velocity_accum;
+
+	/** @brief Velocities history index
+	 *
+	 * The last updated vector in the old_velocities array.
+	 */
+	int old_velocities_current;
+
+	/** @brief Stored velocity vectors
+	 *
+	 * The actual amount of vectors currently stored in the old_velocities
+	 * array.
+	 */
+	int old_velocities_count;
+
 };
 
 struct gmapvote_s
@@ -1549,6 +1773,7 @@ extern qboolean StringToFilter (char *s, ipfilter_t *f);
 //unlagged
 extern	cvar_t	*g_antilag;
 extern  cvar_t	*g_antilagdebug;
+extern	cvar_t	*g_antilagprojectiles;
 
 // ACEBOT_ADD
 #include "acesrc/acebot.h"
