@@ -42,6 +42,7 @@ game_import_t	gi;
 game_export_t	globals;
 spawn_temp_t	st;
 g_vote_t		playervote;
+tactical_t		tacticalScore;
 
 int	sm_meat_index;
 int meansOfDeath;
@@ -69,6 +70,9 @@ cvar_t  *classbased;
 //duel mode
 cvar_t	*g_duel;
 
+//tactical mode
+cvar_t	*g_tactical;
+
 cvar_t	*g_losehealth;
 cvar_t	*g_losehealth_num;
 
@@ -84,6 +88,8 @@ cvar_t	*g_maxrockets;
 cvar_t	*g_maxgrenades;
 cvar_t	*g_maxcells;
 cvar_t	*g_maxslugs;
+cvar_t	*g_maxseekers;
+cvar_t	*g_maxbombs;
 
 //quick weapon change
 cvar_t  *quickweap;
@@ -203,6 +209,7 @@ cvar_t  *sv_custombots;
 //unlagged
 cvar_t	*g_antilag;
 cvar_t	*g_antilagdebug;
+cvar_t	*g_antilagprojectiles;
 
 void SpawnEntities (char *mapname, char *entities, char *spawnpoint);
 void ClientThink (edict_t *ent, usercmd_t *cmd);
@@ -265,6 +272,25 @@ void ShutdownGame (void)
 	gi.FreeTags (TAG_GAME);
 }
 
+/*
+=============
+G_ForceExitIntermission
+
+To prevent those annoying occurrences when you join a server which has been 
+idling at intermission for a long time, then it instantly switches maps as 
+soon as the old one is done loading.
+
+Now new player joins will force intermission to end after twenty seconds, even
+if there are still players in the server.
+=============
+*/
+void ExitLevel (void);
+void G_ForceExitIntermission (void)
+{
+	if (level.time < level.intermissiontime + 20.0 || !level.intermissiontime)
+		return;
+	ExitLevel();
+}
 
 /*
 =================
@@ -292,6 +318,7 @@ game_export_t *GetGameAPI (game_import_t *import)
 	globals.ACESP_FindBotNum = ACESP_FindBotNum;
 
 	globals.RunFrame = G_RunFrame;
+	globals.ForceExitIntermission = G_ForceExitIntermission;
 
 	globals.ServerCommand = ServerCommand;
 
@@ -325,6 +352,9 @@ void ClientEndServerFrames (void)
 	int		i;
 	edict_t	*ent;
 
+	/* make sure bot info in first client is up to date */
+	ACESP_UpdateBots();
+
 	// calc the player views now that all pushing
 	// and damage has been added
 	for (i=0 ; i<g_maxclients->value ; i++)
@@ -379,18 +409,18 @@ edict_t *CreateTargetChangeLevel(char *map)
  * 8: "violator"
  *
  *--- weapon_hit[] incremented ---
- * 0 : blasterball_touch : Weapon_Beamgun, Weapon_Blaster, Weapon_Hover
- * 0 : fire_blaster_beam : Weapon_Beamgun, Weapon_Blaster, Weapon_Strafer
- * 1 : fire_plasma       : Weapon_Disruptor
- * 2 : floater_think     : Weapon_Smartgun
- * 3 : fire_lead         : Weapon_Chain
- * 4 : fireball_touch    : Weapon_Flame
- * 4 : flame_touch       : Weapon_Flame
- * 5 : rocket_touch      : Weapon_RocketLauncher, Weapon_Bomber, Weapon_Strafer
- * 6 : fire_blaster      : Weapon_Beamgun, Weapon_Blaster
- * 7 : bomb_touch        : Weapon_Bomber, Weapon_Vaporizer
- * 7 : fire_energy_field : Weapon_Vaporizer
- * 8 : fire_violator     : Weapon_Violator
+ * 0 : blasterball_touch  : Weapon_Beamgun, Weapon_Blaster, Weapon_Hover
+ * 0 : fire_blaster_beam  : Weapon_Beamgun, Weapon_Blaster, Weapon_Alienblaster, Weapon_Strafer
+ * 1 : fire_disruptor     : Weapon_Disruptor
+ * 2 : smartgrenade_think : Weapon_Smartgun
+ * 3 : fire_lead          : Weapon_Chain
+ * 4 : fireball_touch     : Weapon_Flame
+ * 4 : flame_touch        : Weapon_Flame
+ * 5 : rocket_touch       : Weapon_RocketLauncher, Weapon_Bomber, Weapon_Strafer
+ * 6 : fire_blaster       : Weapon_Beamgun, Weapon_Blaster
+ * 7 : bomb_touch         : Weapon_Bomber, Weapon_Vaporizer
+ * 7 : fire_vaporizer     : Weapon_Vaporizer
+ * 8 : fire_violator      : Weapon_Violator
 
  * --- from sample.cfg accuracy
  * weapon_hit[] :to: bot weapacc[]
@@ -536,7 +566,8 @@ void EndDMLevel (void)
 	}
 
 	//call voting
-	if(g_callvote->value) {
+	if(g_callvote->value) 
+	{
 		playervote.called = false;
 		playervote.yay = 0;
 		playervote.nay = 0;
@@ -544,7 +575,8 @@ void EndDMLevel (void)
 	}
 
 	//map voting
-	if(g_mapvote->value) {
+	if(g_mapvote->value) 
+	{
 		level.changemap = level.mapname;
 
 		// initialise map list using the current map's name
@@ -740,9 +772,14 @@ void EndDMLevel (void)
 	//stick something in here to filter out CTF, and just make it loop back
 	for (i = 0; i < nummaps; i++) {
 		if (Q_strcasecmp(mapnames[i], level.mapname) == 0) {
-			if(mapnames[i+1][0])
-				BeginIntermission (CreateTargetChangeLevel (mapnames[i+1]) );
-			else if(mapnames[0][0]) //no more maps, repeat list
+			if(mapnames[i+1])
+			{
+				if(mapnames[i+1][0])
+					BeginIntermission (CreateTargetChangeLevel (mapnames[i+1]) );
+				else if(mapnames[0][0]) //no more maps, repeat list
+					BeginIntermission (CreateTargetChangeLevel (mapnames[0]) );
+			}
+			else
 				BeginIntermission (CreateTargetChangeLevel (mapnames[0]) );
 		}
 	}
@@ -792,9 +829,9 @@ void CheckNeedPass (void)
 ResetLevel
 =================
 */
-void ResetLevel (void) //for resetting players and items after warmup
+void ResetLevel (qboolean keepscores) //for resetting players and items after warmup
 {
-	int i;
+	int i, backup_score = 0; //initialize to shut gcc up
 	edict_t	*ent;
 	gitem_t *item;
 
@@ -803,6 +840,8 @@ void ResetLevel (void) //for resetting players and items after warmup
 		ent = g_edicts + i + 1;
 		if (!ent->inuse || ent->client->resp.spectator)
 			continue;
+		if (keepscores)
+			backup_score = ent->client->resp.score;
 		// locate ent at a spawn point and reset everything
 		InitClientResp (ent->client);
 		if(ent->is_bot)
@@ -816,6 +855,8 @@ void ResetLevel (void) //for resetting players and items after warmup
 			PutClientInServer (ent);
 		}
 		ent->client->homing_shots = 0;
+		if (keepscores)
+			ent->client->resp.score = backup_score;
 	}
 
 	ACESP_SaveBots(); // update bots.tmp and client bot information
@@ -823,6 +864,21 @@ void ResetLevel (void) //for resetting players and items after warmup
 	blue_team_score = 0;
 	red_team_score = 0;
 
+	if(g_tactical->integer)
+	{
+		tacticalScore.alienAmmoDepot = 
+			tacticalScore.alienComputer = 
+			tacticalScore.alienPowerSource = 
+			tacticalScore.alienBackupGen = 
+			tacticalScore.humanAmmoDepot = 
+			tacticalScore.humanComputer = 
+			tacticalScore.humanPowerSource =
+			tacticalScore.humanBackupGen = 
+			true;
+	}
+
+	mindEraserTime = level.time;
+
 	//reset level items
 	for (i=1, ent=g_edicts+i ; i < globals.num_edicts ; i++,ent++) {
 
@@ -843,15 +899,27 @@ void ResetLevel (void) //for resetting players and items after warmup
 				break;
 			}
 		}
-
+		
+		if (!strcmp(ent->classname, "misc_bluenode") || !strcmp(ent->classname, "misc_rednode"))
+			ent->powered = true;
+		if (!strcmp(ent->classname, "misc_redspidernode"))
+		{
+			gi.unlinkentity (ent);
+			SP_misc_redspidernode (ent);
+		}
+		if (!strcmp(ent->classname, "misc_bluespidernode"))
+		{
+			gi.unlinkentity (ent);
+			SP_misc_bluespidernode (ent);
+		}
 	}
 
-	if(g_callvote->value)
+	if(g_callvote->integer)
 		safe_bprintf(PRINT_HIGH, "Call voting is ^2ENABLED\n");
 	else
 		safe_bprintf(PRINT_HIGH, "Call voting is ^1DISABLED\n");
 
-	if(g_antilag->value)
+	if(g_antilag->integer)
 		safe_bprintf(PRINT_HIGH, "Antilag is ^2ENABLED\n");
 	else
 		safe_bprintf(PRINT_HIGH, "Antilag is ^1DISABLED\n");
@@ -874,7 +942,7 @@ void CheckDMRules (void)
 	
 	gi.cvar_set ("g_teamgame", va("%d", TEAM_GAME));
 
-	if ( !tca->value && !ctf->value && !cp->value && !(dmflags->integer & DF_SKINTEAMS) )
+	if ( !g_tactical->integer && !tca->integer && !ctf->integer && !cp->integer && !(dmflags->integer & DF_SKINTEAMS) )
 	{
 		/*--- non-team game warmup ---*/
 		/*
@@ -895,21 +963,20 @@ void CheckDMRules (void)
 				--warmup_state;
 				switch ( warmup_state )
 				{
-				case 3:
-					gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/three.wav" ), 1, ATTN_NONE, 0 );
-					break;
-				case 2:
-					gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/two.wav" ), 1, ATTN_NONE, 0 );
-					break;
-				case 1:
-					gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/one.wav" ), 1, ATTN_NONE, 0 );
-					break;
-				case 0:
-					gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/fight.wav" ), 1, ATTN_NONE, 0 );
-					break;
-				default:
-					break;
-
+					case 3:
+						gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/three.wav" ), 1, ATTN_NONE, 0 );
+						break;
+					case 2:
+						gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/two.wav" ), 1, ATTN_NONE, 0 );
+						break;
+					case 1:
+						gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/one.wav" ), 1, ATTN_NONE, 0 );
+						break;
+					case 0:
+						gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "misc/fight.wav" ), 1, ATTN_NONE, 0 );
+						break;
+					default:
+						break;
 				}
 				if ( warmup_state > 0 )
 				{
@@ -924,7 +991,7 @@ void CheckDMRules (void)
 					{
 						safe_centerprintf( &g_edicts[i], "FIGHT!\n" );
 					}
-					ResetLevel();
+					ResetLevel(false);
 				}
 			}
 		}
@@ -933,12 +1000,12 @@ void CheckDMRules (void)
 	if (level.intermissiontime)
 		return;
 
-	if (!deathmatch->value)
+	if (!deathmatch->integer)
 		return;
 
 	if (timelimit->value)
 	{
-		if (level.time >= timelimit->value*60 && ((tca->value || ctf->value  || cp->value || (dmflags->integer & DF_SKINTEAMS)) || level.time > warmuptime->value))
+		if (level.time >= timelimit->value*60.0 && ((tca->integer || ctf->integer  || cp->integer || (dmflags->integer & DF_SKINTEAMS)) || level.time > warmuptime->value))
 		{
 			safe_bprintf (PRINT_HIGH, "Timelimit hit.\n");
 			EndDMLevel ();
@@ -946,19 +1013,19 @@ void CheckDMRules (void)
 		}
 	}
 
-	if (fraglimit->value && ((tca->value || ctf->value || cp->value || (dmflags->integer & DF_SKINTEAMS)) || level.time > warmuptime->value))
+	if (fraglimit->integer && ((tca->integer || ctf->integer || cp->integer || (dmflags->integer & DF_SKINTEAMS)) || level.time > warmuptime->value))
 	{
 		//team scores
-		if ((dmflags->integer & DF_SKINTEAMS) || ctf->value || cp->value) //it's all about the team!
+		if ((dmflags->integer & DF_SKINTEAMS) || ctf->integer || cp->integer) //it's all about the team!
 		{
-			if(blue_team_score >= fraglimit->value)
+			if(blue_team_score >= fraglimit->integer)
 			{
 				safe_bprintf(PRINT_HIGH, "Blue Team wins!\n");
 				bot_won = 0; //we don't care if it's a bot that wins
 				EndDMLevel();
 				return;
 			}
-			if(red_team_score >= fraglimit->value)
+			if(red_team_score >= fraglimit->integer)
 			{
 				safe_bprintf(PRINT_HIGH, "Red Team wins!\n");
 				bot_won = 0; //we don't care if it's a bot that wins
@@ -966,9 +1033,10 @@ void CheckDMRules (void)
 				return;
 			}
 		}
-		else {
+		else 
+		{
 			top_score = 0;
-			for (i=0 ; i<g_maxclients->value ; i++)
+			for (i=0 ; i<g_maxclients->integer ; i++)
 			{
 				cl = game.clients + i;
 				if (!g_edicts[i+1].inuse)
@@ -977,9 +1045,10 @@ void CheckDMRules (void)
 				if(cl->resp.score > top_score)
 					top_score = cl->resp.score; //grab the top score
 
-				if (cl->resp.score >= fraglimit->value)
+				if (cl->resp.score >= fraglimit->integer)
 				{
-					if(cl->is_bot){
+					if(cl->is_bot)
+					{
 						bot_won = 1; //a bot has won the match
 						safe_bprintf (PRINT_HIGH, "Fraglimit hit by bot.\n");
 					}
@@ -993,12 +1062,15 @@ void CheckDMRules (void)
 					return;
 				}
 			}
-			if(!tca->value && !ctf->value && !cp->value) {
-				i = fraglimit->value - top_score;
-				switch(i) {
+			if(!tca->integer && !ctf->integer && !cp->integer) 
+			{
+				i = fraglimit->integer - top_score;
+				switch(i) 
+				{
 					case 3:
-						if(!print3){
-							for (i=0 ; i<g_maxclients->value ; i++)
+						if(!print3)
+						{
+							for (i=0 ; i<g_maxclients->integer ; i++)
 							{
 								cl_ent = g_edicts + 1 + i;
 								if (!cl_ent->inuse || cl_ent->is_bot)
@@ -1011,8 +1083,9 @@ void CheckDMRules (void)
 						}
 						break;
 					case 2:
-						if(!print2) {
-							for (i=0 ; i<g_maxclients->value ; i++)
+						if(!print2) 
+						{
+							for (i=0 ; i<g_maxclients->integer ; i++)
 							{
 								cl_ent = g_edicts + 1 + i;
 								if (!cl_ent->inuse || cl_ent->is_bot)
@@ -1025,8 +1098,9 @@ void CheckDMRules (void)
 						}
 						break;
 					case 1:
-						if(!print1) {
-							for (i=0 ; i<g_maxclients->value ; i++)
+						if(!print1) 
+						{
+							for (i=0 ; i<g_maxclients->integer ; i++)
 							{
 								cl_ent = g_edicts + 1 + i;
 								if (!cl_ent->inuse || cl_ent->is_bot)
@@ -1044,19 +1118,57 @@ void CheckDMRules (void)
 			}
 		}
 	}
-	if(tca->value) {
-		if(blue_team_score == 0) {
+
+	if(tca->integer) 
+	{
+		if(red_team_matches == 2) 
+		{
 			safe_bprintf(PRINT_HIGH, "Red Team wins!\n");
 			bot_won = 0; //we don't care if it's a bot that wins
 			EndDMLevel();
 			return;
 		}
-		if(red_team_score == 0) {
+		if(blue_team_matches == 2) 
+		{
 			safe_bprintf(PRINT_HIGH, "Blue Team wins!\n");
 			bot_won = 0; //we don't care if it's a bot that wins
 			EndDMLevel();
 			return;
 		}
+		if (blue_team_score == 0) 
+		{
+			safe_bprintf(PRINT_HIGH, "Red Team wins match %d out of 3!\n", red_team_matches);
+			bot_won = 0;
+			ResetLevel(true);
+			blue_team_score = red_team_score = 4;
+			return;
+		}
+		if (red_team_score == 0) 
+		{
+			safe_bprintf(PRINT_HIGH, "Blue Team wins match %d out of 3!\n", blue_team_matches);
+			bot_won = 0;
+			ResetLevel(true);
+			blue_team_score = red_team_score = 4;
+			return;
+		}
+	}
+
+	if(g_tactical->integer)
+	{
+		if(!tacticalScore.alienAmmoDepot && !tacticalScore.alienComputer && !tacticalScore.alienPowerSource)
+		{
+			safe_bprintf(PRINT_HIGH, "The Humans have defeated the Aliens!\n");
+			bot_won = 0; //we don't care if it's a bot that wins
+			EndDMLevel();
+			return;
+		}
+		if(!tacticalScore.humanAmmoDepot && !tacticalScore.humanComputer && !tacticalScore.humanPowerSource)
+		{
+			safe_bprintf(PRINT_HIGH, "The Aliens have defeated the Humans!\n");
+			bot_won = 0; //we don't care if it's a bot that wins
+			EndDMLevel();
+			return;
+		}
 	}
 }
 
@@ -1071,7 +1183,8 @@ void ExitLevel (void)
 	edict_t		*ent;
 	char		command [256];
 
-	if(strcmp(level.mapname, level.changemap) || timelimit->value) {
+	if(strcmp(level.mapname, level.changemap) || timelimit->value || g_tactical->value) 
+	{
 		Com_sprintf( command, sizeof(command), "map \"%s\"\n", level.changemap );
 		gi.AddCommandString( command );
 	}
@@ -1091,7 +1204,7 @@ void ExitLevel (void)
 	EndIntermission();
 
 	// clear some things before going to next level
-	for (i=0 ; i<g_maxclients->value ; i++)
+	for (i=0 ; i<g_maxclients->integer ; i++)
 	{
 		ent = g_edicts + 1 + i;
 		if (!ent->inuse)
@@ -1106,37 +1219,67 @@ void ExitLevel (void)
 		ent->takedamage = DAMAGE_AIM;
 		ent->solid = SOLID_BBOX;
 		ent->deadflag = DEAD_NO;
-		if(ent->is_bot) {
+		if(ent->is_bot) 
+		{
 			ACESP_PutClientInServer( ent, true );
-		} else {
+		} 
+		else 
+		{
 			PutClientInServer (ent);
 		}
-		if(g_duel->value) {
+		if(g_duel->integer) 
+		{
 			ClientPlaceInQueue(ent);
 			ClientCheckQueue(ent);
 		}
 	}
-	for (i=1, ent=g_edicts+i ; i < globals.num_edicts ; i++,ent++) {
+	for (i=1, ent=g_edicts+i ; i < globals.num_edicts ; i++,ent++) 
+	{
 
 		if (!ent->inuse || ent->client)
 			continue;
 		//remove podiums
 		if(!strcmp(ent->classname, "pad"))
 			G_FreeEdict(ent);
-		if(tca->value) {
+		if(tca->integer) 
+		{
 		    if(strstr(ent->classname, "spidernode"))
 		        ED_CallSpawn (ent);
 		    ent->powered = true;
 		}
 	}
-	if(tca->value) {
-		blue_team_score = 4;
-		red_team_score = 4;
+	if(tca->integer) 
+	{
+		blue_team_score = red_team_score = 4;
+		red_team_matches = blue_team_matches = 0;
 	}
-	else {
+	else 
+	{
 		red_team_score = 0;
 		blue_team_score = 0;
 	}
+
+	if(g_tactical->integer)
+	{
+		tacticalScore.alienAmmoDepot = 
+			tacticalScore.alienComputer = 
+			tacticalScore.alienPowerSource = 
+			tacticalScore.alienBackupGen = 
+			tacticalScore.humanAmmoDepot = 
+			tacticalScore.humanComputer = 
+			tacticalScore.humanPowerSource =
+			tacticalScore.humanBackupGen = 
+			true;
+
+		tacticalScore.alienAmmoDepotHealth = 
+			tacticalScore.alienComputerHealth = 
+			tacticalScore.alienPowerSourceHealth = 
+			tacticalScore.humanAmmoDepotHealth = 
+			tacticalScore.humanComputerHealth = 
+			tacticalScore.humanPowerSourceHealth =
+			100;
+	}
+
 	print1 = print2 = print3 = false;
 
 }
@@ -1453,14 +1596,14 @@ void G_RunFrame (void)
 			}
 		}
 
-		if (i > 0 && i <= g_maxclients->value)
+		if (i > 0 && i <= g_maxclients->integer)
 		{
 			ClientBeginServerFrame (ent);
 		}
 
 		if(ent->inuse && ent->client && !ent->is_bot)
 		{
-			if ( ent->s.number <= g_maxclients->value )
+			if ( ent->s.number <= g_maxclients->integer )
 			{ // count actual players, not deathcam entities
 				numActiveClients++;
 			}
@@ -1476,10 +1619,10 @@ void G_RunFrame (void)
 
 			//unlagged
 			if ( g_antilag->integer)
-				G_UnTimeShiftAllClients( NULL );
+		 		G_UnTimeShiftAllClients( NULL );
 		}
 		else*/
-			G_RunEntity (ent);
+		G_RunEntity (ent, FRAMETIME);
 	}
 
 	// see if it is time to end a deathmatch
@@ -1499,7 +1642,7 @@ void G_RunFrame (void)
 		level.frameStartTime = level.time;
 
 	//call voting
-	if(g_callvote->value && playervote.called) {
+	if(g_callvote->integer && playervote.called) {
 
 		playervote.time = level.time;
 		if(playervote.time-playervote.starttime > 15 ){ //15 seconds
@@ -1527,7 +1670,7 @@ void G_RunFrame (void)
 			playervote.command[0] = 0;
 
 			//do each ent
-			for (i=0 ; i<g_maxclients->value ; i++)
+			for (i=0 ; i<g_maxclients->integer ; i++)
 			{
 				ent = g_edicts + 1 + i;
 				if (!ent->inuse || ent->is_bot)
