@@ -982,10 +982,13 @@ void SP_misc_teleporter (edict_t *ent)
 		return;
 	}
 
+	// The actual misc_teleporter is converted into a cosmetic object for the
+	// glowing effect, while a new entity is spawned to act as the functioning
+	// teleporter. 
 	gi.setmodel (ent, "models/objects/blank/tris.md2");
 	ent->s.skinnum = 1;
 	ent->s.effects = EF_TELEPORTER;
-	ent->solid = SOLID_BBOX;
+	ent->solid     = SOLID_BBOX;
 
 	VectorSet (ent->mins, -32, -32, -24);
 	VectorSet (ent->maxs, 32, 32, -16);
@@ -1016,6 +1019,104 @@ void SP_misc_teleporter_dest (edict_t *ent)
 	gi.linkentity (ent);
 }
 
+void misc_mapmodel_think (edict_t *ent)
+{
+	if(ent->spawnflags & 2)
+		ent->s.frame = (ent->s.frame + 1) % 39;
+	else
+		ent->s.frame = (ent->s.frame + 1) % 24;
+	ent->nextthink = level.time + FRAMETIME;
+}
+void SP_misc_mapmodel (edict_t *ent) //random .md2 map models
+{
+	gi.setmodel (ent, ent->model);
+
+	ent->solid = SOLID_NOT; //will need clipping brushes around it
+
+	//disable shadows(also disables per-pixel dynamic lighting unless minlight set
+	if(ent->spawnflags & 1)
+		ent->s.renderfx = RF_NOSHADOWS;
+
+	if(ent->spawnflags & 16)
+		ent->s.renderfx = RF_TRANSLUCENT;
+
+	if(ent->spawnflags & 32) { //animated mesh
+		if(ent->spawnflags & 128)
+			ent->s.frame = 0;
+		else
+			ent->s.frame = rand()%24;
+		ent->think = misc_mapmodel_think;
+		ent->nextthink = level.time + FRAMETIME;
+	}
+	else //static mesh
+		ent->s.frame = 0;
+
+	//allow dynamic per-pixel lighting
+	if(ent->spawnflags & 64)
+		ent->s.renderfx |= RF_MINLIGHT;
+
+	gi.linkentity (ent);
+}
+
+void watersplash_think (edict_t *ent)
+{
+	vec3_t up;
+
+	up[0] = 0;
+	up[1] = 0;
+	up[2] = 1;
+
+	//write effect
+	gi.WriteByte (svc_temp_entity);
+	gi.WriteByte (TE_SPLASH);
+	gi.WriteByte (8);
+	gi.WritePosition (ent->s.origin);
+	gi.WriteDir (up);
+	gi.WriteByte (SPLASH_BLUE_WATER); //we should allow spawnflags to change this
+	gi.multicast (ent->s.origin, MULTICAST_PVS);
+
+	ent->nextthink = level.time + 1.0;
+}
+
+void SP_misc_watersplash (edict_t *ent)
+{
+	gi.setmodel(ent, NULL);
+	ent->solid = SOLID_NOT;
+	ent->movetype = MOVETYPE_NONE;
+	ent->takedamage = DAMAGE_NO;
+
+	ent->think = watersplash_think;
+	ent->nextthink = level.time + 0.5 + random();
+
+	gi.linkentity (ent);
+}
+
+void misc_electroflash_think (edict_t *ent)
+{
+	gi.WriteByte (svc_muzzleflash);
+	gi.WriteShort (ent-g_edicts);
+	gi.WriteByte (MZ_RAILGUN);
+	gi.multicast (ent->s.origin, MULTICAST_PVS);
+
+	ent->nextthink = level.time + 0.05 + random();
+}
+
+void SP_misc_electroflash (edict_t *ent) //random electrical pulses
+{
+	gi.setmodel (ent, NULL);
+
+	ent->solid = SOLID_NOT; 
+	ent->movetype = MOVETYPE_NONE;
+	ent->takedamage = DAMAGE_NO;
+	ent->s.sound = gi.soundindex("world/electricity.wav");
+	ent->think = misc_electroflash_think;
+	ent->nextthink = level.time + 0.05 + random();
+
+	gi.linkentity (ent);
+}
+
+//Team Core Assault
+
 void spidervolts (edict_t *self)
 {
 
@@ -1147,6 +1248,12 @@ void rednode_think (edict_t *ent)
 }
 void SP_misc_rednode (edict_t *ent)
 {
+	if (!tca->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
 	ent->movetype = MOVETYPE_NONE;
 	ent->solid = SOLID_BBOX;
 	ent->takedamage = DAMAGE_NO;
@@ -1222,6 +1329,12 @@ void bluenode_think (edict_t *ent)
 }
 void SP_misc_bluenode (edict_t *ent)
 {
+	if (!tca->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
 	ent->movetype = MOVETYPE_NONE;
 	ent->solid = SOLID_BBOX;
 	ent->takedamage = DAMAGE_NO;
@@ -1253,8 +1366,10 @@ void redspidernode_think (edict_t *ent)
 	ent->s.frame = (ent->s.frame + 1) % 13;
 	ent->nextthink = level.time + FRAMETIME;
 }
-void red_roundend(edict_t *ent) {
+void red_roundend(edict_t *ent) 
+{
 	red_team_score = 0;
+	blue_team_matches++;
 }
 void redspidernode_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
 {
@@ -1275,13 +1390,20 @@ void redspidernode_die (edict_t *self, edict_t *inflictor, edict_t *attacker, in
 }
 void SP_misc_redspidernode (edict_t *ent)
 {
+	if (!tca->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
 	ent->movetype = MOVETYPE_NONE;
 	ent->solid = SOLID_BBOX;
 	ent->takedamage = DAMAGE_NO;
 
 	ent->s.modelindex = gi.modelindex ("models/misc/spiderpod/tris.md2");
+	ent->s.modelindex3 = gi.modelindex ("models/misc/spiderpod/helmet.md2");
 
-	ent->s.renderfx = (RF_FULLBRIGHT | RF_GLOW | RF_NOSHADOWS);
+	ent->s.renderfx = (RF_FULLBRIGHT | RF_GLOW);
 	VectorSet (ent->mins, -64, -64, 0);
 	VectorSet (ent->maxs, 64, 64, 128);
 	ent->health = 600;
@@ -1303,8 +1425,10 @@ void bluespidernode_think (edict_t *ent)
 	ent->s.frame = (ent->s.frame + 1) % 13;
 	ent->nextthink = level.time + FRAMETIME;
 }
-void blue_roundend(edict_t *self) {
+void blue_roundend(edict_t *self) 
+{
 	blue_team_score = 0;
+	red_team_matches++;
 }
 void bluespidernode_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
 {
@@ -1325,13 +1449,20 @@ void bluespidernode_die (edict_t *self, edict_t *inflictor, edict_t *attacker, i
 }
 void SP_misc_bluespidernode (edict_t *ent)
 {
+	if (!tca->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
 	ent->movetype = MOVETYPE_NONE;
 	ent->solid = SOLID_BBOX;
 	ent->takedamage = DAMAGE_NO;
 
 	ent->s.modelindex = gi.modelindex ("models/misc/spiderpod/tris.md2");
+	ent->s.modelindex3 = gi.modelindex ("models/misc/spiderpod/helmet.md2");
 
-	ent->s.renderfx = (RF_FULLBRIGHT | RF_GLOW | RF_NOSHADOWS);
+	ent->s.renderfx = (RF_FULLBRIGHT | RF_GLOW);
 	VectorSet (ent->mins, -64, -64, 0);
 	VectorSet (ent->maxs, 64, 64, 128);
 	ent->health = 600;
@@ -1343,98 +1474,627 @@ void SP_misc_bluespidernode (edict_t *ent)
 	M_droptofloor (ent);
 }
 
-void misc_mapmodel_think (edict_t *ent)
+//Tactical base items
+//Rules:  When a computer is destroyed, that base's turrents/deathrays will behave eratically, even firing on it's own team on occasion.  Laser barriers shut off.  
+//When a powersource is destroyed, the backup generators(if not already destroyed) for the computer and ammo generator turn on(generator models will animate, emit sound).  Ammo will generate from depot at half speed.  
+//Laser barriers shut off, turrets and deathrays are weak.
+//When an ammo depot is destroyed, ammo stops being produced.
+//When all three are disabled, the other team wins.
+
+//lasers
+void misc_laser_think (edict_t *self)
 {
-	if(ent->spawnflags & 2)
-		ent->s.frame = (ent->s.frame + 1) % 39;
+	edict_t *ent;
+	edict_t	*ignore;
+	vec3_t	start;
+	vec3_t	end;
+	trace_t	tr;
+	vec3_t	point;
+	vec3_t	last_movedir;
+	int		count;
+	
+	if(self->spawnflags & 1)
+	{
+		if(!tacticalScore.humanComputer || !tacticalScore.humanPowerSource)
+		{
+			self->nextthink = 0;
+			return;
+		}
+	}
+	else if(!tacticalScore.alienComputer || !tacticalScore.alienPowerSource)
+	{
+		self->nextthink = 0;
+		return;
+	}
+
+	count = 8;
+
+	if (!self->enemy)
+	{
+		if (self->target)
+		{
+			ent = G_Find (NULL, FOFS(targetname), self->target);
+			if (!ent)
+				gi.dprintf ("%s at %s: %s is a bad target\n", self->classname, vtos(self->s.origin), self->target);
+			self->enemy = ent;
+		}
+		else
+		{
+			G_SetMovedir (self->s.angles, self->movedir);
+		}
+	}
+	else 
+	{
+		VectorCopy (self->movedir, last_movedir);
+		VectorMA (self->enemy->absmin, 0.5, self->enemy->size, point);
+		VectorSubtract (point, self->s.origin, self->movedir);
+		VectorNormalize (self->movedir);
+	}
+
+	ignore = self;
+	VectorCopy (self->s.origin, start);
+	VectorCopy (self->enemy->s.origin, end);
+	while(1)
+	{
+		tr = gi.trace (start, NULL, NULL, end, ignore, CONTENTS_SOLID|CONTENTS_MONSTER|CONTENTS_DEADMONSTER);
+		gi.WriteByte (svc_temp_entity);
+		if (self->spawnflags & 1)
+			gi.WriteByte (TE_REDLASER);
+		else
+			gi.WriteByte (TE_LASERBEAM);
+		gi.WritePosition (start);
+		gi.WritePosition (end);
+		gi.multicast (start, MULTICAST_PHS);
+
+		if (!tr.ent)
+			break;
+
+		//don't hurt anyone on the same team as the laser - spawnflag 1 = human
+		if(self->spawnflags & 1) 
+		{
+			if(tr.ent->ctype == 1)
+				break;
+		}
+		else if(!tr.ent->ctype)
+			break;	
+
+		// hurt it if we can
+		if (tr.ent->takedamage)
+			T_Damage (tr.ent, self, self->activator, self->movedir, tr.endpos, vec3_origin, self->dmg, 1, DAMAGE_ENERGY, MOD_TARGET_LASER);
+
+		// if we hit something that's not a player, we're done
+		if (!tr.ent->client)
+		{
+			break;
+		}
+
+		ignore = tr.ent;
+		VectorCopy (tr.endpos, start);
+	}
+
+	VectorCopy (tr.endpos, self->s.old_origin);
+
+	self->nextthink = level.time + FRAMETIME;
+}
+
+void misc_laser_start (edict_t *self)
+{
+	
+	self->think = misc_laser_think;		
+}
+
+void laser_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
+{
+	self->takedamage = DAMAGE_NO;
+	self->activator = attacker;
+
+	gi.WriteByte (svc_temp_entity);
+	if(self->spawnflags & 1)
+	{
+		gi.WriteByte (TE_ROCKET_EXPLOSION); 		
+	}
 	else
-		ent->s.frame = (ent->s.frame + 1) % 24;
+	{
+		gi.WriteByte (TE_BFG_BIGEXPLOSION);		
+	}
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
+	
+	gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "world/explosion1.wav" ), 1, ATTN_NONE, 0 );
+
+	G_FreeEdict (self);
+}
+
+void SP_misc_laser (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES; 
+	ent->health = 1000;  
+	ent->die = laser_die;
+	
+	ent->dmg = 500; //cause severe damage, especially if multiples(most cases)
+
+	if(ent->spawnflags & 1)
+		ent->s.modelindex = gi.modelindex ("models/tactical/human_laser.iqm");
+	else
+		ent->s.modelindex = gi.modelindex ("models/tactical/alien_laser.iqm");
+
+	VectorSet (ent->mins, -16, -16, -16);
+	VectorSet (ent->maxs, 16, 16, 16);
+
+	// let everything else get spawned before we start firing
+	ent->think = misc_laser_think;
+	ent->nextthink = level.time + 1;
+
+	gi.linkentity (ent);
+}
+
+
+//computers
+void computer_think (edict_t *ent)
+{
+	if(ent->classname == "alien computer")
+		tacticalScore.alienComputerHealth = ent->health/15;
+	else
+		tacticalScore.humanComputerHealth = ent->health/15;
+
+	ent->s.frame = (ent->s.frame + 1) % 24;
 	ent->nextthink = level.time + FRAMETIME;
 }
-void SP_misc_mapmodel (edict_t *ent) //random .md2 map models
+
+void computer_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
 {
-	gi.setmodel (ent, ent->model);
+	edict_t *cl_ent;
+	int i;
 
-	ent->solid = SOLID_NOT; //will need clipping brushes around it
+	self->takedamage = DAMAGE_NO;
+	self->activator = attacker;
 
-	//disable shadows(also disables per-pixel dynamic lighting unless minlight set
-	if(ent->spawnflags & 1)
-		ent->s.renderfx = RF_NOSHADOWS;
-
-	if(ent->spawnflags & 16)
-		ent->s.renderfx = RF_TRANSLUCENT;
-
-	if(ent->spawnflags & 32) { //animated mesh
-		if(ent->spawnflags & 128)
-			ent->s.frame = 0;
-		else
-			ent->s.frame = rand()%24;
-		ent->think = misc_mapmodel_think;
-		ent->nextthink = level.time + FRAMETIME;
-	}
-	else //static mesh
-		ent->s.frame = 0;
-
-	//allow dynamic per-pixel lighting
-	if(ent->spawnflags & 64)
-		ent->s.renderfx |= RF_MINLIGHT;
-
-	gi.linkentity (ent);
-}
-
-void watersplash_think (edict_t *ent)
-{
-	vec3_t up;
-
-	up[0] = 0;
-	up[1] = 0;
-	up[2] = 1;
-
-	//write effect
 	gi.WriteByte (svc_temp_entity);
-	gi.WriteByte (TE_SPLASH);
-	gi.WriteByte (8);
-	gi.WritePosition (ent->s.origin);
-	gi.WriteDir (up);
-	gi.WriteByte (SPLASH_BLUE_WATER); //we should allow spawnflags to change this
-	gi.multicast (ent->s.origin, MULTICAST_PVS);
+	if(self->classname == "alien computer")
+	{
+		tacticalScore.alienComputer = false;
+		tacticalScore.alienComputerHealth = 0;
+		gi.WriteByte (TE_BFG_BIGEXPLOSION); 
+	}
+	else
+	{
+		tacticalScore.humanComputer = false;
+		tacticalScore.humanComputerHealth = 0;
+		gi.WriteByte (TE_ROCKET_EXPLOSION);
+	}
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
 
-	ent->nextthink = level.time + 1.0;
+	if(self->classname == "alien computer")
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Alien Central Computer has been destroyed!");
+		}
+	}
+	else
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Human Central Computer has been destroyed!");
+		}
+	}
+	
+	gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "world/explosion1.wav" ), 1, ATTN_NONE, 0 );
+
+	G_FreeEdict (self);
 }
 
-void SP_misc_watersplash (edict_t *ent)
+void SP_misc_aliencomputer (edict_t *ent)
 {
-	gi.setmodel(ent, NULL);
-	ent->solid = SOLID_NOT;
-	ent->movetype = MOVETYPE_NONE;
-	ent->takedamage = DAMAGE_NO;
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
 
-	ent->think = watersplash_think;
-	ent->nextthink = level.time + 0.5 + random();
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/alien_computer.iqm");
+
+	VectorSet (ent->mins, -64, -64, 0);
+	VectorSet (ent->maxs, 64, 64, 64);
+	ent->health = 1500; 
+	ent->die = computer_die;
+	ent->think = computer_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "alien computer";
 
 	gi.linkentity (ent);
+	M_droptofloor (ent);
 }
 
-void misc_electroflash_think (edict_t *ent)
+void SP_misc_humancomputer (edict_t *ent)
 {
-	gi.WriteByte (svc_muzzleflash);
-	gi.WriteShort (ent-g_edicts);
-	gi.WriteByte (MZ_RAILGUN);
-	gi.multicast (ent->s.origin, MULTICAST_PVS);
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
 
-	ent->nextthink = level.time + 0.05 + random();
-}
-
-void SP_misc_electroflash (edict_t *ent) //random electrical pulses
-{
-	gi.setmodel (ent, NULL);
-
-	ent->solid = SOLID_NOT; 
 	ent->movetype = MOVETYPE_NONE;
-	ent->takedamage = DAMAGE_NO;
-	ent->s.sound = gi.soundindex("world/electricity.wav");
-	ent->think = misc_electroflash_think;
-	ent->nextthink = level.time + 0.05 + random();
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/human_computer.iqm");
+
+	VectorSet (ent->mins, -64, -64, 0);
+	VectorSet (ent->maxs, 64, 64, 64);
+	ent->health = 1500; 
+	ent->die = computer_die;
+	ent->think = computer_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "human computer";
 
 	gi.linkentity (ent);
-}
\ No newline at end of file
+	M_droptofloor (ent);
+}
+
+//power sources
+void powersrc_think (edict_t *ent)
+{
+	if(ent->classname == "alien powersrc")
+		tacticalScore.alienPowerSourceHealth = ent->health/15;
+	else
+		tacticalScore.humanPowerSourceHealth = ent->health/15;
+	ent->s.frame = (ent->s.frame + 1) % 24;
+	ent->nextthink = level.time + FRAMETIME;
+}
+
+void powersrc_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
+{
+	edict_t *cl_ent;
+	int i;
+
+	self->takedamage = DAMAGE_NO;
+	self->activator = attacker;
+
+	gi.WriteByte (svc_temp_entity);
+	if(self->classname == "alien powersrc")
+	{
+		tacticalScore.alienPowerSource = false;
+		tacticalScore.alienPowerSourceHealth = 0;
+		gi.WriteByte (TE_BFG_BIGEXPLOSION); 
+	}
+	else
+	{
+		tacticalScore.humanPowerSource = false;
+		tacticalScore.humanPowerSourceHealth = 0;
+		gi.WriteByte (TE_ROCKET_EXPLOSION);		
+	}
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
+
+	if(self->classname == "alien powersrc")
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Alien Power Source has been destroyed!");
+		}
+	}
+	else
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Human Power Source has been destroyed!");
+		}
+	}
+	
+	gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "world/explosion1.wav" ), 1, ATTN_NONE, 0 );
+
+	G_FreeEdict (self);
+}
+
+void SP_misc_alienpowersrc (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/alien_powersrc.iqm");
+
+	VectorSet (ent->mins, -64, -64, 0);
+	VectorSet (ent->maxs, 64, 64, 72);
+	ent->health = 1500; 
+	ent->die = powersrc_die;
+	ent->think = powersrc_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "alien powersrc";
+
+	gi.linkentity (ent);
+	M_droptofloor (ent);
+}
+
+void SP_misc_humanpowersrc (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/human_powersrc.iqm");
+
+	VectorSet (ent->mins, -32, -32, 0);
+	VectorSet (ent->maxs, 32, 32, 72);
+	ent->health = 1500; 
+	ent->die = powersrc_die;
+	ent->think = powersrc_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "human powersrc";
+
+	gi.linkentity (ent);
+	M_droptofloor (ent);
+}
+
+//ammo depots
+void ammodepot_think (edict_t *ent)
+{
+	if(ent->classname == "alien ammodepot")
+		tacticalScore.alienAmmoDepotHealth = ent->health/15;
+	else
+		tacticalScore.humanAmmoDepotHealth = ent->health/15;
+	ent->nextthink = level.time + FRAMETIME;
+}
+
+void ammodepot_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
+{
+	edict_t	*cl_ent;
+	int i;
+
+	self->takedamage = DAMAGE_NO;
+	self->activator = attacker;
+
+	gi.WriteByte (svc_temp_entity);
+	if(self->classname == "alien ammodepot")
+	{
+		tacticalScore.alienAmmoDepot = false;
+		tacticalScore.alienAmmoDepotHealth = 0;
+		gi.WriteByte (TE_BFG_BIGEXPLOSION); 
+	}
+	else
+	{
+		tacticalScore.humanAmmoDepot = false;
+		tacticalScore.humanAmmoDepotHealth = 0;
+		gi.WriteByte (TE_ROCKET_EXPLOSION);
+	}
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
+
+	if(self->classname == "alien ammodepot")
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Alien Ammo Depot has been destroyed!");
+		}
+	}
+	else
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Human Ammo Depot has been destroyed!");
+		}
+	}
+	
+	gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "world/explosion1.wav" ), 1, ATTN_NONE, 0 );
+
+	G_FreeEdict (self);
+}
+
+void SP_misc_alienammodepot (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/ammopad.md2");
+
+	VectorSet (ent->mins, -32, -32, 0);
+	VectorSet (ent->maxs, 32, 32, 16);
+	ent->health = 1500; 
+	ent->die = ammodepot_die;
+	ent->think = ammodepot_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "alien ammodepot";
+
+	gi.linkentity (ent);
+	M_droptofloor (ent);
+}
+
+void SP_misc_humanammodepot (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("maps/meshes/flagpad.md2");
+
+	VectorSet (ent->mins, -32, -32, 0);
+	VectorSet (ent->maxs, 32, 32, 16);
+	ent->health = 1500; 
+	ent->die = ammodepot_die;
+	ent->think = ammodepot_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "human ammodepot";
+
+	gi.linkentity (ent);
+	M_droptofloor (ent);
+}
+
+//Backup generators
+void backupgen_think (edict_t *ent)
+{
+	if(ent->classname == "alien backupgen")
+	{
+		if(!tacticalScore.alienPowerSource)
+		{
+			//animate
+			ent->s.frame = (ent->s.frame + 1) % 24;
+			gi.sound (ent, CHAN_AUTO, gi.soundindex("misc/reject.wav"), 1, ATTN_STATIC, 0);
+		}
+	}
+	else
+	{
+		if(!tacticalScore.humanPowerSource)
+		{
+			ent->s.frame = (ent->s.frame + 1) % 24;
+			gi.sound (ent, CHAN_AUTO, gi.soundindex("misc/reject.wav"), 1, ATTN_STATIC, 0);
+		}
+	}	
+	ent->nextthink = level.time + FRAMETIME;
+}
+
+void backupgen_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
+{
+	edict_t *cl_ent;
+	int i;
+
+	self->takedamage = DAMAGE_NO;
+	self->activator = attacker;
+
+	gi.WriteByte (svc_temp_entity);
+	if(self->classname == "alien backupgen")
+	{
+		tacticalScore.alienBackupGen = false;
+		gi.WriteByte (TE_BFG_BIGEXPLOSION); 
+	}
+	else
+	{
+		tacticalScore.humanBackupGen = false;
+		gi.WriteByte (TE_ROCKET_EXPLOSION);		
+	}
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
+
+	if(self->classname == "alien backupgen")
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Alien Backup Generator has been destroyed!");
+		}
+	}
+	else
+	{
+		for (i=0 ; i<g_maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			safe_centerprintf(cl_ent, "Human Backup Generator has been destroyed!");
+		}
+	}
+	
+	gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "world/explosion1.wav" ), 1, ATTN_NONE, 0 );
+
+	G_FreeEdict (self);
+}
+
+void SP_misc_alienbackupgen (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/alien_backupgen.iqm");
+
+	VectorSet (ent->mins, -24, -24, 0);
+	VectorSet (ent->maxs, 24, 24, 48);
+	ent->health = 300; 
+	ent->die = backupgen_die;
+	ent->think = backupgen_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "alien backupgen";
+
+	gi.linkentity (ent);
+	M_droptofloor (ent);
+}
+
+void SP_misc_humanbackupgen (edict_t *ent)
+{
+	if (!g_tactical->integer)
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	ent->movetype = MOVETYPE_NONE;
+	ent->solid = SOLID_BBOX;
+	ent->takedamage = DAMAGE_YES;
+
+	ent->s.modelindex = gi.modelindex ("models/tactical/human_backupgen.iqm");
+
+	VectorSet (ent->mins, -24, -24, 0);
+	VectorSet (ent->maxs, 24, 24, 48);
+	ent->health = 300; 
+	ent->die = backupgen_die;
+	ent->think = backupgen_think;
+	ent->nextthink = level.time + FRAMETIME;
+	ent->classname = "human backupgen";
+
+	gi.linkentity (ent);
+	M_droptofloor (ent);
+}
