@@ -8,7 +8,7 @@ of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See the GNU General Public License for more details.
 
@@ -19,6 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // g_phys.c
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "g_local.h"
 
 /*
@@ -26,7 +30,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 pushmove objects do not obey gravity, and do not interact with each other or trigger fields, but block normal movement and push normal objects when they move.
 
-onground is set for toss objects when they come to a complete rest.  it is set for steping or walking objects 
+onground is set for toss objects when they come to a complete rest.  it is set for steping or walking objects
 
 doors, plats, etc are SOLID_BSP, and MOVETYPE_PUSH
 bonus items are SOLID_TRIGGER touch, and MOVETYPE_TOSS
@@ -56,10 +60,10 @@ edict_t	*SV_TestEntityPosition (edict_t *ent)
 	else
 		mask = MASK_SOLID;
 	trace = gi.trace (ent->s.origin, ent->mins, ent->maxs, ent->s.origin, ent, mask);
-	
+
 	if (trace.startsolid)
 		return g_edicts;
-		
+
 	return NULL;
 }
 
@@ -78,7 +82,7 @@ void SV_CheckVelocity (edict_t *ent)
 //
 	for (i=0 ; i<3 ; i++)
 	{
-		if (ent->velocity[i] > sv_maxvelocity->value)
+		if (ent->velocity[i] > sv_maxvelocity->value)		
 			ent->velocity[i] = sv_maxvelocity->value;
 		else if (ent->velocity[i] < -sv_maxvelocity->value)
 			ent->velocity[i] = -sv_maxvelocity->value;
@@ -98,7 +102,7 @@ qboolean SV_RunThink (edict_t *ent)
 
 	if(!strcmp(ent->classname, "func_train")) {
 		//if animated mesh, animate at this target
-		if (ent->spawnflags & 32) 
+		if (ent->spawnflags & 32)
 				ent->s.frame = (ent->s.frame + 1)%24;
 	}
 
@@ -108,7 +112,7 @@ qboolean SV_RunThink (edict_t *ent)
 		return true;
 	if (thinktime > level.time+0.001)
 		return true;
-	
+
 	ent->nextthink = 0;
 	if (!ent->think)
 		gi.error ("NULL ent->think");
@@ -133,7 +137,7 @@ void SV_Impact (edict_t *e1, trace_t *trace)
 
 	if (e1->touch && e1->solid != SOLID_NOT)
 		e1->touch (e1, e2, &trace->plane, trace->surface);
-	
+
 	if (e2->touch && e2->solid != SOLID_NOT)
 		e2->touch (e2, e1, NULL, NULL);
 }
@@ -154,13 +158,13 @@ int ClipVelocity (vec3_t in, vec3_t normal, vec3_t out, float overbounce)
 	float	backoff;
 	float	change;
 	int		i, blocked;
-	
+
 	blocked = 0;
 	if (normal[2] > 0)
 		blocked |= 1;		// floor
 	if (!normal[2])
 		blocked |= 2;		// step
-	
+
 	backoff = DotProduct (in, normal) * overbounce;
 
 	for (i=0 ; i<3 ; i++)
@@ -187,7 +191,7 @@ Returns the clipflags if the velocity was modified (hit something solid)
 ============
 */
 #define	MAX_CLIP_PLANES	5
-int SV_FlyMove (edict_t *ent, float time, int mask)
+int SV_FlyMove (edict_t *ent, float timespan, int mask)
 {
 	edict_t		*hit;
 	int			bumpcount, numbumps;
@@ -201,15 +205,15 @@ int SV_FlyMove (edict_t *ent, float time, int mask)
 	vec3_t		end;
 	float		time_left;
 	int			blocked;
-	
+
 	numbumps = 4;
-	
+
 	blocked = 0;
 	VectorCopy (ent->velocity, original_velocity);
 	VectorCopy (ent->velocity, primal_velocity);
 	numplanes = 0;
-	
-	time_left = time;
+
+	time_left = timespan;
 
 	ent->groundentity = NULL;
 	for (bumpcount=0 ; bumpcount<numbumps ; bumpcount++)
@@ -258,9 +262,9 @@ int SV_FlyMove (edict_t *ent, float time, int mask)
 		if (!ent->inuse)
 			break;		// removed by the impact function
 
-		
+
 		time_left -= time_left * trace.fraction;
-		
+
 	// cliped to another plane
 		if (numplanes >= MAX_CLIP_PLANES)
 		{	// this shouldn't really happen
@@ -287,7 +291,7 @@ int SV_FlyMove (edict_t *ent, float time, int mask)
 			if (j == numplanes)
 				break;
 		}
-		
+
 		if (i != numplanes)
 		{	// go along this plane
 			VectorCopy (new_velocity, ent->velocity);
@@ -326,9 +330,9 @@ SV_AddGravity
 
 ============
 */
-void SV_AddGravity (edict_t *ent)
+void SV_AddGravity (edict_t *ent, float timespan)
 {
-	ent->velocity[2] -= ent->gravity * sv_gravity->value * FRAMETIME;
+	ent->velocity[2] -= ent->gravity * sv_gravity->value * timespan;
 }
 
 /*
@@ -391,7 +395,7 @@ retry:
 		G_TouchTriggers (ent);
 
 	return trace;
-}					
+}
 
 
 typedef struct
@@ -498,7 +502,7 @@ qboolean SV_Push (edict_t *pusher, vec3_t move, vec3_t amove)
 			VectorCopy (check->s.angles, pushed_p->angles);
 			pushed_p++;
 
-			// try moving the contacted entity 
+			// try moving the contacted entity
 			VectorAdd (check->s.origin, move, check->s.origin);
 			if (check->client)
 			{	// FIXME: doesn't rotate monsters?
@@ -536,7 +540,7 @@ qboolean SV_Push (edict_t *pusher, vec3_t move, vec3_t amove)
 				continue;
 			}
 		}
-		
+
 		// save off the obstacle so we can call the block function
 		obstacle = check;
 
@@ -572,7 +576,7 @@ Bmodel objects don't interact with each other, but
 push all box objects
 ================
 */
-void SV_Physics_Pusher (edict_t *ent)
+void SV_Physics_Pusher (edict_t *ent, float timespan)
 {
 	vec3_t		move, amove;
 	edict_t		*part, *mv;
@@ -592,8 +596,8 @@ void SV_Physics_Pusher (edict_t *ent)
 			part->avelocity[0] || part->avelocity[1] || part->avelocity[2]
 			)
 		{	// object is moving
-			VectorScale (part->velocity, FRAMETIME, move);
-			VectorScale (part->avelocity, FRAMETIME, amove);
+			VectorScale (part->velocity, timespan, move);
+			VectorScale (part->avelocity, timespan, amove);
 
 			if (!SV_Push (part, move, amove))
 				break;	// move was blocked
@@ -608,7 +612,7 @@ void SV_Physics_Pusher (edict_t *ent)
 		for (mv = ent ; mv ; mv=mv->teamchain)
 		{
 			if (mv->nextthink > 0)
-				mv->nextthink += FRAMETIME;
+				mv->nextthink += timespan;
 		}
 
 		// if the pusher has a "blocked" function, call it
@@ -653,14 +657,14 @@ SV_Physics_Noclip
 A moving object that doesn't obey physics
 =============
 */
-void SV_Physics_Noclip (edict_t *ent)
+void SV_Physics_Noclip (edict_t *ent, float timespan)
 {
 // regular thinking
 	if (!SV_RunThink (ent))
 		return;
-	
-	VectorMA (ent->s.angles, FRAMETIME, ent->avelocity, ent->s.angles);
-	VectorMA (ent->s.origin, FRAMETIME, ent->velocity, ent->s.origin);
+
+	VectorMA (ent->s.angles, timespan, ent->avelocity, ent->s.angles);
+	VectorMA (ent->s.origin, timespan, ent->velocity, ent->s.origin);
 
 	gi.linkentity (ent);
 }
@@ -680,7 +684,7 @@ SV_Physics_Toss
 Toss, bounce, and fly movement.  When onground, do nothing.
 =============
 */
-void SV_Physics_Toss (edict_t *ent)
+void SV_Physics_Toss (edict_t *ent, float timespan)
 {
 	trace_t		trace;
 	vec3_t		move;
@@ -716,13 +720,13 @@ void SV_Physics_Toss (edict_t *ent)
 // add gravity
 	if (ent->movetype != MOVETYPE_FLY
 	&& ent->movetype != MOVETYPE_FLYMISSILE)
-		SV_AddGravity (ent);
+		SV_AddGravity (ent, timespan);
 
 // move angles
-	VectorMA (ent->s.angles, FRAMETIME, ent->avelocity, ent->s.angles);
+	VectorMA (ent->s.angles, timespan, ent->avelocity, ent->s.angles);
 
 // move origin
-	VectorScale (ent->velocity, FRAMETIME, move);
+	VectorScale (ent->velocity, timespan, move);
 	trace = SV_PushEntity (ent, move);
 	if (!ent->inuse)
 		return;
@@ -738,7 +742,7 @@ void SV_Physics_Toss (edict_t *ent)
 
 	// stop if on ground
 		if (trace.plane.normal[2] > 0.7)
-		{		
+		{
 			if (ent->velocity[2] < 60 || ent->movetype != MOVETYPE_BOUNCE )
 			{
 				ent->groundentity = trace.ent;
@@ -751,7 +755,7 @@ void SV_Physics_Toss (edict_t *ent)
 //		if (ent->touch)
 //			ent->touch (ent, trace.ent, &trace.plane, trace.surface);
 	}
-	
+
 // check for water transition
 	wasinwater = (ent->watertype & MASK_WATER);
 	ent->watertype = gi.pointcontents (ent->s.origin);
@@ -796,13 +800,13 @@ FIXME: is this true?
 #define sv_friction			6
 #define sv_waterfriction	1
 
-void SV_AddRotationalFriction (edict_t *ent)
+void SV_AddRotationalFriction (edict_t *ent, float timespan)
 {
 	int		n;
 	float	adjustment;
 
-	VectorMA (ent->s.angles, FRAMETIME, ent->avelocity, ent->s.angles);
-	adjustment = FRAMETIME * sv_stopspeed * sv_friction;
+	VectorMA (ent->s.angles, timespan, ent->avelocity, ent->s.angles);
+	adjustment = timespan * sv_stopspeed * sv_friction;
 	for (n = 0; n < 3; n++)
 	{
 		if (ent->avelocity[n] > 0)
@@ -820,7 +824,7 @@ void SV_AddRotationalFriction (edict_t *ent)
 	}
 }
 
-void SV_Physics_Step (edict_t *ent)
+void SV_Physics_Step (edict_t *ent, float timespan)
 {
 	qboolean	wasonground;
 	qboolean	hitsound = false;
@@ -842,9 +846,9 @@ void SV_Physics_Step (edict_t *ent)
 		wasonground = true;
 	else
 		wasonground = false;
-		
+
 	if (ent->avelocity[0] || ent->avelocity[1] || ent->avelocity[2])
-		SV_AddRotationalFriction (ent);
+		SV_AddRotationalFriction (ent, timespan);
 
 	// add gravity except:
 	//   flying monsters
@@ -856,7 +860,7 @@ void SV_Physics_Step (edict_t *ent)
 				if (ent->velocity[2] < sv_gravity->value*-0.1)
 					hitsound = true;
 				if (ent->waterlevel == 0)
-					SV_AddGravity (ent);
+					SV_AddGravity (ent, timespan);
 			}
 
 	// friction for flying monsters that have been given vertical velocity
@@ -865,7 +869,7 @@ void SV_Physics_Step (edict_t *ent)
 		speed = fabs(ent->velocity[2]);
 		control = speed < sv_stopspeed ? sv_stopspeed : speed;
 		friction = sv_friction/3;
-		newspeed = speed - (FRAMETIME * control * friction);
+		newspeed = speed - (timespan * control * friction);
 		if (newspeed < 0)
 			newspeed = 0;
 		newspeed /= speed;
@@ -877,7 +881,7 @@ void SV_Physics_Step (edict_t *ent)
 	{
 		speed = fabs(ent->velocity[2]);
 		control = speed < sv_stopspeed ? sv_stopspeed : speed;
-		newspeed = speed - (FRAMETIME * control * sv_waterfriction * ent->waterlevel);
+		newspeed = speed - (timespan * control * sv_waterfriction * ent->waterlevel);
 		if (newspeed < 0)
 			newspeed = 0;
 		newspeed /= speed;
@@ -898,7 +902,7 @@ void SV_Physics_Step (edict_t *ent)
 					friction = sv_friction;
 
 					control = speed < sv_stopspeed ? sv_stopspeed : speed;
-					newspeed = speed - FRAMETIME*control*friction;
+					newspeed = speed - timespan*control*friction;
 
 					if (newspeed < 0)
 						newspeed = 0;
@@ -913,7 +917,7 @@ void SV_Physics_Step (edict_t *ent)
 			mask = MASK_MONSTERSOLID;
 		else
 			mask = MASK_SOLID;
-		SV_FlyMove (ent, FRAMETIME, mask);
+		SV_FlyMove (ent, timespan, mask);
 
 		gi.linkentity (ent);
 		G_TouchTriggers (ent);
@@ -932,7 +936,7 @@ G_RunEntity
 
 ================
 */
-void G_RunEntity (edict_t *ent)
+void G_RunEntity (edict_t *ent, float timespan)
 {
 	if (ent->prethink)
 		ent->prethink (ent);
@@ -946,24 +950,24 @@ void G_RunEntity (edict_t *ent)
 // ACEBOT_END
 	case MOVETYPE_PUSH:
 	case MOVETYPE_STOP:
-		SV_Physics_Pusher (ent);
+		SV_Physics_Pusher (ent, timespan);
 		break;
 	case MOVETYPE_NONE:
 		SV_Physics_None (ent);
 		break;
 	case MOVETYPE_NOCLIP:
-		SV_Physics_Noclip (ent);
+		SV_Physics_Noclip (ent, timespan);
 		break;
 	case MOVETYPE_STEP:
-		SV_Physics_Step (ent);
+		SV_Physics_Step (ent, timespan);
 		break;
 	case MOVETYPE_TOSS:
 	case MOVETYPE_BOUNCE:
 	case MOVETYPE_FLY:
 	case MOVETYPE_FLYMISSILE:
-		SV_Physics_Toss (ent);
+		SV_Physics_Toss (ent, timespan);
 		break;
 	default:
-		gi.error ("SV_Physics: bad movetype %i", (int)ent->movetype);			
+		gi.error ("SV_Physics: bad movetype %i", (int)ent->movetype);
 	}
 }
