@@ -30,88 +30,6 @@ typedef struct
 	void	(*spawn)(edict_t *ent);
 } spawn_t;
 
-
-void SP_item_health (edict_t *self);
-void SP_item_health_small (edict_t *self);
-void SP_item_health_large (edict_t *self);
-void SP_item_health_mega (edict_t *self);
-
-void SP_info_player_start (edict_t *ent);
-void SP_info_player_deathmatch (edict_t *ent);
-void SP_info_player_intermission (edict_t *ent);
-void SP_info_player_red (edict_t *ent);
-void SP_info_player_blue(edict_t *ent);
-
-void SP_func_plat (edict_t *ent);
-void SP_func_rotating (edict_t *ent);
-void SP_func_button (edict_t *ent);
-void SP_func_door (edict_t *ent);
-void SP_func_door_secret (edict_t *ent);
-void SP_func_door_rotating (edict_t *ent);
-void SP_func_water (edict_t *ent);
-void SP_func_train (edict_t *ent);
-void SP_func_conveyor (edict_t *self);
-void SP_func_wall (edict_t *self);
-void SP_func_object (edict_t *self);
-void SP_func_explosive (edict_t *self);
-void SP_func_timer (edict_t *self);
-void SP_func_areaportal (edict_t *ent);
-void SP_func_killbox (edict_t *ent);
-
-void SP_trigger_always (edict_t *ent);
-void SP_trigger_once (edict_t *ent);
-void SP_trigger_multiple (edict_t *ent);
-void SP_trigger_relay (edict_t *ent);
-void SP_trigger_push (edict_t *ent);
-void SP_trigger_hurt (edict_t *ent);
-void SP_trigger_key (edict_t *ent);
-void SP_trigger_counter (edict_t *ent);
-void SP_trigger_elevator (edict_t *ent);
-void SP_trigger_gravity (edict_t *ent);
-void SP_trigger_monsterjump (edict_t *ent);
-void SP_trigger_deathballtarget (edict_t *ent);
-void SP_trigger_reddeathballtarget (edict_t *ent);
-void SP_trigger_bluedeathballtarget (edict_t *ent);
-void SP_trigger_redcowtarget ( edict_t *ent);
-void SP_trigger_bluecowtarget (edict_t *ent);
-
-void SP_target_temp_entity (edict_t *ent);
-void SP_target_speaker (edict_t *ent);
-void SP_target_explosion (edict_t *ent);
-void SP_target_secret (edict_t *ent);
-void SP_target_splash (edict_t *ent);
-void SP_target_steam (edict_t *ent);
-void SP_target_spawner (edict_t *ent);
-void SP_target_blaster (edict_t *ent);
-void SP_target_laser (edict_t *self);
-void SP_target_lightramp (edict_t *self);
-void SP_target_earthquake (edict_t *ent);
-void SP_target_fire (edict_t *ent);
-void SP_target_changelevel (edict_t *ent);
-
-void SP_worldspawn (edict_t *ent);
-
-void SP_light (edict_t *self);
-void SP_info_null (edict_t *self);
-void SP_info_notnull (edict_t *self);
-void SP_path_corner (edict_t *self);
-void SP_point_combat (edict_t *self);
-
-void SP_misc_teleporter (edict_t *self);
-void SP_misc_teleporter_dest (edict_t *self);
-
-void SP_npc_cow (edict_t *self);
-void SP_npc_deathray(edict_t *self);
-
-void SP_misc_spiderpod (edict_t *self);
-void SP_misc_rednode (edict_t *self);
-void SP_misc_bluenode (edict_t *self);
-void SP_misc_bluespidernode (edict_t *self);
-void SP_misc_redspidernode (edict_t *self);
-void SP_misc_mapmodel (edict_t *self);
-void SP_misc_watersplash (edict_t *ent);
-void SP_misc_electroflash (edict_t *ent);
-
 spawn_t	spawns[] = {
 	{"item_health", SP_item_health},
 	{"item_health_small", SP_item_health_small},
@@ -164,8 +82,10 @@ spawn_t	spawns[] = {
 	{"target_splash", SP_target_splash},
 	{"target_steam", SP_target_steam},
 	{"target_spawner", SP_target_spawner},
+#ifndef ALTERIA
 	{"target_blaster", SP_target_blaster},
 	{"target_laser", SP_target_laser},
+#endif
 	{"target_lightramp", SP_target_lightramp},
 	{"target_earthquake", SP_target_earthquake},
 	{"target_fire", SP_target_fire},
@@ -186,11 +106,25 @@ spawn_t	spawns[] = {
 	{"npc_cow", SP_npc_cow},
 	{"npc_deathray", SP_npc_deathray},
 
+	//TCA
 	{"misc_spiderpod", SP_misc_spiderpod},
 	{"misc_rednode", SP_misc_rednode},
 	{"misc_bluenode", SP_misc_bluenode},
 	{"misc_redspidernode", SP_misc_redspidernode},
 	{"misc_bluespidernode", SP_misc_bluespidernode},
+
+	//Tactical
+	{"misc_aliencomputer", SP_misc_aliencomputer},
+	{"misc_humancomputer", SP_misc_humancomputer},
+	{"misc_alienpowersrc", SP_misc_alienpowersrc},
+	{"misc_humanpowersrc", SP_misc_humanpowersrc},
+	{"misc_alienammodepot", SP_misc_alienammodepot},
+	{"misc_humanammodepot", SP_misc_humanammodepot},
+	{"misc_alienbackupgen", SP_misc_alienbackupgen},
+	{"misc_humanbackupgen", SP_misc_humanbackupgen},
+	{"misc_deathray", SP_misc_deathray}, //note - spawnflags determine team for this item(1 for human)
+	{"misc_laser", SP_misc_laser}, //spawnflag 1 for human
+
 	{"misc_mapmodel", SP_misc_mapmodel},
 	{"misc_watersplash", SP_misc_watersplash},
 	{"misc_electroflash", SP_misc_electroflash},
@@ -227,6 +161,8 @@ void ED_CallSpawn (edict_t *ent)
 			ent->classname = "weapon_rocketlauncher"; //hack to remove old weapons
 		if(!Q_strcasecmp(ent->classname, "weapon_machinegun"))
 			ent->classname = "weapon_bfg"; //hack to remove old weapons
+		if(!Q_strcasecmp(ent->classname, "weapon_minderaser"))
+			continue; //never place one of these traditionally
 
 		if (!strcmp(item->classname, ent->classname))
 		{	// found it
@@ -460,6 +396,7 @@ Creates a server's entity / program execution context by
 parsing textual entity definitions out of an ent file.
 ==============
 */
+int levelnum;
 void SpawnEntities (char *mapname, char *entities, char *spawnpoint)
 {
 	edict_t		*ent;
@@ -491,6 +428,7 @@ void SpawnEntities (char *mapname, char *entities, char *spawnpoint)
 
 	ent = NULL;
 	inhibit = 0;
+	levelnum++;
 
 // parse ents
 	while (1)
@@ -546,6 +484,9 @@ void SpawnEntities (char *mapname, char *entities, char *spawnpoint)
 
 	gi.dprintf ("%i entities inhibited\n", inhibit);
 
+	//reset minderaser
+	mindEraserTime = level.time;
+
 	ACEND_InitNodes();
 	ACEND_LoadNodes();
 
@@ -776,6 +717,196 @@ char *team_statusbar =
 "yt 413 "
 "pic 31 "
 "endif "
+;
+
+char *tca_statusbar =
+// background
+"yb -256 "
+"xl	 0 "
+"pic 0 "
+"xr  -130 "
+"yt  2 "
+"pic 18 "
+
+// health
+"yb	-29 "
+"xl	11 "
+"hnum "
+
+// ammo
+"if 2 "
+"	xl	76 "
+"	anum "
+"endif "
+
+// armor
+"	xl	142 "
+"	rnum "
+
+// timer
+"if 9 "
+"	xv	324 "
+"   yb  -24 "
+"	num	2	10 "
+"	xv	358 "
+"   yb  -32 "
+"	pic	9 "
+"endif "
+
+// weapon icon
+"if 11 "
+"	xr	-72 "
+"   yt  196 "
+"	pic	11 "
+"endif "
+
+//  frags
+"xr	-67 "
+"yt 16 "
+"num 3 14"
+
+//  deaths
+"xr	-67 "
+"yt 48 "
+"num 3 19 "
+
+//  high scorer
+"yt 80 "
+"num 3 20 "
+
+//  red team
+"yt 132 "
+"num 1 6 "
+"xr -27 "
+"num 1 21 "
+//  blue team
+"yt 166 "
+"num 1 22 "
+"xr -67 "
+"num 1 7 "
+//  red label
+"yt 124 "
+"xr -30 "
+"string nodes "
+"xr -70 "
+"string matches "
+//  blue label
+"yt 158 "
+"xr -30 "
+"string nodes "
+"xr -70 "
+"string matches "
+
+//  flag
+"   xv 128 "
+"   yb -64 "
+"   pic 23 "
+
+//  weapon stats
+"if 25 "
+"xr -72 "
+"yt 227 "
+"pic 25 "
+"endif"
+
+"if 26 "
+"yt 258 "
+"pic 26 "
+"endif "
+
+"if 27 "
+"yt 289 "
+"pic 27 "
+"endif "
+
+"if 28 "
+"yt 320 "
+"pic 28 "
+"endif "
+
+"if 29 "
+"yt 351 "
+"pic 29 "
+"endif "
+
+"if 30 "
+"yt 382 "
+"pic 30 "
+"endif "
+
+"if 31 "
+"yt 413 "
+"pic 31 "
+"endif "
+;
+
+char *tactical_statusbar =
+// background
+"yb -256 "
+"xl	 0 "
+"pic 0 "
+"xr  -130 "
+"yt  2 "
+"pic 8 "
+
+// health
+"yb	-29 "
+"xl	11 "
+"hnum "
+
+// ammo
+"if 2 "
+"	xl	76 "
+"	anum "
+"endif "
+
+// armor
+"	xl	142 "
+"	rnum "
+
+//  Human Computer
+"xr	-67 "
+"yt 16 "
+"num 3 25 "
+
+//  Human Power
+"yt 48 "
+"num 3 26 "
+
+//  Human Ammo Depot
+"yt 80 "
+"num 3 27 "
+
+//  Alien Computer
+"yt 122 "
+"num 3 28 "
+
+//  Alien Power
+"yt 154 "
+"num 3 29 "
+
+//  Alien Ammo Depot
+"yt 186 "
+"num 3 30 "
+
+// timer
+"if 9 "
+"	xv	324 "
+"   yb  -24 "
+"	num	2	10 "
+"	xv	358 "
+"   yb  -32 "
+"	pic	9 "
+"endif "
+
+// weapon icon
+"if 11 "
+"	xr	-72 "
+"   yt  196 "
+"	pic	11 "
+"endif "
+
+
 ;
 /*QUAKED worldspawn (0 0 0) ?
 
@@ -828,12 +959,16 @@ void SP_worldspawn (edict_t *ent)
 	gi.configstring (CS_MAXCLIENTS, va("%i", g_maxclients->integer ) );
 
 	// status bar program
-	if ((dmflags->integer & DF_SKINTEAMS) || ctf->value || tca->value || cp->value)
+	if ((dmflags->integer & DF_SKINTEAMS) || ctf->integer || cp->integer)
 	{
 		gi.configstring (CS_STATUSBAR, team_statusbar);
 		if(ctf->value)
 			CTFPrecache();
 	}
+	else if (tca->integer)
+		gi.configstring (CS_STATUSBAR, tca_statusbar);
+	else if (g_tactical->integer)
+		gi.configstring (CS_STATUSBAR, tactical_statusbar);
 	else
 		gi.configstring (CS_STATUSBAR, dm_statusbar);
 
@@ -911,6 +1046,7 @@ void SP_worldspawn (edict_t *ent)
 	gi.soundindex ("misc/godlike.wav");
 	gi.soundindex ("misc/rampage.wav");
 	gi.soundindex ("misc/fight.wav");
+	gi.soundindex ("misc/minderaser.wav");
 
     //if maxclients is 0 or 1, it means someone is probably just testing a map
     //or testing some code, so we really don't need to precache all this crap
@@ -958,6 +1094,18 @@ void SP_worldspawn (edict_t *ent)
 	    gi.soundindex ("taunts/martiancyborg/taunt4.wav");
 	    gi.soundindex ("taunts/martiancyborg/taunt5.wav");
 
+		gi.soundindex ("taunts/martianoverlord/taunt1.wav");
+	    gi.soundindex ("taunts/martianoverlord/taunt2.wav");
+	    gi.soundindex ("taunts/martianoverlord/taunt3.wav");
+	    gi.soundindex ("taunts/martianoverlord/taunt4.wav");
+	    gi.soundindex ("taunts/martianoverlord/taunt5.wav");
+
+		gi.soundindex ("taunts/martianwarrior/taunt1.wav");
+	    gi.soundindex ("taunts/martianwarrior/taunt2.wav");
+	    gi.soundindex ("taunts/martianwarrior/taunt3.wav");
+	    gi.soundindex ("taunts/martianwarrior/taunt4.wav");
+	    gi.soundindex ("taunts/martianwarrior/taunt5.wav");
+
 	    //precache any gibs
 	    sm_meat_index = gi.modelindex ("models/objects/gibs/sm_meat/tris.md2");
 	    gi.modelindex ("models/objects/gibs/mart_gut/tris.md2");
@@ -968,6 +1116,8 @@ void SP_worldspawn (edict_t *ent)
 	    //this should be here for the cases of 7.45 clients and older
 	    gi.modelindex ("players/martianenforcer/tris.md2");
 	    gi.modelindex ("players/martiancyborg/tris.md2");
+		gi.modelindex ("players/martianoverlord/tris.md2");
+		gi.modelindex ("players/martianwarrior/tris.md2");
 	    gi.modelindex ("players/enforcer/tris.md2");
 	    gi.modelindex ("players/lauren/tris.md2");
 	    gi.modelindex ("players/slashbot/tris.md2");
@@ -985,6 +1135,7 @@ void SP_worldspawn (edict_t *ent)
 	    gi.modelindex ("players/martianenforcer/w_railgun.md2");
 	    gi.modelindex ("players/martianenforcer/w_bfg.md2");
 	    gi.modelindex ("players/martianenforcer/w_violator.md2");
+		gi.modelindex ("players/martianenforcer/w_minderaser.md2");
 
 	    gi.modelindex ("players/martiancyborg/weapon.md2");
 	    gi.modelindex ("players/martiancyborg/w_blaster.md2");
@@ -997,6 +1148,7 @@ void SP_worldspawn (edict_t *ent)
 	    gi.modelindex ("players/martiancyborg/w_railgun.md2");
 	    gi.modelindex ("players/martiancyborg/w_bfg.md2");
 	    gi.modelindex ("players/martiancyborg/w_violator.md2");
+		gi.modelindex ("players/martiancyborg/w_minderaser.md2");
 
 	    gi.modelindex ("players/enforcer/weapon.md2");
 	    gi.modelindex ("players/enforcer/w_blaster.md2");
@@ -1009,6 +1161,7 @@ void SP_worldspawn (edict_t *ent)
 	    gi.modelindex ("players/enforcer/w_railgun.md2");
 	    gi.modelindex ("players/enforcer/w_bfg.md2");
 	    gi.modelindex ("players/enforcer/w_violator.md2");
+		gi.modelindex ("players/enforcer/w_minderaser.md2");
 
 	    gi.modelindex ("players/lauren/weapon.md2");
 	    gi.modelindex ("players/lauren/w_blaster.md2");
@@ -1021,6 +1174,7 @@ void SP_worldspawn (edict_t *ent)
 	    gi.modelindex ("players/lauren/w_railgun.md2");
 	    gi.modelindex ("players/lauren/w_bfg.md2");
 	    gi.modelindex ("players/lauren/w_violator.md2");
+		gi.modelindex ("players/lauren/w_minderaser.md2");
 
 	    gi.modelindex ("players/slashbot/weapon.md2");
 	    gi.modelindex ("players/slashbot/w_blaster.md2");
@@ -1033,6 +1187,7 @@ void SP_worldspawn (edict_t *ent)
 	    gi.modelindex ("players/slashbot/w_railgun.md2");
 	    gi.modelindex ("players/slashbot/w_bfg.md2");
 	    gi.modelindex ("players/slashbot/w_violator.md2");
+		gi.modelindex ("players/slashbot/w_minderaser.md2");
 
 	    gi.modelindex ("players/commander/weapon.md2");
 	    gi.modelindex ("players/commander/w_blaster.md2");
@@ -1045,8 +1200,42 @@ void SP_worldspawn (edict_t *ent)
 	    gi.modelindex ("players/commander/w_railgun.md2");
 	    gi.modelindex ("players/commander/w_bfg.md2");
 	    gi.modelindex ("players/commander/w_violator.md2");
+		gi.modelindex ("players/commander/w_minderaser.md2");
+
+		gi.modelindex ("players/martianoverlord/weapon.md2");
+	    gi.modelindex ("players/martianoverlord/w_blaster.md2");
+	    gi.modelindex ("players/martianoverlord/w_shotgun.md2");
+	    gi.modelindex ("players/martianoverlord/w_sshotgun.md2");
+	    gi.modelindex ("players/martianoverlord/w_machinegun.md2");
+	    gi.modelindex ("players/martianoverlord/w_chaingun.md2");
+	    gi.modelindex ("players/martianoverlord/w_rlauncher.md2");
+	    gi.modelindex ("players/martianoverlord/w_hyperblaster.md2");
+	    gi.modelindex ("players/martianoverlord/w_railgun.md2");
+	    gi.modelindex ("players/martianoverlord/w_bfg.md2");
+	    gi.modelindex ("players/martianoverlord/w_violator.md2");
+		gi.modelindex ("players/martianoverlord/w_minderaser.md2");
+
+		gi.modelindex ("players/martianwarrior/weapon.md2");
+	    gi.modelindex ("players/martianwarrior/w_blaster.md2");
+	    gi.modelindex ("players/martianwarrior/w_shotgun.md2");
+	    gi.modelindex ("players/martianwarrior/w_sshotgun.md2");
+	    gi.modelindex ("players/martianwarrior/w_machinegun.md2");
+	    gi.modelindex ("players/martianwarrior/w_chaingun.md2");
+	    gi.modelindex ("players/martianwarrior/w_rlauncher.md2");
+	    gi.modelindex ("players/martianwarrior/w_hyperblaster.md2");
+	    gi.modelindex ("players/martianwarrior/w_railgun.md2");
+	    gi.modelindex ("players/martianwarrior/w_bfg.md2");
+	    gi.modelindex ("players/martianwarrior/w_violator.md2");
+		gi.modelindex ("players/martianwarrior/w_minderaser.md2");
+
+		//weapon models that aren't placed in levels
+		gi.modelindex ("models/weapons/v_minderaser/tris.md2");
+		gi.modelindex ("models/weapons/g_minderaser/tris.md2");
+		gi.modelindex ("models/objects/spud/tris.md2");
+		gi.modelindex ("models/weapons/v_blast/tris.md2");
+		gi.modelindex ("models/weapons/v_violator/tris.md2");
 	}
-	
+
 	//these are needed for custom player models for when they enter a game
 	gi.modelindex ("#w_blaster.md2");
 	gi.modelindex ("#w_shotgun.md2");
@@ -1060,6 +1249,7 @@ void SP_worldspawn (edict_t *ent)
 	gi.modelindex ("#w_railgun.md2");
 	gi.modelindex ("#w_bfg.md2");
 	gi.modelindex ("#w_violator.md2");
+	gi.modelindex ("#w_minderaser.md2");
 
 //
 // Setup light animation tables. 'a' is total darkness, 'z' is doublebright.
