@@ -90,10 +90,11 @@ void Target_Speaker_Think (edict_t *ent)
 
 	ent->nextthink = level.time + 1;
 }
+extern int levelnum;
 void SP_target_speaker (edict_t *ent)
 {
 	char	buffer[MAX_QPATH];
-
+	static int prev_levelnum;
 
 	if(!st.noise)
 	{
@@ -102,7 +103,13 @@ void SP_target_speaker (edict_t *ent)
 	}
 	if (strstr (st.noise, "music"))
 	{
-		gi.dprintf("SHUT UP SHUT UP SHUT UP YOU INFERNAL BONGO DRUMS!\n");
+		if (levelnum != prev_levelnum) //only warn once per level
+		{
+			prev_levelnum = levelnum;
+			gi.dprintf("SHUT UP SHUT UP SHUT UP YOU INFERNAL BONGO DRUMS!\n");
+			gi.dprintf("Map contains one or more target_speakers with \"music\" in its sound file name.\n");
+			gi.dprintf("These have been disabled.\n");
+		}
 		return;
 	}
 	if (!strstr (st.noise, ".wav"))
@@ -357,6 +364,7 @@ dmg		default is 15
 speed	default is 1000
 */
 
+#ifndef ALTERIA
 void use_target_blaster (edict_t *self, edict_t *other, edict_t *activator)
 {
 	int effect;
@@ -368,7 +376,7 @@ void use_target_blaster (edict_t *self, edict_t *other, edict_t *activator)
 	else
 		effect = EF_BLASTER;
 
-	fire_blaster (self, self->s.origin, self->movedir, self->dmg, self->speed, EF_BLASTER, MOD_TARGET_BLASTER);
+	fire_blaster (self, self->s.origin, self->s.origin, self->movedir, self->dmg, self->speed, EF_BLASTER, MOD_TARGET_BLASTER);
 	gi.sound (self, CHAN_VOICE, self->noise_index, 1, ATTN_NORM, 0);
 }
 
@@ -385,7 +393,7 @@ void SP_target_blaster (edict_t *self)
 
 	self->svflags = SVF_NOCLIENT;
 }
-
+#endif
 //==========================================================
 
 /*QUAKED target_laser (0 .5 .8) (-8 -8 -8) (8 8 8) START_ON RED GREEN BLUE YELLOW ORANGE FAT
