@@ -8,7 +8,7 @@ of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See the GNU General Public License for more details.
 
@@ -19,6 +19,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // g_utils.c -- misc utility functions for game module
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "g_local.h"
 
 
@@ -58,7 +62,7 @@ edict_t *G_Find (edict_t *from, int fieldofs, char *match)
 		s = *(char **) ((byte *)from + fieldofs);
 		if (!s)
 			continue;
-		if (!Q_stricmp (s, match))
+		if (!Q_strcasecmp (s, match))
 			return from;
 	}
 
@@ -192,8 +196,8 @@ void G_UseTargets (edict_t *ent, edict_t *activator)
 		t->killtarget = ent->killtarget;
 		return;
 	}
-	
-	
+
+
 //
 // print the message
 //
@@ -232,8 +236,8 @@ void G_UseTargets (edict_t *ent, edict_t *activator)
 		while ((t = G_Find (t, FOFS(targetname), ent->target)))
 		{
 			// doors fire area portals in a specific way
-			if (!Q_stricmp(t->classname, "func_areaportal") &&
-				(!Q_stricmp(ent->classname, "func_door") || !Q_stricmp(ent->classname, "func_door_rotating")))
+			if (!Q_strcasecmp(t->classname, "func_areaportal") &&
+				(!Q_strcasecmp(ent->classname, "func_door") || !Q_strcasecmp(ent->classname, "func_door_rotating")))
 				continue;
 
 			if (t == ent)
@@ -333,15 +337,15 @@ void G_SetMovedir (vec3_t angles, vec3_t movedir)
 float vectoyaw (vec3_t vec)
 {
 	float	yaw;
-	
-	if (/*vec[YAW] == 0 &&*/ vec[PITCH] == 0) 
+
+	if (/*vec[YAW] == 0 &&*/ vec[PITCH] == 0)
 	{
 		yaw = 0;
 		if (vec[YAW] > 0)
 			yaw = 90;
 		else if (vec[YAW] < 0)
 			yaw = -90;
-	} 
+	}
 	else
 	{
 		yaw = (int) (atan2(vec[YAW], vec[PITCH]) * 180 / M_PI);
@@ -352,46 +356,12 @@ float vectoyaw (vec3_t vec)
 	return yaw;
 }
 
-
-void vectoangles (vec3_t value1, vec3_t angles)
-{
-	float	forward;
-	float	yaw, pitch;
-	
-	if (value1[1] == 0 && value1[0] == 0)
-	{
-		yaw = 0;
-		if (value1[2] > 0)
-			pitch = 90;
-		else
-			pitch = 270;
-	}
-	else
-	{
-		if (value1[0])
-			yaw = (int) (atan2(value1[1], value1[0]) * 180 / M_PI);
-		else if (value1[1] > 0)
-			yaw = 90;
-		else
-			yaw = -90;
-		if (yaw < 0)
-			yaw += 360;
-
-		forward = sqrt (value1[0]*value1[0] + value1[1]*value1[1]);
-		pitch = (int) (atan2(value1[2], forward) * 180 / M_PI);
-		if (pitch < 0)
-			pitch += 360;
-	}
-
-	angles[PITCH] = -pitch;
-	angles[YAW] = yaw;
-	angles[ROLL] = 0;
-}
+// void vectoangles (vec3_t value1, vec3_t angles)  Moved to q_shared.c
 
 char *G_CopyString (const char *in)
 {
 	char	*out;
-	
+
 	out = gi.TagMalloc (strlen(in)+1, TAG_LEVEL);
 	strcpy (out, in);
 	return out;
@@ -422,8 +392,8 @@ edict_t *G_Spawn (void)
 	int			i;
 	edict_t		*e;
 
-	e = &g_edicts[(int)maxclients->value+1];
-	for ( i=maxclients->value+1 ; i<globals.num_edicts ; i++, e++)
+	e = &g_edicts[g_maxclients->integer+1];
+	for ( i=g_maxclients->integer+1 ; i<globals.num_edicts ; i++, e++)
 	{
 		// the first couple seconds of server time can involve a lot of
 		// freeing and allocating, so relax the replacement policy
@@ -433,10 +403,10 @@ edict_t *G_Spawn (void)
 			return e;
 		}
 	}
-	
+
 	if (i == game.maxentities)
 		gi.error ("ED_Alloc: no free edicts");
-		
+
 	globals.num_edicts++;
 	G_InitEdict (e);
 	return e;
@@ -453,7 +423,7 @@ void G_FreeEdict (edict_t *ed)
 {
 	gi.unlinkentity (ed);		// unlink from world
 
-	if ((ed - g_edicts) <= (maxclients->value + BODY_QUEUE_SIZE))
+	if ((ed - g_edicts) <= (g_maxclients->value + BODY_QUEUE_SIZE))
 	{
 //		gi.dprintf("tried to free special edict\n");
 		return;
@@ -574,32 +544,31 @@ G_CleanPlayerName
 
 Removes escape characters from a player's name and prepares it
 to be displayed in green through safe_centerprintf.
+
+!Note!: dest must be at least PLAYERNAME_SIZE  bytes long.
+  (could be PLAYERNAME_GLYPHS + 1 long, but safer to use max. string length)
 =============
 */
 void G_CleanPlayerName ( const char *source, char *dest )
 {
-	int i, j, k;
+	const char *pch_src = source;
+	char *pch_dst = dest;
+	int count = strlen( pch_src );
 
-	memset( dest, 0, 16 );
-	for (i = j = k = 0; i < strlen(source) && j < 16; i ++)
+	memset( pch_dst, 0, PLAYERNAME_SIZE );
+	while ( count > 0 )
 	{
-		if ( !k && source[i] == '^' )
+		if ( Q_IsColorString( pch_src ) )
 		{
-			if ( source[i + 1] != '^' )
-			{
-				i += 1;
-				continue;
-			}
-			else
-			{
-				k = 1;
-			}
+			pch_src += 2;
+			count -= 2;
 		}
-		else if ( k )
+		else
 		{
-			k = 0;
+			*pch_dst++ = *pch_src | 0x80;
+			++pch_src;
+			--count;
 		}
-		dest[j ++] = source[i] + 128;
 	}
 }
 
