@@ -24,8 +24,50 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "g_local.h"
 
-int floater_timer;
-int prox_timer;
+#ifdef ALTERIA 
+
+	//add Alteria weapon code
+
+void fire_punch(edict_t *self, vec3_t start, vec3_t aimdir, int damage)
+{
+
+	vec3_t		from;
+	vec3_t		end;
+	trace_t		tr;
+	edict_t		*ignore;
+
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
+	VectorMA (start, 32, aimdir, end);
+	
+	VectorCopy (start, from);
+	ignore = self;
+
+	tr = gi.trace (from, NULL, NULL, end, ignore, MASK_SHOT);
+
+	if (tr.ent != self) 
+	{
+		if(tr.ent->takedamage || (tr.fraction < 1.0))
+		{
+			if(tr.ent->takedamage)
+				T_Damage (tr.ent, self, self, aimdir, tr.endpos, tr.plane.normal, damage, 4, 0, MOD_VIOLATOR); //add alteria mod types
+
+			gi.sound (self, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0); //change to impact sound
+
+			//change this to smoke puffs or similar
+			gi.WriteByte (svc_temp_entity);
+			gi.WriteByte (TE_CHAINGUNSMOKE); //note TE_SMOKE is available and unused if we want better impact effects
+			gi.WritePosition (tr.endpos);
+			gi.multicast (start, MULTICAST_PVS);
+		}
+	}
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
+}
+
+#else
 
 /*
 =================
@@ -299,7 +341,48 @@ void blasterball_touch (edict_t *self, edict_t *other, cplane_t *plane, csurface
 	G_FreeEdict (self);
 }
 
-void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, int effect, qboolean hyper)
+void alienblasterball_touch (edict_t *self, edict_t *other, cplane_t *plane, csurface_t *surf)
+{
+	int		mod;
+
+	if (other == self->owner)
+		return;
+
+	if (surf && (surf->flags & SURF_SKY))
+	{
+		G_FreeEdict (self);
+		return;
+	}
+
+	if (self->owner->client)
+		PlayerNoise(self->owner, self->s.origin, PNOISE_IMPACT);
+
+	if (other->takedamage)
+	{
+		if (self->spawnflags & 1)
+			mod = MOD_BEAMGUN;
+		else
+			mod = MOD_BLASTER;
+		T_Damage (other, self, self->owner, self->velocity, self->s.origin, plane->normal, self->dmg, 1, DAMAGE_ENERGY, mod);
+		self->owner->client->resp.weapon_hits[0]++;
+		gi.sound (self->owner, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
+	}
+	else
+	{
+		gi.WriteByte (svc_temp_entity);
+		gi.WriteByte (TE_SCREEN_SPARKS);
+		gi.WritePosition (self->s.origin);
+		if (!plane)
+			gi.WriteDir (vec3_origin);
+		else
+			gi.WriteDir (plane->normal);
+		gi.multicast (self->s.origin, MULTICAST_PVS);
+	}
+	T_RadiusDamage(self, self->owner, 95, other, 150, MOD_PLASMA_SPLASH, 0);
+	G_FreeEdict (self);
+}
+
+void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, int effect, qboolean hyper, qboolean alien)
 {
 	edict_t	*bolt;
 	trace_t	tr;
@@ -317,19 +400,28 @@ void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int
 	bolt->movetype = MOVETYPE_FLYMISSILE;
 	bolt->clipmask = MASK_SHOT;
 	bolt->solid = SOLID_BBOX;
-	if(effect == EF_ROCKET) { //ack, kinda assbackwards, but the past mistakes haunt us
+	if(effect == EF_ROCKET) 
+	{	//ack, kinda assbackwards, but the past mistakes haunt us
 		bolt->s.effects |= EF_BLASTER;
 		bolt->s.modelindex = gi.modelindex ("models/objects/laser/tris.md2");
 	}
-	else {
+	else 
+	{
 		bolt->s.effects |= EF_PLASMA;
 		bolt->s.modelindex = gi.modelindex ("models/objects/fireball/tris.md2");
 	}
+	// All we care about is the effects. However, old clients will refuse to 
+	// draw them unless the modelindex is set. To work around this, newer
+	// clients have an RF_NODRAW flag. 
+	bolt->s.renderfx |= RF_NODRAW;
 	VectorClear (bolt->mins);
 	VectorClear (bolt->maxs);
 	bolt->s.sound = gi.soundindex ("misc/lasfly.wav");
 	bolt->owner = self;
-	bolt->touch = blasterball_touch;
+	if(alien)
+		bolt->touch = alienblasterball_touch;
+	else
+		bolt->touch = blasterball_touch;
 	bolt->nextthink = level.time + 2;
 	bolt->think = G_FreeEdict;
 	bolt->dmg = damage;
@@ -344,9 +436,11 @@ void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int
 		VectorMA (bolt->s.origin, -10, dir, bolt->s.origin);
 		bolt->touch (bolt, tr.ent, NULL, NULL);
 	}
+	
+	if ( g_antilag->integer && g_antilagprojectiles->integer)
+		G_AntilagProjectile (bolt);
 }
-void fire_blaster (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
-int speed, int effect, qboolean hyper)
+void fire_blaster (edict_t *self, vec3_t start, vec3_t muzzle, vec3_t aimdir, int damage, int speed, int effect, qboolean hyper)
 {
 	vec3_t		from;
 	vec3_t		end;
@@ -375,7 +469,8 @@ int speed, int effect, qboolean hyper)
 	}
 	else
 	{
-		if ((tr.ent != self) && (tr.ent->takedamage)) {
+		if ((tr.ent != self) && (tr.ent->takedamage)) 
+		{
 			T_Damage (tr.ent, self, self, aimdir, tr.endpos, tr.plane.normal, damage, 0, 0, MOD_BLASTER);
 			self->client->resp.weapon_hits[6]++;
 			gi.sound (self, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
@@ -395,13 +490,13 @@ int speed, int effect, qboolean hyper)
 
 	gi.WriteByte (svc_temp_entity);
 	gi.WriteByte (TE_LASERBEAM);
-	gi.WritePosition (start);
+	gi.WritePosition (muzzle);
 	gi.WritePosition (tr.endpos);
 	gi.multicast (self->s.origin, MULTICAST_PHS);
 
 	if ((tr.ent != self) && (tr.ent->takedamage))
 	{
-        	T_Damage (tr.ent, self, self, aimdir, tr.endpos, tr.plane.normal, damage, 0, 0, MOD_BEAMGUN);
+        T_Damage (tr.ent, self, self, aimdir, tr.endpos, tr.plane.normal, damage, 0, 0, MOD_BEAMGUN);
 		self->client->resp.weapon_hits[6]++;
 		gi.sound (self, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
 	}
@@ -476,7 +571,11 @@ void fire_rocket (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed
 	rocket->movetype = MOVETYPE_FLYMISSILE;
 	rocket->clipmask = MASK_SHOT;
 	rocket->solid = SOLID_BBOX;
-	rocket->s.effects |= EF_ROCKET | EF_ROCKETEXHAUST ;
+	rocket->s.effects |= EF_ROCKETEXHAUST ;
+	rocket->s.renderfx |= RF_FULLBRIGHT ;
+	if (!excessive->integer && self->client->invincible_framenum <= level.framenum)
+		//With too many rockets, lots of smoke trails hurt performance.
+		rocket->s.effects |= EF_ROCKET;
 	VectorClear (rocket->mins);
 	VectorClear (rocket->maxs);
 	rocket->s.modelindex = gi.modelindex ("models/objects/rocket/tris.md2");
@@ -491,6 +590,9 @@ void fire_rocket (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed
 	rocket->classname = "rocket";
 
 	gi.linkentity (rocket);
+	
+	if ( g_antilag->integer && g_antilagprojectiles->integer)
+		G_AntilagProjectile (rocket);
 }
 
 /*
@@ -551,7 +653,7 @@ void stinger_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *s
 	G_FreeEdict (ent);
 }
 
-void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, qboolean detonate)
+void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, qboolean detonate, qboolean alien)
 {
 	vec3_t		from;
 	vec3_t		end;
@@ -583,8 +685,8 @@ void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 	}
 	else
 	{
-
-		if ((tr.ent != self) && (tr.ent->takedamage)) {
+		if ((tr.ent != self) && (tr.ent->takedamage)) 
+		{
 			T_Damage (tr.ent, self, self, aimdir, tr.endpos, tr.plane.normal, damage, kick, 0, MOD_BLASTER);
 			self->client->resp.weapon_hits[0]++;
 			gi.sound (self, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
@@ -604,18 +706,25 @@ void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 	VectorCopy (tr.endpos, from);
 
 	gi.WriteByte (svc_temp_entity);
-	gi.WriteByte (TE_BLASTERBEAM);
+	if(alien)
+		gi.WriteByte (TE_LASERBEAM);
+	else
+		gi.WriteByte (TE_BLASTERBEAM);
 	gi.WritePosition (start);
 	gi.WritePosition (tr.endpos);
 	gi.multicast (self->s.origin, MULTICAST_PHS);
 
 	gi.WriteByte (svc_temp_entity);
-	gi.WriteByte (TE_BLASTER);
+	if(alien)
+		gi.WriteByte (TE_SCREEN_SPARKS);
+	else
+		gi.WriteByte (TE_BLASTER);
 	gi.WritePosition (tr.endpos);
 	gi.WriteDir (tr.plane.normal);
 	gi.multicast (tr.endpos, MULTICAST_PVS);
 
-	if(detonate) {
+	if(detonate) 
+	{
 		//spawn a new ent at end of explosion - and detonate it
 		bomb = G_Spawn();
 		VectorCopy (tr.endpos, bomb->s.origin);
@@ -631,11 +740,11 @@ void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 	}
 
 	if (gi.pointcontents (start) & MASK_WATER)
-		{
-			water = true;
-			VectorCopy (start, water_start);
-			content_mask &= ~MASK_WATER;
-		}
+	{
+		water = true;
+		VectorCopy (start, water_start);
+		content_mask &= ~MASK_WATER;
+	}
 
 	tr = gi.trace (start, NULL, NULL, end, self, content_mask);
 
@@ -857,7 +966,7 @@ void fire_hover_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, in
 
 }
 
-void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick)
+void fire_disruptor (edict_t *self, vec3_t start, vec3_t muzzle, vec3_t aimdir, int damage, int kick)
 {
 	vec3_t		from;
 	vec3_t		end;
@@ -907,7 +1016,7 @@ void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int ki
 
 	gi.WriteByte (svc_temp_entity);
 	gi.WriteByte (TE_RAILTRAIL);
-	gi.WritePosition (start);
+	gi.WritePosition (muzzle);
 	gi.WritePosition (tr.endpos);
 	gi.multicast (self->s.origin, MULTICAST_PHS);
 
@@ -932,7 +1041,7 @@ void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int ki
 }
 
 //vaporizer code
-void fire_energy_field (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick)
+void fire_vaporizer (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick)
 {
 
 	vec3_t		from;
@@ -1058,7 +1167,7 @@ void homing_think (edict_t *ent)
 		VectorScale(targetdir, speed, ent->velocity);
 	}
 
-	ent->nextthink = level.time + .1;
+	ent->nextthink = level.time + FRAMETIME;
 }
 
 void fire_homingrocket (edict_t *self, vec3_t start, vec3_t dir, int damage, int speed, float damage_radius, int radius_damage)
@@ -1108,11 +1217,191 @@ void fire_homingrocket (edict_t *self, vec3_t start, vec3_t dir, int damage, int
 
 /*
 =================
-fire_floater
+fire_minderaser
 =================
 */
 
-void floater_think (edict_t *self)
+void minderaser_think (edict_t *ent)
+{
+	edict_t *target = NULL;
+	edict_t *blip = NULL;
+	vec3_t  targetdir, blipdir;
+	vec_t   speed;
+
+	while ((blip = findradius(blip, ent->s.origin, 1000)) != NULL)
+	{
+		if (!(blip->svflags & SVF_MONSTER) && !blip->client)
+			continue;
+		if (blip == ent->owner)
+			continue;
+		if (!blip->takedamage)
+			continue;
+		if (blip->health <= 0)
+			continue;
+		if (!visible(ent, blip))
+			continue;
+
+		VectorSubtract(blip->s.origin, ent->s.origin, blipdir);
+		blipdir[2] += 16;
+		if ((target == NULL) || (VectorLength(blipdir) < VectorLength(targetdir)))
+		{
+			target = blip;
+			VectorCopy(blipdir, targetdir);
+		}
+	}
+
+	if (target != NULL)
+	{
+		trace_t tr;
+
+		tr = gi.trace (ent->s.origin, NULL, NULL, target->s.origin, ent, MASK_SOLID);
+
+		// only move to it if path is not blocked
+		if (tr.fraction == 1.0)
+		{
+			// target acquired, nudge our direction toward it
+			VectorNormalize(targetdir);
+			VectorScale(targetdir, 0.2, targetdir);
+			VectorAdd(targetdir, ent->movedir, targetdir);
+			VectorNormalize(targetdir);
+			VectorCopy(targetdir, ent->movedir);
+			vectoangles(targetdir, ent->s.angles);
+			speed = 450;  //speed it up we have a target
+			VectorScale(targetdir, speed, ent->velocity);
+			ent->s.frame = (ent->s.frame + 1) % 24;
+			ent->s.effects = EF_SHIPEXHAUST; 
+			ent->s.sound = gi.soundindex ("weapons/seeker_fast.wav");
+			ent->s.renderfx = 0;
+		}
+	
+		//zap it if close enough and clear
+		if(VectorLength(blipdir) < 128)
+		{
+			//we are close, slow it down
+			speed = 30;  
+			VectorScale(targetdir, speed, ent->velocity);
+		
+			ent->s.effects = EF_COLOR_SHELL;
+			ent->s.renderfx = RF_SHELL_BLUE;
+		
+			// hurt it if we can
+			if ((target->takedamage) && (target != ent->owner)) 
+			{
+				T_Damage (target, ent, ent->owner, targetdir, target->s.origin, vec3_origin, 35, 1, DAMAGE_ENERGY, MOD_MINDERASER);
+				gi.sound (ent->owner, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
+			}
+
+			ent->s.sound = gi.soundindex ("weapons/seeker_zap.wav");
+
+			gi.WriteByte (svc_temp_entity);
+			gi.WriteByte (TE_LIGHTNING);
+			gi.WritePosition (ent->s.origin);
+			gi.WritePosition (target->s.origin);
+			gi.multicast (ent->s.origin, MULTICAST_PHS);
+
+			ent->s.frame = 27;
+		}
+	}
+	else
+	{
+		speed = 30;  
+		VectorScale(ent->movedir, speed, ent->velocity);
+		ent->s.effects = 0;
+		ent->s.renderfx = 0;
+		ent->s.sound = gi.soundindex ("weapons/seeker.wav");
+		ent->s.frame = (ent->s.frame + 1) % 24;
+	}
+	
+	ent->nade_timer++;
+	ent->nextthink = level.time + FRAMETIME;
+
+	if(ent->nade_timer > 300) 
+	{	//explode
+		gi.WriteByte (svc_temp_entity);
+		gi.WriteByte (TE_BFG_BIGEXPLOSION);
+		gi.WritePosition (ent->s.origin);
+		gi.multicast (ent->s.origin, MULTICAST_PHS);
+
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/explodebomb.wav"), 1, ATTN_NORM, 0);
+
+		T_RadiusDamage(ent, ent->owner, 400, ent->enemy, 120, MOD_R_SPLASH, 2);
+
+		G_FreeEdict (ent);
+	}
+}
+
+void minderaser_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
+{
+	if (other == ent->owner)
+		return;
+
+	if (surf && (surf->flags & SURF_SKY))
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+	else if(surf)
+	{
+		//explode
+		gi.WriteByte (svc_temp_entity);
+		gi.WriteByte (TE_BFG_BIGEXPLOSION);
+		gi.WritePosition (ent->s.origin);
+		gi.multicast (ent->s.origin, MULTICAST_PHS);
+
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/explodebomb.wav"), 1, ATTN_NORM, 0);
+		
+		T_RadiusDamage(ent, ent->owner, 400, ent->enemy, 120, MOD_R_SPLASH, 2);
+		
+		G_FreeEdict (ent);
+		return;
+	}
+
+	//just bounce off of anything else
+	gi.sound (ent, CHAN_VOICE, gi.soundindex ("weapons/clank.wav"), 1, ATTN_NORM, 0);
+	return;
+
+}
+
+void fire_minderaser (edict_t *self, vec3_t start, vec3_t dir, float timer)
+{
+	edict_t	*spud;
+	float *v;
+
+	spud = G_Spawn();
+	VectorCopy (start, spud->s.origin);
+	VectorCopy (dir, spud->movedir);
+	vectoangles (dir, spud->s.angles);
+	VectorScale (dir, 30, spud->velocity);
+	spud->movetype = MOVETYPE_FLYMISSILE;
+	spud->clipmask = MASK_SHOT;
+	spud->solid = SOLID_BBOX;
+	spud->s.effects = 0;
+	spud->s.renderfx = 0;
+	v = tv(-8,-8,-8);
+	VectorCopy (v, spud->mins);
+	v = tv(8,8,8);
+	VectorCopy (v, spud->maxs);
+	spud->s.modelindex = gi.modelindex ("models/objects/spud/tris.md2");
+	spud->owner = self;
+	spud->touch = minderaser_touch;
+	spud->nade_timer = 0;
+		
+	spud->nextthink = level.time + FRAMETIME;
+	spud->think = minderaser_think;
+
+	spud->s.sound = gi.soundindex ("weapons/seeker.wav"); 
+	spud->classname = "seeker"; //to do - make sure bots know to run like hell away from these things
+
+	gi.linkentity (spud);
+}
+
+/*
+=================
+fire_smartgrenade
+=================
+*/
+
+void smartgrenade_think (edict_t *self)
 {
 	edict_t	*ent;
 	edict_t *target = NULL;
@@ -1193,9 +1482,9 @@ void floater_think (edict_t *self)
 
 	self->s.frame = (self->s.frame + 1) % 23; //pulse grotesquely
 	self->nextthink = level.time + FRAMETIME;
-	floater_timer++;
+	self->nade_timer++;
 
-	if(floater_timer > 10) { //explode
+	if(self->nade_timer > 10) { //explode
 		T_RadiusDamage(self, self->owner, self->radius_dmg, self->enemy, self->dmg_radius, MOD_R_SPLASH, 2);
 
 		gi.WriteByte (svc_temp_entity);
@@ -1207,6 +1496,7 @@ void floater_think (edict_t *self)
 	}
 
 }
+void prox_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point);
 void prox_think (edict_t *self)
 {
 	edict_t	*ent;
@@ -1222,8 +1512,14 @@ void prox_think (edict_t *self)
 
 		if (!ent->takedamage)
 			continue;
+		
+		if (ent->die == prox_die)
+			continue;
+		
+		//avoid infinite recursion
+		self->takedamage = DAMAGE_NO;
 
-		T_RadiusDamage(self, self->owner, self->radius_dmg, self->enemy, self->dmg_radius, MOD_R_SPLASH, -1);
+		T_RadiusDamage(self, self->owner, self->radius_dmg, NULL, self->dmg_radius, MOD_R_SPLASH, -1);
 		self->owner->client->resp.weapon_hits[2]++;
 
 		gi.WriteByte (svc_temp_entity);
@@ -1237,11 +1533,16 @@ void prox_think (edict_t *self)
 
 	self->s.frame = (self->s.frame + 1) % 23; //pulse grotesquely
 	self->nextthink = level.time + FRAMETIME;
-	prox_timer++;
+	self->nade_timer++;
 
-	if(prox_timer > 300)
+	if(self->nade_timer > 300)
 	{	//explode
-		T_RadiusDamage(self, self->owner, self->radius_dmg, self->enemy, self->dmg_radius, MOD_R_SPLASH, 2);
+		
+		//avoid infinite recursion
+		self->takedamage = DAMAGE_NO;
+		
+		T_RadiusDamage(self, self->owner, self->radius_dmg, NULL, self->dmg_radius, MOD_R_SPLASH, 2);
+
 
 		gi.WriteByte (svc_temp_entity);
 		gi.WriteByte (TE_BFG_BIGEXPLOSION);
@@ -1252,7 +1553,7 @@ void prox_think (edict_t *self)
 	}
 
 }
-void floater_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
+void smartgrenade_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
 {
 	if (other == ent->owner)
 		return;
@@ -1268,8 +1569,23 @@ void floater_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *s
 	return;
 
 }
+void prox_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
+{
+	//avoid infinite recursion
+	self->takedamage = DAMAGE_NO;
+	
+	//explode
+	T_RadiusDamage(self, self->owner, self->radius_dmg, NULL, self->dmg_radius, MOD_R_SPLASH, 2);
 
-void fire_floater (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int speed, float damage_radius, int radius_damage, float timer)
+	gi.WriteByte (svc_temp_entity);
+	gi.WriteByte (TE_BFG_BIGEXPLOSION);
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
+
+	G_FreeEdict (self);
+}
+
+void fire_smartgrenade (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int speed, float damage_radius, int radius_damage, float timer)
 {
 	edict_t	*floater;
     vec3_t	dir, forward, right, up;
@@ -1294,16 +1610,15 @@ void fire_floater (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int s
 	VectorClear (floater->maxs);
 	floater->s.modelindex = gi.modelindex ("models/objects/electroball/tris.md2");
 	floater->owner = self;
-	floater->touch = floater_touch;
+	floater->touch = smartgrenade_touch;
 	floater->nextthink = level.time + .1;
-    floater->think = floater_think;
+    floater->think = smartgrenade_think;
     floater->dmg = damage;
 	floater->radius_dmg = radius_damage;
 	floater->dmg_radius = damage_radius;
 	floater->s.sound = gi.soundindex ("weapons/electroball.wav");
 	floater->classname = "grenade";
-	floater_timer = 0;
-
+	floater->nade_timer = 0;
 
 	gi.linkentity (floater);
 }
@@ -1330,14 +1645,17 @@ void fire_prox (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int spee
 	VectorClear (prox->maxs);
 	prox->s.modelindex = gi.modelindex ("models/objects/electroball/tris.md2");
 	prox->owner = self;
-	prox->touch = floater_touch;
+	prox->touch = smartgrenade_touch;
 	prox->nextthink = level.time + .1;
     prox->think = prox_think;
     prox->dmg = damage;
 	prox->radius_dmg = radius_damage;
 	prox->dmg_radius = damage_radius;
 	prox->classname = "mine";
-	prox_timer = 0;
+	prox->takedamage = DAMAGE_YES;
+	prox->health = 20;
+	prox->die = prox_die;
+	prox->nade_timer = 0;
 
 	gi.linkentity (prox);
 }
@@ -1574,6 +1892,10 @@ void fire_flamethrower(edict_t *self, vec3_t start, vec3_t dir, int damage, int
 	VectorClear (flame->mins);
 	VectorClear (flame->maxs);
 	flame->s.modelindex = gi.modelindex ("models/objects/fireball/tris.md2");
+	// All we care about is the effects. However, old clients will refuse to 
+	// draw them unless the modelindex is set. To work around this, newer
+	// clients have an RF_NODRAW flag.
+	flame->s.renderfx |= RF_NODRAW;
 	flame->owner = self;
 	flame->touch = flame_touch;
 	flame->nextthink = level.time + 500/speed;
@@ -1783,3 +2105,185 @@ void fire_violator(edict_t *self, vec3_t start, vec3_t aimdir, int damage, int k
 	if ( g_antilag->integer)
 		G_UndoTimeShiftFor( self );
 }
+
+//Tactical - bombs
+
+void tactical_bomb_think (edict_t *self)
+{	
+	self->nextthink = level.time + FRAMETIME;
+
+	if(self->armed)
+	{		
+		self->s.frame++;
+		if(self->s.frame > 10)
+			self->s.frame = 10;
+		self->nade_timer++; //this should only start after armed
+	}
+	else //in cases of being disarmed
+	{
+		self->s.frame--;
+		if(self->s.frame < 0)
+			self->s.frame = 0;
+		self->nade_timer = 0;
+	}
+
+	if(self->nade_timer > 100) //10 seconds
+	{	//explode
+		
+		//avoid infinite recursion
+		self->takedamage = DAMAGE_NO;
+		
+		T_RadiusDamage(self, self->owner, self->radius_dmg, NULL, self->dmg_radius, MOD_R_SPLASH, 0);
+
+		//just send this bad boy to whole level, make it sound BIG
+		gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "world/explosion1.wav" ), 1, ATTN_NONE, 0 );
+
+		gi.WriteByte (svc_temp_entity);
+		if(self->classname == "abomb")
+			gi.WriteByte (TE_BFG_BIGEXPLOSION);
+		else
+			gi.WriteByte (TE_ROCKET_EXPLOSION); //might want different, massive effect here
+		gi.WritePosition (self->s.origin);
+		gi.multicast (self->s.origin, MULTICAST_PHS);
+
+		G_FreeEdict (self);
+	}
+
+}
+void abomb_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
+{
+	if (other == ent->owner)
+		return;
+
+	//detonator will set arming
+
+	if (surf && (surf->flags & SURF_SKY))
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	//do we just want players with detonators to just touch them?  Touch for now...maybe actually fire one off in future?
+	if(other->has_detonator && other->ctype == 0)
+	{
+		
+		ent->armed = true;
+		other->has_detonator = false; //only get one
+		safe_bprintf(PRINT_HIGH, "Alien bomb is armed!\n"); 
+		gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "weapons/adetonatorup.wav" ), 1, ATTN_NONE, 0 );
+	}
+	else if((other->has_minderaser || other->has_vaporizor) && ent->armed)
+	{
+		ent->armed = false;
+		safe_bprintf(PRINT_HIGH, "Alien bomb is disarmed!\n");
+		gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "weapons/adetonatordown.wav" ), 1, ATTN_NONE, 0 );
+	}
+
+	return;
+}
+void hbomb_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf)
+{
+	if (other == ent->owner)
+		return;
+
+	//detonator will set arming
+
+	if (surf && (surf->flags & SURF_SKY))
+	{
+		G_FreeEdict (ent);
+		return;
+	}
+
+	//do we just want players with detonators to just touch them?  Touch for now...maybe actually fire one off in future?
+	if(other->has_detonator && other->ctype == 1)
+	{
+		ent->armed = true;
+		other->has_detonator = false; //only get one
+		safe_bprintf(PRINT_HIGH, "Bomb is armed!\n"); 
+		gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "weapons/hdetonatorup.wav" ), 1, ATTN_NONE, 0 );
+	}
+	else if((other->has_minderaser || other->has_vaporizor) && ent->armed)
+	{
+		ent->armed = false;
+		safe_bprintf(PRINT_HIGH, "Bomb is disarmed!\n");
+		gi.sound( &g_edicts[1], CHAN_AUTO, gi.soundindex( "weapons/hdetonatordown.wav" ), 1, ATTN_NONE, 0 );
+	}
+
+	return;
+}
+void bomb_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damage, vec3_t point)
+{
+	float div;
+
+	//blowing up an armed bomb is still pretty bad, but it would be smarter to get it prior to being armed.
+	if(self->armed)
+		div = 2;
+	else
+		div = 10;
+
+	//avoid infinite recursion
+	self->takedamage = DAMAGE_NO;
+	
+	//explode - much smaller explosion.
+	T_RadiusDamage(self, self->owner, self->radius_dmg/div, NULL, self->dmg_radius/div, MOD_R_SPLASH, 0);
+
+	gi.WriteByte (svc_temp_entity);
+	if(self->classname == "abomb")
+		gi.WriteByte (TE_BFG_BIGEXPLOSION);
+	else
+		gi.WriteByte (TE_ROCKET_EXPLOSION);
+	gi.WritePosition (self->s.origin);
+	gi.multicast (self->s.origin, MULTICAST_PHS);
+
+	G_FreeEdict (self);
+}
+void fire_tacticalbomb (edict_t *self, vec3_t start, vec3_t aimdir, int speed)
+{
+	edict_t	*bomb;
+    vec3_t	dir, forward, right, up;
+	float *v;
+
+	vectoangles(aimdir, dir);
+	AngleVectors(dir, forward, right, up);
+
+	bomb = G_Spawn();
+	VectorCopy (start, bomb->s.origin);
+	VectorScale (aimdir, speed, bomb->velocity);
+	VectorMA (bomb->velocity, 20 + crandom() * 10.0, up, bomb->velocity);
+	VectorMA (bomb->velocity, crandom() * 10.0, right, bomb->velocity);
+	VectorSet (bomb->avelocity, 20, 20, 20);
+	bomb->movetype = MOVETYPE_BOUNCE;
+	bomb->clipmask = MASK_SHOT;
+	bomb->solid = SOLID_BBOX;
+	v = tv(-8,-8,0);
+	VectorCopy (v, bomb->mins);
+	v = tv(8,8,16);
+	VectorCopy (v, bomb->maxs);
+	if(self->ctype)
+	{
+		bomb->s.modelindex = gi.modelindex ("models/tactical/human_bomb.iqm"); 
+		bomb->touch = hbomb_touch;
+		bomb->classname = "hbomb";
+	}
+	else
+	{
+		bomb->s.modelindex = gi.modelindex ("models/tactical/alien_bomb.iqm");
+		bomb->touch = abomb_touch;
+		bomb->classname = "abomb";
+	}
+	bomb->owner = self;
+	bomb->think = tactical_bomb_think; 
+	bomb->nextthink = level.time + .1;    
+    bomb->dmg = 1000; //insane amount of damage power
+	bomb->radius_dmg = 1000;
+	bomb->dmg_radius = 512;	
+	bomb->takedamage = DAMAGE_YES;
+	bomb->health = 500; //make it somewhat hard to destroy
+	bomb->die = bomb_die; 
+	bomb->armed = false;
+	bomb->nade_timer = 0;
+
+	gi.linkentity (bomb);
+}
+
+#endif
