@@ -90,10 +90,18 @@ struct edict_s
 	solid_t		solid;
 	int			clipmask;
 	edict_t		*owner;
+	
+	int			redirect_number;	//for ghost mode
+	
+	// must be accessible to server code for collision detection
+	int			dmteam;
+	int			teamset;
+#define RED_TEAM				0
+#define BLUE_TEAM				1
+#define NO_TEAM					2
 
 	// the game dll can add anything it wants after
 	// this point in the structure
-	edict_t     *orb;
 };
 
 #endif		// GAME_INCLUDE
@@ -126,6 +134,11 @@ typedef struct
 	int		(*modelindex) (char *name);
 	int		(*soundindex) (char *name);
 	int		(*imageindex) (char *name);
+	
+	// for checking if assets are already loaded - return 0 if they are not
+	int		(*checkmodelindex) (char *name);
+	int		(*checksoundindex) (char *name);
+	int		(*checkimageindex) (char *name);
 
 	void	(*setmodel) (edict_t *ent, char *name);
 
@@ -165,8 +178,9 @@ typedef struct
 
 	// console variable interaction
 	cvar_t	*(*cvar) (char *var_name, char *value, int flags);
-	cvar_t	*(*cvar_set) (char *var_name, char *value);
+	void	(*cvar_set) (char *var_name, char *value);
 	cvar_t	*(*cvar_forceset) (char *var_name, char *value);
+	void	(*cvar_describe) (cvar_t *var, char *description_string);
 
 	// ClientCommand and ServerCommand parameter access
 	int		(*argc) (void);
@@ -177,7 +191,7 @@ typedef struct
 	// for map changing, etc
 	void	(*AddCommandString) (char *text);
 
-	void	(*DebugGraph) (float value, int color);
+	void	(*DebugGraph) (float value, const float color[]);
 
 	int		(*Sys_Milliseconds) (void);
 
@@ -214,6 +228,7 @@ typedef struct
 	int			(*ACESP_FindBotNum) (void);
 
 	void		(*RunFrame) (void);
+	void		(*ForceExitIntermission) (void); 
 
 	// ServerCommand will be called when an "sv <command>" command is issued on the
 	// server console.
