@@ -36,7 +36,7 @@ INTERMISSION
 
 void MoveClientToIntermission (edict_t *ent)
 {
-	if (deathmatch->value)
+	if (deathmatch->integer)
 		ent->client->showscores = true;
 	VectorCopy (level.intermission_origin, ent->s.origin);
 	ent->client->ps.pmove.origin[0] = level.intermission_origin[0]*8;
@@ -44,6 +44,11 @@ void MoveClientToIntermission (edict_t *ent)
 	ent->client->ps.pmove.origin[2] = level.intermission_origin[2]*8;
 	VectorCopy (level.intermission_angle, ent->client->ps.viewangles);
 
+	// 2011-05 stop spectator from drifting,
+	//  and, possibly, other intermission bogus movement.
+	VectorClear( ent->velocity );
+	VectorClear( ent->avelocity);
+
 	ent->client->ps.pmove.pm_type = PM_FREEZE;
 	ent->client->ps.gunindex = 0;
 	ent->client->ps.blend[3] = 0;
@@ -62,19 +67,16 @@ void MoveClientToIntermission (edict_t *ent)
 	ent->s.modelindex = 0;
 	ent->s.modelindex2 = 0;
 	ent->s.modelindex3 = 0;
-	ent->s.modelindex = 0;
+	ent->s.modelindex4 = 0;
 	ent->s.effects = 0;
 	ent->s.sound = 0;
 	ent->solid = SOLID_NOT;
 
 	// add the layout
 
-	if (deathmatch->value)
+	if (deathmatch->integer)
 	{
-		if(g_mapvote->value)
-			DeathmatchScoreboardMessage (ent, NULL, true);
-		else
-			DeathmatchScoreboardMessage (ent, NULL, false);
+		DeathmatchScoreboardMessage (ent, NULL, g_mapvote->integer);
 		gi.unicast (ent, true);
 	}
 
@@ -91,15 +93,9 @@ void PlaceWinnerOnVictoryPad(edict_t *winner, int offset)
 	edict_t *chasecam;
 	gclient_t *cl;
 	vec3_t forward, right, movedir, origin;
-	int zoffset; //for moving dead players to the right place
 
-	if(winner->deadflag == DEAD_DEAD)
-		zoffset = -40;
-	else  {
-		zoffset = 0;
-		if(winner->in_vehicle)
-			Reset_player(winner);
-	}
+	if(winner->in_vehicle)
+		Reset_player(winner);
 
 	VectorCopy (level.intermission_angle, winner->s.angles);
 
@@ -114,7 +110,10 @@ void PlaceWinnerOnVictoryPad(edict_t *winner, int offset)
 	winner->client->ps.pmove.origin[1] = winner->s.origin[1];
 	winner->client->ps.pmove.origin[2] = winner->s.origin[2];
 
-	if (deathmatch->value)
+	VectorClear( winner->velocity );
+	VectorClear( winner->avelocity );
+
+	if (deathmatch->integer)
 		winner->client->showscores = true;
 
 	winner->client->ps.gunindex = 0;
@@ -139,12 +138,9 @@ void PlaceWinnerOnVictoryPad(edict_t *winner, int offset)
 
 	// add the layout
 
-	if (deathmatch->value)
+	if (deathmatch->integer)
 	{
-		if(g_mapvote->value)
-			DeathmatchScoreboardMessage (winner, NULL, true);
-		else
-			DeathmatchScoreboardMessage (winner, NULL, false);
+		DeathmatchScoreboardMessage (winner, NULL, g_mapvote->integer);
 		gi.unicast (winner, true);
 	}
 
@@ -182,8 +178,6 @@ void PlaceWinnerOnVictoryPad(edict_t *winner, int offset)
 	//now we want to allow the winners to actually see themselves on the podium, creating a
 	//simple chasecam
 
-	winner->s.origin[2] += zoffset;
-
 	if(winner->is_bot) { //who cares if bots can not see themselves!
 		winner->takedamage = DAMAGE_NO; //so they stop burning and suiciding
 		gi.linkentity(winner); //link because we changed position!
@@ -264,17 +258,34 @@ void BeginIntermission (edict_t *targ)
 
 	game.autosaved = false;
 
-	// respawn any dead clients
-	for (i=0 ; i<g_maxclients->value ; i++)
+	for (i=0 ; i<g_maxclients->integer ; i++)
 	{
 		client = g_edicts + 1 + i;
 		if (!client->inuse)
 			continue;
-		if (client->health <= 0) {
-			respawn(client);
-			client->deadflag = DEAD_DEAD; //so we can know if he's dead for placement offsetting
+		// remove from vehicle.
+		//  note: vehicle hud is client-side, so may remain in view
+		if (client->in_vehicle)
+		{
+			Reset_player( client );
 		}
-		if(!client->is_bot && g_mapvote->value)
+		// respawn the dead
+		if (client->health <= 0)
+		{
+			respawn(client);
+		}
+		client->s.frame = 0;
+		// disconnect spectators from target to prevent point-of-view errors
+		if ( client->client && client->client->resp.spectator )
+		{
+			if ( client->client->chase_target != NULL )
+			{
+				client->client->chase_target = NULL;
+				client->client->ps.pmove.pm_flags &= ~PMF_NO_PREDICTION;
+			}
+		}
+
+		if(!client->is_bot && g_mapvote->integer)
 			safe_centerprintf(client, "Use F1-F4 to vote for next map!");
 	}
 
@@ -303,6 +314,10 @@ void BeginIntermission (edict_t *targ)
 				ent = G_Find (ent, FOFS(classname), "info_player_intermission");
 		}
 	}
+	if ( ent == NULL )
+	{ /* can occur when running a demo .dm2 file */
+		return;
+	}
 
 	VectorCopy (ent->s.origin, level.intermission_origin);
 	VectorCopy (ent->s.angles, level.intermission_angle);
@@ -368,7 +383,7 @@ void BeginIntermission (edict_t *targ)
 		secondrunnerup = g_edicts;
 
 	// move all clients but the winners to the intermission point
-	for (i=0 ; i<g_maxclients->value ; i++)
+	for (i=0 ; i<g_maxclients->integer ; i++)
 	{
 		client = g_edicts + 1 + i;
 		if (!client->inuse)
@@ -377,21 +392,36 @@ void BeginIntermission (edict_t *targ)
 			MoveClientToIntermission (client);
 	}
 
-	if ((dmflags->integer & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) //team stuff
+	if ((dmflags->integer & DF_SKINTEAMS) || ctf->integer || tca->integer || cp->integer) //team stuff
 	{
-		if(blue_team_score > red_team_score) {
-			if(ctf->value || tca->value || cp->value)
+		if ( blue_team_score > red_team_score )
+		{
+			if(ctf->integer || tca->integer || cp->integer)
 				gi.sound (client, CHAN_AUTO, gi.soundindex("misc/blue_wins_ctf.wav"), 1, ATTN_NONE, 0);
 			else
 				gi.sound (client, CHAN_AUTO, gi.soundindex("misc/blue_wins.wav"), 1, ATTN_NONE, 0);
 		}
-		else {
-			if(ctf->value || tca->value || cp->value)
+		else if ( blue_team_score < red_team_score )
+		{
+			if(ctf->integer || tca->integer || cp->integer)
 				gi.sound (client, CHAN_AUTO, gi.soundindex("misc/red_wins_ctf.wav"), 1, ATTN_NONE, 0);
 
 			else
 				gi.sound (client, CHAN_AUTO, gi.soundindex("misc/red_wins.wav"), 1, ATTN_NONE, 0);
 		}
+		else
+		{
+			if ( ctf->integer )
+			{
+				gi.sound( client, CHAN_AUTO,
+						gi.soundindex("misc/game_tied_ctf.wav"), 1, ATTN_NONE, 0);
+			}
+			else
+			{
+				gi.sound( client, CHAN_AUTO,
+						gi.soundindex("misc/game_tied.wav"), 1, ATTN_NONE, 0);
+			}
+		}
 	}
 	else if ( !(dmflags->integer & DF_BOT_LEVELAD) )
 	{
@@ -416,7 +446,7 @@ int highestpos, numplayers;
 void MoveEveryoneDownQueue(void) {
 	int i, induel = 0;
 
-	for (i = 0; i < g_maxclients->value; i++) {
+	for (i = 0; i < g_maxclients->integer; i++) {
 		if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
 			//move everyone else down a notch(never less than 0)
 			if(induel > 1 && g_edicts[i+1].client->pers.queue <= 3) //houston, we have a problem
@@ -437,7 +467,7 @@ void CheckDuelWinner(void) {
 	highestpos = 0;
 	induel = 0;
 
-	for (i = 0; i < g_maxclients->value; i++) {
+	for (i = 0; i < g_maxclients->integer; i++) {
 		if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
 			if(g_edicts[i+1].client->resp.score > highscore)
 				highscore = g_edicts[i+1].client->resp.score;
@@ -449,7 +479,7 @@ void CheckDuelWinner(void) {
 	if(highestpos < numplayers)
 		highestpos = numplayers;
 
-	for (i = 0; i < g_maxclients->value; i++) {
+	for (i = 0; i < g_maxclients->integer; i++) {
 		if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
 			if((g_edicts[i+1].client->resp.score < highscore) && g_edicts[i+1].client->pers.queue < 3) {
 				g_edicts[i+1].client->pers.queue = highestpos+1; //loser, kicked to the back of the line
@@ -464,7 +494,7 @@ void CheckDuelWinner(void) {
 	//check for any screwups and correct the queue
 	while(induel < 2 && numplayers > 1) {
 		induel = 0;
-		for (i = 0; i < g_maxclients->value; i++) {
+		for (i = 0; i < g_maxclients->integer; i++) {
 			if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
 				if(g_edicts[i+1].client->pers.queue < 3 && g_edicts[i+1].client->pers.queue)
 					induel++;
@@ -480,10 +510,10 @@ void EndIntermission(void)
 	int		i;
 	edict_t	*ent;
 
-	if(g_duel->value)
+	if(g_duel->integer)
 		CheckDuelWinner();
 
-	for (i=0 ; i<g_maxclients->value; i++)
+	for (i=0 ; i<g_maxclients->integer; i++)
 	{
 		ent = g_edicts + 1 + i;
         if (!ent->inuse || ent->client->resp.spectator)
@@ -536,6 +566,9 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer, int mapvote)
 	if (ent->is_bot)
 		return;
 
+	if (g_tactical->integer)
+		return; //no scoreboard in tactical mode
+
 	if ((dmflags->integer & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) {
 		CTFScoreboardMessage (ent, killer, mapvote);
 		return;
@@ -545,7 +578,7 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer, int mapvote)
 	for (i=0 ; i<game.maxclients ; i++)
 	{
 		cl_ent = g_edicts + 1 + i;
-		if (!cl_ent->inuse || (!g_duel->value && game.clients[i].resp.spectator))
+		if (!cl_ent->inuse || (!g_duel->integer && game.clients[i].resp.spectator))
 			continue;
 
 		score = game.clients[i].resp.score;
@@ -755,10 +788,8 @@ Note that it isn't that hard to overflow the 1400 byte message limit!
 */
 void DeathmatchScoreboard (edict_t *ent)
 {
-// ACEBOT_ADD
 	if (ent->is_bot)
 		return;
-// ACEBOT_END
 
 	DeathmatchScoreboardMessage (ent, ent->enemy, false);
 	gi.unicast (ent, true);
@@ -777,7 +808,9 @@ void Cmd_Score_f (edict_t *ent)
 	ent->client->showinventory = false;
 	ent->client->showhelp = false;
 
-	if (!deathmatch->value)
+	if (!deathmatch->integer)
+		// Should we ever wish to do a singleplayer game, we could bring the 
+		// "help computer" back here. 
 		return;
 
 	if (ent->client->showscores)
@@ -790,39 +823,70 @@ void Cmd_Score_f (edict_t *ent)
 	DeathmatchScoreboard (ent);
 }
 
-/*
-==================
-Cmd_Help_f
+//=======================================================================
 
-Display the current help message
-==================
+/*
+===============
+G_SetScoreStats
+Common with G_SetStats and G_SetSpectatorStats
+===============
 */
-void Cmd_Help_f (edict_t *ent)
+
+void G_SetScoreStats (edict_t *ent)
 {
-	// this is for backwards compatability
-	if (deathmatch->value)
+	edict_t *e2;
+	int i;
+	int high_score = 0;
+	
+	// highest scorer
+	for (i = 0, e2 = g_edicts + 1; i < g_maxclients->integer; i++, e2++) 
 	{
-		Cmd_Score_f (ent);
-		return;
+		if (!e2->inuse)
+			continue;
+
+		if(e2->client->resp.score > high_score)
+			high_score = e2->client->resp.score;
+	}
+	ent->client->ps.stats[STAT_HIGHSCORE] = high_score;
+	
+	ent->client->ps.stats[STAT_SCOREBOARD] = gi.imageindex ("i_score");
+
+	//team
+	ent->client->ps.stats[STAT_REDSCORE] = red_team_score;
+	ent->client->ps.stats[STAT_BLUESCORE] = blue_team_score;
+	
+	if (tca->integer) 
+	{
+		ent->client->ps.stats[STAT_RED_MATCHES] = red_team_matches;
+		ent->client->ps.stats[STAT_BLUE_MATCHES] = blue_team_matches;
+	}
+
+	if (g_tactical->integer) //just use the weapon slots since these aren't displayed in tactical anyway
+	{
+		ent->client->ps.stats[STAT_TACTICAL_SCORE] = gi.imageindex ("i_tactical");
+
+		ent->client->ps.stats[STAT_WEAPN1] = tacticalScore.humanComputerHealth; 
+		ent->client->ps.stats[STAT_WEAPN2] = tacticalScore.humanPowerSourceHealth; 
+		ent->client->ps.stats[STAT_WEAPN3] = tacticalScore.humanAmmoDepotHealth; 
+
+		ent->client->ps.stats[STAT_WEAPN4] = tacticalScore.alienComputerHealth; 
+		ent->client->ps.stats[STAT_WEAPN5] = tacticalScore.alienPowerSourceHealth; 
+		ent->client->ps.stats[STAT_WEAPN6] = tacticalScore.alienAmmoDepotHealth; 
 	}
 }
 
-
-//=======================================================================
-
 /*
 ===============
 G_SetStats
 ===============
 */
+
 void G_SetStats (edict_t *ent)
 {
 	gitem_t		*item;
 	int			index;
-	edict_t *e2;
-	int i, j;
-	int high_score = 0;
-	gitem_t *flag1_item, *flag2_item;
+	int			i;
+	gitem_t		*flag1_item, *flag2_item;
 
 
 	flag1_item = FindItemByClassname("item_flag_red");
@@ -866,15 +930,6 @@ void G_SetStats (edict_t *ent)
         ent->client->ps.stats[STAT_ARMOR] = 0;
     }
 
-	//
-	// pickup message
-	//
-	if (level.time > ent->client->pickup_msg_time)
-	{
-		ent->client->ps.stats[STAT_PICKUP_ICON] = 0;
-		ent->client->ps.stats[STAT_PICKUP_STRING] = 0;
-	}
-
 	//
 	// timers
 	//
@@ -917,15 +972,10 @@ void G_SetStats (edict_t *ent)
 	//
 	// selected item
 	//
-	if (ent->client->pers.selected_item == -1)
-		ent->client->ps.stats[STAT_SELECTED_ICON] = 0;
-	else
-		ent->client->ps.stats[STAT_SELECTED_ICON] = gi.imageindex (itemlist[ent->client->pers.selected_item].icon);
-
 	ent->client->ps.stats[STAT_SELECTED_ITEM] = ent->client->pers.selected_item;
 
 	//ctf
-	if(ctf->value) {
+	if(ctf->integer) {
 		if (ent->client->pers.inventory[ITEM_INDEX(flag1_item)])
 			ent->client->ps.stats[STAT_FLAG_ICON] = gi.imageindex ("i_flag1");
 		else if (ent->client->pers.inventory[ITEM_INDEX(flag2_item)])
@@ -953,7 +1003,7 @@ void G_SetStats (edict_t *ent)
 	//
 	ent->client->ps.stats[STAT_LAYOUTS] = 0;
 
-	if (deathmatch->value)
+	if (deathmatch->integer)
 	{
 		if (ent->client->pers.health <= 0 || level.intermissiontime
 			|| ent->client->showscores)
@@ -974,69 +1024,53 @@ void G_SetStats (edict_t *ent)
 	//
 	ent->client->ps.stats[STAT_FRAGS] = ent->client->resp.score;
 	ent->client->ps.stats[STAT_DEATHS] = ent->client->resp.deaths;
+	
+	G_SetScoreStats (ent);
 
-	// highest scorer
-	for (i = 0, e2 = g_edicts + 1; i < g_maxclients->value; i++, e2++) {
-		if (!e2->inuse)
-			continue;
+#ifndef ALTERIA
 
-		if(e2->client->resp.score > high_score)
-			high_score = e2->client->resp.score;
-	}
-	ent->client->ps.stats[STAT_HIGHSCORE] = high_score;
-
-	//bot score info
-	ent->client->ps.botnum = ent->client->resp.botnum;
-	if(ent->client->resp.botnum) {
-		for(i = 0; i < ent->client->resp.botnum; i++) {
-			strcpy(ent->client->ps.bots[i].name, ent->client->resp.bots[i].name);
-			//tally bot scores
-			for(j = 0, e2 = g_edicts + 1; j < g_maxclients->value; j++, e2++) {
-				if(!strcmp(ent->client->resp.bots[i].name, e2->client->pers.netname))
-					ent->client->ps.bots[i].score = e2->client->resp.score;
-			}
+	if(!g_tactical->integer)
+	{
+		//weapon/ammo inventories
+		for(i = 0; i < 7; i++)
+			ent->client->ps.stats[STAT_WEAPN1+i] = 0;
+		i = 0;
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Disruptor"))] &&
+			ent->client->pers.weapon != (FindItem("Alien Disruptor"))) {
+				ent->client->ps.stats[STAT_WEAPN1] = gi.imageindex ("disruptor");
+				i++;
+		}
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Smartgun"))] &&
+			ent->client->pers.weapon != (FindItem("Alien Smartgun"))) {
+				ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("smartgun");
+				i++;
+		}
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Pulse Rifle"))] &&
+			ent->client->pers.weapon != (FindItem("Pulse Rifle"))) {
+				ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("chaingun");
+				i++;
+		}
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Flame Thrower"))] &&
+			ent->client->pers.weapon != (FindItem("Flame Thrower"))) {
+				ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("flamethrower");
+				i++;
+		}
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Rocket Launcher"))] &&
+			ent->client->pers.weapon != (FindItem("Rocket Launcher"))) {
+				ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("rocketlauncher");
+				i++;
+		}
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Disruptor"))] &&
+			ent->client->pers.weapon != (FindItem("Disruptor"))) {
+				ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("beamgun");
+				i++;
+		}
+		if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Vaporizer"))] &&
+			ent->client->pers.weapon != (FindItem("Alien Vaporizer"))) {
+				ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("vaporizor");
 		}
 	}
-	//end bot score info
-
-	//weapon/ammo inventories
-	for(i = 0; i < 7; i++)
-		ent->client->ps.stats[STAT_WEAPN1+i] = 0;
-	i = 0;
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Disruptor"))] &&
-		ent->client->pers.weapon != (FindItem("Alien Disruptor"))) {
-			ent->client->ps.stats[STAT_WEAPN1] = gi.imageindex ("disruptor");
-			i++;
-	}
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Smartgun"))] &&
-		ent->client->pers.weapon != (FindItem("Alien Smartgun"))) {
-			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("smartgun");
-			i++;
-	}
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Pulse Rifle"))] &&
-		ent->client->pers.weapon != (FindItem("Pulse Rifle"))) {
-			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("chaingun");
-			i++;
-	}
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Flame Thrower"))] &&
-		ent->client->pers.weapon != (FindItem("Flame Thrower"))) {
-			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("flamethrower");
-			i++;
-	}
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Rocket Launcher"))] &&
-		ent->client->pers.weapon != (FindItem("Rocket Launcher"))) {
-			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("rocketlauncher");
-			i++;
-	}
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Disruptor"))] &&
-		ent->client->pers.weapon != (FindItem("Disruptor"))) {
-			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("beamgun");
-			i++;
-	}
-	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Vaporizer"))] &&
-		ent->client->pers.weapon != (FindItem("Alien Vaporizer"))) {
-			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("vaporizor");
-	}
+#endif
 
 	//
 	// current weapon
@@ -1045,36 +1079,15 @@ void G_SetStats (edict_t *ent)
 		ent->client->ps.stats[STAT_HELPICON] = gi.imageindex (ent->client->pers.weapon->icon);
 	else
 		ent->client->ps.stats[STAT_HELPICON] = 0;
-
-	ent->client->ps.stats[STAT_SPECTATOR] = 0;
-
-	ent->client->ps.stats[STAT_SCOREBOARD] = gi.imageindex ("i_score");
-
-	//team
-	ent->client->ps.stats[STAT_REDSCORE] = red_team_score;
-	ent->client->ps.stats[STAT_BLUESCORE] = blue_team_score;
-
-	//spectator mode
-	ent->client->ps.stats[STAT_SPECTATOR] = 0;
-}
-
-/*
-===============
-G_CheckChaseStats
-===============
-*/
-void G_CheckChaseStats (edict_t *ent)
-{
-	int i;
-	gclient_t *cl;
-
-	for (i = 1; i <= g_maxclients->value; i++) {
-		cl = g_edicts[i].client;
-		if (!g_edicts[i].inuse || cl->chase_target != ent)
-			continue;
-		memcpy(cl->ps.stats, ent->client->ps.stats, sizeof(cl->ps.stats));
-		G_SetSpectatorStats(g_edicts + i);
-	}
+	
+	ent->client->ps.stats[STAT_FLAGS] &= ~STAT_FLAGS_CROSSHAIRPOSITION;
+	if	(	ent->client->pers.weapon == FindItem ("Alien Disruptor") || 
+			ent->client->pers.weapon == FindItem ("Violator") || 
+			ent->client->pers.weapon == FindItem ("Disruptor") ||
+			ent->client->pers.weapon == FindItem ("Pulse Rifle"))
+		ent->client->ps.stats[STAT_FLAGS] |= STAT_FLAGS_CROSSHAIRCENTER;
+	else if (ent->client->pers.weapon == FindItem ("Rocket Launcher"))
+		ent->client->ps.stats[STAT_FLAGS] |= STAT_FLAGS_CROSSHAIRPOS2;
 }
 
 /*
@@ -1086,22 +1099,59 @@ void G_SetSpectatorStats (edict_t *ent)
 {
 	gclient_t *cl = ent->client;
 
-	if (!cl->chase_target)
-		G_SetStats (ent);
-
-	cl->ps.stats[STAT_SPECTATOR] = 1;
-
-	// layouts are independant in spectator
-	cl->ps.stats[STAT_LAYOUTS] = 0;
-	if (cl->pers.health <= 0 || level.intermissiontime || cl->showscores)
-		cl->ps.stats[STAT_LAYOUTS] |= 1;
-	if (cl->showinventory && cl->pers.health > 0)
-		cl->ps.stats[STAT_LAYOUTS] |= 2;
+	if (ent->client->showscores)
+		cl->ps.stats[STAT_LAYOUTS] = 1;
+	else
+		cl->ps.stats[STAT_LAYOUTS] = 0;
 
 	if (cl->chase_target && cl->chase_target->inuse)
+	{
+		// XXX: is this code ever active?
 		cl->ps.stats[STAT_CHASE] = CS_PLAYERSKINS +
 			(cl->chase_target - g_edicts) - 1;
+	}
 	else
+	{
 		cl->ps.stats[STAT_CHASE] = 0;
+		G_SetScoreStats (ent);
+	}
 }
 
+/**
+ * @brief  Update stats for scoreboard display on
+ *
+ * @detail  Spectator issues complicate things. Spectator sees chase target's
+ *          HUD information. But spectator needs to show zeroed scoring
+ *          data in status responses, player lists, etc. Real player clients
+ *          contain bot scoring info, which is updated by
+ *          acebot_spawn.c::ACESP_UpdateBots(), called for every server frame
+ *          and would be redundant if updated here. The server expects to see
+ *          the bot info in the first client, regardless of it being in use
+ *          or being a spectator.
+ *
+ * @param ent  entity for player or bot
+ */
+void G_UpdateStats( edict_t *ent )
+{
+	gclient_t *gcl = ent->client;
+
+	if ( gcl->resp.spectator && !ent->is_bot )
+	{
+		if ( gcl->chase_target != NULL && level.intermissiontime <= 0.0f )
+		{ // clone chase target's stats for hud
+			memcpy( gcl->ps.stats, gcl->chase_target->client->ps.stats,
+					sizeof( gcl->ps.stats ) );
+		}
+		else
+		{
+			memset( gcl->ps.stats, 0, sizeof(gcl->ps.stats));
+			G_SetSpectatorStats( ent);
+		}
+		gcl->ps.stats[STAT_SPECTATOR] = 1;
+	}
+	else
+	{
+		ent->client->ps.stats[STAT_SPECTATOR] = 0;
+		G_SetStats( ent );
+	}
+}
