@@ -85,7 +85,7 @@ void P_DamageFeedback (edict_t *player)
 	client = player->client;
 
 	// flash the backgrounds behind the status numbers
-	client->ps.stats[STAT_FLASHES] = 0;
+	client->ps.stats[STAT_FLASHES] &= ~(1|2);
 	if (client->damage_blood)
 		client->ps.stats[STAT_FLASHES] |= 1;
 	if (client->damage_armor && !(player->flags & FL_GODMODE) && (client->invincible_framenum <= level.framenum))
@@ -436,6 +436,8 @@ void ResetWeaponModel (edict_t *ent)
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_rlauncher.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_blast/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_blaster.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_alienblast/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_alienblaster.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_bfg/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_bfg.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_rail/tris.md2"))
@@ -448,6 +450,8 @@ void ResetWeaponModel (edict_t *ent)
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_hyperblaster.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_chain/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_chaingun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_minderaser/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_minderaser.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "vehicles/deathball/v_wep.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_machinegun.md2");
 
@@ -833,6 +837,9 @@ TeamEffects
 */
 void TeamEffects(edict_t *player)
 {
+	if(g_tactical->integer)
+		return;
+
 	if(player->dmteam == RED_TEAM)
 		player->s.effects |= EF_TEAM1;
 	else if(player->dmteam == BLUE_TEAM)
@@ -859,7 +866,7 @@ void G_SetClientEffects (edict_t *ent)
 
 	if ((dmflags->integer & DF_SKINTEAMS) || ctf->value || tca->value || cp->value )
 		TeamEffects(ent);
-	else if (g_dmlights->integer)
+	else if (g_dmlights->integer && !g_tactical->integer)
 	    ent->s.effects |= EF_TEAM2;
 
 	if (ent->client->quad_framenum > level.framenum)
@@ -887,7 +894,7 @@ void G_SetClientEffects (edict_t *ent)
 		ent->s.effects |= EF_PENT;
 
 	if(ent->client->kill_streak >= 8)
-		ent->s.effects |= EF_FLIES;
+		ent->s.effects |= EF_BUBBLES;
 
 	//invisibility
 	if(ent->client->invis_framenum > level.framenum) {
@@ -1094,6 +1101,21 @@ void ClientEndServerFrame (edict_t *ent)
 	int		i;
 	current_player = ent;
 	current_client = ent->client;
+	
+	if (ent->client->chase_target != NULL && ent->client->resp.spectator != 0)
+	{
+		ent->redirect_number = ent->client->chase_target->s.number;
+		// This isn't needed to make spectators follow the chase target 
+		// around; the server code does that already. This is so that when you
+		// un-follow someone, you're dropped off right where they were. Due to
+		// the weird way bots are done, there's no easy way to copy view
+		// angles over.
+		VectorCopy (ent->client->chase_target->s.origin, ent->s.origin);
+		ent->s.origin[2] += 24; // a little above the target
+
+	}
+	else
+		ent->redirect_number = ent->s.number;
 
 	//
 	// If the origin or velocity have changed since ClientThink(),
@@ -1115,10 +1137,9 @@ void ClientEndServerFrame (edict_t *ent)
 	//
 	if (level.intermissiontime)
 	{
-		// FIXME: add view drifting here?
 		current_client->ps.blend[3] = 0;
 		current_client->ps.fov = 90;
-		G_SetStats (ent);
+		G_UpdateStats (ent);
 		return;
 	}
 
@@ -1146,12 +1167,12 @@ void ClientEndServerFrame (edict_t *ent)
 
 	xyspeed = sqrt(ent->velocity[0]*ent->velocity[0] + ent->velocity[1]*ent->velocity[1]);
 
-	if (xyspeed < 5)
+	if (xyspeed < 5 || !ent->groundentity)
 	{
 		bobmove = 0;
 		current_client->bobtime = 0;	// start at beginning of cycle again
 	}
-	else if (ent->groundentity)
+	else
 	{	// so bobbing only cycles when on ground
 		if (xyspeed > 210)
 			bobmove = 0.25;
@@ -1188,12 +1209,7 @@ void ClientEndServerFrame (edict_t *ent)
 	// should be determined by the client
 	SV_CalcBlend (ent);
 
-	// chase cam stuff
-	if (ent->client->resp.spectator)
-		G_SetSpectatorStats(ent);
-	else
-		G_SetStats (ent);
-	G_CheckChaseStats(ent);
+	G_UpdateStats (ent);
 
 	G_SetClientEvent (ent);
 
