@@ -45,7 +45,6 @@ void P_ProjectSource (gclient_t *client, vec3_t point, vec3_t distance, vec3_t f
 	G_ProjectSource (point, _distance, forward, right, result);
 }
 
-
 /*
 ===============
 PlayerNoise
@@ -117,7 +116,6 @@ void PlayerNoise(edict_t *who, vec3_t where, int type)
 	gi.linkentity (noise);
 }
 
-
 qboolean Pickup_Weapon (edict_t *ent, edict_t *other)
 {
 	int		index;
@@ -135,6 +133,29 @@ qboolean Pickup_Weapon (edict_t *ent, edict_t *other)
 		return false; //why pick them up in these modes?
 	}
 
+	if( g_tactical->integer)
+	{
+		//certain classes can only use certain weapons
+		if(other->ctype == 0)
+		{
+			if(!strcmp(ent->classname, "weapon_rocketlauncher") || !strcmp(ent->classname, "weapon_chaingun") || !strcmp(ent->classname, "weapon_bfg")
+				|| !strcmp(ent->classname, "weapon_supershotgun"))
+				return false;
+		}
+		else if (!strcmp(ent->classname, "weapon_shotgun") || !strcmp(ent->classname, "weapon_hyperblaster") || !strcmp(ent->classname, "weapon_railgun")
+				|| !strcmp(ent->classname, "weapon_minderaser"))
+				return false;
+
+		//do not pick up a weapon if you already have one - the premise behind this is that it will give others opportunities to pick up weapons since they do not respawn
+		if(other->client->pers.inventory[ITEM_INDEX(FindItem("Alien Disruptor"))] || other->client->pers.inventory[ITEM_INDEX(FindItem("Alien Smartgun"))]
+			|| other->client->pers.inventory[ITEM_INDEX(FindItem("Rocket Launcher"))] || other->client->pers.inventory[ITEM_INDEX(FindItem("Disruptor"))]
+			|| other->client->pers.inventory[ITEM_INDEX(FindItem("Pulse Rifle"))] || other->client->pers.inventory[ITEM_INDEX(FindItem("Flame Thrower"))] )
+		{
+			safe_centerprintf(other, "Cannot pick up weapon, you already have a weapon");
+			return false;
+		}
+	}
+
 	if ( ( (dmflags->integer & DF_WEAPONS_STAY))
 		&& other->client->pers.inventory[index])
 	{
@@ -144,7 +165,6 @@ qboolean Pickup_Weapon (edict_t *ent, edict_t *other)
 
 	other->client->pers.inventory[index]++;
 
-
 	if (!(ent->spawnflags & DROPPED_ITEM) )
 	{
 		// give them some ammo with it
@@ -156,13 +176,21 @@ qboolean Pickup_Weapon (edict_t *ent, edict_t *other)
 		else
 			Add_Ammo (other, ammo, ammo->quantity, true, false);
 
-		if (! (ent->spawnflags & DROPPED_PLAYER_ITEM) )
+		//if ME, make sure original weapon gets respawned
+		if(!strcmp(ent->classname, "weapon_minderaser"))
+		{
+			if(ent->replaced_weapon != NULL)
+				SetRespawn(ent->replaced_weapon, 5);
+		}
+
+		if ( !(ent->spawnflags & DROPPED_PLAYER_ITEM) )
 		{
 			if (deathmatch->value)
 			{
 				if (dmflags->integer & DF_WEAPONS_STAY)
 					ent->flags |= FL_RESPAWN;
-				else {
+				else 
+				{
 					//weapon = FindItem (ent->item->weapon);
 					if(ent->item->weapmodel == WEAP_VAPORIZER)
 						SetRespawn (ent, 10);
@@ -175,8 +203,12 @@ qboolean Pickup_Weapon (edict_t *ent, edict_t *other)
 
 	if (other->client->pers.weapon != ent->item &&
 		(other->client->pers.inventory[index] == 1) &&
-		( !deathmatch->value || other->client->pers.weapon == FindItem("blaster") ) )
+		( !deathmatch->value || other->client->pers.weapon == FindItem("blaster") || other->client->pers.weapon == FindItem("Alien Blaster")) )
 		other->client->newweapon = ent->item;
+	
+	if (other->client->pers.lastfailedswitch == ent->item &&
+	    (level.framenum - other->client->pers.failedswitch_framenum) < 5)
+	    other->client->newweapon = ent->item;
 
 	return true;
 }
@@ -212,14 +244,8 @@ void Q2_FindFile (char *filename, FILE **file)
 			gi.dprintf("Q2_FindFile: failed fopen for read: %s", full_path );
 		}
 	}
-
 }
 
-
-
-
-
-
 /*
 ===============
 ChangeWeapon
@@ -293,12 +319,17 @@ void ChangeWeapon (edict_t *ent)
 
 	sprintf(weaponmodel, "players/%s%s", weaponame, "weapon.md2"); //default
 
+#ifdef ALTERIA
+	//add in Alteria definitions
+#else
 	if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_violator/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_violator.md2");
 	else if( !Q_strcasecmp( ent->client->pers.weapon->view_model,"models/weapons/v_rocket/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_rlauncher.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_blast/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_blaster.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_alienblast/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_alienblaster.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_bfg/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_bfg.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_rail/tris.md2"))
@@ -311,26 +342,34 @@ void ChangeWeapon (edict_t *ent)
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_hyperblaster.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_chain/tris.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_chaingun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_minderaser/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_minderaser.md2");
 	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"vehicles/deathball/v_wep.md2"))
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_machinegun.md2");
+#endif
 
 	sprintf(weaponpath, "%s", weaponmodel);
-	Q2_FindFile (weaponpath, &file); //does it really exist?
-	if(!file)
+	ent->s.modelindex2 = gi.checkmodelindex(weaponmodel);
+	
+	if (ent->s.modelindex2 == 0) // check if hasn't already been loaded
 	{
-		sprintf(weaponpath, "%s%s", weaponame, "weapon.md2"); //no w_weaps, do we have this model?
-		Q2_FindFile (weaponpath, &file);
-		if(!file) //server does not have this player model
-			sprintf(weaponmodel, "players/martianenforcer/weapon.md2");//default player(martian)
-		else
-		{ //have the model, but it has no w_weaps
-			sprintf(weaponmodel, "players/%s%s", weaponame, "weapon.md2"); //custom weapon
-			fclose(file);
+		Q2_FindFile (weaponpath, &file); //does it really exist?
+		if(!file)
+		{
+			sprintf(weaponpath, "%s%s", weaponame, "weapon.md2"); //no w_weaps, do we have this model?
+			Q2_FindFile (weaponpath, &file);
+			if(!file) //server does not have this player model
+				sprintf(weaponmodel, "players/martianenforcer/weapon.md2");//default player(martian)
+			else
+			{ //have the model, but it has no w_weaps
+				sprintf(weaponmodel, "players/%s%s", weaponame, "weapon.md2"); //custom weapon
+				fclose(file);
+			}
 		}
+		else
+			fclose(file);
+		ent->s.modelindex2 = gi.modelindex(weaponmodel);
 	}
-	else
-		fclose(file);
-	ent->s.modelindex2 = gi.modelindex(weaponmodel);
 
 	//play a sound like in Q3, except for blaster, so it doesn't do it on spawn.
 	if( Q_strcasecmp( ent->client->pers.weapon->view_model,"models/weapons/v_blast/tris.md2") )
@@ -346,9 +385,7 @@ void ChangeWeapon (edict_t *ent)
 	{
 			ent->s.frame = FRAME_pain301;
 			ent->client->anim_end = FRAME_pain304;
-
 	}
-
 }
 
 /*
@@ -400,7 +437,10 @@ void NoAmmoWeaponChange (edict_t *ent)
 		return;
 	}
 
-	ent->client->newweapon = FindItem ("blaster");
+	if(g_tactical->integer && ent->ctype == 0)
+		ent->client->newweapon = FindItem ("Alien Blaster");
+	else
+		ent->client->newweapon = FindItem ("Blaster");
 }
 
 /*
@@ -460,12 +500,16 @@ void Use_Weapon (edict_t *ent, gitem_t *item)
 		if (!ent->client->pers.inventory[ammo_index])
 		{
 			safe_cprintf (ent, PRINT_HIGH, "No %s for %s.\n", ammo_item->pickup_name, item->pickup_name);
+			ent->client->pers.lastfailedswitch = item;
+			ent->client->pers.failedswitch_framenum = level.framenum;
 			return;
 		}
 
 		if (ent->client->pers.inventory[ammo_index] < item->quantity)
 		{
 			safe_cprintf (ent, PRINT_HIGH, "Not enough %s for %s.\n", ammo_item->pickup_name, item->pickup_name);
+			ent->client->pers.lastfailedswitch = item;
+			ent->client->pers.failedswitch_framenum = level.framenum;
 			return;
 		}
 	}
@@ -474,8 +518,6 @@ void Use_Weapon (edict_t *ent, gitem_t *item)
 	ent->client->newweapon = item;
 }
 
-
-
 /*
 ================
 Drop_Weapon
@@ -513,25 +555,15 @@ A generic function to handle the basics of weapon thinking
 */
 #define FRAME_FIRE_FIRST		(FRAME_ACTIVATE_LAST + 1)
 #define FRAME_IDLE_FIRST		(FRAME_FIRE_LAST + 1)
-#define FRAME_DEACTIVATE_FIRST	(FRAME_IDLE_LAST + 1)
 
-void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST, int FRAME_IDLE_LAST, int FRAME_DEACTIVATE_LAST, int *pause_frames, int *fire_frames, void (*fire)(edict_t *ent))
+void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST, int FRAME_IDLE_LAST, int FRAME_DEACTIVATE, int *pause_frames, int *fire_frames, void (*fire)(edict_t *ent))
 {
 	int		n;
+	#define gunframe ent->client->ps.gunframe
 
 	if (ent->client->weaponstate == WEAPON_DROPPING)
 	{
-		if(excessive->value || quickweap->value || ent->client->invincible_framenum > level.framenum) {
-			ChangeWeapon (ent);
-			return;
-		}
-		else if (ent->client->ps.gunframe == FRAME_DEACTIVATE_LAST)
-		{
-			ChangeWeapon (ent);
-			return;
-		}
-
-		ent->client->ps.gunframe++;
+		ChangeWeapon (ent);
 		return;
 	}
 
@@ -539,29 +571,43 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 	{
 		if(excessive->value || quickweap->value || ent->client->invincible_framenum > level.framenum) {
 			ent->client->weaponstate = WEAPON_READY;
-			ent->client->ps.gunframe = FRAME_IDLE_FIRST;
-			return;
+			gunframe = FRAME_IDLE_FIRST;
+			goto fire_begin; //velociraptors be damned
 		}
-		else if (ent->client->ps.gunframe == FRAME_ACTIVATE_LAST-2)
+		else if ((ent->client->latched_buttons|ent->client->buttons) & (BUTTON_ATTACK|BUTTON_ATTACK2))
+		{
+			if (gunframe >= FRAME_ACTIVATE_LAST-3)
+			{
+				ent->client->weaponstate = WEAPON_READY;
+				gunframe = FRAME_IDLE_FIRST;
+				goto fire_begin; //velociraptors be damned
+			}
+		}
+		else if (gunframe == FRAME_ACTIVATE_LAST)
 		{
 			ent->client->weaponstate = WEAPON_READY;
-			ent->client->ps.gunframe = FRAME_IDLE_FIRST;
+			gunframe = FRAME_IDLE_FIRST;
 			return;
 		}
-
-		ent->client->ps.gunframe++;
+		
+		gunframe++;
 		return;
 	}
 
-	if ((ent->client->newweapon) && (ent->client->weaponstate != WEAPON_FIRING))
+	if ((ent->client->newweapon) && (ent->client->weaponstate != WEAPON_FIRING || gunframe == FRAME_FIRE_FIRST || gunframe == FRAME_FIRE_LAST))
 	{
+		if(excessive->value || quickweap->value || ent->client->invincible_framenum > level.framenum) {
+			ChangeWeapon (ent);
+			return;
+		}
 		ent->client->weaponstate = WEAPON_DROPPING;
-		ent->client->ps.gunframe = FRAME_DEACTIVATE_FIRST+2;
+		gunframe = FRAME_DEACTIVATE-1;
 		return;
 	}
 
 	if (ent->client->weaponstate == WEAPON_READY)
 	{
+fire_begin:
 		if ( ((ent->client->latched_buttons|ent->client->buttons) & BUTTON_ATTACK) )
 		{
 			ent->client->spawnprotected = false;
@@ -570,7 +616,7 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 			if ((!ent->client->ammo_index) ||
 				( ent->client->pers.inventory[ent->client->ammo_index] >= ent->client->pers.weapon->quantity))
 			{
-				ent->client->ps.gunframe = FRAME_FIRE_FIRST;
+				gunframe = FRAME_FIRE_FIRST;
 				ent->client->weaponstate = WEAPON_FIRING;
 
 				// start the animation
@@ -586,6 +632,7 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 						ent->s.frame = FRAME_attack1-1;
 						ent->client->anim_end = FRAME_attack8;
 					}
+
 				}
 			}
 			else
@@ -607,7 +654,7 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 			if ((!ent->client->ammo_index) ||
 				( ent->client->pers.inventory[ent->client->ammo_index] >= ent->client->pers.weapon->quantity))
 			{
-				ent->client->ps.gunframe = FRAME_FIRE_FIRST;
+				gunframe = FRAME_FIRE_FIRST;
 				ent->client->weaponstate = WEAPON_FIRING;
 
 				// start the animation
@@ -637,9 +684,11 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 		}
 		else
 		{
-			if (ent->client->ps.gunframe == FRAME_IDLE_LAST)
+			if (gunframe == FRAME_IDLE_LAST)
 			{
-				ent->client->ps.gunframe = FRAME_IDLE_FIRST;
+#ifdef ALTERIA
+				gunframe = FRAME_IDLE_FIRST; //we can return this to alien arena if we ever decide to have idle animations again for weapons
+#endif
 				return;
 			}
 
@@ -647,7 +696,7 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 			{
 				for (n = 0; pause_frames[n]; n++)
 				{
-					if (ent->client->ps.gunframe == pause_frames[n])
+					if (gunframe == pause_frames[n])
 					{
 						if (rand()&15)
 							return;
@@ -655,7 +704,7 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 				}
 			}
 
-			ent->client->ps.gunframe++;
+			gunframe++;
 			return;
 		}
 	}
@@ -664,7 +713,7 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 	{
 		for (n = 0; fire_frames[n]; n++)
 		{
-			if (ent->client->ps.gunframe == fire_frames[n])
+			if (gunframe == fire_frames[n])
 			{
 				if (ent->client->quad_framenum > level.framenum)
 					gi.sound(ent, CHAN_ITEM, gi.soundindex("items/damage3.wav"), 1, ATTN_NORM, 0);
@@ -675,16 +724,69 @@ void Weapon_Generic (edict_t *ent, int FRAME_ACTIVATE_LAST, int FRAME_FIRE_LAST,
 		}
 
 		if (!fire_frames[n])
-			ent->client->ps.gunframe++;
+			gunframe++;
 
-		if (ent->client->ps.gunframe == FRAME_IDLE_FIRST+1)
+		if (gunframe == FRAME_IDLE_FIRST+1)
 			ent->client->weaponstate = WEAPON_READY;
 	}
+	#undef gunframe
 }
 
-void weapon_plasma_fire (edict_t *ent)
+#ifdef ALTERIA
+	//add Alteria weapons here
+/*
+======================================================================
+
+VIOALATOR
+
+======================================================================
+*/
+
+void punch_fire (edict_t *ent)
 {
-	vec3_t		start;
+	vec3_t	offset, start;
+	vec3_t	forward, right;
+	int		damage;
+
+	damage = 30; 
+	
+	AngleVectors (ent->client->v_angle, forward, right, NULL);
+
+	VectorScale (forward, 2, ent->client->kick_origin);
+	ent->client->kick_angles[0] = -1;
+
+	VectorSet(offset, 4, 4, ent->viewheight-2);
+	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
+
+	fire_punch (ent, start, forward, damage);
+
+	ent->client->ps.gunframe++;
+
+	PlayerNoise(ent, start, PNOISE_WEAPON);
+
+	//punch does not use ammo
+}
+
+void Weapon_Punch (edict_t *ent)
+{
+	static int	pause_frames[]	= {52, 0};
+	static int	fire_frames[]	= {9, 0};
+	
+	Weapon_Generic (ent, 5, 14, 52, 56, pause_frames, fire_frames, punch_fire);
+}
+
+void Weapon_Wizard_Punch (edict_t *ent)
+{
+	static int	pause_frames[]	= {52, 0};
+	static int	fire_frames[]	= {9, 0};
+	
+	Weapon_Generic (ent, 5, 14, 52, 56, pause_frames, fire_frames, punch_fire);
+}
+
+#else
+void weapon_disruptor_fire (edict_t *ent)
+{
+	vec3_t		start, muzzle;
 	vec3_t		forward, right;
 	vec3_t		offset;
 
@@ -726,8 +828,10 @@ void weapon_plasma_fire (edict_t *ent)
 	ent->client->kick_angles[0] = -3;
 
 	VectorSet(offset, 32, 5,  ent->viewheight-5);
+	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, muzzle);
+	VectorSet(offset, 32, 0, ent->viewheight);
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
-	fire_plasma (ent, start, forward, damage*damage_buildup, kick);
+	fire_disruptor (ent, start, muzzle, forward, damage*damage_buildup, kick);
 
 	//alt fire - reset some things
 	ent->client->ps.fov = atoi(Info_ValueForKey(ent->client->pers.userinfo, "fov")); //alt fire - reset the fov;
@@ -758,12 +862,12 @@ void Weapon_Disruptor (edict_t *ent)
 	static int	excessive_fire_frames[] = {5,7,9,11,0};
 
 	if(excessive->value || ent->client->invincible_framenum > level.framenum)
-		Weapon_Generic (ent, 4, 12, 42, 46, pause_frames, excessive_fire_frames, weapon_plasma_fire);
+		Weapon_Generic (ent, 4, 12, 42, 46, pause_frames, excessive_fire_frames, weapon_disruptor_fire);
 	else
-		Weapon_Generic (ent, 4, 12, 42, 46, pause_frames, fire_frames, weapon_plasma_fire);
+		Weapon_Generic (ent, 4, 12, 42, 46, pause_frames, fire_frames, weapon_disruptor_fire);
 }
 
-void weapon_energy_field_fire (edict_t *ent)
+void weapon_vaporizer_fire (edict_t *ent)
 {
 	vec3_t		start;
 	vec3_t		forward, right;
@@ -827,7 +931,7 @@ void weapon_energy_field_fire (edict_t *ent)
 
 		}
 		else {
-				fire_energy_field (ent, start, forward, damage, kick);
+				fire_vaporizer (ent, start, forward, damage, kick);
 				if (! ( dmflags->integer & DF_INFINITE_AMMO ) )
 					ent->client->pers.inventory[ent->client->ammo_index]= ent->client->pers.inventory[ent->client->ammo_index]-2;
 		}
@@ -858,7 +962,7 @@ void Weapon_Vaporizer (edict_t *ent)
 	static int	pause_frames[]	= {48, 0};
 	static int	fire_frames[]	= {6, 7, 12, 13, 0};
 
-	Weapon_Generic (ent, 5, 18, 48, 52, pause_frames, fire_frames, weapon_energy_field_fire);
+	Weapon_Generic (ent, 5, 18, 48, 52, pause_frames, fire_frames, weapon_vaporizer_fire);
 }
 
 /*
@@ -904,7 +1008,7 @@ void weapon_flamethrower_fire (edict_t *ent)
 		return;
 	}
 
-	if (!(ent->client->buttons & BUTTON_ATTACK))
+	if (!(ent->client->buttons & BUTTON_ATTACK) || (!ent->is_bot && ent->client->newweapon))
 	{
 		ent->client->ps.gunframe = 17;
 		return;
@@ -1033,10 +1137,10 @@ BLASTER
 ======================================================================
 */
 
-void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, int effect)
+void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, qboolean alien, int effect)
 {
 	vec3_t	forward, right;
-	vec3_t	start;
+	vec3_t	start, muzzle;
 	vec3_t	offset;
 
 	if (is_quad)
@@ -1044,7 +1148,8 @@ void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, in
 
 	AngleVectors (ent->client->v_angle, forward, right, NULL);
 
-	if(!hyper) {
+	if(!hyper) 
+	{
 		VectorScale (forward, -3, ent->client->kick_origin);
 		ent->client->kick_angles[0] = -3;
 	}
@@ -1056,28 +1161,43 @@ void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, in
 	else
 		VectorSet(offset, 30, 6, ent->viewheight-5);
 	VectorAdd (offset, g_offset, offset);
-	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
+	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, muzzle);
+	if (hyper && (ent->client->buttons & BUTTON_ATTACK))
+	{
+		VectorSet(offset, 32, 0, ent->viewheight);
+		P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
+	}
+	else
+		VectorCopy (muzzle, start);
 
-	if(hyper) {
-		if(ent->client->buttons & BUTTON_ATTACK2) {//alt fire
+	if(hyper) 
+	{
+		if(ent->client->buttons & BUTTON_ATTACK2) 
+		{
+			//alt fire
 			ent->altfire = !ent->altfire;
-			if(ent->altfire) {
+			if(ent->altfire) 
+			{
 				gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/blastf1a.wav"), 1, ATTN_NORM, 0);
-				fire_blasterball (ent, start, forward, damage*3, 1000, effect, hyper);
+				fire_blasterball (ent, start, forward, damage*3, 1000, effect, hyper, false);
 			}
 		}
-		else {
+		else 
+		{
 			// [no file]gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/hyprbd1a.wav"), 1, ATTN_NORM, 0);
-			fire_blaster (ent, start, forward, damage, 2800, effect, hyper);
+			fire_blaster (ent, start, muzzle, forward, damage, 2800, effect, hyper);
 		}
 	}
-	else {
-		if(ent->client->buttons & BUTTON_ATTACK2) { //alt fire
-			fire_blaster_beam (ent, start, forward, (int)((float)damage/1.4), 0, false);
+	else 
+	{
+		if(ent->client->buttons & BUTTON_ATTACK2) 
+		{ 
+			//alt fire
+			fire_blaster_beam (ent, start, forward, (int)((float)damage/1.4), 0, false, alien);
 			gi.sound(ent, CHAN_AUTO, gi.soundindex("vehicles/shootlaser.wav"), 1, ATTN_NORM, 0);
 		}
 		else
-			fire_blasterball (ent, start, forward, damage, 1200, effect, hyper);
+			fire_blasterball (ent, start, forward, damage, 1200, effect, hyper, alien);
 	}
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
@@ -1094,11 +1214,11 @@ void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, in
 	//create visual muzzle flash sprite!
 	if(!hyper || (ent->client->buttons & BUTTON_ATTACK2))
 	{
-		VectorAdd(start, forward, start);
+		VectorAdd(muzzle, forward, muzzle);
 		gi.WriteByte (svc_temp_entity);
 		gi.WriteByte (TE_BLUE_MUZZLEFLASH);
-		gi.WritePosition (start);
-		gi.multicast (start, MULTICAST_PVS);
+		gi.WritePosition (muzzle);
+		gi.multicast (muzzle, MULTICAST_PVS);
 	}
 }
 
@@ -1109,7 +1229,7 @@ void Weapon_Blaster_Fire (edict_t *ent)
 
 	damage = 30;
 
-	Blaster_Fire (ent, vec3_origin, damage, false, EF_BLASTER);
+	Blaster_Fire (ent, vec3_origin, damage, false, false, EF_BLASTER);
 	ent->client->ps.gunframe++;
 }
 
@@ -1125,6 +1245,29 @@ void Weapon_Blaster (edict_t *ent)
 		Weapon_Generic (ent, 4, 8, 52, 55, pause_frames, fire_frames, Weapon_Blaster_Fire);
 }
 
+//to do - tactical - create different effect(i think same damage and behavior though).
+void Weapon_AlienBlaster_Fire (edict_t *ent)
+{
+	int		damage;
+
+	damage = 30;
+
+	Blaster_Fire (ent, vec3_origin, damage, false, true, EF_ROCKET);
+	ent->client->ps.gunframe++;
+}
+
+void Weapon_AlienBlaster (edict_t *ent)
+{
+	static int	pause_frames[]	= {52, 0};
+	static int	fire_frames[]	= {5,0};
+	static int  excessive_fire_frames[] = {5,6,7,8,0};
+
+	if(excessive->value || ent->client->invincible_framenum > level.framenum)
+		Weapon_Generic (ent, 4, 8, 52, 55, pause_frames, excessive_fire_frames, Weapon_AlienBlaster_Fire);
+	else
+		Weapon_Generic (ent, 4, 8, 52, 55, pause_frames, fire_frames, Weapon_AlienBlaster_Fire);
+}
+
 //vehicles
 void Weapon_Bomber_Fire (edict_t *ent)
 {
@@ -1222,7 +1365,7 @@ void Weapon_Strafer_Fire (edict_t *ent)
 	if(ent->client->buttons & BUTTON_ATTACK2)
 		fire_rocket (ent, start, forward, damage, 1200, damage_radius, radius_damage);
 	else
-		fire_blaster_beam (ent, start, forward, damage, 0, true);
+		fire_blaster_beam (ent, start, forward, damage, 0, true, false);
 
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
@@ -1250,12 +1393,14 @@ void Weapon_Strafer_Fire (edict_t *ent)
 
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 
-	if(ent->client->buttons & BUTTON_ATTACK2) {
+	if(ent->client->buttons & BUTTON_ATTACK2) 
+	{
 		fire_rocket (ent, start, forward, damage, 1200, damage_radius, radius_damage);
 		gi.sound (ent, CHAN_WEAPON, gi.soundindex("weapons/rocklr1b.wav"), 1, ATTN_NORM, 0);
 	}
-	else {
-		fire_blaster_beam (ent, start, forward, damage, 0, true);
+	else 
+	{
+		fire_blaster_beam (ent, start, forward, damage, 0, true, false);
 		gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/shootlaser.wav"), 1, ATTN_NORM, 0);
 	}
 
@@ -1309,7 +1454,7 @@ void Weapon_Hover_Fire (edict_t *ent)
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 
 	if(ent->client->buttons & BUTTON_ATTACK2) {
-		fire_blasterball (ent, start, forward, damage*3, 1500, EF_ROCKET, false);
+		fire_blasterball (ent, start, forward, damage*3, 1500, EF_ROCKET, false, false);
 		gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/hypbrl1a.wav"), 1, ATTN_NORM, 0);
 	}
 	else if(ent->client->ps.gunframe == 6) {
@@ -1359,7 +1504,7 @@ void Weapon_Beamgun_Fire (edict_t *ent)
 	int		effect;
 	int		damage;
 
-	if (!((ent->client->buttons & BUTTON_ATTACK) || (ent->client->buttons & BUTTON_ATTACK2)))
+	if (!(ent->client->buttons & (BUTTON_ATTACK|BUTTON_ATTACK2)) || (!ent->is_bot && ent->client->newweapon))
 	{
 		ent->client->ps.gunframe = 25;
 	}
@@ -1390,7 +1535,7 @@ void Weapon_Beamgun_Fire (edict_t *ent)
 			else
 				damage = 7;
 
-			Blaster_Fire (ent, offset, damage, true, effect);
+			Blaster_Fire (ent, offset, damage, true, false, effect);
 			if (! ( dmflags->integer & DF_INFINITE_AMMO ) )
 				ent->client->pers.inventory[ent->client->ammo_index]--;
 		}
@@ -1404,7 +1549,7 @@ void Weapon_Beamgun_Fire (edict_t *ent)
 void Weapon_Beamgun (edict_t *ent)
 {
 	static int	pause_frames[]	= {53, 0};
-	static int	fire_frames[]	= {6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 23, 24, 0};
+	static int	fire_frames[]	= {6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 0};
 
 	Weapon_Generic (ent, 5, 24, 53, 57, pause_frames, fire_frames, Weapon_Beamgun_Fire);
 }
@@ -1543,14 +1688,19 @@ void Machinegun_Fire (edict_t *ent)
 
 	}
 	else if(!ent->altfire){
+		
 		if (!(ent->client->buttons & BUTTON_ATTACK) && ent->client->ps.gunframe > 6) {
-			//Fast-forward through firing animation if not firing anymore.
-			//This is purely cosmetic, the player can resume firing at any
-			//point in this animation. 
-			//TODO: special sound effect here? 
-			ent->client->ps.gunframe+=2;
+			
+			// Make it easier to escape from the animation while not firing. 
+			// Since this is a rapid-fire weapon, there's no reason not to do
+			// this. (If it wasn't, allowing this would make it be possible to
+			// cheat the maximum fire rate.)
+			if (!ent->is_bot && ent->client->newweapon)
+				ent->client->ps.gunframe = 14;
+			
 			return; //Don't waste ammo
 		}
+		
 		for (i=0 ; i<shots ; i++)
 		{
 			VectorSet(offset, 1, 1, ent->viewheight-0.5);
@@ -1580,6 +1730,13 @@ void Machinegun_Fire (edict_t *ent)
 		gi.multicast (start, MULTICAST_PVS);
 		if (! ( dmflags->integer & DF_INFINITE_AMMO ) )
 			ent->client->pers.inventory[ent->client->ammo_index] -= shots;
+		
+		// Make it easier to escape from the animation while firing. Since 
+		// this is a rapid-fire weapon, there's no reason not to do this. (If
+		// it wasn't, allowing this would make it be possible to cheat the
+		// maximum fire rate.)
+		if (!ent->is_bot && ent->client->newweapon)
+			ent->client->ps.gunframe = 14;
 	}
 
 }
@@ -1593,7 +1750,7 @@ void Weapon_Chain (edict_t *ent)
 
 }
 
-void weapon_floater_fire (edict_t *ent)
+void weapon_smartgun_fire (edict_t *ent)
 {
 	vec3_t	offset, start;
 	vec3_t	forward, right;
@@ -1621,14 +1778,15 @@ void weapon_floater_fire (edict_t *ent)
 	forward[0] = forward[0] * 2.6;
 	forward[1] = forward[1] * 2.6;
 	forward[2] = forward[2] * 2.6;
-	if(ent->altfire) {
+	if(ent->altfire) 
+	{
 		if(excessive->value)
-			fire_floater (ent, start, forward, damage, 400, damage_radius, radius_damage, 8);
+			fire_smartgrenade (ent, start, forward, damage, 400, damage_radius, radius_damage, 8);
 		else
 			fire_prox (ent, start, forward, damage-50, 200, damage_radius, radius_damage-50, 8);
 	}
 	else
-		fire_floater (ent, start, forward, damage, 500, damage_radius, radius_damage, 8);
+		fire_smartgrenade (ent, start, forward, damage, 500, damage_radius, radius_damage, 8);
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
 	gi.WriteShort (ent-g_edicts);
@@ -1664,7 +1822,85 @@ void Weapon_Smartgun (edict_t *ent)
 	else if(ent->client->buttons & BUTTON_ATTACK)
 		ent->altfire = false;
 
-	Weapon_Generic (ent, 3, 11, 31, 35, pause_frames, fire_frames, weapon_floater_fire);
+	Weapon_Generic (ent, 3, 11, 31, 35, pause_frames, fire_frames, weapon_smartgun_fire);
+}
+
+void weapon_minderaser_fire (edict_t *ent)
+{
+	vec3_t	offset, start;
+	vec3_t	forward, right;
+	int		damage;
+	float	damage_radius;
+	int		radius_damage;
+
+	damage        = 110; 
+	radius_damage = 120;
+	damage_radius = 120;
+	if (is_quad || excessive->value)
+	{
+		damage *= 2;
+		radius_damage *= 2;
+	}
+
+	AngleVectors (ent->client->v_angle, forward, right, NULL);
+
+	VectorScale (forward, 2, ent->client->kick_origin);
+	ent->client->kick_angles[0] = -1;
+
+	VectorSet(offset, 8, 8, ent->viewheight-4);
+	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
+	forward[0] = forward[0] * 12.0;
+	forward[1] = forward[1] * 12.0;
+	forward[2] = forward[2] * 12.0;
+	
+	if(ent->altfire) 
+		fire_spider (ent, start, forward, 25);
+	else
+		fire_minderaser (ent, start, forward, 30);
+
+	gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/minderaserfire.wav"), 1, ATTN_NORM, 0);
+	
+	// send muzzle flash
+	gi.WriteByte (svc_muzzleflash);
+	gi.WriteShort (ent-g_edicts);
+	gi.WriteByte (MZ_RAILGUN | is_silenced);
+	gi.multicast (ent->s.origin, MULTICAST_PVS);
+
+	ent->client->ps.gunframe++;
+
+	PlayerNoise(ent, start, PNOISE_WEAPON);
+
+	//create visual muzzle flash sprite!
+	forward[0] = forward[0] * 2;
+	forward[1] = forward[1] * 2;
+
+	VectorAdd(start, forward, start);
+	start[2]+=6;
+
+	gi.WriteByte (svc_temp_entity);
+	gi.WriteByte (TE_BOSSTPORT);
+	gi.WritePosition (start);
+	gi.multicast (start, MULTICAST_PVS);
+
+	gi.WriteByte (svc_temp_entity);
+	gi.WriteByte (TE_BLUE_MUZZLEFLASH);
+	gi.WritePosition (start);
+	gi.multicast (start, MULTICAST_PVS);
+
+	ent->client->pers.inventory[ent->client->ammo_index]--;
+}
+
+void Weapon_Minderaser (edict_t *ent)
+{
+	static int	pause_frames[]	= {31, 0};
+	static int	fire_frames[]	= {6, 0};
+
+	if(ent->client->buttons & BUTTON_ATTACK2)
+		ent->altfire = true;
+	else if(ent->client->buttons & BUTTON_ATTACK)
+		ent->altfire = false;
+
+	Weapon_Generic (ent, 3, 11, 31, 35, pause_frames, fire_frames, weapon_minderaser_fire);
 }
 
 void Weapon_Deathball_Fire (edict_t *ent)
@@ -1681,7 +1917,6 @@ void Weapon_Deathball_Fire (edict_t *ent)
 	VectorSet(offset, 32, 5,  ent->viewheight-5);
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 
-
 	if(ent->client->ps.gunframe == 7) {
 
 		fire_deathball (ent, start, forward, 550);
@@ -1698,13 +1933,9 @@ void Weapon_Deathball_Fire (edict_t *ent)
 		gi.WriteByte (TE_BLUE_MUZZLEFLASH);
 		gi.WritePosition (start);
 		gi.multicast (start, MULTICAST_PVS);
-
-
+		
 		gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/energyfield.wav"), 1, ATTN_NORM, 0);
 		ent->client->weapon_sound = 0;
-
-
-
 	}
 	ent->client->ps.gunframe++;
 
@@ -1720,7 +1951,7 @@ void Weapon_Deathball (edict_t *ent)
 /*
 ======================================================================
 
-VIOALATOR
+VIOLATOR
 
 ======================================================================
 */
@@ -1733,18 +1964,36 @@ void Violator_Fire (edict_t *ent)
 	int			damage;
 	int			kick = 4;
 
-	if(excessive->value || instagib->value)
+	if(excessive->value || instagib->integer)
 		damage = 200;
 	else
 		damage = 40;
 
-	if ((ent->client->ps.gunframe == 6) && !(ent->client->buttons & BUTTON_ATTACK || ent->client->buttons & BUTTON_ATTACK2))
+	if ((ent->client->ps.gunframe == 6) && !(ent->client->buttons & (BUTTON_ATTACK|BUTTON_ATTACK2)))
 	{
 		ent->client->ps.gunframe = 14;
 		ent->client->weapon_sound = 0;
 		return;
 	}
-	else if (ent->client->ps.gunframe == 14 && (ent->client->buttons & BUTTON_ATTACK || ent->client->buttons & BUTTON_ATTACK2))
+	else if (!ent->altfire && ent->client->ps.gunframe > 6 && !(ent->client->buttons & (BUTTON_ATTACK|BUTTON_ATTACK2)))
+	{
+		//Fast-forward through firing animation if not firing anymore.
+		//This is purely cosmetic, the player can resume firing at any point
+		//point in this animation. 
+		//TODO: special sound effect here? 
+		ent->client->ps.gunframe+=1;
+		ent->client->weapon_sound = 0;
+		
+		// Make it easier to escape from the animation while not firing. Since 
+		// this is a rapid-fire weapon, there's no reason not to do this. (If
+		// it wasn't, allowing this would make it be possible to cheat the
+		// maximum fire rate.)
+		if (!ent->is_bot && ent->client->newweapon)
+			ent->client->ps.gunframe = 14;
+		
+		return;
+	}
+	else if (ent->client->ps.gunframe == 14 && (ent->client->buttons & (BUTTON_ATTACK|BUTTON_ATTACK2)))
 	{
 		ent->client->ps.gunframe = 6;
 	}
@@ -1851,6 +2100,13 @@ void Violator_Fire (edict_t *ent)
 		gi.WriteByte (TE_BLUE_MUZZLEFLASH);
 		gi.WritePosition (start);
 		gi.multicast (start, MULTICAST_PVS);
+		
+		// Make it easier to escape from the animation while firing. Since 
+		// this is a rapid-fire weapon, there's no reason not to do this. (If
+		// it wasn't, allowing this would make it be possible to cheat the
+		// maximum fire rate.)
+		if (!ent->is_bot && ent->client->newweapon)
+			ent->client->ps.gunframe = 14;
 	}
 }
 
@@ -1866,3 +2122,52 @@ void Weapon_Violator (edict_t *ent)
 
 }
 
+//Tactical weapons - bombs, detonators, etc
+
+void Weapon_TacticalBomb_Fire (edict_t *ent)
+{
+	vec3_t		start;
+	vec3_t		forward, right;
+	vec3_t		offset;
+
+	AngleVectors (ent->client->v_angle, forward, right, NULL);
+
+	VectorScale (forward, -3, ent->client->kick_origin);
+	ent->client->kick_angles[0] = -3;
+
+	VectorSet(offset, 32, 5,  ent->viewheight-5);
+	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
+
+	if(ent->client->ps.gunframe == 7) {
+
+		fire_tacticalbomb (ent, start, forward, 100);
+
+		// send muzzle flash
+		gi.WriteByte (svc_muzzleflash);
+		gi.WriteShort (ent-g_edicts);
+		gi.WriteByte (MZ_RAILGUN | is_silenced);
+		gi.multicast (ent->s.origin, MULTICAST_PVS);
+
+		VectorAdd(start, forward, start);
+		start[2]+=6;
+		gi.WriteByte (svc_temp_entity);
+		gi.WriteByte (TE_BLUE_MUZZLEFLASH);
+		gi.WritePosition (start);
+		gi.multicast (start, MULTICAST_PVS);
+		
+		gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/energyfield.wav"), 1, ATTN_NORM, 0); // to do - change me
+		ent->client->weapon_sound = 0;
+	}
+	ent->client->ps.gunframe++;
+
+	ent->client->pers.inventory[ent->client->ammo_index]--;
+
+}
+void Weapon_TacticalBomb (edict_t *ent)
+{
+	static int	pause_frames[]	= {33, 0};
+	static int	fire_frames[]	= {7,0};
+
+	Weapon_Generic (ent, 5, 11, 33, 39, pause_frames, fire_frames, Weapon_TacticalBomb_Fire);
+}
+#endif
