@@ -356,6 +356,9 @@ CVARS (console variables)
 #ifndef CVAR
 #define	CVAR
 
+// These are all the flags that actually do anything.
+// NOTE: there is no need to maintain any kind of compatibility, we can change
+// these if we want to.
 #define	CVAR_ARCHIVE	1	// set to cause it to be saved to vars.rc
 #define	CVAR_USERINFO	2	// added to userinfo  when changed
 #define	CVAR_SERVERINFO	4	// added to serverinfo when changed
@@ -367,6 +370,16 @@ CVARS (console variables)
                             // string when changed
 #define CVAR_PROFILE	128 // profile information
 
+// These flags are purely for documentation (used by the "help" command.)
+// Usually, at most one of these will be set. They are not "enforced" by the
+// code in any way.
+// TODO: go through and add all these flags for cvars as appropriate.
+// TODO: others like address, path, IP address, etc?
+#define CVARDOC_BOOL	512
+#define CVARDOC_STR		1024
+#define CVARDOC_FLOAT	2048
+#define CVARDOC_INT		4096
+
 // nothing outside the Cvar_*() functions should modify these fields!
 typedef struct cvar_s
 {
@@ -378,7 +391,8 @@ typedef struct cvar_s
 	qboolean	modified;		// set each time the cvar is changed
 	float		value;
 	float		default_value;
-	int		integer;		//BLOOMS
+	int		integer;
+	char		*description; // Optional (may be NULL)
 	struct cvar_s	*next;
 } cvar_t;
 
@@ -425,8 +439,6 @@ COLLISION DETECTION
 #define	CONTENTS_TRANSLUCENT	0x10000000	// auto set if any surface has trans
 #define	CONTENTS_LADDER			0x20000000
 
-
-
 #define	SURF_LIGHT		0x1		// value will hold the light strength
 
 #define	SURF_SLICK		0x2		// effects game physics
@@ -437,8 +449,11 @@ COLLISION DETECTION
 #define	SURF_TRANS66	0x20
 #define	SURF_FLOWING	0x40	// scroll towards angle
 #define	SURF_NODRAW		0x80	// don't bother referencing the texture
-
-
+#define SURF_BLOOD		0x400	// dripping blood surface
+#define SURF_WATER		0x800	// dripping water surface
+#define SURF_UNDERWATER	0x2000	// reflecting ripples; applied automatically 
+								// to underwater surfs but could be applied
+								// manually as well
 
 // content masks
 #define	MASK_ALL				(-1)
@@ -452,13 +467,11 @@ COLLISION DETECTION
 #define MASK_CURRENT			(CONTENTS_CURRENT_0|CONTENTS_CURRENT_90|CONTENTS_CURRENT_180|CONTENTS_CURRENT_270|CONTENTS_CURRENT_UP|CONTENTS_CURRENT_DOWN)
 #define MASK_VISIBILILITY   (CONTENTS_SOLID/*|CONTENTS_WINDOW*/|CONTENTS_WATER|CONTENTS_LAVA|CONTENTS_SLIME)
 
-
 // gi.BoxEdicts() can return a list of either solid or trigger entities
 // FIXME: eliminate AREA_ distinction?
 #define	AREA_SOLID		1
 #define	AREA_TRIGGERS	2
 
-
 // plane_t structure
 // !!! if this is changed, it must be changed in asm code too !!!
 typedef struct cplane_s
@@ -513,8 +526,6 @@ typedef struct
 	struct edict_s	*ent;		// not set by CM_*() functions
 } trace_t;
 
-
-
 // pmove_state_t is the information necessary for client side movement
 // prediction
 typedef enum
@@ -546,7 +557,7 @@ typedef struct
 {
 	pmtype_t	pm_type;
 
-	short		origin[3];		// 12.3
+	int 		origin[3];		// 12.3
 	short		velocity[3];	// 12.3
 	byte		pm_flags;		// ducked, jump_held, etc
 	byte		pm_time;		// each unit = 8 ms
@@ -555,7 +566,6 @@ typedef struct
 									// changed by spawns, rotating objects, and teleporters
 } pmove_state_t;
 
-
 //
 // button bits
 //
@@ -564,7 +574,6 @@ typedef struct
 #define BUTTON_ATTACK2		4
 #define	BUTTON_ANY			128			// any key whatsoever
 
-
 // usercmd_t is sent to the server each client frame
 typedef struct usercmd_s
 {
@@ -576,7 +585,6 @@ typedef struct usercmd_s
 	byte	lightlevel;		// light level the player is standing on
 } usercmd_t;
 
-
 #define	MAXTOUCH	32
 typedef struct
 {
@@ -624,7 +632,7 @@ typedef struct
 #define	EF_ANIM23			0x00000800		// automatically cycle between frames 2 and 3 at 2 hz
 #define EF_ANIM_ALL			0x00001000		// automatically cycle through all frames at 2hz
 #define EF_ANIM_ALLFAST		0x00002000		// automatically cycle through all frames at 10hz
-#define	EF_FLIES			0x00004000
+#define	EF_BUBBLES			0x00004000
 #define	EF_QUAD				0x00008000
 #define	EF_PENT				0x00010000
 #define	EF_TELEPORTER		0x00020000		// particle fountain
@@ -654,6 +662,7 @@ typedef struct
 #define	RF_SHELL_HALF_DAM	0x00020000
 #define RF_NOSHADOWS		0x00040000 //use this one for turning off shadows, etc.
 #define RF_MONSTER  		0x00080000
+#define	RF_NODRAW			0x00100000 //use this instead of a 0 modelindex for compatibility.
 #define RF_MENUMODEL		0x01280000 //for player menu
 
 
@@ -710,7 +719,7 @@ typedef struct
 #define TE_ROCKET_EXPLOSION_WATER	17
 #define	TE_REDLASER					19
 #define	TE_BFG_BIGEXPLOSION			21
-#define	TE_BOSSTPORT				22			// used as '22' in a map, so DON'T RENUMBER!!!
+#define	TE_BOSSTPORT				22			
 #define	TE_DONTUSE2					23
 #define	TE_GREENBLOOD				26
 #define	TE_LIGHTNING				33
@@ -765,9 +774,9 @@ typedef struct
 #define	STAT_AMMO				3
 #define	STAT_ARMOR_ICON			4
 #define	STAT_ARMOR				5
-#define	STAT_SELECTED_ICON		6
-#define	STAT_PICKUP_ICON		7
-#define	STAT_PICKUP_STRING		8
+#define	STAT_RED_MATCHES		6
+#define	STAT_BLUE_MATCHES		7
+#define STAT_TACTICAL_SCORE		8
 #define	STAT_TIMER_ICON			9
 #define	STAT_TIMER				10
 #define	STAT_HELPICON			11
@@ -775,6 +784,7 @@ typedef struct
 #define	STAT_LAYOUTS			13
 #define	STAT_FRAGS				14
 #define	STAT_FLASHES			15		// cleared each frame, 1 = health, 2 = armor
+#define STAT_FLAGS				15		// misc boolean values, use only bits 3-16
 #define STAT_CHASE				16
 #define STAT_SPECTATOR			17
 #define STAT_SCOREBOARD			18
@@ -794,6 +804,13 @@ typedef struct
 
 #define	MAX_STATS				32
 
+// bit flags for use in STAT_FLAGS
+#define STAT_FLAGS_CROSSHAIRPOSITION	(4|8)
+#define STAT_FLAGS_CROSSHAIRPOS1		0 //default
+#define STAT_FLAGS_CROSSHAIRCENTER		4
+#define STAT_FLAGS_CROSSHAIRPOS2		8
+#define STAT_FLAGS_CROSSHAIRPOS3		12
+
 
 // dmflags->value flags
 #define	DF_NO_HEALTH		0x00000001	// 1
@@ -833,6 +850,12 @@ typedef struct
 #define	ANGLE2SHORT(x)	((int)((x)*65536/360) & 65535)
 #define	SHORT2ANGLE(x)	((x)*(360.0/65536))
 
+// Number of bytes per axis of world coordinates in the net protocol.
+// 2 is the default, backward-compatible number.
+// TODO: make this a variable, have it set based on the map size, server kicks
+// clients that don't support big maps.
+#define	coord_bytes		2
+
 
 //
 // config strings are a general means of communication from
@@ -987,4 +1010,14 @@ typedef struct
 #define MAX_LATENT_CMDS 64
 //unlagged - lag simulation #2
 
+//
+// types of compression
+//
+enum compressiontypes_e  {
+    compression_zlib_raw,
+    compression_lzo,
+    compression_zlib_header,
+    last_compressiontype_known
+};
+
 #endif  /* Q_SHARED_H_ */
