@@ -59,7 +59,7 @@ void Cmd_ForwardToServer (void)
 	Com_Printf ("Unknown command \"%s\"\n", cmd);
 }
 
-void SCR_DebugGraph (float value, int color)
+void SCR_DebugGraph (float value, const float color[])
 {
 }
 
@@ -76,3 +76,8 @@ void Key_Init (void)
 	Cmd_AddCommand ("bind", Key_Bind_Null_f);
 }
 
+netadr_t *CL_GetRemoteServer (void)
+{
+	return NULL;
+}
+
