@@ -12,9 +12,9 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See the GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 */
 // cmodel.c -- model loading
@@ -123,6 +123,24 @@ int		c_pointcontents;
 int		c_traces, c_brush_traces;
 
 
+/**
+ * calculate the 3 signbits for a plane normal.
+ *
+ * @param  plane
+ * @return the signbits
+ */
+static byte signbits_for_plane(const cplane_t *plane)
+{
+	byte bits;
+
+	bits =  plane->normal[0] < 0.0f ? (byte)0x01 : 0 ;
+	bits |= plane->normal[1] < 0.0f ? (byte)0x02 : 0 ;
+	bits |= plane->normal[2] < 0.0f ? (byte)0x04 : 0 ;
+
+	return bits;
+}
+
+
 /*
 ===============================================================================
 
@@ -133,6 +151,34 @@ int		c_traces, c_brush_traces;
 
 byte	*cmod_base;
 
+//Performs a sanity check on the BSP file, making sure all the lumps are nice
+//and shipshape and won't lead off a cliff somewhere. This does mean that the 
+//BSP file format has been made less extensible, but we've tended to add more
+//data using separate files instead of adding lumps to the BSP, so I think we
+//can live with that.
+qboolean checkLumps (lump_t *l, size_t header_size, int *lump_order, void *_file_base, int num_lumps, int file_len)
+{
+	int i = 0;
+	lump_t *in;
+	byte *lumpdata_base;
+	byte *file_base = (byte *)_file_base;
+	byte *lumpdata_next = file_base+header_size+sizeof(lump_t)*num_lumps;
+	byte *file_end = file_base + file_len;
+	for (i = 0; i < num_lumps; i++)
+	{
+		in = l+lump_order[i];
+		lumpdata_base = file_base+in->fileofs;
+		if (lumpdata_base != lumpdata_next)
+			return true;
+		lumpdata_next = lumpdata_base+((in->filelen+3)&~3);
+		if (lumpdata_next < lumpdata_base)
+			return true;
+	}
+	if (lumpdata_next != file_end)
+		return true;
+	return false;
+}
+
 /*
 =================
 CMod_LoadSubmodels
@@ -146,7 +192,7 @@ void CMod_LoadSubmodels (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadSubmodels: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count < 1)
@@ -184,7 +230,7 @@ void CMod_LoadSurfaces (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadSurfaces: funny lump size");
 	count = l->filelen / sizeof(*in);
 	if (count < 1)
 		Com_Error (ERR_DROP, "Map with no surfaces");
@@ -219,7 +265,7 @@ void CMod_LoadNodes (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadNodes: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count < 1)
@@ -257,7 +303,7 @@ void CMod_LoadBrushes (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadBrushes: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count > MAX_MAP_BRUSHES)
@@ -290,14 +336,14 @@ void CMod_LoadLeafs (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadLeafs: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count < 1)
 		Com_Error (ERR_DROP, "Map with no leafs");
 	// need to save space for box planes
-	if (count > MAX_MAP_PLANES)
-		Com_Error (ERR_DROP, "Map has too many planes");
+	if (count > MAX_MAP_LEAFS)
+		Com_Error (ERR_DROP, "Map has too many leafs");
 
 	out = map_leafs;
 	numleafs = count;
@@ -338,15 +384,14 @@ CMod_LoadPlanes
 */
 void CMod_LoadPlanes (lump_t *l)
 {
-	int			i, j;
+	int			i;
 	cplane_t	*out;
 	dplane_t 	*in;
 	int			count;
-	int			bits;
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadPlanes: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count < 1)
@@ -357,20 +402,15 @@ void CMod_LoadPlanes (lump_t *l)
 
 	out = map_planes;
 	numplanes = count;
-
-	for ( i=0 ; i<count ; i++, in++, out++)
+	for ( ; count-- ; in++, out++)
 	{
-		bits = 0;
-		for (j=0 ; j<3 ; j++)
+		for (i=0 ; i<3 ; i++)
 		{
-			out->normal[j] = LittleFloat (in->normal[j]);
-			if (out->normal[j] < 0)
-				bits |= 1<<j;
+			out->normal[i] = LittleFloat( in->normal[i] );
 		}
-
 		out->dist = LittleFloat (in->dist);
 		out->type = LittleLong (in->type);
-		out->signbits = bits;
+		out->signbits = signbits_for_plane( out );
 	}
 }
 
@@ -388,11 +428,11 @@ void CMod_LoadLeafBrushes (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadLeafBrushes: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count < 1)
-		Com_Error (ERR_DROP, "Map with no planes");
+		Com_Error (ERR_DROP, "Map with no leafbrushes");
 	// need to save space for box planes
 	if (count > MAX_MAP_LEAFBRUSHES)
 		Com_Error (ERR_DROP, "Map has too many leafbrushes");
@@ -419,12 +459,12 @@ void CMod_LoadBrushSides (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadBrushSides: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	// need to save space for box planes
 	if (count > MAX_MAP_BRUSHSIDES)
-		Com_Error (ERR_DROP, "Map has too many planes");
+		Com_Error (ERR_DROP, "Map has too many brushsides");
 
 	out = map_brushsides;
 	numbrushsides = count;
@@ -454,7 +494,7 @@ void CMod_LoadAreas (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadAreas: funny lump size");
 	count = l->filelen / sizeof(*in);
 
 	if (count > MAX_MAP_AREAS)
@@ -486,11 +526,11 @@ void CMod_LoadAreaPortals (lump_t *l)
 
 	in = (void *)(cmod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
-		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size");
+		Com_Error (ERR_DROP, "CMod_LoadAreaPortals: funny lump size");
 	count = l->filelen / sizeof(*in);
 
-	if (count > MAX_MAP_AREAS)
-		Com_Error (ERR_DROP, "Map has too many areas");
+	if (count > MAX_MAP_AREAPORTALS)
+		Com_Error (ERR_DROP, "Map has too many areaportals");
 
 	out = map_areaportals;
 	numareaportals = count;
@@ -521,7 +561,15 @@ void CMod_LoadVisibility (lump_t *l)
 	for (i=0 ; i<map_vis->numclusters ; i++)
 	{
 		map_vis->bitofs[i][0] = LittleLong (map_vis->bitofs[i][0]);
+		if (map_vis->bitofs[i][0] < 0 || map_vis->bitofs[i][0] >= numvisibility)
+			Com_Error (	ERR_DROP, 
+				"Map contains invalid PVS bit offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
 		map_vis->bitofs[i][1] = LittleLong (map_vis->bitofs[i][1]);
+		if (map_vis->bitofs[i][1] < 0 || map_vis->bitofs[i][1] >= numvisibility)
+			Com_Error (	ERR_DROP, 
+				"Map contains invalid PHS bit offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
 	}
 }
 
@@ -574,10 +622,19 @@ Loads in the map and all submodels
 cmodel_t *CM_LoadBSP (char *name, qboolean clientload, unsigned *checksum)
 {
 	unsigned		*buf;
-	int				i;
+	unsigned int	i;
 	dheader_t		header;
 	int				length;
 	static unsigned	last_checksum;
+	int				bsp_lump_order[HEADER_LUMPS] = 
+	{
+		LUMP_PLANES, LUMP_LEAFS, LUMP_VERTEXES, LUMP_NODES, 
+		LUMP_TEXINFO, LUMP_FACES, LUMP_BRUSHES,
+		LUMP_BRUSHSIDES, LUMP_LEAFFACES, LUMP_LEAFBRUSHES,
+		LUMP_SURFEDGES, LUMP_EDGES, LUMP_MODELS, LUMP_AREAS,
+		LUMP_AREAPORTALS, LUMP_LIGHTING, LUMP_VISIBILITY,
+		LUMP_ENTITIES, LUMP_POP
+	};
 
 	map_noareas = Cvar_Get ("map_noareas", "0", 0);
 
@@ -631,6 +688,10 @@ cmodel_t *CM_LoadBSP (char *name, qboolean clientload, unsigned *checksum)
 		, name, header.version, BSPVERSION);
 
 	cmod_base = (byte *)buf;
+	
+	if (checkLumps(header.lumps, 2*sizeof(int), bsp_lump_order, cmod_base, HEADER_LUMPS, length))
+		Com_Error (ERR_DROP,"CMod_LoadBrushModel: lumps in %s don't add up right!\n"
+							"The file is likely corrupt, please obtain a fresh copy.",name);
 
 	// load into heap
 	CMod_LoadSurfaces (&header.lumps[LUMP_TEXINFO]);
@@ -773,7 +834,10 @@ int		CM_LeafContents (int leafnum)
 int		CM_LeafCluster (int leafnum)
 {
 	if (leafnum < 0 || leafnum >= numleafs)
+	{
 		Com_Error (ERR_DROP, "CM_LeafCluster: bad number");
+		return 0; /* unreachable. quiets bogus compiler array index warning */
+	}
 	return map_leafs[leafnum].cluster;
 }
 
@@ -850,15 +914,15 @@ void CM_InitBoxHull (void)
 		// planes
 		p = &box_planes[i*2];
 		p->type = i>>1;
-		p->signbits = 0;
 		VectorClear (p->normal);
 		p->normal[i>>1] = 1;
+		p->signbits = signbits_for_plane( p );
 
 		p = &box_planes[i*2+1];
 		p->type = 3 + (i>>1);
-		p->signbits = 0;
 		VectorClear (p->normal);
 		p->normal[i>>1] = -1;
+		p->signbits = signbits_for_plane( p );
 	}
 }
 
@@ -1080,26 +1144,30 @@ trace_t	trace_trace;
 int		trace_contents;
 qboolean	trace_ispoint;		// optimized case
 
-/*
-================
-CM_ClipBoxToBrush
-================
-*/
-void CM_ClipBoxToBrush (vec3_t mins, vec3_t maxs, vec3_t p1, vec3_t p2,
-					  trace_t *trace, cbrush_t *brush)
+/**
+ * @brief  intersect test for axis-aligned box and a brush in a leaf.
+ *
+ * This is the innermost loop for CM_BoxTrace() general and point cases.
+ * Does not use file level static variables; not sure why.
+ *
+ * @returns  if intersection occurred, information about the brush in the
+ *           trace_t record. otherwise does not touch trace_t record
+ *
+ */
+void CM_ClipBoxToBrush( vec3_t mins, vec3_t maxs,vec3_t p1, vec3_t p2,
+		trace_t *trace, cbrush_t *brush )
 {
-	int			i, j;
-	cplane_t	*plane, *clipplane;
-	float		dist;
-	float		enterfrac, leavefrac;
-	vec3_t		ofs;
-	float		d1, d2;
-	qboolean	getout, startout;
-	float		f;
-	cbrushside_t	*side, *leadside;
+	int           i;
+	cplane_t     *plane, *clipplane;
+	float         dist;
+	float         enterfrac, leavefrac;
+	float         d1, d2;
+	qboolean      getout, startout;
+	float         f;
+	cbrushside_t *side, *leadside;
 
-	enterfrac = -1;
-	leavefrac = 1;
+	enterfrac = -1.0f;
+	leavefrac = 1.0f;
 	clipplane = NULL;
 
 	if (!brush->numsides)
@@ -1115,24 +1183,72 @@ void CM_ClipBoxToBrush (vec3_t mins, vec3_t maxs, vec3_t p1, vec3_t p2,
 	{
 		side = &map_brushsides[brush->firstbrushside+i];
 		plane = side->plane;
-
-		// FIXME: special case for axial
-
-		if (!trace_ispoint)
-		{	// general box case
-
-			// push the plane out apropriately for mins/maxs
-
-			// FIXME: use signbits into 8 way lookup for each mins/maxs
-			for (j=0 ; j<3 ; j++)
+		if ( !trace_ispoint )
+		{
+			switch ( plane->type )
 			{
-				if (plane->normal[j] < 0)
-					ofs[j] = maxs[j];
+			case PLANE_X:
+				if ( plane->signbits == 0x01 )
+					dist = plane->dist - (maxs[0]*plane->normal[0]);
 				else
-					ofs[j] = mins[j];
+					dist = plane->dist - (mins[0]*plane->normal[0]);
+				break;
+
+			case PLANE_Y:
+				if ( plane->signbits == 0x02 )
+					dist = plane->dist - (maxs[1]*plane->normal[1]);
+				else
+					dist = plane->dist - (mins[1]*plane->normal[1]);
+				break;
+
+			case PLANE_Z:
+				if ( plane->signbits == 0x04 )
+					dist = plane->dist - (maxs[2]*plane->normal[2]);
+				else
+					dist = plane->dist - (mins[2]*plane->normal[2]);
+				break;
+
+			default:
+				switch ( plane->signbits ) /* bit2=z<0, bit1=y<0, bit0=x<0 */
+				{
+				case 0: /* 000b */
+					dist = plane->dist - (mins[0]*plane->normal[0] +
+							mins[1]*plane->normal[1] + mins[2]*plane->normal[2]);
+					break;
+				case 1: /* 001b */
+					dist = plane->dist - (maxs[0]*plane->normal[0] +
+							mins[1]*plane->normal[1] + mins[2]*plane->normal[2]);
+					break;
+				case 2: /* 010b */
+					dist = plane->dist - (mins[0]*plane->normal[0] +
+							maxs[1]*plane->normal[1] + mins[2]*plane->normal[2]);
+					break;
+				case 3: /* 011b */
+					dist = plane->dist - (maxs[0]*plane->normal[0] +
+							maxs[1]*plane->normal[1] + mins[2]*plane->normal[2]);
+					break;
+				case 4: /* 100b */
+					dist = plane->dist - (mins[0]*plane->normal[0] +
+							mins[1]*plane->normal[1] + maxs[2]*plane->normal[2]);
+					break;
+				case 5: /* 101b */
+					dist = plane->dist - (maxs[0]*plane->normal[0] +
+							mins[1]*plane->normal[1] + maxs[2]*plane->normal[2]);
+					break;
+				case 6: /* 110b */
+					dist = plane->dist - (mins[0]*plane->normal[0] +
+							maxs[1]*plane->normal[1] + maxs[2]*plane->normal[2]);
+					break;
+				case 7: /* 111b */
+					dist = plane->dist - (maxs[0]*plane->normal[0] +
+							maxs[1]*plane->normal[1] + maxs[2]*plane->normal[2]);
+					break;
+
+				default:
+					Com_Error( ERR_DROP, "CM_ClipBoxToBrush: bad plane signbits\n");
+					return; // unreachable. suppress bogus compiler warning
+				}
 			}
-			dist = DotProduct (ofs, plane->normal);
-			dist = plane->dist - dist;
 		}
 		else
 		{	// special point case
@@ -1142,16 +1258,16 @@ void CM_ClipBoxToBrush (vec3_t mins, vec3_t maxs, vec3_t p1, vec3_t p2,
 		d1 = DotProduct (p1, plane->normal) - dist;
 		d2 = DotProduct (p2, plane->normal) - dist;
 
-		if (d2 > 0)
-			getout = true;	// endpoint is not in solid
-		if (d1 > 0)
+		if ( d2 > 0.0f )
+			getout = true; // endpoint is not in solid
+		if ( d1 > 0.0f )
 			startout = true;
 
 		// if completely in front of face, no intersection
-		if (d1 > 0 && d2 >= d1)
+		if ( d1 > 0.0f && d2 >= d1 )
 			return;
 
-		if (d1 <= 0 && d2 <= 0)
+		if ( d1 <= 0.0f && d2 <= 0.0f )
 			continue;
 
 		// crosses face
@@ -1165,12 +1281,15 @@ void CM_ClipBoxToBrush (vec3_t mins, vec3_t maxs, vec3_t p1, vec3_t p2,
 				leadside = side;
 			}
 		}
-		else
+		else if ( d1 < d2 )
 		{	// leave
 			f = (d1+DIST_EPSILON) / (d1-d2);
 			if (f < leavefrac)
 				leavefrac = f;
 		}
+		/* else d1 == d2, line segment is parallel to plane, cannot intersect.
+		   formerly processed like d1 < d2. now just continue to next
+		   brushside plane.  */
 	}
 
 	if (!startout)
@@ -1330,10 +1449,12 @@ void CM_RecursiveHullCheck (int num, float p1f, float p2f, vec3_t p1, vec3_t p2)
 	float		t1, t2, offset;
 	float		frac, frac2;
 	float		idist;
-	int			i;
 	vec3_t		mid;
 	int			side;
 	float		midf;
+	vec3_t		p1_copy;
+
+re_test:
 
 	if (trace_trace.fraction <= p1f)
 		return;		// already hit something nearer
@@ -1380,13 +1501,13 @@ return;
 	// see which sides we need to consider
 	if (t1 >= offset && t2 >= offset)
 	{
-		CM_RecursiveHullCheck (node->children[0], p1f, p2f, p1, p2);
-		return;
+		num = node->children[0];
+		goto re_test;
 	}
 	if (t1 < -offset && t2 < -offset)
 	{
-		CM_RecursiveHullCheck (node->children[1], p1f, p2f, p1, p2);
-		return;
+		num = node->children[1];
+		goto re_test;
 	}
 
 	// put the crosspoint DIST_EPSILON pixels on the near side
@@ -1418,10 +1539,15 @@ return;
 		frac = 1;
 
 	midf = p1f + (p2f - p1f)*frac;
-	for (i=0 ; i<3 ; i++)
-		mid[i] = p1[i] + frac*(p2[i] - p1[i]);
+	mid[0] = p1[0] + frac*(p2[0] - p1[0]);
+	mid[1] = p1[1] + frac*(p2[1] - p1[1]);
+	mid[2] = p1[2] + frac*(p2[2] - p1[2]);
+	
+	// so we can modify p1
+	VectorCopy (p1, p1_copy);
 
-	CM_RecursiveHullCheck (node->children[side], p1f, midf, p1, mid);
+	// this is the only case where the function actually recurses
+	CM_RecursiveHullCheck (node->children[side], p1f, midf, p1_copy, mid);
 
 
 	// go past the node
@@ -1430,11 +1556,13 @@ return;
 	if (frac2 > 1)
 		frac2 = 1;
 
-	midf = p1f + (p2f - p1f)*frac2;
-	for (i=0 ; i<3 ; i++)
-		mid[i] = p1[i] + frac2*(p2[i] - p1[i]);
-
-	CM_RecursiveHullCheck (node->children[side^1], midf, p2f, mid, p2);
+	p1f += (p2f - p1f)*frac2;
+	p1[0] += frac2*(p2[0] - p1[0]);
+	p1[1] += frac2*(p2[1] - p1[1]);
+	p1[2] += frac2*(p2[2] - p1[2]);
+	
+	num = node->children[side^1];
+	goto re_test;
 }
 
 
@@ -1451,6 +1579,7 @@ trace_t		CM_BoxTrace (vec3_t start, vec3_t end,
 						  int headnode, int brushmask)
 {
 	int		i;
+	vec3_t	local_start, local_end;
 
 	checkcount++;		// for multi-check avoidance
 
@@ -1469,12 +1598,43 @@ trace_t		CM_BoxTrace (vec3_t start, vec3_t end,
 	VectorCopy (end, trace_end);
 	VectorCopy (mins, trace_mins);
 	VectorCopy (maxs, trace_maxs);
-
-	//
-	// check for position test special case
-	//
-	if (start[0] == end[0] && start[1] == end[1] && start[2] == end[2])
+	
+	if ( !(start == end) &&
+			(start[0] != end[0] || start[1] != end[1] || start[2] != end[2]) )
 	{
+		if (( mins == maxs ) ||
+				( mins[0] == 0.0f && mins[1] == 0.0f && mins[2] == 0.0f &&
+				maxs[0] == 0.0f && maxs[1] == 0.0f && maxs[2] == 0.0f ))
+		{ // point special case
+			trace_ispoint = true;
+			VectorClear (trace_extents);
+		}
+		else
+		{ // general axis-aligned box case
+			trace_ispoint = false;
+			trace_extents[0] = -mins[0] > maxs[0] ? -mins[0] : maxs[0];
+			trace_extents[1] = -mins[1] > maxs[1] ? -mins[1] : maxs[1];
+			trace_extents[2] = -mins[2] > maxs[2] ? -mins[2] : maxs[2];
+		}
+		//
+		// general sweeping through world
+		//
+		VectorCopy (start, local_start); // so the function can modify these in-place
+		VectorCopy (end, local_end);
+		CM_RecursiveHullCheck (headnode, 0, 1, local_start, local_end);
+
+		if (trace_trace.fraction == 1)
+		{
+			VectorCopy (end, trace_trace.endpos);
+		}
+		else
+		{
+			for (i=0 ; i<3 ; i++)
+				trace_trace.endpos[i] = start[i] + trace_trace.fraction * (end[i] - start[i]);
+		}
+	}
+	else
+	{ // position test special case
 		int		leafs[1024];
 		int		i, numleafs;
 		vec3_t	c1, c2;
@@ -1496,40 +1656,8 @@ trace_t		CM_BoxTrace (vec3_t start, vec3_t end,
 				break;
 		}
 		VectorCopy (start, trace_trace.endpos);
-		return trace_trace;
 	}
 
-	//
-	// check for point special case
-	//
-	if (mins[0] == 0 && mins[1] == 0 && mins[2] == 0
-		&& maxs[0] == 0 && maxs[1] == 0 && maxs[2] == 0)
-	{
-		trace_ispoint = true;
-		VectorClear (trace_extents);
-	}
-	else
-	{
-		trace_ispoint = false;
-		trace_extents[0] = -mins[0] > maxs[0] ? -mins[0] : maxs[0];
-		trace_extents[1] = -mins[1] > maxs[1] ? -mins[1] : maxs[1];
-		trace_extents[2] = -mins[2] > maxs[2] ? -mins[2] : maxs[2];
-	}
-
-	//
-	// general sweeping through world
-	//
-	CM_RecursiveHullCheck (headnode, 0, 1, start, end);
-
-	if (trace_trace.fraction == 1)
-	{
-		VectorCopy (end, trace_trace.endpos);
-	}
-	else
-	{
-		for (i=0 ; i<3 ; i++)
-			trace_trace.endpos[i] = start[i] + trace_trace.fraction * (end[i] - start[i]);
-	}
 	return trace_trace;
 }
 
@@ -1677,6 +1805,11 @@ byte	phsrow[MAX_MAP_LEAFS/8];
 
 byte	*CM_ClusterPVS (int cluster)
 {
+	static int old_cluster = -1;
+	if (cluster == old_cluster)
+		return pvsrow;
+	old_cluster = cluster;
+	
 	if (cluster == -1)
 		memset (pvsrow, 0, (numclusters+7)>>3);
 	else
@@ -1686,6 +1819,11 @@ byte	*CM_ClusterPVS (int cluster)
 
 byte	*CM_ClusterPHS (int cluster)
 {
+	static int old_cluster = -1;
+	if (cluster == old_cluster)
+		return phsrow;
+	old_cluster = cluster;
+
 	if (cluster == -1)
 		memset (phsrow, 0, (numclusters+7)>>3);
 	else
@@ -1877,3 +2015,82 @@ qboolean CM_HeadnodeVisible (int nodenum, byte *visbits)
 	return CM_HeadnodeVisible(node->children[1], visbits);
 }
 
+
+/*
+=================
+CM_inPVS
+
+Also checks portalareas so that doors block sight
+=================
+*/
+qboolean CM_inPVS (vec3_t p1, vec3_t p2)
+{
+	int		leafnum;
+	int		cluster;
+	int		area1, area2;
+	byte	*mask;
+
+	leafnum = CM_PointLeafnum (p1);
+	cluster = CM_LeafCluster (leafnum);
+	area1 = CM_LeafArea (leafnum);
+	mask = CM_ClusterPVS (cluster);
+
+	leafnum = CM_PointLeafnum (p2);
+	cluster = CM_LeafCluster (leafnum);
+	area2 = CM_LeafArea (leafnum);
+	if ( mask && (!(mask[cluster>>3] & (1<<(cluster&7)) ) ) )
+		return false;
+	if (!CM_AreasConnected (area1, area2))
+		return false;		// a door blocks sight
+	return true;
+}
+
+//similar to CM_inPVS but with leafnums
+qboolean CM_inPVS_leafs (int leafnum1, int leafnum2)
+{
+	int		cluster;
+	int		area1, area2;
+	byte	*mask;
+
+	cluster = CM_LeafCluster (leafnum1);
+	area1 = CM_LeafArea (leafnum1);
+	mask = CM_ClusterPVS (cluster);
+
+	cluster = CM_LeafCluster (leafnum2);
+	area2 = CM_LeafArea (leafnum2);
+	if ( mask && (!(mask[cluster>>3] & (1<<(cluster&7)) ) ) )
+		return false;
+	if (!CM_AreasConnected (area1, area2))
+		return false;		// a door blocks sight
+	return true;
+}
+
+/*
+=================
+CM_inPHS
+
+Also checks portalareas so that doors block sound
+=================
+*/
+qboolean CM_inPHS (vec3_t p1, vec3_t p2)
+{
+	int		leafnum;
+	int		cluster;
+	int		area1, area2;
+	byte	*mask;
+
+	leafnum = CM_PointLeafnum (p1);
+	cluster = CM_LeafCluster (leafnum);
+	area1 = CM_LeafArea (leafnum);
+	mask = CM_ClusterPHS (cluster);
+
+	leafnum = CM_PointLeafnum (p2);
+	cluster = CM_LeafCluster (leafnum);
+	area2 = CM_LeafArea (leafnum);
+	if ( mask && (!(mask[cluster>>3] & (1<<(cluster&7)) ) ) )
+		return false;		// more than one bounce away
+	if (!CM_AreasConnected (area1, area2))
+		return false;		// a door blocks hearing
+
+	return true;
+}
