@@ -27,13 +27,22 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <setjmp.h>
 
+//zlib currently not needed server-side (FOR NOW!)
+//Also, due to the ld flags for zlib not being used server side, it won't
+//compile without this workaround.
+#ifdef DEDICATED_ONLY
+#undef HAVE_ZLIB
+#endif 
+
+#ifdef HAVE_ZLIB
+#include <zlib.h>
+#endif
+
 #define	MAXPRINTMSG	8192
 
 #define MAX_NUM_ARGVS	50
 
 #ifndef DEDICATED_ONLY
-extern void S_StartLocalSound (char *sound);
-extern void S_StartMusic ( char * sound );
 extern void S_StartMenuMusic( void );
 #endif
 
@@ -351,6 +360,8 @@ void MSG_WriteByte (sizebuf_t *sb, int c)
 
 void MSG_WriteShort (sizebuf_t *sb, int c)
 {
+#if 0
+	/* Original Version */
 	byte	*buf;
 
 #ifdef PARANOID
@@ -364,10 +375,19 @@ void MSG_WriteShort (sizebuf_t *sb, int c)
 	buf = SZ_GetSpace (sb, 2);
 	buf[0] = c&0xff;
 	buf[1] = c>>8;
+#else
+	/* Experimental Version */
+	short *buf;
+
+	buf  = (short*)SZ_GetSpace( sb , 2 );
+	*buf = LittleShort( c );
+#endif
 }
 
 void MSG_WriteLong (sizebuf_t *sb, int c)
 {
+#if 0
+	/* Original Version */
 	byte	*buf;
 
 	buf = SZ_GetSpace (sb, 4);
@@ -375,6 +395,27 @@ void MSG_WriteLong (sizebuf_t *sb, int c)
 	buf[1] = (c>>8)&0xff;
 	buf[2] = (c>>16)&0xff;
 	buf[3] = c>>24;
+#else
+	/* Experimental Version */
+	int *buf;
+
+	buf  = (int*)SZ_GetSpace( sb, 4 );
+	*buf = LittleLong( c );
+#endif
+}
+
+void MSG_WriteSizeInt (sizebuf_t *sb, int bytes, int c)
+{
+	byte		*buf;
+	int			i;
+	
+	// manually construct two's complement encoding with appropriate bit depth
+	if (c < 0 && bytes != sizeof(c))
+		c += 1<<(8*bytes);
+	
+	buf = SZ_GetSpace (sb, bytes);
+	for (i = 0; i < bytes; i++)
+		buf[i] = (byte)((c>>(8*i))&0xff);
 }
 
 void MSG_WriteFloat (sizebuf_t *sb, float f)
@@ -402,14 +443,14 @@ void MSG_WriteString (sizebuf_t *sb, char *s)
 
 void MSG_WriteCoord (sizebuf_t *sb, float f)
 {
-	MSG_WriteShort (sb, (int)(f*8));
+	MSG_WriteSizeInt (sb, coord_bytes, (int)(f*8));
 }
 
 void MSG_WritePos (sizebuf_t *sb, vec3_t pos)
 {
-	MSG_WriteShort (sb, (int)(pos[0]*8));
-	MSG_WriteShort (sb, (int)(pos[1]*8));
-	MSG_WriteShort (sb, (int)(pos[2]*8));
+	MSG_WriteSizeInt (sb, coord_bytes, (int)(pos[0]*8));
+	MSG_WriteSizeInt (sb, coord_bytes, (int)(pos[1]*8));
+	MSG_WriteSizeInt (sb, coord_bytes, (int)(pos[2]*8));
 }
 
 void MSG_WriteAngle (sizebuf_t *sb, float f)
@@ -796,6 +837,8 @@ int MSG_ReadByte (sizebuf_t *msg_read)
 
 int MSG_ReadShort (sizebuf_t *msg_read)
 {
+#if 0
+	/* Origninal Version */
 	int	c;
 
 	if (msg_read->readcount+2 > msg_read->cursize)
@@ -807,10 +850,27 @@ int MSG_ReadShort (sizebuf_t *msg_read)
 	msg_read->readcount += 2;
 
 	return c;
+#else
+	/* Experimental Version */
+	int c     = -1;
+	int rc    = msg_read->readcount;
+	short *pi = (short*)(&msg_read->data[ rc ]);
+
+	rc += 2;
+	if ( rc <= msg_read->cursize )
+	{
+		c = LittleShort( *pi );
+		msg_read->readcount = rc;
+	}
+
+	return c;
+#endif
 }
 
 int MSG_ReadLong (sizebuf_t *msg_read)
 {
+#if 0
+	/* Original Version */
 	int	c;
 
 	if (msg_read->readcount+4 > msg_read->cursize)
@@ -823,6 +883,41 @@ int MSG_ReadLong (sizebuf_t *msg_read)
 
 	msg_read->readcount += 4;
 
+	return c;
+#else
+	/* Experimental Version */
+	int c   = -1;
+	int rc  = msg_read->readcount;
+	int *pi = (int*)(&msg_read->data[rc]);
+
+	rc += 4;
+	if ( rc <= msg_read->cursize )
+	{
+		c = LittleLong( *pi );
+		msg_read->readcount = rc;
+	}
+
+	return c;
+#endif
+}
+
+int MSG_ReadSizeInt (sizebuf_t *msg_read, int bytes)
+{
+	int c, i;
+	
+	if (msg_read->readcount+bytes > msg_read->cursize)
+		c = -1;
+	else
+	{
+		c = 0;
+		for (i = 0; i < bytes; i++)
+			c += msg_read->data[msg_read->readcount++] << (8*i);
+		// check sign bit and sign-extend to 32 bits if needed
+		if (bytes != sizeof(c) && (c & 1<<(8*bytes-1)))
+			for (; i < sizeof(c); i++)
+				c += 0xff<<(8*i);
+	}
+	
 	return c;
 }
 
@@ -897,14 +992,14 @@ char *MSG_ReadStringLine (sizebuf_t *msg_read)
 
 float MSG_ReadCoord (sizebuf_t *msg_read)
 {
-	return MSG_ReadShort(msg_read) * (1.0/8);
+	return MSG_ReadSizeInt(msg_read, coord_bytes) * (1.0/8);
 }
 
 void MSG_ReadPos (sizebuf_t *msg_read, vec3_t pos)
 {
-	pos[0] = MSG_ReadShort(msg_read) * (1.0/8);
-	pos[1] = MSG_ReadShort(msg_read) * (1.0/8);
-	pos[2] = MSG_ReadShort(msg_read) * (1.0/8);
+	pos[0] = MSG_ReadSizeInt(msg_read, coord_bytes) * (1.0/8);
+	pos[1] = MSG_ReadSizeInt(msg_read, coord_bytes) * (1.0/8);
+	pos[2] = MSG_ReadSizeInt(msg_read, coord_bytes) * (1.0/8);
 }
 
 float MSG_ReadAngle (sizebuf_t *msg_read)
@@ -1133,7 +1228,7 @@ int	memsearch (byte *start, int count, int search)
 }
 
 
-char *CopyString (char *in)
+char *CopyString (const char *in)
 {
 	char	*out;
 
@@ -1149,15 +1244,21 @@ void Info_Print (char *s)
 	char	key[512];
 	char	value[512];
 	char	*o;
-	int		l;
+	int		l, i;
 
 	if (*s == '\\')
 		s++;
 	while (*s)
 	{
 		o = key;
-		while (*s && *s != '\\')
+		i = 512;
+		while (--i && *s && *s != '\\')
 			*o++ = *s++;
+		if (*s && *s != '\\')
+		{
+			Com_Printf ("KEY TOO LONG\n");
+			return;
+		}
 
 		l = o - key;
 		if (l < 20)
@@ -1177,8 +1278,14 @@ void Info_Print (char *s)
 
 		o = value;
 		s++;
-		while (*s && *s != '\\')
+		i = 512;
+		while (--i && *s && *s != '\\')
 			*o++ = *s++;
+		if (*s && *s != '\\')
+		{
+			Com_Printf ("VALUE TOO LONG\n");
+			return;
+		}
 		*o = 0;
 
 		if (*s)
@@ -1603,7 +1710,7 @@ void Qcommon_Init (int argc, char **argv)
 	if (!dedicated->value)
 		S_StartMenuMusic();
 #endif
-	Com_Printf ("======== CRX Initialized ========\n\n");
+	Com_Printf ("======== Alien Arena Initialized ========\n\n");
 }
 
 /*
@@ -1660,10 +1767,13 @@ void Qcommon_Frame (int msec)
 		extern	int c_traces, c_brush_traces;
 		extern	int	c_pointcontents;
 
-		Com_Printf ("%4i traces  %4i points\n", c_traces, c_pointcontents);
-		c_traces = 0;
-		c_brush_traces = 0;
-		c_pointcontents = 0;
+		if ( c_traces > 0 || c_brush_traces > 0 || c_pointcontents > 0 )
+		{
+			Com_Printf ("%4i traces %4i brush %4i points\n", c_traces, c_brush_traces, c_pointcontents);
+			c_traces = 0;
+			c_brush_traces = 0;
+			c_pointcontents = 0;
+		}
 	}
 
 #if defined UNIX_VARIANT
@@ -1738,3 +1848,114 @@ Qcommon_Shutdown
 void Qcommon_Shutdown (void)
 {
 }
+
+
+
+
+
+
+
+
+
+#ifdef HAVE_ZLIB
+//This compression code was originally from the Lua branch. More of it will
+//eventually be merged when packet compression is added, but currently we 
+//are only decompressing zlib data and only need it on the client side.
+
+//Following function ripped off from R1Q2, then modified to use sizebufs
+int ZLibDecompress (sizebuf_t *in, sizebuf_t *out, int wbits) {
+	z_stream zs;
+	char *err_summary;
+	int result;
+
+	memset (&zs, 0, sizeof(zs));
+
+
+	zs.next_in = in->data + in->readcount;
+	zs.avail_in = 0;
+
+	zs.next_out = out->data + out->cursize;
+	zs.avail_out = out->maxsize - out->cursize;
+
+	result = inflateInit2(&zs, wbits);
+	if (result != Z_OK)
+	{
+		switch (result)
+		{
+			default: err_summary = "unknown"; break;
+			case Z_MEM_ERROR: err_summary = "system memory exhausted"; break;
+			case Z_STREAM_ERROR: err_summary = "invalid z_stream"; break;
+			case Z_VERSION_ERROR: err_summary = "incompatible zlib version"; break;
+		}
+		Com_Printf ("ZLib data error! Error %d on inflateInit.\nMessage: %s\n", result, zs.msg);
+		return 0;
+	}
+
+	zs.avail_in = in->cursize;
+
+	result = inflate(&zs, Z_FINISH);
+	if (result != Z_STREAM_END)
+	{
+	    switch (result)
+		{
+			default: err_summary = "unknown"; break;
+			case Z_STREAM_END: err_summary = "premature end of stream"; break;
+			case Z_NEED_DICT: err_summary = "missing preset dictionary"; break;
+			case Z_STREAM_ERROR: err_summary = "invalid z_stream"; break;
+			case Z_DATA_ERROR: err_summary = "corrupted data"; break;
+			case Z_BUF_ERROR: err_summary = "too much compressed data"; break;
+			case Z_OK: err_summary = "shouldn't happen with Z_FINISH"; break;
+		}
+		Com_Printf ("ZLib data error! Error %d (%s) on inflate.\nMessage: %s\n", result, err_summary, zs.msg);
+		return 0;
+	}
+
+	result = inflateEnd(&zs);
+	if (result != Z_OK)
+	{
+		switch (result)
+		{
+			default: err_summary = "unknown"; break;
+			case Z_STREAM_ERROR: err_summary = "invalid z_stream"; break;
+			case Z_DATA_ERROR: err_summary = "prematurely freed z_stream"; break;
+		}
+		Com_Printf ("ZLib data error! Error %d (%s) on inflateEnd.\nMessage: %s\n", result, err_summary, zs.msg);
+		return 0;
+	}
+
+	return zs.total_out;
+}
+#endif //HAVE_ZLIB
+
+void qdecompress (sizebuf_t *src, sizebuf_t *dst, int type){
+	int newsize = 0;
+
+	switch (type) {
+#ifdef HAVE_ZLIB
+		case compression_zlib_raw:
+			//raw DEFLATE data, no header/trailer
+			newsize = ZLibDecompress (src, dst, -15);
+			break;
+#endif
+#if 0 
+        case compression_lzo:
+            newsize = LzoDecompress (src, dst);
+            break;
+#endif
+#ifdef HAVE_ZLIB
+		case compression_zlib_header: 
+			//automatically detect gzip/zlib header/trailer
+			newsize = ZLibDecompress (src, dst, 47); 
+			break;
+#endif
+	} 
+        
+	if ( newsize == 0)
+		dst->cursize = 0;
+	else
+		dst->cursize += newsize;
+	
+}
+
+
+
