@@ -32,7 +32,7 @@ cvar_t	*cvar_vars;
 Cvar_InfoValidate
 ============
 */
-static qboolean Cvar_InfoValidate (char *s)
+static qboolean Cvar_InfoValidate (const char *s)
 {
 	if (strstr (s, "\\"))
 		return false;
@@ -48,7 +48,7 @@ static qboolean Cvar_InfoValidate (char *s)
 Cvar_FindVar
 ============
 */
-static cvar_t *Cvar_FindVar (char *var_name)
+static cvar_t *Cvar_FindVar (const char *var_name)
 {
 	cvar_t		*var;
 	unsigned int	i, hash_key;
@@ -66,7 +66,7 @@ static cvar_t *Cvar_FindVar (char *var_name)
 Cvar_VariableValue
 ============
 */
-float Cvar_VariableValue (char *var_name)
+float Cvar_VariableValue (const char *var_name)
 {
 	cvar_t	*var;
 
@@ -82,7 +82,7 @@ float Cvar_VariableValue (char *var_name)
 Cvar_VariableString
 ============
 */
-char *Cvar_VariableString (char *var_name)
+char *Cvar_VariableString (const char *var_name)
 {
 	cvar_t *var;
 
@@ -98,7 +98,7 @@ char *Cvar_VariableString (char *var_name)
 Cvar_CompleteVariable
 ============
 */
-char *Cvar_CompleteVariable (char *partial)
+char *Cvar_CompleteVariable (const char *partial)
 {
 	cvar_t		*cvar;
 	int		len, i;
@@ -130,7 +130,7 @@ Cvar_Allocate
 Creates a new variable's record
 ============
 */
-inline static cvar_t *Cvar_Allocate(char *var_name, char *var_value, int flags, unsigned int hash_key)
+inline static cvar_t *Cvar_Allocate(const char *var_name, const char *var_value, int flags, unsigned int hash_key)
 {
 	cvar_t *nvar;
 
@@ -143,11 +143,31 @@ inline static cvar_t *Cvar_Allocate(char *var_name, char *var_value, int flags,
 	nvar->integer = atoi (nvar->string);
 	nvar->flags = flags;
 	nvar->hash_key = hash_key;
+	nvar->description = NULL;
 
 	return nvar;
 }
 
 
+/*
+============
+Anon_Cvar_Allocate
+
+Initializes a new cvar struct holding just a value, with no name in it.
+Used in some places where a weak-typed variable is needed.
+============
+*/ 
+void Anon_Cvar_Set(cvar_t *nvar, const char *var_value)
+{
+	if (nvar->string)
+		Z_Free (nvar->string);
+	nvar->string = CopyString (var_value);
+	nvar->value = atof (nvar->string);
+	nvar->default_value = atof (nvar->string);
+	nvar->integer = atoi (nvar->string);
+}
+
+
 /*
 ============
 Cvar_AddBetween
@@ -155,7 +175,7 @@ Cvar_AddBetween
 Adds a variable between two others.
 ============
 */
-inline static cvar_t *Cvar_AddBetween(char *var_name, char *var_value, int flags, unsigned int hash_key,cvar_t **prev, cvar_t *next)
+inline static cvar_t *Cvar_AddBetween(const char *var_name, const char *var_value, int flags, unsigned int hash_key,cvar_t **prev, cvar_t *next)
 {
 	cvar_t *nvar;
 
@@ -191,7 +211,7 @@ If the variable already exists, the value will not be set
 The flags will be or'ed in if the variable exists.
 ============
 */
-cvar_t *Cvar_Get (char *var_name, char *var_value, int flags)
+cvar_t *Cvar_Get (const char *var_name, const char *var_value, int flags)
 {
 	cvar_t		*var, **prev;
 	unsigned int	i, hash_key;
@@ -236,7 +256,7 @@ sets the pointer to the variable through a parameter.
 ============
 */
 inline static qboolean Cvar_FindOrCreate (
-		char *var_name, char *var_value, int flags, cvar_t **found)
+		const char *var_name, const char *var_value, int flags, cvar_t **found)
 {
 	cvar_t		*var, **prev;
 	unsigned int	i, hash_key;
@@ -265,7 +285,7 @@ inline static qboolean Cvar_FindOrCreate (
 Cvar_Set2
 ============
 */
-cvar_t *Cvar_Set2 (char *var_name, char *value, qboolean force)
+cvar_t *Cvar_Set2 (const char *var_name, const char *value, qboolean force)
 {
 	cvar_t	*var;
 
@@ -355,7 +375,7 @@ cvar_t *Cvar_Set2 (char *var_name, char *value, qboolean force)
 Cvar_ForceSet
 ============
 */
-cvar_t *Cvar_ForceSet (char *var_name, char *value)
+cvar_t *Cvar_ForceSet (const char *var_name, const char *value)
 {
 	return Cvar_Set2 (var_name, value, true);
 }
@@ -365,9 +385,9 @@ cvar_t *Cvar_ForceSet (char *var_name, char *value)
 Cvar_Set
 ============
 */
-cvar_t *Cvar_Set (char *var_name, char *value)
+void Cvar_Set (const char *var_name, const char *value)
 {
-	return Cvar_Set2 (var_name, value, false);
+	(void)Cvar_Set2 (var_name, value, false);
 }
 
 /*
@@ -375,12 +395,12 @@ cvar_t *Cvar_Set (char *var_name, char *value)
 Cvar_FullSet
 ============
 */
-cvar_t *Cvar_FullSet (char *var_name, char *value, int flags)
+void Cvar_FullSet (const char *var_name, const char *value, int flags)
 {
 	cvar_t	*var;
 
 	if (!Cvar_FindOrCreate (var_name, value, flags, &var))
-		return var;
+		return;
 
 	var->modified = true;
 
@@ -394,8 +414,6 @@ cvar_t *Cvar_FullSet (char *var_name, char *value, int flags)
 	var->default_value = var->value;
 	var->integer = atoi(var->string);
 	var->flags = flags;
-
-	return var;
 }
 
 /*
@@ -403,7 +421,7 @@ cvar_t *Cvar_FullSet (char *var_name, char *value, int flags)
 Cvar_SetValue
 ============
 */
-void Cvar_SetValue (char *var_name, float value)
+void Cvar_SetValue (const char *var_name, float value)
 {
 	char	val[32];
 
@@ -545,6 +563,7 @@ void Cvar_WriteVariables (char *path)
 ============
 Cvar_List_f
 
+List all cvars, also displaying their documentation if appropriate
 ============
 */
 void Cvar_List_f (void)
@@ -574,11 +593,84 @@ void Cvar_List_f (void)
 			Com_Printf ("L");
 		else
 			Com_Printf (" ");
-		Com_Printf (" %s \"%s\"\n", var->name, var->string);
+		Com_Printf (" %s \"%s\"", var->name, var->string);
+		if (var->description)
+			Com_Printf (" - %s", var->description);
+		Com_Printf ("\n");
 	}
 	Com_Printf ("%i cvars\n", i);
 }
 
+/*
+============
+Cvar_Help_f
+
+If the cvar exists, display its current value and documentation
+============
+*/
+void Cvar_Help_f (void)
+{
+	cvar_t	*var;
+	char	*var_name;
+	int		flags;
+	
+	if (Cmd_Argc() != 2)
+	{
+		Com_Printf ("Usage: help <variable>\n");
+		Com_Printf ("Displays the cvar's type, value, and help string if present.\n");
+		return;
+	}
+	
+	var_name = Cmd_Argv(1);
+	var = Cvar_Get (var_name, 0, 0);
+	
+	if (var == NULL)
+	{
+		Com_Printf ("Nonexistant cvar \"%s\"\n", var_name);
+		return;
+	}
+	
+	Com_Printf ("Name: \"%s\"\n", var_name);
+	
+	flags = var->flags;
+	if (flags != 0)
+	{
+		Com_Printf ("Type: ");
+		
+#define HANDLE_FLAG(flag,str)\
+		if (flags & (flag))\
+		{\
+			Com_Printf (str);\
+			flags &= ~(flag);\
+			if (flags != 0) \
+				Com_Printf (", ");\
+		}
+		
+		HANDLE_FLAG (CVAR_ARCHIVE, "saved in config.cfg");
+		HANDLE_FLAG (CVAR_USERINFO, "player setting");
+		HANDLE_FLAG (CVAR_SERVERINFO, "server setting");
+		HANDLE_FLAG (CVAR_NOSET, "can be set only at the command line");
+		HANDLE_FLAG (CVAR_LATCH, "changes not applied until next game");
+		HANDLE_FLAG (CVAR_ROM, "read-only");
+		HANDLE_FLAG (CVAR_GAMEINFO, "gameplay setting");
+		HANDLE_FLAG (CVAR_PROFILE, "saved in profile.cfg");
+		
+		// Usually, at most one of these will be set.
+		HANDLE_FLAG (CVARDOC_BOOL, "0 for disabled, nonzero for enabled");
+		HANDLE_FLAG (CVARDOC_STR, "not a number");
+		HANDLE_FLAG (CVARDOC_FLOAT, "floating-point number");
+		HANDLE_FLAG (CVARDOC_INT, "integer");
+			
+#undef HANDLE_FLAG
+		
+		Com_Printf ("\n");
+	}
+	
+	Com_Printf ("Value: \"%s\"\n", var->string);
+	
+	if (var->description != NULL)
+		Com_Printf ("Description: %s\n", var->description);
+}
 
 qboolean userinfo_modified;
 
@@ -648,6 +740,27 @@ char	*Cvar_Serverinfo (void)
     return info;
 }
 
+void	Cvar_Describe(cvar_t *var, const char *description_string)
+{
+	if (var == NULL)
+		return;
+	if (var->description != NULL)
+		Z_Free (var->description);
+	if (description_string == NULL)
+		var->description = NULL;
+	else
+		var->description = CopyString (description_string);
+}
+
+void	Cvar_Describe_ByName(const char *var_name, const char *description_string)
+{
+	cvar_t *var;
+	if (var_name == NULL)
+		return;
+	var = Cvar_Get (var_name, 0, 0);
+	Cvar_Describe (var, description_string);
+}
+
 /*
 ============
 Cvar_Init
@@ -659,5 +772,6 @@ void Cvar_Init (void)
 {
 	Cmd_AddCommand ("set", Cvar_Set_f);
 	Cmd_AddCommand ("cvarlist", Cvar_List_f);
+	Cmd_AddCommand ("help", Cvar_Help_f);
 
 }
