@@ -137,6 +137,10 @@ they shouldn't.
 #include <sys/stat.h>
 #endif
 
+#if defined HAVE_ZLIB
+# include <zlib.h>
+#endif
+
 #include <errno.h>
 
 #include "qcommon.h"
@@ -163,9 +167,10 @@ char fs_gamesearch[GAME_SEARCH_SLOTS][MAX_OSPATH];
 #define BOT_SEARCH_SLOTS 3
 char fs_botsearch[BOT_SEARCH_SLOTS][MAX_OSPATH];
 
-/** \brief Initialize paths: data1, arena, botinfo, etc.
+/**
+ * @brief Initialize paths: data1, arena, botinfo, etc.
  *
- * \note This gets executed after command line "+set" commands are executed
+ * @note This gets executed after command line "+set" commands are executed
  *       the first time. But, before any other commands from command line
  *       or .cfg files are executed.
  */
@@ -269,16 +274,9 @@ static void FS_init_paths( void )
 	}
 	else
 	{ // was empty, set to "arena"
-#if 0
-		// this can't be right because it could recurse back here
-		fs_gamedirvar = Cvar_ForceSet("game", GAME_GAMEDATA );
-		Com_sprintf( game_gamedata, sizeof(game_gamedata), "%s/%s",
-				fs_datadir, GAME_GAMEDATA );
-#else
 		fs_gamedirvar = Cvar_ForceSet("gamedir", GAME_GAMEDATA );
 		Com_sprintf( game_gamedata, sizeof(game_gamedata), "%s/%s",
 				fs_datadir, GAME_GAMEDATA );
-#endif
 	}
 
 	// set path for "botinfo"
@@ -477,8 +475,11 @@ FS_FullPath
 Given a relative path to a file
   search for the file in the installation-dependent filesystem hierarchy
   if found, return true, with the constructed full path
-  otherwise, return false, with a constructed writeable full path
+  otherwise return false, with full_path set to zero-length string
 
+Corrected 2012-11, was:
+  otherwise, return false, with a constructed writeable full path
+  (which clearly it might not, and could not do).
 ===
  */
 qboolean FS_FullPath( char *full_path, size_t pathsize, const char *relative_path )
@@ -487,6 +488,8 @@ qboolean FS_FullPath( char *full_path, size_t pathsize, const char *relative_pat
 	char * to_search;
 	qboolean found = false;
 
+	*full_path = 0;
+
 	if ( strlen( relative_path ) >= MAX_QPATH )
 	{
 		Com_DPrintf("FS_FullPath: relative path size error: %s\n", relative_path );
@@ -749,6 +752,76 @@ int FS_LoadFile (char *path, void **buffer)
 	return len;
 }
 
+/*
+============
+FS_LoadFile_TryStatic
+
+Given relative path, and assuming statbuffer is already allocated with at 
+least statbuffer_len bytes,
+ if buffer or statbuffer arg is NULL, just return the file length. Otherwise,
+   If the file length is less than statbuffer_len, load file contents into it
+   and return it in the buffer output pointer. Otherwise,
+     Z_malloc a buffer, read the file into it, return pointer to the new
+     buffer.
+
+This is a variation on FS_LoadFile which attempts to reduce needless dynamic 
+memory allocations, by using a static buffer when possible.
+
+The calling function is responsible for comparing the value of *buffer with
+statbuffer, and determining whether or not to use FS_FreeFile on it.
+============
+*/
+int FS_LoadFile_TryStatic (char *path, void **buffer, void *statbuffer, size_t statbuffer_len)
+{
+	FILE	*h;
+	byte	*buf = NULL;
+	int		len;
+	char lc_path[MAX_OSPATH];
+
+	len = FS_FOpenFile (path, &h);
+
+	//-JR
+	if (!h)
+	{
+		Q_strncpyz2( lc_path, FS_TolowerPath( path ), sizeof(lc_path) );
+		if ( strcmp( path, lc_path ) )
+		{ // lowercase conversion changed something
+			len = FS_FOpenFile( lc_path, &h);
+		}
+	}
+
+	if (!h)
+	{
+		if (buffer)
+			*buffer = NULL;
+		return -1;
+	}
+
+	if (buffer == NULL || statbuffer == NULL || statbuffer_len == 0)
+	{
+		FS_FCloseFile (h);
+		return len;
+	}
+	
+	if (statbuffer_len > len)
+	{
+		memset (statbuffer, 0, statbuffer_len);
+		buf = statbuffer;
+	}
+	else
+	{
+		buf = Z_Malloc(len+1);
+	}
+	buf[len] = 0;
+	*buffer = buf;
+
+	FS_Read (buf, len, h);
+
+	FS_FCloseFile (h);
+
+	return len;
+}
+
 /*
 =============
 FS_FreeFile
@@ -840,10 +913,10 @@ void FS_SetGamedir (char *dir)
 		Cbuf_AddText ("vid_restart\nsnd_restart\n");
 	}
 
+	// note: FullSet triggers sending info to client
 	if ( !strcmp( dir, BASE_GAMEDATA ) || ( *dir == 0 ) )
-	{
+	{ // either "data1" or "", set to empty string.
 		Cvar_FullSet ("gamedir", "", CVAR_SERVERINFO|CVAR_NOSET);
-		Cvar_FullSet ("game", "", CVAR_LATCH|CVAR_SERVERINFO);
 	}
 	else
 	{
@@ -936,6 +1009,7 @@ FS_ListFilesInFS(char *findname, int *numfiles, unsigned musthave, unsigned cant
 	int		tmpnfiles;		/* Temp number of files. */
 	char		**tmplist;		/* Temporary list of files. */
 	char		**list;			/* List of files found. */
+	char		**new_list;
 	char		path[MAX_OSPATH];	/* Temporary path. */
 	int s;
 
@@ -949,7 +1023,13 @@ FS_ListFilesInFS(char *findname, int *numfiles, unsigned musthave, unsigned cant
 		if (tmplist != NULL)
 		{
 			nfiles += tmpnfiles;
-			list = realloc(list, nfiles * sizeof(char *));
+			new_list = realloc(list, nfiles * sizeof(char *));
+			if (new_list == NULL) {
+				FS_FreeFileList (tmplist, tmpnfiles);
+				Com_Printf ("WARN: SYSTEM MEMORY EXHAUSTION!\n");
+				break;
+			}
+			list = new_list;
 			for (i = 0, j = nfiles - tmpnfiles; i < tmpnfiles; i++, j++)
 			{ // copy from full path to relative path
 				list[j] = strdup( tmplist[i] + strlen( fs_gamesearch[s] ) + 1  );
@@ -987,7 +1067,13 @@ FS_ListFilesInFS(char *findname, int *numfiles, unsigned musthave, unsigned cant
 	if (nfiles > 0)
 	{
 		nfiles++;
-		list = realloc(list, nfiles * sizeof(char *));
+		new_list = realloc(list, nfiles * sizeof(char *));
+		if (new_list == NULL) {
+			Com_Printf ("WARN: SYSTEM MEMORY EXHAUSTION!\n");
+			nfiles--; // convert previous entry into a guard
+		} else {
+			list = new_list;
+		}
 		list[nfiles - 1] = NULL;
 	} else
 	{
@@ -1153,4 +1239,8 @@ void FS_InitFilesystem (void)
 	FS_CreatePath( dummy_path );
 #endif
 
+#if defined HAVE_ZLIB && !defined DEDICATED_ONLY
+	Com_Printf("using zlib version %s\n", zlibVersion() );
+#endif
+
 }
