@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "game/q_shared.h"
 
 #if !defined VERSION
-#define	VERSION		"7.45"
+#define	VERSION		"7.66"
 #endif
 
 /* ---- Default locations of game data ---*/
@@ -33,7 +33,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define BASE_GAMEDATA "data1"
 #endif
 
-#if !defined MOD_GAMEDATA
+#if !defined GAME_GAMEDATA
 #define GAME_GAMEDATA "arena"
 #endif
 
@@ -49,7 +49,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
 #define MENU_STATIC_WIDTH	720.0f
-#define MENU_FONT_SIZE 12
 
 #define DEFAULTMODEL		"martianenforcer"
 #define DEFAULTSKIN			"default"
@@ -150,6 +149,7 @@ int	MSG_ReadChar (sizebuf_t *sb);
 int	MSG_ReadByte (sizebuf_t *sb);
 int	MSG_ReadShort (sizebuf_t *sb);
 int	MSG_ReadLong (sizebuf_t *sb);
+int MSG_ReadSizeInt (sizebuf_t *msg_read, int bytes);
 float	MSG_ReadFloat (sizebuf_t *sb);
 char	*MSG_ReadString (sizebuf_t *sb);
 char	*MSG_ReadStringLine (sizebuf_t *sb);
@@ -187,7 +187,7 @@ void COM_AddParm (char *parm);
 void COM_Init (void);
 void COM_InitArgv (int argc, char **argv);
 
-char *CopyString (char *in);
+char *CopyString (const char *in);
 
 //============================================================================
 
@@ -500,55 +500,68 @@ interface from being ambiguous.
 
 extern	cvar_t	*cvar_vars;
 
-cvar_t *Cvar_Get (char *var_name, char *value, int flags);
-// creates the variable if it doesn't exist, or returns the existing one
-// if it exists, the value will not be changed, but flags will be ORed in
-// that allows variables to be unarchived without needing bitflags
+// Creates the variable if it doesn't exist, or returns the existing one
+// if it exists, the value will not be changed, but flags will be ORed in.
+// That allows variables to be unarchived without needing bitflags. NOTE: the
+// value and name strings are copied, so you don't need to keep track of any
+// strings you use with this function.
+cvar_t *Cvar_Get (const char *var_name, const char *value, int flags);
 
-cvar_t 	*Cvar_Set (char *var_name, char *value);
 // will create the variable if it doesn't exist
+void	Cvar_Set (const char *var_name, const char *value);
 
-cvar_t *Cvar_ForceSet (char *var_name, char *value);
 // will set the variable even if NOSET or LATCH
+cvar_t *Cvar_ForceSet (const char *var_name, const char *value);
 
-cvar_t 	*Cvar_FullSet (char *var_name, char *value, int flags);
+void	Cvar_FullSet (const char *var_name, const char *value, int flags);
 
-void	Cvar_SetValue (char *var_name, float value);
 // expands value to a string and calls Cvar_Set
+void	Cvar_SetValue (const char *var_name, float value);
 
-float	Cvar_VariableValue (char *var_name);
 // returns 0 if not defined or non numeric
+float	Cvar_VariableValue (const char *var_name);
 
-char	*Cvar_VariableString (char *var_name);
 // returns an empty string if not defined
+char	*Cvar_VariableString (const char *var_name);
 
-char 	*Cvar_CompleteVariable (char *partial);
 // attempts to match a partial variable name for command line completion
 // returns NULL if nothing fits
+char 	*Cvar_CompleteVariable (const char *partial);
 
-void	Cvar_GetLatchedVars (void);
 // any CVAR_LATCHED variables that have been set will now take effect
+void	Cvar_GetLatchedVars (void);
 
-qboolean Cvar_Command (void);
 // called by Cmd_ExecuteString when Cmd_Argv(0) doesn't match a known
 // command.  Returns true if the command was a variable reference that
 // was handled. (print or change)
+qboolean Cvar_Command (void);
 
-void 	Cvar_WriteVariables (char *path);
 // appends lines containing "set variable value" for all variables
 // with the archive flag set to true.
+void 	Cvar_WriteVariables (char *path);
 
+// setup the cvar subsystem and related console commands
 void	Cvar_Init (void);
 
-char	*Cvar_Userinfo (void);
 // returns an info string containing all the CVAR_USERINFO cvars
+char	*Cvar_Userinfo (void);
 
-char	*Cvar_Serverinfo (void);
 // returns an info string containing all the CVAR_SERVERINFO cvars
+char	*Cvar_Serverinfo (void);
+
+// Initializes a new cvar struct holding just a value, with no name in it.
+// Used in some places where a weak-typed variable is needed.
+void	Anon_Cvar_Set(cvar_t *nvar, const char *var_value);
+
+// Copies the description string into the cvar. Useful for documenting cvars.
+// The description string is copied, so it need not be kept in memory for the
+// life of the cvar.
+void	Cvar_Describe(cvar_t *var, const char *description_string);
+void	Cvar_Describe_ByName(const char *var_name, const char *description_string);
 
-extern	qboolean	userinfo_modified;
 // this is set each time a CVAR_USERINFO variable is changed
 // so that the client knows to send it to the server
+extern	qboolean	userinfo_modified;
 
 /*
 ==============================================================
@@ -576,7 +589,7 @@ typedef struct
 	byte	ip[4];
 	byte	ipx[10];
 
-	unsigned short	port;
+	unsigned short	port; /* network byte order (bigendian) */
 } netadr_t;
 
 void		NET_Init (void);
@@ -711,6 +724,10 @@ qboolean	CM_HeadnodeVisible (int headnode, byte *visbits);
 void		CM_WritePortalState (FILE *f);
 void		CM_ReadPortalState (FILE *f);
 
+qboolean 	CM_inPVS (vec3_t p1, vec3_t p2);
+qboolean	CM_inPVS_leafs (int leafnum1, int leafnum2);
+qboolean 	CM_inPHS (vec3_t p1, vec3_t p2);
+
 extern char map_name[MAX_QPATH];
 
 /*
@@ -724,6 +741,8 @@ Common between server and client so prediction matches
 */
 
 extern float pm_airaccelerate;
+qboolean remoteserver_jousting;
+int	remoteserver_runspeed;
 
 void Pmove (pmove_t *pmove);
 
@@ -751,9 +770,7 @@ void	FS_FCloseFile (FILE *f);
 // note: this can't be called from another DLL, due to MS libc issues
 
 int		FS_LoadFile (char *path, void **buffer);
-int		FS_LoadFileZ (const char *path, void **buffer); // jit - null-terminated
-// a null buffer will just return the file length without loading
-// a -1 length is not present
+int		FS_LoadFile_TryStatic (char *path, void **buffer, void *statbuffer, size_t statbuffer_len);
 
 void	FS_Read (void *buffer, int len, FILE *f);
 // properly handles partial reads
@@ -810,6 +827,7 @@ float	crand(void);	// -1 to 1
 extern	cvar_t	*developer;
 extern	cvar_t	*dedicated;
 extern	cvar_t	*host_speeds;
+
 extern	cvar_t	*log_stats;
 
 extern	FILE *log_stats_file;
@@ -833,11 +851,13 @@ void Qcommon_Shutdown (void);
 extern	vec3_t	bytedirs[NUMVERTEXNORMALS];
 
 // this is in the client code, but can be used for debugging from server
-void SCR_DebugGraph (float value, int color);
+void SCR_DebugGraph (float value, const float color[]);
 
 //used for render effect
 void CL_GlassShards(vec3_t org, vec3_t dir, int count);
 
+#define static_array_size(array) (sizeof(array)/sizeof((array)[0]))
+
 /*
 ==============================================================
 
@@ -884,6 +904,12 @@ void SV_Init (void);
 void SV_Shutdown (char *finalmsg, qboolean reconnect);
 void SV_Frame (int msec);
 
+
+//
+// compression
+//
+void qdecompress (sizebuf_t *src, sizebuf_t *dst, int type);
+
 #endif /* Q_COMMON_H_ */
 
 
