@@ -108,6 +108,7 @@ extern  void ( APIENTRY * qglDisableClientState )(GLenum array);
 extern  void ( APIENTRY * qglDrawArrays )(GLenum mode, GLint first, GLsizei count);
 extern  void ( APIENTRY * qglDrawBuffer )(GLenum mode);
 extern  void ( APIENTRY * qglDrawElements )(GLenum mode, GLsizei count, GLenum type, const GLvoid *indices);
+extern  void ( APIENTRY * qglDrawRangeElements )(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid *indices);
 extern  void ( APIENTRY * qglDrawPixels )(GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels);
 extern  void ( APIENTRY * qglEdgeFlag )(GLboolean flag);
 extern  void ( APIENTRY * qglEdgeFlagPointer )(GLsizei stride, const GLvoid *pointer);
@@ -488,6 +489,7 @@ typedef void (APIENTRY * PFNGLUNIFORM2FARBPROC) (GLint location, GLfloat v0, GLf
 typedef void (APIENTRY * PFNGLUNIFORM1IARBPROC) (GLint location, GLint v0);
 typedef void (APIENTRY * PFNGLUNIFORM1FARBPROC) (GLint location, GLfloat v0);
 typedef void (APIENTRY * PFNGLUNIFORMMATRIX3FVARBPROC) (GLint location, GLsizei count, GLboolean transpose, GLfloat [3][3]); 
+typedef void (APIENTRY * PFNGLUNIFORMMATRIX3X4FVPROC) (GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
 typedef void (APIENTRY * PFNGLVERTEXATTRIBPOINTERARBPROC) (GLuint, GLint, GLenum, GLboolean, GLsizei, const GLvoid *);
 typedef void (APIENTRY * PFNGLENABLEVERTEXATTRIBARRAYARBPROC) (GLuint);
 typedef void (APIENTRY * PFNGLBINDATTRIBLOCATIONARBPROC) (GLhandleARB programObj, GLuint index, const GLcharARB *name);
@@ -711,7 +713,7 @@ extern	PFNGLDELETEOBJECTARBPROC			glDeleteObjectARB;
 extern	PFNGLUSEPROGRAMOBJECTARBPROC		glUseProgramObjectARB;
 extern	PFNGLCREATESHADEROBJECTARBPROC		glCreateShaderObjectARB;
 extern  PFNGLSHADERSOURCEARBPROC			glShaderSourceARB;		
-extern	PFNGLCOMPILESHADERARBPROC			glCompileShaderARB;
+extern	PFNGLCOMPILESHADERPROC				glCompileShader;
 extern	PFNGLGETOBJECTPARAMETERIVARBPROC	glGetObjectParameterivARB;
 extern	PFNGLATTACHOBJECTARBPROC			glAttachObjectARB;
 extern	PFNGLGETINFOLOGARBPROC				glGetInfoLogARB;
@@ -722,6 +724,10 @@ extern	PFNGLUNIFORM2FARBPROC				glUniform2fARB;
 extern	PFNGLUNIFORM1IARBPROC				glUniform1iARB;
 extern  PFNGLUNIFORM1FARBPROC				glUniform1fARB;
 extern  PFNGLUNIFORMMATRIX3FVARBPROC		glUniformMatrix3fvARB;
+#ifndef PFNGLUNIFORMMATRIX3X4FVARBPROC //FIXME HACK to get it compiling on Linux
+typedef void (APIENTRYP PFNGLUNIFORMMATRIX3X4FVARBPROC) (GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
+#endif
+extern  PFNGLUNIFORMMATRIX3X4FVARBPROC		glUniformMatrix3x4fvARB;
 extern	PFNGLVERTEXATTRIBPOINTERARBPROC		glVertexAttribPointerARB;
 extern	PFNGLENABLEVERTEXATTRIBARRAYARBPROC glEnableVertexAttribArrayARB;
 extern	PFNGLDISABLEVERTEXATTRIBARRAYARBPROC glDisableVertexAttribArrayARB;
