@@ -108,191 +108,6 @@ void Draw_InitLocal (void)
 	RefreshFont();
 }
 
-/*
-================
-Draw_Char
-
-Draws one 8*8 graphics character with 0 being transparent.
-It can be clipped to the top of the screen to allow the console to be
-smoothly scrolled off.
-================
-*/
-void Draw_Char (int x, int y, int num)
-{
-	int				row, col;
-	float			frow, fcol, size;
-
-	num &= 255;
-
-	if ( (num&127) == 32 )
-		return;		// space
-
-	if (y <= -8)
-		return;			// totally off screen
-
-	row = num>>4;
-	col = num&15;
-
-	frow = row*0.0625;
-	fcol = col*0.0625;
-	size = 0.0625;
-
-	GL_Bind (draw_chars->texnum);
-
-	qglBegin (GL_QUADS);
-	qglColor4f( 1,1,1,1 );
-	qglTexCoord2f (fcol, frow);
-	qglVertex2f (x, y);
-	qglTexCoord2f (fcol + size, frow);
-	qglVertex2f (x+8, y);
-	qglTexCoord2f (fcol + size, frow + size);
-	qglVertex2f (x+8, y+8);
-	qglTexCoord2f (fcol, frow + size);
-	qglVertex2f (x, y+8);
-	qglEnd ();
-}
-static byte R_FloatToByte( float x )
-{
-	union {
-		float			f;
-		unsigned int	i;
-	} f2i;
-
-	// shift float to have 8bit fraction at base of number
-	f2i.f = x + 32768.0f;
-	f2i.i &= 0x7FFFFF;
-
-	// then read as integer and kill float bits...
-	return ( byte )min( f2i.i, 255 );
-}
-
-void Draw_ColorChar (int x, int y, int num, vec4_t color)
-{
-	int				row, col;
-	float			frow, fcol, size;
-	byte			colors[4];
-
-	colors[0] = R_FloatToByte( color[0] );
-	colors[1] = R_FloatToByte( color[1] );
-	colors[2] = R_FloatToByte( color[2] );
-	colors[3] = R_FloatToByte( color[3] );
-
-	num &= 255;
-
-	if ( (num&127) == 32 )
-		return;		// space
-
-	if (y <= -8)
-		return;			// totally off screen
-
-	row = num>>4;
-	col = num&15;
-
-	frow = row*0.0625;
-	fcol = col*0.0625;
-	size = 0.0625;
-
-	GL_Bind (draw_chars->texnum);
-
-	qglColor4ubv( colors );
-	GL_TexEnv(GL_MODULATE);
-	qglBegin (GL_QUADS);
-	qglTexCoord2f (fcol, frow);
-	qglVertex2f (x, y);
-	qglTexCoord2f (fcol + size, frow);
-	qglVertex2f (x+8, y);
-	qglTexCoord2f (fcol + size, frow + size);
-	qglVertex2f (x+8, y+8);
-	qglTexCoord2f (fcol, frow + size);
-	qglVertex2f (x, y+8);
-	qglEnd ();
-	qglColor4f( 1,1,1,1 );
-}
-
-void Draw_ScaledChar (float x, float y, int num, float scale, int from_menu)
-
-{
-	int				row, col;
-	float			frow, fcol, size;
-
-	num &= 255;
-
-	if ( (num&127) == 32 )
-		return;		// space
-
-	if (y <= -8)
-		return;			// totally off screen
-
-	row = num>>4;
-	col = num&15;
-
-	frow = row*0.0625;
-	fcol = col*0.0625;
-	size = 0.0625;
-
-	if(from_menu)
-		GL_Bind(menu_chars->texnum);
-	else
-		GL_Bind(draw_chars->texnum);
-
-	qglColor4f( 1,1,1,1 );
-	qglBegin (GL_QUADS);
-	qglTexCoord2f (fcol, frow);
-	qglVertex2f (x, y);
-	qglTexCoord2f (fcol + size, frow);
-	qglVertex2f (x+scale, y);
-	qglTexCoord2f (fcol + size, frow + size);
-	qglVertex2f (x+scale, y+scale);
-	qglTexCoord2f (fcol, frow + size);
-	qglVertex2f (x, y+scale);
-	qglEnd();
-
-}
-void Draw_ScaledColorChar (float x, float y, int num, vec4_t color, float scale, int from_menu)
-{
-	int				row, col;
-	float			frow, fcol, size;
-	byte			colors[4];
-
-	colors[0] = R_FloatToByte( color[0] );
-	colors[1] = R_FloatToByte( color[1] );
-	colors[2] = R_FloatToByte( color[2] );
-	colors[3] = R_FloatToByte( color[3] );
-
-	num &= 255;
-
-	if ( (num&127) == 32 )
-		return;		// space
-
-	if (y <= -8)
-		return;			// totally off screen
-
-	row = num>>4;
-	col = num&15;
-
-	frow = row*0.0625;
-	fcol = col*0.0625;
-	size = 0.0625;
-
-	if(from_menu)
-		GL_Bind(menu_chars->texnum);
-	else
-		GL_Bind(draw_chars->texnum);
-
-	qglColor4ubv( colors );
-	GL_TexEnv(GL_MODULATE);
-	qglBegin (GL_QUADS);
-	qglTexCoord2f (fcol, frow);
-	qglVertex2f (x, y);
-	qglTexCoord2f (fcol + size, frow);
-	qglVertex2f (x+scale, y);
-	qglTexCoord2f (fcol + size, frow + size);
-	qglVertex2f (x+scale, y+scale);
-	qglTexCoord2f (fcol, frow + size);
-	qglVertex2f (x, y+scale);
-	qglEnd ();
-	qglColor4f( 1,1,1,1 );
-}
 /*
 =============
 R_RegisterPic
@@ -327,12 +142,12 @@ image_t	*R_RegisterParticlePic (const char *name)
 	if (name[0] != '/' && name[0] != '\\')
 	{
 		Com_sprintf (fullname, sizeof(fullname), "particles/%s.tga", name);
-		gl = GL_FindImage (fullname, it_pic);
+		gl = GL_FindImage (fullname, it_particle);
 	}
 	else
 	{ // 2010-10 match above workaround (paranoid)
 		strcpy( fullname, &name[1] );
-		gl = GL_FindImage( fullname, it_pic );
+		gl = GL_FindImage( fullname, it_particle );
 	}
 
 	return gl;
@@ -395,13 +210,23 @@ image_t	*R_RegisterPlayerIcon (const char *name)
 
 	return gl;
 }
+
+/*
+=============
+Draw_PicExists
+=============
+*/
+qboolean Draw_PicExists (const char *pic)
+{
+	return R_RegisterPic (pic) != NULL;
+}
+
 /*
 =============
 Draw_GetPicSize
 =============
 */
-
-void Draw_GetPicSize (int *w, int *h, char *pic)
+void Draw_GetPicSize (int *w, int *h, const char *pic)
 {
 	image_t *gl;
 
@@ -416,15 +241,35 @@ void Draw_GetPicSize (int *w, int *h, char *pic)
 }
 
 #define DIV254BY255 (0.9960784313725490196078431372549f)
-void Draw_ShaderPic (image_t *gl, float alphaval)
+/*
+=============
+Draw_AlphaStretchPic
+- Note: If tiling is true, the texture wrapping flags are adjusted to prevent
+        gaps from appearing if the texture is tiled with itself or with other
+        textures. This adjustment is permanent, although it would be easy to
+        change the code to undo it after rendering.
+=============
+*/
+enum draw_tiling_s
 {
-	rscript_t *rs = NULL;
+	draw_without_tiling,
+	draw_with_tiling
+};
+void Draw_AlphaStretchImage (float x, float y, float w, float h, const image_t *gl, float alphaval, enum draw_tiling_s tiling)
+{
+	rscript_t *rs;
 	float	alpha,s,t;
 	rs_stage_t *stage;
 	char shortname[MAX_QPATH];
-
+	float xscale, yscale;
+	float cropped_x, cropped_y, cropped_w, cropped_h;
+	
+	if (scrap_dirty)
+		Scrap_Upload ();
+	
 	COM_StripExtension ( gl->name, shortname );
-	rs=RS_FindScript(shortname);
+	
+	rs = gl->script;
 
 	//if we draw the red team bar, we are on red team
 	if(!strcmp(shortname, "pics/i_team1"))
@@ -433,7 +278,7 @@ void Draw_ShaderPic (image_t *gl, float alphaval)
 		r_teamColor = 2;
 
 	R_InitQuadVarrays();
-
+	
 	if (!rs)
 	{
 		qglDisable (GL_ALPHA_TEST);
@@ -441,7 +286,21 @@ void Draw_ShaderPic (image_t *gl, float alphaval)
 		GLSTATE_DISABLE_ALPHATEST
 		GLSTATE_ENABLE_BLEND
 		GL_TexEnv( GL_MODULATE );
-
+		
+		xscale = (float)w/(float)gl->upload_width;
+		yscale = (float)h/(float)gl->upload_height;
+		
+		cropped_x = x + xscale*(float)gl->crop_left;
+		cropped_y = y + yscale*(float)gl->crop_top;
+	
+		cropped_w = xscale*(float)gl->crop_width; 
+		cropped_h = yscale*(float)gl->crop_height;
+		
+		VA_SetElem2(vert_array[0], cropped_x,			cropped_y);
+		VA_SetElem2(vert_array[1], cropped_x+cropped_w, cropped_y);
+		VA_SetElem2(vert_array[2], cropped_x+cropped_w, cropped_y+cropped_h);
+		VA_SetElem2(vert_array[3], cropped_x,			cropped_y+cropped_h);
+	
 		qglColor4f(1,1,1, alphaval);
 
 		//set color of hud by team
@@ -459,17 +318,29 @@ void Draw_ShaderPic (image_t *gl, float alphaval)
 		VA_SetElem4(col_array[2], 1,1,1,1);
 		VA_SetElem4(col_array[3], 1,1,1,1);
 		GL_Bind (gl->texnum);
-		VA_SetElem2(tex_array[0],gl->sl, gl->tl);
-		VA_SetElem2(tex_array[1],gl->sh, gl->tl);
-		VA_SetElem2(tex_array[2],gl->sh, gl->th);
-		VA_SetElem2(tex_array[3],gl->sl, gl->th);
-		qglDrawArrays (GL_QUADS, 0, 4);
+		if (tiling == draw_with_tiling)
+		{
+			qglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
+			qglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
+		}
+		VA_SetElem2(tex_array[0], gl->crop_sl, gl->crop_tl);
+		VA_SetElem2(tex_array[1], gl->crop_sh, gl->crop_tl);
+		VA_SetElem2(tex_array[2], gl->crop_sh, gl->crop_th);
+		VA_SetElem2(tex_array[3], gl->crop_sl, gl->crop_th);
+		R_DrawVarrays (GL_QUADS, 0, 4);
 		qglEnable (GL_ALPHA_TEST);
 		qglDisable (GL_BLEND);
+		GLSTATE_DISABLE_BLEND
+		GLSTATE_ENABLE_ALPHATEST
 		R_KillVArrays();
 	}
 	else
 	{
+		VA_SetElem2(vert_array[0], x,	y);
+		VA_SetElem2(vert_array[1], x+w, y);
+		VA_SetElem2(vert_array[2], x+w, y+h);
+		VA_SetElem2(vert_array[3], x,	y+h);
+	
 		RS_ReadyScript(rs);
 
 		stage=rs->stage;
@@ -545,7 +416,7 @@ void Draw_ShaderPic (image_t *gl, float alphaval)
 			RS_SetTexcoords2D (stage, &s, &t);
 			VA_SetElem2(tex_array[2],s, t);
 
-			qglDrawArrays(GL_QUADS,0,4);
+			R_DrawVarrays(GL_QUADS, 0, 4);
 
 			qglColor4f(1,1,1,1);
 			if (stage->colormap.enabled)
@@ -563,12 +434,7 @@ void Draw_ShaderPic (image_t *gl, float alphaval)
 	}
 }
 
-/*
-=============
-Draw_StretchPic
-=============
-*/
-void Draw_StretchPic (int x, int y, int w, int h, char *pic)
+void Draw_AlphaStretchTilingPic (float x, float y, float w, float h, const char *pic, float alphaval)
 {
 	image_t *gl;
 
@@ -577,16 +443,8 @@ void Draw_StretchPic (int x, int y, int w, int h, char *pic)
 	{
 		return;
 	}
-
-	if (scrap_dirty)
-		Scrap_Upload ();
-
-	VA_SetElem2(vert_array[0],x, y);
-	VA_SetElem2(vert_array[1],x+w, y);
-	VA_SetElem2(vert_array[2],x+w, y+h);
-	VA_SetElem2(vert_array[3],x, y+h);
-
-	Draw_ShaderPic(gl, DIV254BY255);
+	
+	Draw_AlphaStretchImage (x, y, w, h, gl, alphaval, draw_with_tiling);
 }
 
 /*
@@ -594,7 +452,7 @@ void Draw_StretchPic (int x, int y, int w, int h, char *pic)
 Draw_AlphaStretchPic
 =============
 */
-void Draw_AlphaStretchPic (int x, int y, int w, int h, char *pic, float alphaval)
+void Draw_AlphaStretchPic (float x, float y, float w, float h, const char *pic, float alphaval)
 {
 	image_t *gl;
 
@@ -603,47 +461,18 @@ void Draw_AlphaStretchPic (int x, int y, int w, int h, char *pic, float alphaval
 	{
 		return;
 	}
-
-	if (scrap_dirty)
-		Scrap_Upload ();
-
-	VA_SetElem2(vert_array[0],x, y);
-	VA_SetElem2(vert_array[1],x+w, y);
-	VA_SetElem2(vert_array[2],x+w, y+h);
-	VA_SetElem2(vert_array[3],x, y+h);
-
-	Draw_ShaderPic(gl, alphaval);
+	
+	Draw_AlphaStretchImage (x, y, w, h, gl, alphaval, draw_without_tiling);
 }
 
 /*
 =============
-Draw_Pic
+Draw_StretchPic
 =============
 */
-void Draw_Pic (int x, int y, char *pic)
+void Draw_StretchPic (float x, float y, float w, float h, const char *pic)
 {
-	image_t *gl;
-	int w, h;
-
-	gl = R_RegisterPic (pic);
-	if (!gl)
-	{
-//		Com_Printf ("Can't find pic: %s\n", pic);
-		return;
-	}
-
-	w = gl->width;
-	h = gl->height;
-
-	if (scrap_dirty)
-		Scrap_Upload ();
-
-	VA_SetElem2(vert_array[0],x, y);
-	VA_SetElem2(vert_array[1],x+w, y);
-	VA_SetElem2(vert_array[2],x+w, y+h);
-	VA_SetElem2(vert_array[3],x, y+h);
-
-	Draw_ShaderPic(gl, DIV254BY255);
+	Draw_AlphaStretchPic (x, y, w, h, pic, DIV254BY255);
 }
 
 /*
@@ -651,35 +480,31 @@ void Draw_Pic (int x, int y, char *pic)
 Draw_ScaledPic
 =============
 */
-
-void Draw_ScaledPic (int x, int y, float scale, char *pic)
+void Draw_ScaledPic (float x, float y, float scale, const char *pic)
 {
 	image_t *gl;
-	int w, h;
-	float xoff, yoff;
+	float w, h;
 
 	gl = R_RegisterPic (pic);
 	if (!gl)
 	{
-//		Com_Printf ("Can't find pic: %s\n", pic);
 		return;
 	}
 
-	w = gl->width;
-	h = gl->height;
+	w = (float)gl->width*scale;
+	h = (float)gl->height*scale;
+	
+	Draw_AlphaStretchImage (x, y, w, h, gl, DIV254BY255, draw_without_tiling);
+}
 
-	xoff = (w*scale-w);
-	yoff = (h*scale-h);
-
-	if (scrap_dirty)
-		Scrap_Upload ();
-
-	VA_SetElem2(vert_array[0],x, y);
-	VA_SetElem2(vert_array[1],x+w+xoff, y);
-	VA_SetElem2(vert_array[2],x+w+xoff, y+h+yoff);
-	VA_SetElem2(vert_array[3],x, y+h+yoff);
-
-	Draw_ShaderPic(gl, DIV254BY255);
+/*
+=============
+Draw_Pic
+=============
+*/
+void Draw_Pic (float x, float y, const char *pic)
+{
+	Draw_ScaledPic (x, y, 1.0, pic);
 }
 
 /*
@@ -687,7 +512,7 @@ void Draw_ScaledPic (int x, int y, float scale, char *pic)
 Draw_AlphaStretchPlayerIcon
 =============
 */
-void Draw_AlphaStretchPlayerIcon (int x, int y, int w, int h, char *pic, float alphaval)
+void Draw_AlphaStretchPlayerIcon (int x, int y, int w, int h, const char *pic, float alphaval)
 {
 	image_t *gl;
 
@@ -697,46 +522,7 @@ void Draw_AlphaStretchPlayerIcon (int x, int y, int w, int h, char *pic, float a
 		return;
 	}
 
-	if (scrap_dirty)
-		Scrap_Upload ();
-
-	VA_SetElem2(vert_array[0],x, y);
-	VA_SetElem2(vert_array[1],x+w, y);
-	VA_SetElem2(vert_array[2],x+w, y+h);
-	VA_SetElem2(vert_array[3],x, y+h);
-
-	Draw_ShaderPic(gl, alphaval);
-}
-/*
-=============
-Draw_TileClear
-
-This repeats a 64*64 tile graphic to fill the screen around a sized down
-refresh window.
-=============
-*/
-void Draw_TileClear (int x, int y, int w, int h, char *pic)
-{
-	image_t	*image;
-
-	image = R_RegisterPic (pic);
-	if (!image)
-	{
-//		Com_Printf ("Can't find pic: %s\n", pic);
-		return;
-	}
-
-	GL_Bind (image->texnum);
-	qglBegin (GL_QUADS);
-	qglTexCoord2f (x/64.0, y/64.0);
-	qglVertex2f (x, y);
-	qglTexCoord2f ( (x+w)/64.0, y/64.0);
-	qglVertex2f (x+w, y);
-	qglTexCoord2f ( (x+w)/64.0, (y+h)/64.0);
-	qglVertex2f (x+w, y+h);
-	qglTexCoord2f ( x/64.0, (y+h)/64.0 );
-	qglVertex2f (x, y+h);
-	qglEnd ();
+	Draw_AlphaStretchImage (x, y, w, h, gl, alphaval, draw_without_tiling);
 }
 
 
@@ -747,42 +533,32 @@ Draw_Fill
 Fills a box of pixels with a single color
 =============
 */
-void Draw_Fill (int x, int y, int w, int h, int c)
+void Draw_Fill (float x, float y, float w, float h, const float rgba[])
 {
-	union
-	{
-		unsigned	c;
-		byte		v[4];
-	} color;
-
-	if ( (unsigned)c > 255)
-		Com_Error (ERR_FATAL, "Draw_Fill: bad color");
-
 	qglDisable (GL_TEXTURE_2D);
-
-	color.c = d_8to24table[c];
-	qglColor3f (color.v[0]/255.0,
-		color.v[1]/255.0,
-		color.v[2]/255.0);
-
+	// FIXME HACK
+	qglDisable (GL_ALPHA_TEST);
+	qglEnable (GL_BLEND);
+	GLSTATE_ENABLE_BLEND;
+	GLSTATE_DISABLE_ALPHATEST;
+	qglColor4fv (rgba);
+	
 	qglBegin (GL_QUADS);
-
-	qglVertex2f (x,y);
-	qglVertex2f (x+w, y);
-	qglVertex2f (x+w, y+h);
-	qglVertex2f (x, y+h);
-
+		qglVertex2f (x,y);
+		qglVertex2f (x+w, y);
+		qglVertex2f (x+w, y+h);
+		qglVertex2f (x, y+h);
 	qglEnd ();
+	
+	GLSTATE_DISABLE_BLEND;
 	qglColor3f (1,1,1);
 	qglEnable (GL_TEXTURE_2D);
 }
-
 //=============================================================================
 
 /*
 ================
 Draw_FadeScreen
-
 ================
 */
 void Draw_FadeScreen (void)
@@ -804,3 +580,22 @@ void Draw_FadeScreen (void)
 }
 
 
+//=============================================================================
+
+/*
+================
+RGBA - This really should be a macro, but MSVC doesn't support C99.
+================
+*/
+
+float *RGBA (float r, float g, float b, float a)
+{
+	static float ret[4];
+	
+	ret[0] = r;
+	ret[1] = g;
+	ret[2] = b;
+	ret[3] = a;
+	
+	return ret;
+}
