@@ -45,8 +45,8 @@ extern cvar_t	*cl_hudimage2;
 
 unsigned	d_8to24table[256];
 
-qboolean GL_Upload8 (byte *data, int width, int height,  qboolean mipmap, qboolean is_sky );
-qboolean GL_Upload32 (unsigned *data, int width, int height,  qboolean mipmap, qboolean is_normalmap);
+qboolean GL_Upload8 (byte *data, int width, int height, int picmip, qboolean filter);
+qboolean GL_Upload32 (byte *data, int width, int height, int picmip, qboolean filter, qboolean force_standard_mipmap);
 
 int		gl_solid_format = 3;
 int		gl_alpha_format = 4;
@@ -59,7 +59,7 @@ int		gl_filter_max = GL_LINEAR;
 
 void R_InitImageSubsystem(void)
 {
-	int		aniso_level, max_aniso;
+	int		max_aniso;
 
 	if ( strstr( gl_config.extensions_string, "GL_ARB_multitexture" ) )
 	{
@@ -80,7 +80,7 @@ void R_InitImageSubsystem(void)
 	}
 	else
 	{
-		Com_Printf ("...GL_ARB_multitexture not found\n" );
+		Com_Error (ERR_FATAL, "...GL_ARB_multitexture not found\n" );
 	}
 	
 	gl_config.mtexcombine = false;
@@ -110,16 +110,22 @@ void R_InitImageSubsystem(void)
 	{
 		qglGetIntegerv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &max_aniso);
 
-		r_ext_max_anisotropy = Cvar_Get("r_ext_max_anisotropy", "0", CVAR_ARCHIVE );
+		r_ext_max_anisotropy = Cvar_Get("r_ext_max_anisotropy", "1", CVAR_ARCHIVE );
 		Cvar_SetValue("r_ext_max_anisotropy", max_aniso);
 
-		r_anisotropic = Cvar_Get("r_anisotropic", "16", CVAR_ARCHIVE);
-		if (r_anisotropic->value >= r_ext_max_anisotropy->value)
-			Cvar_SetValue("r_anisotropic", r_ext_max_anisotropy->value);
+		r_anisotropic = Cvar_Get("r_anisotropic", "1", CVAR_ARCHIVE);
+		if (r_anisotropic->integer >= r_ext_max_anisotropy->integer)
+			Cvar_SetValue("r_anisotropic", r_ext_max_anisotropy->integer);
+		if (r_anisotropic->integer <= 0)
+			Cvar_SetValue("r_anisotropic", 1);
+		
+		r_alphamasked_anisotropic = Cvar_Get("r_alphamasked_anisotropic", "1", CVAR_ARCHIVE);
+		if (r_alphamasked_anisotropic->integer >= r_ext_max_anisotropy->integer)
+			Cvar_SetValue("r_alphamasked_anisotropic", r_ext_max_anisotropy->integer);
+		if (r_alphamasked_anisotropic->integer <= 0)
+			Cvar_SetValue("r_alphamasked_anisotropic", 1);
 
-		aniso_level = r_anisotropic->value;
-
-		if (r_anisotropic->value == 1)
+		if (r_anisotropic->integer == 1 && r_alphamasked_anisotropic->integer == 1)
 			Com_Printf("...ignoring GL_EXT_texture_filter_anisotropic\n");
 		else
 			Com_Printf("...using GL_EXT_texture_filter_anisotropic\n");
@@ -128,6 +134,7 @@ void R_InitImageSubsystem(void)
 	{
 		Com_Printf("...GL_EXT_texture_filter_anisotropic not found\n");
 		r_anisotropic = Cvar_Get("r_anisotropic", "0", CVAR_ARCHIVE);
+		r_alphamasked_anisotropic = Cvar_Get("r_alphamasked_anisotropic", "0", CVAR_ARCHIVE);
 		r_ext_max_anisotropy = Cvar_Get("r_ext_max_anisotropy", "0", CVAR_ARCHIVE);
 	}
 }
@@ -156,6 +163,14 @@ void GL_ShadeModel (GLenum mode)
 	}
 }
 
+// FIXME:
+// This centralized texture bind batching system is a great idea if you use it
+// *everywhere*. Otherwise you just end up binding a texture elsewhere and
+// this system doesn't know you've done it, so when you count on this system
+// to bindanother texture, it doesn't know it has to, and it'll happily let
+// you use the wrong texture. Anywhere we use glActiveTexture and 
+// glBindTexture directly can potentially screw things up. 
+
 void GL_EnableMultitexture( qboolean enable )
 {
 	if ( !qglActiveTextureARB )
@@ -238,7 +253,7 @@ void GL_Bind (int texnum)
 {
 	extern	image_t	*draw_chars;
 
-	if (gl_nobind->value && draw_chars)		// performance evaluation option
+	if (gl_nobind->integer && draw_chars)		// performance evaluation option
 		texnum = draw_chars->texnum;
 	if ( gl_state.currenttextures[gl_state.currenttmu] == texnum)
 		return;
@@ -338,7 +353,7 @@ void GL_TextureMode( char *string )
 	// change all the existing mipmap texture objects
 	for (i=0, glt=gltextures ; i<numgltextures ; i++, glt++)
 	{
-		if (glt->type != it_pic && glt->type != it_sky )
+		if (glt->type != it_pic && glt->type != it_particle && glt->type != it_sky )
 		{
 			GL_Bind (glt->texnum);
 			qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
@@ -433,6 +448,9 @@ void	GL_ImageList_f (void)
 		case it_pic:
 			Com_Printf ("P");
 			break;
+		case it_particle:
+			Com_Printf ("A");
+			break;
 		default:
 			Com_Printf (" ");
 			break;
@@ -457,11 +475,11 @@ void	GL_ImageList_f (void)
 */
 
 #define	MAX_SCRAPS		1
-#define	BLOCK_WIDTH		256
-#define	BLOCK_HEIGHT	256
+#define	BLOCK_WIDTH		1024
+#define	BLOCK_HEIGHT	512
 
 int			scrap_allocated[MAX_SCRAPS][BLOCK_WIDTH];
-byte		scrap_texels[MAX_SCRAPS][BLOCK_WIDTH*BLOCK_HEIGHT];
+byte		scrap_texels[MAX_SCRAPS][BLOCK_WIDTH*BLOCK_HEIGHT*4];
 qboolean	scrap_dirty;
 
 // returns a texture number and the position inside it
@@ -503,19 +521,23 @@ int Scrap_AllocBlock (int w, int h, int *x, int *y)
 	}
 
 	return -1;
-//	Sys_Error ("Scrap_AllocBlock: full");
+/*	Sys_Error ("Scrap_AllocBlock: full");*/
 }
 
 int	scrap_uploads;
 
 void Scrap_Upload (void)
 {
-	scrap_uploads++;
 	GL_Bind(TEXNUM_SCRAPS);
-	GL_Upload8 (scrap_texels[0], BLOCK_WIDTH, BLOCK_HEIGHT, false, false );
+	GL_Upload32 (scrap_texels[scrap_uploads++], BLOCK_WIDTH, BLOCK_HEIGHT, 0, false, true);
 	scrap_dirty = false;
 }
 
+//just a guessed size-- this isn't necessarily raw RGBA data, it's the
+//encoded image data.
+#define	STATIC_RAWDATA_SIZE	(1024*1024*4+256)
+byte	static_rawdata[STATIC_RAWDATA_SIZE];
+
 /*
 =================================================================
 
@@ -545,7 +567,8 @@ void LoadPCX (char *filename, byte **pic, byte **palette, int *width, int *heigh
 	//
 	// load the file
 	//
-	len = FS_LoadFile (filename, (void **)&raw);
+	len = FS_LoadFile_TryStatic (	filename, (void **)&raw, static_rawdata, 
+									STATIC_RAWDATA_SIZE);
 	if (!raw)
 	{
 		return;
@@ -575,7 +598,8 @@ void LoadPCX (char *filename, byte **pic, byte **palette, int *width, int *heigh
 		|| pcx->ymax >= 512)
 	{
 		Com_Printf ("Bad pcx file %s\n", filename);
-		FS_FreeFile (pcx);
+		if ((byte *)pcx != static_rawdata)
+			FS_FreeFile (pcx);
 		return;
 	}
 
@@ -623,7 +647,8 @@ void LoadPCX (char *filename, byte **pic, byte **palette, int *width, int *heigh
 		*pic = NULL;
 	}
 
-	FS_FreeFile (pcx);
+	if ((byte *)pcx != static_rawdata)
+		FS_FreeFile (pcx);
 }
 /*
 =================================================================
@@ -635,6 +660,9 @@ By Robert 'Heffo' Heffernan
 =================================================================
 */
 
+static JOCTET eoi_buffer[2] = {(JOCTET)0xFF, (JOCTET)JPEG_EOI};
+static qboolean crjpg_corrupted;
+
 void crjpg_null(j_decompress_ptr cinfo)
 {
 }
@@ -642,6 +670,9 @@ void crjpg_null(j_decompress_ptr cinfo)
 int crjpg_fill_input_buffer(j_decompress_ptr cinfo)
 {
     Com_Printf("Premature end of JPEG data\n");
+    cinfo->src->next_input_byte = eoi_buffer;
+    cinfo->src->bytes_in_buffer = 2;
+    crjpg_corrupted = true;
     return 1;
 }
 
@@ -652,7 +683,12 @@ void crjpg_skip_input_data(j_decompress_ptr cinfo, long num_bytes)
     cinfo->src->bytes_in_buffer -= (size_t) num_bytes;
 
     if (cinfo->src->bytes_in_buffer < 0)
+    {
 		Com_Printf("Premature end of JPEG data\n");
+		cinfo->src->next_input_byte = eoi_buffer;
+    	cinfo->src->bytes_in_buffer = 2;
+    	crjpg_corrupted = true;
+    }
 }
 
 void crjpg_mem_src(j_decompress_ptr cinfo, byte *mem, int len)
@@ -675,6 +711,8 @@ void crjpg_mem_src(j_decompress_ptr cinfo, byte *mem, int len)
 LoadJPG
 ==============
 */
+#define	STATIC_SCANLINE_SIZE	(1024*3)
+byte	static_scanline[STATIC_SCANLINE_SIZE];
 void LoadJPG (char *filename, byte **pic, int *width, int *height)
 {
 	struct jpeg_decompress_struct	cinfo;
@@ -682,8 +720,10 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 	byte							*rawdata, *rgbadata, *scanline, *p, *q;
 	int								rawsize, i;
 
+	crjpg_corrupted = false;
 	// Load JPEG file into memory
-	rawsize = FS_LoadFile(filename, (void **)&rawdata);
+	rawsize = FS_LoadFile_TryStatic	(	filename, (void **)&rawdata,
+										static_rawdata, STATIC_RAWDATA_SIZE);
 	if (!rawdata)
 	{
 		return;
@@ -694,7 +734,8 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 		||	rawdata[7] != 'F'
 		||	rawdata[8] != 'I'
 		||	rawdata[9] != 'F') {
-		FS_FreeFile(rawdata);
+		if (rawdata != static_rawdata)
+			FS_FreeFile(rawdata);
 		return;
 	}
 
@@ -715,7 +756,8 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 	if(cinfo.output_components != 3)
 	{
 		jpeg_destroy_decompress(&cinfo);
-		FS_FreeFile(rawdata);
+		if (rawdata != static_rawdata)
+			FS_FreeFile(rawdata);
 		return;
 	}
 
@@ -724,7 +766,8 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 	if(!rgbadata)
 	{
 		jpeg_destroy_decompress(&cinfo);
-		FS_FreeFile(rawdata);
+		if (rawdata != static_rawdata)
+			FS_FreeFile(rawdata);
 		return;
 	}
 
@@ -732,13 +775,21 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 	*width = cinfo.output_width; *height = cinfo.output_height;
 
 	// Allocate Scanline buffer
-	scanline = malloc(cinfo.output_width * 3);
-	if(!scanline)
+	if (cinfo.output_width * 3 < STATIC_SCANLINE_SIZE)
 	{
-		free(rgbadata);
-		jpeg_destroy_decompress(&cinfo);
-		FS_FreeFile(rawdata);
-		return;
+		scanline = &static_scanline[0];
+	}
+	else
+	{
+		scanline = malloc(cinfo.output_width * 3);
+		if(!scanline)
+		{
+			free(rgbadata);
+			jpeg_destroy_decompress(&cinfo);
+			if (rawdata != static_rawdata)
+				FS_FreeFile(rawdata);
+			return;
+		}
 	}
 
 	// Read Scanlines, and expand from RGB to RGBA
@@ -760,7 +811,8 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 	}
 
 	// Free the scanline buffer
-	free(scanline);
+	if (scanline != &static_scanline[0])
+		free(scanline);
 
 	// Finish Decompression
 	jpeg_finish_decompress(&cinfo);
@@ -769,7 +821,11 @@ void LoadJPG (char *filename, byte **pic, int *width, int *height)
 	jpeg_destroy_decompress(&cinfo);
 
 	// Free the raw data now that it's done being processed
-    FS_FreeFile(rawdata);
+	if (rawdata != static_rawdata)
+    	FS_FreeFile(rawdata);
+    
+    if (crjpg_corrupted)
+    	Com_Printf ("JPEG file %s is likely corrupted, please obtain a fresh copy.\n", filename);
 
 	// Return the 'rgbadata'
 	*pic = rgbadata;
@@ -805,6 +861,7 @@ void LoadTGA (char *name, byte **pic, int *width, int *height)
 	int		row, column;
 	byte	*buf_p;
 	byte	*buffer;
+	size_t	buf_end;
 	int		length;
 	TargaHeader		targa_header;
 	byte			*targa_rgba;
@@ -815,49 +872,71 @@ void LoadTGA (char *name, byte **pic, int *width, int *height)
 	//
 	// load the file
 	//
-	length = FS_LoadFile (name, (void **)&buffer);
+	length = FS_LoadFile_TryStatic	(	name, (void **)&buffer, 
+										static_rawdata, STATIC_RAWDATA_SIZE);
 	if (!buffer)
 	{
 		return;
 	}
 
 	buf_p = buffer;
+	buf_end = (size_t)(buffer+length);
 
-	targa_header.id_length = *buf_p++;
-	targa_header.colormap_type = *buf_p++;
-	targa_header.image_type = *buf_p++;
+#define GET_TGA_BYTE(dest) \
+	{\
+		if ((size_t)buf_p<buf_end)\
+		{\
+			(dest) = *buf_p++;\
+		}\
+		else\
+		{\
+			/* We don't set *pic to NULL, so if the image is mostly intact,
+			 * whatever pixels were already there will still show up.
+			 */\
+			Com_Printf ("LoadTGA: %s is either truncated or corrupt, please obtain a fresh copy!\n", name);\
+			return;\
+		}\
+	}
 
-	tmp[0] = buf_p[0];
-	tmp[1] = buf_p[1];
+	GET_TGA_BYTE (targa_header.id_length);
+	GET_TGA_BYTE (targa_header.colormap_type);
+	GET_TGA_BYTE (targa_header.image_type);
+
+	GET_TGA_BYTE (tmp[0]);
+	GET_TGA_BYTE (tmp[1]);
 	targa_header.colormap_index = LittleShort ( *((short *)tmp) );
-	buf_p+=2;
-	tmp[0] = buf_p[0];
-	tmp[1] = buf_p[1];
+	GET_TGA_BYTE (tmp[0]);
+	GET_TGA_BYTE (tmp[1]);
 	targa_header.colormap_length = LittleShort ( *((short *)tmp) );
-	buf_p+=2;
-	targa_header.colormap_size = *buf_p++;
-	targa_header.x_origin = LittleShort ( *((short *)buf_p) );
-	buf_p+=2;
-	targa_header.y_origin = LittleShort ( *((short *)buf_p) );
-	buf_p+=2;
-	targa_header.width = LittleShort ( *((short *)buf_p) );
-	buf_p+=2;
-	targa_header.height = LittleShort ( *((short *)buf_p) );
-	buf_p+=2;
-	targa_header.pixel_size = *buf_p++;
-	targa_header.attributes = *buf_p++;
+	GET_TGA_BYTE (targa_header.colormap_size);
+	GET_TGA_BYTE (tmp[0]);
+	GET_TGA_BYTE (tmp[1]);
+	targa_header.x_origin = LittleShort ( *((short *)tmp) );
+	GET_TGA_BYTE (tmp[0]);
+	GET_TGA_BYTE (tmp[1]);
+	targa_header.y_origin = LittleShort ( *((short *)tmp) );
+	GET_TGA_BYTE (tmp[0]);
+	GET_TGA_BYTE (tmp[1]);
+	targa_header.width = LittleShort ( *((short *)tmp) );
+	GET_TGA_BYTE (tmp[0]);
+	GET_TGA_BYTE (tmp[1]);
+	targa_header.height = LittleShort ( *((short *)tmp) );
+	GET_TGA_BYTE (targa_header.pixel_size);
+	GET_TGA_BYTE (targa_header.attributes);
 
 	if (targa_header.image_type!=2
 		&& targa_header.image_type!=10) {
 		//Com_Error (ERR_DROP, "LoadTGA: Only type 2 and 10 targa RGB images supported\n");
-		FS_FreeFile (buffer);
+		if (buffer != static_rawdata)
+			FS_FreeFile (buffer);
 		return;
 	}
 
 	if (targa_header.colormap_type !=0
 		|| (targa_header.pixel_size!=32 && targa_header.pixel_size!=24)) {
 		//Com_Error (ERR_DROP, "LoadTGA: Only 32 or 24 bit images supported (no colormaps)\n");
-		FS_FreeFile (buffer);
+		if (buffer != static_rawdata)
+			FS_FreeFile (buffer);
 		return;
 	}
 
@@ -871,8 +950,9 @@ void LoadTGA (char *name, byte **pic, int *width, int *height)
 		*height = rows;
 
 	targa_rgba = malloc (numPixels*4);
+	memset (targa_rgba, 0, numPixels*4);
 	*pic = targa_rgba;
-
+	
 	if (targa_header.id_length != 0)
 		buf_p += targa_header.id_length;  // skip TARGA image comment
 
@@ -880,60 +960,40 @@ void LoadTGA (char *name, byte **pic, int *width, int *height)
 		for(row=rows-1; row>=0; row--) {
 			pixbuf = targa_rgba + row*columns*4;
 			for(column=0; column<columns; column++) {
-				unsigned char red,green,blue,alphabyte;
-				switch (targa_header.pixel_size) {
-					case 24:
-
-							blue = *buf_p++;
-							green = *buf_p++;
-							red = *buf_p++;
-							*pixbuf++ = red;
-							*pixbuf++ = green;
-							*pixbuf++ = blue;
-							*pixbuf++ = 255;
-							break;
-					case 32:
-							blue = *buf_p++;
-							green = *buf_p++;
-							red = *buf_p++;
-							alphabyte = *buf_p++;
-							*pixbuf++ = red;
-							*pixbuf++ = green;
-							*pixbuf++ = blue;
-							*pixbuf++ = alphabyte;
-							break;
+				unsigned char red,green,blue,alphabyte=255;
+				GET_TGA_BYTE (blue);
+				GET_TGA_BYTE (green);
+				GET_TGA_BYTE (red);
+				if (targa_header.pixel_size == 32)
+				{
+					GET_TGA_BYTE (alphabyte);
 				}
+				*pixbuf++ = red;
+				*pixbuf++ = green;
+				*pixbuf++ = blue;
+				*pixbuf++ = alphabyte;
 			}
 		}
 	}
 	else if (targa_header.image_type==10) {   // Runlength encoded RGB images
-		unsigned char red=0,green=0,blue=0,alphabyte=0,packetHeader,packetSize,j;
+		unsigned char red=0,green=0,blue=0,alphabyte=255,packetHeader,packetSize,j;
 		for(row=rows-1; row>=0; row--) {
 			pixbuf = targa_rgba + row*columns*4;
 			for(column=0; column<columns; ) {
-				packetHeader= *buf_p++;
+				GET_TGA_BYTE (packetHeader);
 				packetSize = 1 + (packetHeader & 0x7f);
 				if (packetHeader & 0x80) {        // run-length packet
-					switch (targa_header.pixel_size) {
-						case 24:
-								blue = *buf_p++;
-								green = *buf_p++;
-								red = *buf_p++;
-								alphabyte = 255;
-								break;
-						case 32:
-								blue = *buf_p++;
-								green = *buf_p++;
-								red = *buf_p++;
-								alphabyte = *buf_p++;
-								break;
-					}
+					GET_TGA_BYTE (blue);
+					GET_TGA_BYTE (green);
+					GET_TGA_BYTE (red);
+					if (targa_header.pixel_size == 32)
+						GET_TGA_BYTE (alphabyte);
 
 					for(j=0;j<packetSize;j++) {
-						*pixbuf++=red;
-						*pixbuf++=green;
-						*pixbuf++=blue;
-						*pixbuf++=alphabyte;
+						*pixbuf++ = red;
+						*pixbuf++ = green;
+						*pixbuf++ = blue;
+						*pixbuf++ = alphabyte;
 						column++;
 						if (column==columns) { // run spans across rows
 							column=0;
@@ -947,27 +1007,15 @@ void LoadTGA (char *name, byte **pic, int *width, int *height)
 				}
 				else {                            // non run-length packet
 					for(j=0;j<packetSize;j++) {
-						switch (targa_header.pixel_size) {
-							case 24:
-									blue = *buf_p++;
-									green = *buf_p++;
-									red = *buf_p++;
-									*pixbuf++ = red;
-									*pixbuf++ = green;
-									*pixbuf++ = blue;
-									*pixbuf++ = 255;
-									break;
-							case 32:
-									blue = *buf_p++;
-									green = *buf_p++;
-									red = *buf_p++;
-									alphabyte = *buf_p++;
-									*pixbuf++ = red;
-									*pixbuf++ = green;
-									*pixbuf++ = blue;
-									*pixbuf++ = alphabyte;
-									break;
-						}
+						GET_TGA_BYTE (blue);
+						GET_TGA_BYTE (green);
+						GET_TGA_BYTE (red);
+						if (targa_header.pixel_size == 32)
+							GET_TGA_BYTE (alphabyte);
+						*pixbuf++ = red;
+						*pixbuf++ = green;
+						*pixbuf++ = blue;
+						*pixbuf++ = alphabyte;
 						column++;
 						if (column==columns) { // pixel packet run spans across rows
 							column=0;
@@ -984,7 +1032,8 @@ void LoadTGA (char *name, byte **pic, int *width, int *height)
 		}
 	}
 
-	FS_FreeFile (buffer);
+	if (buffer != static_rawdata)
+		FS_FreeFile (buffer);
 }
 
 
@@ -1079,10 +1128,10 @@ void R_FloodFillSkin( byte *skin, int skinwidth, int skinheight )
 GL_ResampleTexture
 ================
 */
-void GL_ResampleTexture (unsigned *in, int inwidth, int inheight, unsigned *out,  int outwidth, int outheight)
+void GL_ResampleTexture (byte *in, int inwidth, int inheight, byte *out, int outwidth, int outheight)
 {
 	int		i, j;
-	unsigned	*inrow, *inrow2;
+	byte	*inrow, *inrow2;
 	unsigned	frac, fracstep;
 	unsigned	p1[1024], p2[1024];
 	byte		*pix1, *pix2, *pix3, *pix4;
@@ -1102,21 +1151,21 @@ void GL_ResampleTexture (unsigned *in, int inwidth, int inheight, unsigned *out,
 		frac += fracstep;
 	}
 
-	for (i=0 ; i<outheight ; i++, out += outwidth)
+	for (i=0 ; i<outheight ; i++, out += outwidth*4)
 	{
-		inrow = in + inwidth*(int)((i+0.25)*inheight/outheight);
-		inrow2 = in + inwidth*(int)((i+0.75)*inheight/outheight);
+		inrow = in + 4*inwidth*(int)((i+0.25)*inheight/outheight);
+		inrow2 = in + 4*inwidth*(int)((i+0.75)*inheight/outheight);
 		frac = fracstep >> 1;
 		for (j=0 ; j<outwidth ; j++)
 		{
-			pix1 = (byte *)inrow + p1[j];
-			pix2 = (byte *)inrow + p2[j];
-			pix3 = (byte *)inrow2 + p1[j];
-			pix4 = (byte *)inrow2 + p2[j];
-			((byte *)(out+j))[0] = (pix1[0] + pix2[0] + pix3[0] + pix4[0])>>2;
-			((byte *)(out+j))[1] = (pix1[1] + pix2[1] + pix3[1] + pix4[1])>>2;
-			((byte *)(out+j))[2] = (pix1[2] + pix2[2] + pix3[2] + pix4[2])>>2;
-			((byte *)(out+j))[3] = (pix1[3] + pix2[3] + pix3[3] + pix4[3])>>2;
+			pix1 = inrow + p1[j];
+			pix2 = inrow + p2[j];
+			pix3 = inrow2 + p1[j];
+			pix4 = inrow2 + p2[j];
+			*(out+4*j+0) = (pix1[0] + pix2[0] + pix3[0] + pix4[0])>>2;
+			*(out+4*j+1) = (pix1[1] + pix2[1] + pix3[1] + pix4[1])>>2;
+			*(out+4*j+2) = (pix1[2] + pix2[2] + pix3[2] + pix4[2])>>2;
+			*(out+4*j+3) = (pix1[3] + pix2[3] + pix3[3] + pix4[3])>>2;
 		}
 	}
 }
@@ -1128,9 +1177,8 @@ Applies brightness and contrast to the specified image while optionally computin
 the image's average color.  Also handles image inversion and monochrome.  This is
 all munged into one function to reduce loops on level load.
 */
-void R_FilterTexture(unsigned *in, int width, int height)
+void R_FilterTexture(byte *in, int width, int height)
 {
-	byte *pcb;
 	int count;
 	float fcb;
 	int ix;
@@ -1163,40 +1211,105 @@ void R_FilterTexture(unsigned *in, int width, int height)
 	}
 
 	// apply to image
-	pcb = (byte*)in;
 	count = width * height;
 	while ( count-- ) {
-		*pcb = lut[*pcb];
-		++pcb;
-		*pcb = lut[*pcb];
-		++pcb;
-		*pcb = lut[*pcb];
-		++pcb;
-		++pcb;
+		*in = lut[*in];
+		++in;
+		*in = lut[*in];
+		++in;
+		*in = lut[*in];
+		++in;
+		++in;
 	}
 
 }
 
+
+#define ALPHATEST_THRESHOLD 169
+// Uses the canonical mipmap generation algorithm, but unlike OpenGL's built-
+// in mipmapping, it this function knows when to quit-- that is, when the 
+// mipmaps have gotten too "mushy." Mushy mipmaps usually aren't a problem, 
+// but they're a huge one when dealing with alphatest surfaces.
+int GL_RecursiveGenerateAlphaMipmaps (byte *data, int width, int height, int depth)
+{
+	int			ret;
+	int			x, y;
+	int			newwidth, newheight;
+	int			upper_left_alpha;
+	qboolean	uniform = true;
+	byte		*mipmap, *scan, *scan2, *out;
+	
+	if (width == 1 && height == 1)
+		return depth;
+	
+	newwidth = width;
+	if (newwidth > 1)
+		newwidth /= 2;
+	newheight = height;
+	if (newheight > 1)
+		newheight /= 2;
+	
+	mipmap = malloc (newwidth*newheight*4); 
+	
+	for (y = 0; (y+1) < height; y += 2)
+	{
+		for (x = 0; (x+1) < width; x += 2)
+		{
+			scan = data + (y*width+x)*4;
+			scan2 = data + ((y+1)*width+x)*4;
+			out = mipmap + ((y/2)*(width/2)+(x/2))*4;
+			out[0] = (scan[0]+scan[4]+scan2[0]+scan2[4])/4;
+			out[1] = (scan[1]+scan[5]+scan2[1]+scan2[5])/4;
+			out[2] = (scan[2]+scan[6]+scan2[2]+scan2[6])/4;
+			out[3] = (scan[3]+scan[7]+scan2[3]+scan2[7])/4;
+			if (x == 0 && y == 0)
+				upper_left_alpha = out[3];
+			else if ((upper_left_alpha > ALPHATEST_THRESHOLD) != (out[3] > ALPHATEST_THRESHOLD))
+				uniform = false;
+		}
+	}
+	
+	if (uniform)
+	{
+		free (mipmap);
+		return depth;
+	}
+	
+	ret = GL_RecursiveGenerateAlphaMipmaps (mipmap, newwidth, newheight, depth+1);
+	
+	qglTexImage2D (GL_TEXTURE_2D, depth+1, gl_tex_alpha_format, newwidth, newheight, 0, GL_RGBA, GL_UNSIGNED_BYTE, mipmap);
+	
+	free (mipmap);
+	return ret;
+}
+
+int GL_GenerateAlphaMipmaps (byte *data, int width, int height)
+{
+	return GL_RecursiveGenerateAlphaMipmaps (data, width, height, 0);
+}
+
+
 static int	powers_of_two[] = {16,32,64,128,256,512,1024,2048,4096};
 int		upload_width, upload_height;
+int		crop_left, crop_right, crop_top, crop_bottom;
 qboolean	uploaded_paletted;
 
 
-qboolean GL_Upload32 (unsigned *data, int width, int height,  qboolean mipmap, qboolean is_normalmap)
+qboolean GL_Upload32 (byte *data, int width, int height, int picmip, qboolean filter, qboolean force_standard_mipmap)
 {
 	int		samples;
-	unsigned 	*scaled;
+	byte 	*scaled;
 	int		scaled_width, scaled_height;
 	int		i, c;
 	byte		*scan;
 	int comp;
 
-	if(mipmap && !is_normalmap)
+	if(filter)
 		R_FilterTexture(data, width, height);
 
 	uploaded_paletted = false;    // scan the texture for any non-255 alpha
 	c = width*height;
-	scan = ((byte *)data) + 3;
+	scan = data + 3;
 	samples = gl_solid_format;
 	for (i=0 ; i<c ; i++, scan += 4)
 	{
@@ -1239,10 +1352,12 @@ qboolean GL_Upload32 (unsigned *data, int width, int height,  qboolean mipmap, q
 				break;
 		}
 		// let people sample down the world textures for speed
-		if (mipmap)
+		for (i = 0; i < picmip; i++)
 		{
-			scaled_width >>= gl_picmip->integer;
-			scaled_height >>= gl_picmip->integer;
+			if (scaled_width <= 1 || scaled_height <= 1)
+				break;
+			scaled_width >>= 1;
+			scaled_height >>= 1;
 		}
 		if (scaled_width > max_size)
 			scaled_width = max_size;
@@ -1257,21 +1372,68 @@ qboolean GL_Upload32 (unsigned *data, int width, int height,  qboolean mipmap, q
 		scaled=data;
 	}
 
-	if (mipmap)
+	if (samples != gl_alpha_format || force_standard_mipmap)
 		qglTexParameteri( GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE );
 	qglTexImage2D (GL_TEXTURE_2D, 0, comp, scaled_width, scaled_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, scaled);
+	
+	crop_left = crop_right = crop_top = crop_bottom = 0;
+	if (samples == gl_alpha_format)
+	{
+		int			x, y;
+		qboolean 	found;
+		
+		scan = (byte*)scaled+3;
+		
+		for (y = 0; y < scaled_height; y++, crop_top++)
+		{
+			found = false;
+			for (x = 0; x < scaled_width; x++)
+			{
+				scan = ((byte*)scaled + (y*scaled_width+x)*4+3);
+				if (*scan != 0)
+					found = true;
+			}
+			if (found)
+				break;
+		}
+		
+		crop_left = scaled_width;
+		crop_right = scaled_width;
+		
+		for (; y < scaled_height; y++)
+		{
+			for (x = 0; x < scaled_width; x++)
+			{
+				scan = ((byte*)scaled + (y*scaled_width+x)*4+3);
+				if (*scan != 0)
+				{
+					if (x < crop_left)
+						crop_left = x;
+					if (scaled_width-x-1 < crop_right)
+						crop_right = scaled_width-x-1;
+					crop_bottom = scaled_height-y-1;
+				}
+			}
+		}
+	}
+	
+	if (samples == gl_alpha_format && !force_standard_mipmap)
+	{
+		int generated_mipmaps = 
+			GL_GenerateAlphaMipmaps (scaled, scaled_width, scaled_height);
+		qglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, generated_mipmaps);
+		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, r_alphamasked_anisotropic->integer);
+	}
+	else
+		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, r_anisotropic->integer);
 
 	if (scaled_width != width || scaled_height != height)
 		free(scaled);
 
 	upload_width = scaled_width;
 	upload_height = scaled_height;
-	if ( mipmap )
-		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
-	else
-		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max);
+	qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
 	qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
-	qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, r_anisotropic->value);
 
 	return (samples == gl_alpha_format);
 }
@@ -1283,9 +1445,9 @@ GL_Upload8
 Returns has_alpha
 ===============
 */
-qboolean GL_Upload8 (byte *data, int width, int height,  qboolean mipmap, qboolean is_sky )
+qboolean GL_Upload8 (byte *data, int width, int height, int picmip, qboolean filter)
 {
-	unsigned	trans[512*256];
+	unsigned	trans[512*256]; // contains 32 bit RGBA color
 	int			i, s;
 	int			p;
 
@@ -1319,21 +1481,21 @@ qboolean GL_Upload8 (byte *data, int width, int height,  qboolean mipmap, qboole
             ((byte *)&trans[i])[2] = ((byte *)&d_8to24table[p])[2];
         }
     }
-    return GL_Upload32 (trans, width, height, mipmap, false);
+    return GL_Upload32 ((byte*)trans, width, height, picmip, filter, false);
 }
 
 /*
 ================
-GL_LoadPic
+GL_ReservePic
 
-This is also used as an entry point for the generated r_notexture
+Find a free image_t
 ================
 */
-image_t *GL_LoadPic (char *name, byte *pic, int width, int height, imagetype_t type, int bits)
+image_t *GL_FindFreeImage (char *name, int width, int height, imagetype_t type)
 {
 	image_t		*image;
 	int			i;
-
+	
 	// find a free image_t
 	for (i=0, image=gltextures ; i<numgltextures ; i++,image++)
 	{
@@ -1363,54 +1525,101 @@ image_t *GL_LoadPic (char *name, byte *pic, int width, int height, imagetype_t t
 	image->width = width;
 	image->height = height;
 	image->type = type;
+	
+	image->texnum = TEXNUM_IMAGES + (image - gltextures);
+	
+	return image;
+}
+
+/*
+================
+GL_LoadPic
+
+This is also used as an entry point for the generated r_notexture
+================
+*/
+image_t *GL_LoadPic (char *name, byte *pic, int width, int height, imagetype_t type, int bits)
+{
+	image_t		*image;
+	int			i;
+	int			picmip;
+
+	image = GL_FindFreeImage (name, width, height, type);
 
 	if (type == it_skin && bits == 8)
 		R_FloodFillSkin(pic, width, height);
+	
+	if (type == it_particle || type == it_pic)
+		picmip = 0;
+	else
+		picmip = gl_picmip->integer;
 
-	// load little pics into the scrap
-	if (image->type == it_pic && bits == 8
-		&& image->width < 64 && image->height < 64)
+	// load little particles into the scrap
+	if (type == it_particle && bits != 8
+		&& image->width <= 128 && image->height <= 128)
 	{
 		int		x, y;
-		int		i, j, k;
+		int		i, j, k, l;
 		int		texnum;
 
 		texnum = Scrap_AllocBlock (image->width, image->height, &x, &y);
 		if (texnum == -1)
 			goto nonscrap;
 		scrap_dirty = true;
-
+		
 		// copy the texels into the scrap block
 		k = 0;
 		for (i=0 ; i<image->height ; i++)
-			for (j=0 ; j<image->width ; j++, k++)
-				scrap_texels[texnum][(y+i)*BLOCK_WIDTH + x + j] = pic[k];
-		image->texnum = TEXNUM_SCRAPS + texnum;
+			for (j=0 ; j<image->width ; j++)
+			    for (l = 0; l < 4; l++, k++)
+				    scrap_texels[texnum][((y+i)*BLOCK_WIDTH + x + j)*4+l] = pic[k];
+		image->texnum = TEXNUM_SCRAPS + texnum; // overwrite old texnum
 		image->scrap = true;
 		image->has_alpha = true;
-		image->sl = (x+0.01)/(float)BLOCK_WIDTH;
-		image->sh = (x+image->width-0.01)/(float)BLOCK_WIDTH;
-		image->tl = (y+0.01)/(float)BLOCK_WIDTH;
-		image->th = (y+image->height-0.01)/(float)BLOCK_WIDTH;
+		image->sl = (double)(x+0.5)/(double)BLOCK_WIDTH;
+		image->sh = (double)(x+image->width-0.5)/(double)BLOCK_WIDTH;
+		image->tl = (double)(y+0.5)/(double)BLOCK_HEIGHT;
+		image->th = (double)(y+image->height-0.5)/(double)BLOCK_HEIGHT;
 	}
 	else
 	{
 nonscrap:
 		image->scrap = false;
-		image->texnum = TEXNUM_IMAGES + (image - gltextures);
 		GL_Bind(image->texnum);
 		if (bits == 8) {
-			image->has_alpha = GL_Upload8 (pic, width, height, (image->type != it_pic && image->type != it_sky), image->type == it_sky );
+			image->has_alpha = GL_Upload8 (pic, width, height, picmip, type <= it_wall);
 		} else {
-			image->has_alpha = GL_Upload32 ((unsigned *)pic, width, height, (image->type != it_pic && image->type != it_sky), image->type == it_bump );
+			image->has_alpha = GL_Upload32 (pic, width, height, picmip, type <= it_wall, type >= it_bump);
 		}
-		image->upload_width = upload_width;		// after power of 2 and scales
-		image->upload_height = upload_height;
+
+		if (type == it_pic)
+			qglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+		
 		image->paletted = uploaded_paletted;
+
+		// size in pixels after power of 2 and scales
+		image->upload_width = upload_width;
+		image->upload_height = upload_height;
+		
+		// vertex offset to apply when cropping
+		image->crop_left = crop_left;
+		image->crop_top = crop_top;
+		
+		// size in pixels after cropping
+		image->crop_width = upload_width-crop_left-crop_right;
+		image->crop_height = upload_height-crop_top-crop_bottom;
+		
+		// texcoords to use when not cropping
 		image->sl = 0;
 		image->sh = 1;
 		image->tl = 0;
 		image->th = 1;
+		
+		// texcoords to use when cropping
+		image->crop_sl = 0 + (float)crop_left/(float)upload_width;
+		image->crop_sh = 1 - (float)crop_right/(float)upload_width;
+		image->crop_tl = 0 + (float)crop_top/(float)upload_height;
+		image->crop_th = 1 - (float)crop_bottom/(float)upload_height;
 	}
 
 	COMPUTE_HASH_KEY( image->hash_key, name, i );
@@ -1534,20 +1743,29 @@ image_t	*GL_FindImage (char *name, imagetype_t type)
 	pic = NULL;
 	palette = NULL;
 
-	// try to load .tga first
+	// Try to load the image with different file extensions, in the following
+	// order of decreasing preference: TGA, JPEG, PCX, and WAL.
+	
 	LoadTGA (va("%s.tga", shortname), &pic, &width, &height);
 	if (pic)
 	{
 		image = GL_LoadPic (name, pic, width, height, type, 32);
 		goto done;
 	}
+	
 	LoadJPG (va("%s.jpg", shortname), &pic, &width, &height);
 	if (pic)
 	{
 		image = GL_LoadPic (name, pic, width, height, type, 32);
 		goto done;
 	}
-	// then comes .pcx
+	
+	// TGA and JPEG are the only file types used for heightmaps and
+	// normalmaps, so if we haven't found it yet, it isn't there, and we can
+	// save ourselves a file lookup.
+	if (type == it_bump)
+		goto done;
+	
 	LoadPCX (va("%s.pcx", shortname), &pic, &palette, &width, &height);
 	if (pic)
 	{
@@ -1555,7 +1773,6 @@ image_t	*GL_FindImage (char *name, imagetype_t type)
 		goto done;
 	}
 
-	// then comes .wal
 	if (type == it_wall)
 		image = GL_LoadWal (va("%s.wal", shortname));
 
@@ -1564,6 +1781,9 @@ done:
 		free(pic);
 	if (palette)
 		free(palette);
+	
+	if (image != NULL)
+		image->script = RS_FindScript (shortname);
 
 ret_image:
 	return image;
@@ -1604,7 +1824,7 @@ void GL_FreeUnusedImages (void)
 			continue;		// used this sequence
 		if (!image->registration_sequence)
 			continue;		// free image_t slot
-		if (image->type == it_pic || image->type == it_sprite)
+		if (image->type == it_pic || image->type == it_sprite || image->type == it_particle)
 			continue;		// don't free pics or particles
 		// free it
 		qglDeleteTextures (1, (unsigned *)&image->texnum );
@@ -1629,7 +1849,21 @@ int Draw_GetPalette (void)
 
 	LoadPCX ("pics/colormap.pcx", &pic, &pal, &width, &height);
 	if (!pal)
-		Com_Error (ERR_FATAL, "Couldn't load pics/colormap.pcx");
+	{
+		Com_Error (ERR_FATAL,
+				"Couldn't load pics/colormap.pcx\n"
+				"The game executable cannot find its data files. This means\n"
+				"the game is likely not installed correctly."
+				// TODO: automatically generate a list of paths where the data
+				// files should be
+#if defined UNIX_VARIANT
+				" If you\ninstalled the game from the repository of a Linux\n"
+				"distribution, please report this to ther package\n"
+				"maintainers and have them check the README for packaging\n"
+				"instructions."
+#endif //UNIX_VARIANT
+			);
+	}
 
 	for (i=0 ; i<256 ; i++)
 	{
@@ -1649,16 +1883,21 @@ int Draw_GetPalette (void)
 	return 0;
 }
 
+// TODO: have r_newrefdef available before calling these, put r_mirroretxture
+// in that struct, base it on the viewport specified in that struct. (Needed
+// for splitscreen.)
 void R_InitMirrorTextures( void )
 {
 	byte	*data;
 	int		size;
+	int		size_oneside;
 
 	//init the partial screen texture
-	size = 512 * 512 * 4;
+	size_oneside = ceil((512.0f/1080.0f)*(float)viddef.height);
+	size = size_oneside * size_oneside * 4;
 	data = malloc( size );
 	memset( data, 255, size );
-	r_mirrortexture = GL_LoadPic( "***r_mirrortexture***", (byte *)data, 512, 512, it_pic, 32 );
+	r_mirrortexture = GL_LoadPic( "***r_mirrortexture***", (byte *)data, size_oneside, size_oneside, it_pic, 32 );
 	free ( data );
 }
 
@@ -1702,10 +1941,11 @@ void	GL_InitImages (void)
 
 	Draw_GetPalette ();
 
-	R_InitBloomTextures();//BLOOMS
 	R_InitMirrorTextures();//MIRRORS
 	R_InitDepthTextures();//DEPTH(SHADOWMAPS)
 	R_FB_InitTextures();//FULLSCREEN EFFECTS
+	R_SI_InitTextures();//SIMPLE ITEMS
+	R_InitBloomTextures();//BLOOMS
 }
 
 /*
@@ -1726,5 +1966,10 @@ void	GL_ShutdownImages (void)
 		qglDeleteTextures (1, (unsigned *)&image->texnum);
 		memset (image, 0, sizeof(*image));
 	}
+	
+	memset (scrap_allocated, 0, sizeof(scrap_allocated));
+	memset (scrap_texels, 0, sizeof(scrap_texels));
+	scrap_dirty = false;
+	scrap_uploads = 0;
 }
 
