@@ -81,7 +81,7 @@ void GL_UpdateSwapInterval( void );
 
 extern	float	gldepthmin, gldepthmax;
 
-#define	MAX_LBM_HEIGHT		512
+#define	MAX_LBM_HEIGHT		1024
 
 #define BACKFACE_EPSILON	0.01
 
@@ -103,12 +103,12 @@ extern	int			gl_filter_min, gl_filter_max;
 //
 extern	refdef_t	r_newrefdef;
 extern	int		r_viewcluster, r_viewcluster2, r_oldviewcluster, r_oldviewcluster2;
+mleaf_t			*r_viewleaf, *r_viewleaf2;
 
 extern	cvar_t	*r_norefresh;
 extern	cvar_t	*r_lefthand;
 extern	cvar_t	*r_drawentities;
 extern	cvar_t	*r_drawworld;
-extern	cvar_t	*r_speeds;
 extern	cvar_t	*r_fullbright;
 extern	cvar_t	*r_novis;
 extern	cvar_t	*r_nocull;
@@ -132,7 +132,6 @@ extern	cvar_t	*gl_bitdepth;
 extern	cvar_t	*gl_mode;
 extern	cvar_t	*gl_log;
 extern	cvar_t	*gl_lightmap;
-extern	cvar_t	*gl_shadows;
 extern	cvar_t	*gl_dynamic;
 extern	cvar_t	*gl_nobind;
 extern	cvar_t	*gl_picmip;
@@ -160,27 +159,41 @@ extern	cvar_t	*vid_gamma;
 extern  cvar_t	*vid_contrast;
 
 extern cvar_t *r_anisotropic;
+extern cvar_t *r_alphamasked_anisotropic;
 extern cvar_t *r_ext_max_anisotropy;
 
 extern cvar_t	*r_overbrightbits;
 
 extern cvar_t	*gl_normalmaps;
+extern cvar_t	*gl_bspnormalmaps;
 extern cvar_t	*gl_shadowmaps;
-extern cvar_t	*gl_glsl_postprocess;
+extern cvar_t	*gl_fog;
 
 extern	cvar_t	*r_shaders;
 extern	cvar_t	*r_bloom;
 extern	cvar_t	*r_lensflare;
 extern	cvar_t	*r_lensflare_intens;
 extern	cvar_t	*r_drawsun;
+extern  cvar_t	*r_godrays;
+extern  cvar_t	*r_godray_intensity;
+extern	cvar_t	*r_optimize;
+
+extern	cvar_t	*r_lightmapfiles;
 
 extern  cvar_t	*r_ragdolls;
 extern  cvar_t  *r_ragdoll_debug;
 
 extern	qboolean	map_fog;
-extern	char		map_music[128];
+#if defined WIN32_VARIANT
+extern	char		map_music[MAX_PATH];
+#else
+extern	char		map_music[MAX_OSPATH];
+#endif
 extern  unsigned	r_weather;
 extern unsigned		r_nosun;
+extern float		r_sunX;
+extern float		r_sunY;
+extern float		r_sunZ;
 
 extern  cvar_t		*r_minimap;
 extern  cvar_t		*r_minimap_size;
@@ -214,19 +227,26 @@ extern	float	r_project_matrix[16];
 extern	int		r_viewport[4];
 extern	float	r_farclip, r_farclip_min, r_farclip_bias;
 
+extern	int		r_origin_leafnum;
+
 //Image
-extern void R_InitImageSubsystem(void);
-extern void GL_Bind (int texnum);
-extern void GL_MBind( GLenum target, int texnum );
-extern void GL_TexEnv( GLenum value );
-extern void GL_EnableMultitexture( qboolean enable );
-extern void GL_SelectTexture( GLenum );
+void R_InitImageSubsystem(void);
+void GL_Bind (int texnum);
+void GL_MBind( GLenum target, int texnum );
+void GL_TexEnv( GLenum value );
+void GL_EnableMultitexture( qboolean enable );
+void GL_SelectTexture( GLenum );
+void RefreshFont (void);
+
 extern void vectoangles (vec3_t value1, vec3_t angles);
-extern void R_LightPoint (vec3_t p, vec3_t color, qboolean addDynamic);
-extern void R_PushDlights (void);
-extern void R_PushDlightsForBModel (entity_t *e);
-extern void SetVertexOverbrights (qboolean toggle);
-extern void RefreshFont (void);
+
+// dynamic lights
+void R_StaticLightPoint (vec3_t p, vec3_t color);
+void R_DynamicLightPoint (vec3_t p, vec3_t color);
+void R_LightPoint (vec3_t p, vec3_t color, qboolean addDynamic);
+void R_PushDlights (void);
+void R_PushDlightsForBModel (entity_t *e);
+void SetVertexOverbrights (qboolean toggle);
 
 //====================================================================
 extern	model_t	*r_worldmodel;
@@ -241,14 +261,14 @@ extern void V_AddBlend (float r, float g, float b, float a, float *v_blend);
 extern int	R_Init( void *hinstance, void *hWnd );
 extern void R_Shutdown( void );
 
+extern void R_SetupViewport (void);
 extern void R_RenderView (refdef_t *fd);
 extern void GL_ScreenShot_f (void);
 extern void R_DrawAliasModel (void);
 extern void R_DrawBrushModel (void);
-extern void R_DrawWorld (void);
+extern void R_DrawWorldSurfs (void);
 extern void R_RenderDlights (void);
 extern void R_DrawAlphaSurfaces (void);
-extern void R_DrawRSSurfaces(void);
 extern void R_InitParticleTexture (void);
 extern void R_DrawParticles (void);
 extern void R_DrawRadar(void);
@@ -258,12 +278,12 @@ extern void R_DrawSkyBox (void);
 extern void Draw_InitLocal (void);
 
 //Renderer utils
-extern void R_SubdivideSurface (msurface_t *fa);
+extern void R_SubdivideSurface (msurface_t *fa, int firstedge, int numedges);
 extern qboolean R_CullBox (vec3_t mins, vec3_t maxs);
 extern qboolean R_CullOrigin(vec3_t origin);
 extern qboolean R_CullSphere( const vec3_t centre, const float radius, const int clipflags );
 extern void R_RotateForEntity (entity_t *e);
-extern void R_MarkLeaves (void);
+extern void R_MarkWorldSurfs (void);
 extern void R_AddSkySurface (msurface_t *fa);
 extern void R_RenderWaterPolys (msurface_t *fa, int texnum, float scaleX, float scaleY);
 extern void R_ReadFogScript(char config_file[128]);
@@ -278,16 +298,21 @@ extern float VLight_GetLightValue ( vec3_t normal, vec3_t dir, float apitch, flo
 
 //BSP 
 extern GLuint normalisationCubeMap;
+extern image_t *r_droplets;
+extern image_t *r_droplets_nm;
+extern image_t *r_blooddroplets;
+extern image_t *r_blooddroplets_nm;
 extern void BSP_DrawTexturelessPoly (msurface_t *fa);
 extern void BSP_DrawTexturelessBrushModel (entity_t *e);
-extern void BSP_RenderBrushPoly (msurface_t *fa);
 
 //Postprocess
 void R_GLSLPostProcess(void);
 void R_FB_InitTextures(void);
 
 //VBO
+extern qboolean use_vbo;
 extern GLuint vboId;
+extern GLuint eboId;
 extern int totalVBOsize;
 extern int	vboPosition;
 extern void R_LoadVBOSubsystem(void);
@@ -295,10 +320,11 @@ extern void R_VCShutdown(void);
 extern void VB_VCInit(void);
 extern void VB_BuildVBOBufferSize(msurface_t *surf);
 extern void VB_BuildWorldVBO(void);
+void GL_SetupWorldVBO (void);
 void GL_BindVBO(vertCache_t *cache);
 void GL_BindIBO(vertCache_t *cache);
-vertCache_t *R_VCFindCache(vertStoreMode_t store, entity_t *ent);
-vertCache_t *R_VCLoadData(vertCacheMode_t mode, int size, void *buffer, vertStoreMode_t store, entity_t *ent);
+vertCache_t *R_VCFindCache(vertStoreMode_t store, model_t *mod);
+vertCache_t *R_VCLoadData(vertCacheMode_t mode, int size, void *buffer, vertStoreMode_t store, model_t *mod);
 
 //Light Bloom
 extern void R_BloomBlend( refdef_t *fd );
@@ -329,6 +355,12 @@ qboolean r_hasleaves;
 extern void Mod_AddVegetationSurface (msurface_t *surf, int texnum, vec3_t color, float size, char name[MAX_QPATH], int type);
 extern void R_DrawVegetationSurface (void);
 extern void R_ClearGrasses(void);
+extern void R_FinalizeGrass(model_t *mod); 
+
+//Simple Items
+extern void R_SI_InitTextures( void );
+extern void R_DrawSimpleItems( void );
+void R_SetSimpleTexnum (model_t *loadmodel, const char *pathname);
 
 //Light beams/volumes
 int r_numbeams;
@@ -344,17 +376,13 @@ extern float	scr_playericonalpha;
 
 //Team colors
 int r_teamColor;
+qboolean r_gotFlag;
+qboolean r_lostFlag;
 
-extern void	Draw_GetPicSize (int *w, int *h, char *name);
-extern void	Draw_Pic (int x, int y, char *name);
-extern void	Draw_ScaledPic (int x, int y, float scale, char *pic);
-extern void	Draw_StretchPic (int x, int y, int w, int h, char *name);
-extern void	Draw_Char (int x, int y, int c);
-extern void	Draw_ColorChar (int x, int y, int num, vec4_t color);
-extern void	Draw_ScaledChar(float x, float y, int num, float scale, int from_menu);
-extern void	Draw_ScaledColorChar (float x, float y, int num, vec4_t color, float scale, int from_menu);
-extern void	Draw_TileClear (int x, int y, int w, int h, char *name);
-extern void	Draw_Fill (int x, int y, int w, int h, int c);
+extern void	Draw_GetPicSize (int *w, int *h, const char *name);
+extern void	Draw_Pic (float x, float y, const char *name);
+extern void	Draw_ScaledPic (float x, float y, float scale, const char *pic);
+extern void	Draw_StretchPic (float x, float y, float w, float h, const char *name);
 extern void	Draw_FadeScreen (void);
 
 extern void	R_BeginFrame( float camera_separation );
@@ -362,14 +390,13 @@ extern void	R_SwapBuffers( int );
 extern void	R_SetPalette ( const unsigned char *palette);
 extern int	Draw_GetPalette (void);
 
-extern void	GL_ResampleTexture (unsigned *in, int inwidth, int inheight, unsigned *out,  int outwidth, int outheight);
-
 image_t *R_RegisterSkin (char *name);
 image_t *R_RegisterParticlePic(const char *name);
 image_t *R_RegisterParticleNormal(const char *name);
 image_t *R_RegisterGfxPic(const char *name);
 
 extern void	LoadPCX (char *filename, byte **pic, byte **palette, int *width, int *height);
+extern image_t *GL_FindFreeImage (char *name, int width, int height, imagetype_t type);
 extern image_t *GL_LoadPic (char *name, byte *pic, int width, int height, imagetype_t type, int bits);
 extern image_t *GL_GetImage( const char * name );
 extern image_t	*GL_FindImage (char *name, imagetype_t type);
@@ -453,31 +480,33 @@ typedef struct
 
 typedef struct
 {
-	float		inverse_intensity;
-	qboolean	fullscreen;
+	 float       inverse_intensity;
+    qboolean    fullscreen;
 
-	int			prev_mode;
+    int         prev_mode;
 
-	unsigned char *d_16to8table;
+    unsigned char *d_16to8table;
 
-	int			lightmap_textures;
+    int         lightmap_textures;
 
-	int			currenttextures[3];
-	int			currenttmu;
+    int         currenttextures[3];
+    int         currenttmu;
 
-	float		camera_separation;
-	qboolean	stereo_enabled;
+    float       camera_separation;
+    qboolean    stereo_enabled;
 
-	qboolean	alpha_test;
-	qboolean	blend;
-	qboolean	texgen;
-	qboolean	fragment_program;
-	qboolean	glsl_shaders;
-	qboolean	separateStencil;
-	qboolean	stencil_wrap;
-	qboolean	vbo;
-	qboolean	fbo;
-	qboolean	hasFBOblit;
+    qboolean    alpha_test;
+    qboolean    blend;
+    qboolean    texgen;
+    qboolean    fragment_program;
+    qboolean    glsl_shaders;
+    qboolean    separateStencil;
+    qboolean    stencil_wrap;
+    qboolean    vbo;
+    qboolean    fbo;
+    qboolean    hasFBOblit;
+
+	qboolean	ati;
 
 } glstate_t;
 
@@ -523,48 +552,42 @@ extern vec3_t ShadowArray[MAX_SHADOW_VERTS];
 #define VERT_BUMPMAPPED_COLOURED		8		// verts and st for 1 tmu, 2 texoord pointers and colour
 #define VERT_NORMAL_COLOURED_TEXTURED	9		// verts and st for 1tmu and color, with normals
 
-#if 0
-// vertex array kill flags
+// looks like these should be bit flags (2010-08)
+// apparently not - this was not working as bit flags, works fine as ints
 #define KILL_TMU0_POINTER	1
 #define KILL_TMU1_POINTER	2
 #define KILL_TMU2_POINTER	3
 #define KILL_TMU3_POINTER	4
-#define KILL_RGBA_POINTER	5
-#define KILL_NORMAL_POINTER 6
-#else
-// looks like these should be bit flags (2010-08)
-#define KILL_TMU0_POINTER	0x01
-#define KILL_TMU1_POINTER	0x02
-#define KILL_TMU2_POINTER	0x04
-#define KILL_TMU3_POINTER	0x08
-#define KILL_RGBA_POINTER	0x10
-#define KILL_NORMAL_POINTER 0x20
-#endif
+#define KILL_TMU4_POINTER   5
+#define KILL_TMU5_POINTER	6
+#define KILL_RGBA_POINTER	7
+#define KILL_NORMAL_POINTER 8
 
 // vertex array subsystem
-extern void R_InitVArrays (int varraytype);
-extern void R_KillVArrays (void);
-extern void R_DrawVarrays(GLenum mode, GLint first, GLsizei count, qboolean vbo);
-extern void R_InitQuadVarrays(void);
-extern void R_AddSurfToVArray (msurface_t *surf);
-extern void R_AddTexturedSurfToVArray (msurface_t *surf, float scroll);
-extern void R_AddLightMappedSurfToVArray (msurface_t *surf, float scroll);
-extern void R_AddGLSLShadedWarpSurfToVArray (msurface_t *surf, float scroll);
-extern void R_KillNormalTMUs(void);
+void R_InitVArrays (int varraytype);
+void R_KillVArrays (void);
+void R_DrawVarrays(GLenum mode, GLint first, GLsizei count);
+void R_InitQuadVarrays(void);
+void R_AddSurfToVArray (msurface_t *surf);
+void R_AddShadowSurfToVArray (msurface_t *surf, vec3_t origin);
+void R_AddTexturedSurfToVArray (msurface_t *surf, float scroll);
+void R_AddLightMappedSurfToVArray (msurface_t *surf, float scroll);
+void R_AddGLSLShadedWarpSurfToVArray (msurface_t *surf, float scroll);
+void R_KillNormalTMUs(void);
 
 //shadows
-extern	cvar_t		*r_shadowmapratio;
+extern  void R_InitShadowSubsystem(void);
+extern  void R_CastShadow(void);
+extern	cvar_t		*r_shadowmapscale;
 extern  int			r_lightgroups;
 extern  image_t		*r_depthtexture;
 extern	image_t		*r_depthtexture2;
 extern  image_t		*r_colorbuffer;
-extern  image_t		*r_shadowbuffer;
-extern	image_t		*r_shadowbufferBlur;
 extern GLuint   fboId[3];
 extern GLuint	rboId;
 extern vec3_t	r_worldLightVec;
-extern qboolean have_stencil;
-typedef struct	LightGroup {
+typedef struct	LightGroup 
+{
 	vec3_t	group_origin;
 	vec3_t	accum_origin;
 	float	avg_intensity;
@@ -572,16 +595,21 @@ typedef struct	LightGroup {
 } LightGroup_t;
 extern			LightGroup_t LightGroups[MAX_LIGHTS];
 
+extern void		R_CheckFBOExtensions (void);
 extern void		R_GenerateShadowFBO(void);
-extern void		R_InitShadowSubsystem(void);
 extern void		MD2_DrawCaster (void);
 extern void		IQM_DrawCaster (void);
 extern void		IQM_DrawRagDollCaster (int);
 extern void		R_DrawDynamicCaster(void);
 extern void		R_DrawVegetationCaster(void);
-extern void		R_CastShadow(void);
-extern float	SHD_ShadowLight (vec3_t pos, vec3_t angles, vec3_t lightAdd, int type);
+extern void		R_DrawEntityCaster(entity_t *ent);
+extern void		R_GenerateEntityShadow( void );
+extern void		R_GenerateRagdollShadow( int RagDollID );
+extern void		BSP_DrawShadowPoly (msurface_t *fa, vec3_t origin);
+extern void		R_DrawShadowMapWorld(qboolean forEnt, vec3_t origin);
 int				FB_texture_width, FB_texture_height;
+float			fadeShadow;
+cvar_t			*r_shadowcutoff;
 
 //shader programs
 extern void R_LoadARBPrograms(void);
@@ -592,11 +620,16 @@ extern unsigned int g_water_program_id;
 
 //glsl
 extern GLhandleARB	g_programObj;
+extern GLhandleARB  g_shadowprogramObj;
 extern GLhandleARB	g_waterprogramObj;
 extern GLhandleARB	g_meshprogramObj;
+extern GLhandleARB  g_glassprogramObj;
+extern GLhandleARB	g_blankmeshprogramObj;
 extern GLhandleARB	g_fbprogramObj;
 extern GLhandleARB	g_blurprogramObj;
 extern GLhandleARB	g_rblurprogramObj;
+extern GLhandleARB  g_dropletsprogramObj;
+extern GLhandleARB  g_godraysprogramObj;
 
 extern GLhandleARB	g_vertexShader;
 extern GLhandleARB	g_fragmentShader;
@@ -616,16 +649,31 @@ extern GLuint		g_location_parallax;
 extern GLuint		g_location_dynamic;
 extern GLuint		g_location_shadowmap;
 extern GLuint		g_Location_statshadow;
+extern GLuint		g_location_xOffs;
+extern GLuint		g_location_yOffs;
 extern GLuint		g_location_lightPosition;
 extern GLuint		g_location_staticLightPosition;
 extern GLuint		g_location_lightColour;
 extern GLuint		g_location_lightCutoffSquared;
+extern GLuint		g_location_liquid;
+extern GLuint		g_location_shiny;
+extern GLuint		g_location_rsTime;
+extern GLuint		g_location_liquidTexture;
+extern GLuint		g_location_liquidNormTex;
+extern GLuint		g_location_chromeTex;
+
+//shadows on white bsp surface
+extern GLuint		g_location_entShadow;
+extern GLuint		g_location_fadeShadow;
+extern GLuint		g_location_xOffset;
+extern GLuint		g_location_yOffset;
 
 //water
 extern GLuint		g_location_baseTexture;
 extern GLuint		g_location_normTexture;
 extern GLuint		g_location_refTexture;
-extern GLuint		g_location_tangent;
+extern GLuint		g_location_waterEyePos;
+extern GLuint		g_location_tangentSpaceTransform;
 extern GLuint		g_location_time;
 extern GLuint		g_location_lightPos;
 extern GLuint		g_location_reflect;
@@ -637,13 +685,28 @@ extern GLuint		g_location_meshlightPosition;
 extern GLuint		g_location_baseTex;
 extern GLuint		g_location_normTex;
 extern GLuint		g_location_fxTex;
+extern GLuint		g_location_fx2Tex;
 extern GLuint		g_location_color;
-extern GLuint		g_location_minLight;
 extern GLuint		g_location_meshNormal;
 extern GLuint		g_location_meshTime;
 extern GLuint		g_location_meshFog;
 extern GLuint		g_location_useFX;
 extern GLuint		g_location_useGlow;
+extern GLuint		g_location_useShell;
+extern GLuint		g_location_useCube;
+extern GLuint		g_location_useGPUanim;
+extern GLuint		g_location_outframe;
+extern GLuint		g_location_fromView;
+
+//glass
+extern GLuint		g_location_gmirTexture;
+extern GLuint		g_location_grefTexture;
+extern GLuint		g_location_gLightPos;
+extern GLuint		g_location_gFog;
+extern GLuint		g_location_gOutframe;
+
+//blank mesh
+extern GLuint		g_location_bmOutframe;
 
 //fullscreen distortion effects
 extern GLuint		g_location_framebuffTex;
@@ -660,6 +723,18 @@ extern GLuint		g_location_rscale;
 extern GLuint		g_location_rsource;
 extern GLuint		g_location_rparams;
 
+//water droplets
+extern GLuint		g_location_drSource;
+extern GLuint		g_location_drTex;
+extern GLuint		g_location_drTime;
+extern GLuint		g_location_drParams;
+
+//god rays
+extern GLuint		g_location_lightPositionOnScreen;
+extern GLuint		g_location_sunTex;
+extern GLuint		g_location_godrayScreenAspect;
+extern GLuint		g_location_sunRadius;
+
 //MD2
 extern void Mod_LoadMD2Model (model_t *mod, void *buffer);
 
@@ -671,8 +746,10 @@ extern m_dlight_t model_dlights[128];
 extern image_t	*r_mirrortexture;
 extern cvar_t	*cl_gun;
 vec3_t lightPosition;
+vec3_t statLightPosition;
+float statLightIntensity;
 float  dynFactor;
-extern void		R_GetLightVals(vec3_t origin, qboolean RagDoll, qboolean dynamic);
+extern void	R_GetLightVals(vec3_t origin, qboolean RagDoll);
 extern void R_ModelViewTransform(const vec3_t in, vec3_t out);
 extern void GL_BlendFunction (GLenum sfactor, GLenum dfactor);
 
@@ -686,8 +763,8 @@ extern void R_DrawINTERQUAKEMODEL(void);
 extern void IQM_AnimateFrame(float curframe, int nextframe);
 extern qboolean IQM_InAnimGroup(int frame, int oldframe);
 extern int IQM_NextFrame(int frame);
-extern void IQM_AnimateRagdoll(int RagDollID);
-extern void IQM_DrawRagDollFrame(int RagDollID, int skinnum, float shellAlpha, int shellEffect);
+extern void IQM_AnimateRagdoll(int RagDollID, int shellEffect);
+extern void IQM_DrawFrame(int skinnum, qboolean ragdoll, float shellAlpha);
 extern void IQM_DrawShadow(vec3_t origin);
 
 //Ragdoll
