@@ -46,13 +46,14 @@ glconfig_t		gl_config;
 glstate_t		gl_state;
 
 cvar_t	*gl_normalmaps;
+cvar_t	*gl_bspnormalmaps;
 cvar_t  *gl_shadowmaps;
-cvar_t	*gl_glsl_postprocess;
 cvar_t	*gl_arb_fragment_program;
 cvar_t	*gl_glsl_shaders;
+cvar_t	*gl_fog;
 
 entity_t	*currententity;
-model_t	*currentmodel;
+model_t		*currentmodel;
 
 cplane_t	frustum[4];
 
@@ -66,6 +67,7 @@ int last_c_alias_polys, c_alias_polys;
 int c_flares;
 int c_grasses;
 int c_beams;
+extern int c_vbo_batches;
 extern int c_visible_lightmaps;
 extern int c_visible_textures;
 
@@ -82,6 +84,7 @@ vec3_t	vup;
 vec3_t	vpn;
 vec3_t	vright;
 vec3_t	r_origin;
+int		r_origin_leafnum;
 
 float	r_world_matrix[16];
 float	r_project_matrix[16];
@@ -96,7 +99,6 @@ int		r_viewcluster, r_viewcluster2, r_oldviewcluster, r_oldviewcluster2;
 cvar_t	*r_norefresh;
 cvar_t	*r_drawentities;
 cvar_t	*r_drawworld;
-cvar_t	*r_speeds;
 cvar_t	*r_fullbright;
 cvar_t	*r_novis;
 cvar_t	*r_nocull;
@@ -105,7 +107,7 @@ cvar_t	*r_lefthand;
 
 cvar_t  *r_wave; // Water waves
 
-cvar_t	*r_shadowmapratio;
+cvar_t	*r_shadowmapscale;
 
 cvar_t	*r_overbrightbits;
 
@@ -122,16 +124,13 @@ cvar_t	*gl_particle_att_a;
 cvar_t	*gl_particle_att_b;
 cvar_t	*gl_particle_att_c;
 
-cvar_t	*gl_ext_swapinterval;
 cvar_t	*gl_ext_pointparameters;
-cvar_t	*gl_ext_compiled_vertex_array;
 
 cvar_t	*gl_log;
 cvar_t	*gl_bitdepth;
 cvar_t	*gl_drawbuffer;
 cvar_t	*gl_driver;
 cvar_t	*gl_lightmap;
-cvar_t	*gl_shadows;
 cvar_t	*gl_mode;
 cvar_t	*gl_dynamic;
 cvar_t	*gl_modulate;
@@ -156,6 +155,7 @@ cvar_t  *vid_contrast;
 cvar_t	*vid_ref;
 
 cvar_t *r_anisotropic;
+cvar_t *r_alphamasked_anisotropic;
 cvar_t *r_ext_max_anisotropy;
 
 cvar_t	*r_shaders;
@@ -163,6 +163,12 @@ cvar_t	*r_bloom;
 cvar_t	*r_lensflare;
 cvar_t	*r_lensflare_intens;
 cvar_t	*r_drawsun;
+cvar_t	*r_lightbeam;
+cvar_t	*r_godrays;
+cvar_t  *r_godray_intensity;
+cvar_t	*r_optimize;
+
+cvar_t	*r_lightmapfiles;
 
 qboolean	map_fog;
 
@@ -191,7 +197,7 @@ cvar_t  *r_test;
 //ODE initialization error check
 int r_odeinit_success; // 0 if dODEInit2() fails, 1 otherwise.
 
-//fog stuff
+//fog script stuff
 struct r_fog
 {
 	float red;
@@ -203,6 +209,9 @@ struct r_fog
 } fog;
 unsigned r_weather;
 unsigned r_nosun;
+float r_sunX;
+float r_sunY;
+float r_sunZ;
 
 /*
 =================
@@ -217,7 +226,7 @@ void R_ReadFogScript( char *config_file )
 	char a_string[128];
 	char *buffer;
 	char *s;
-	size_t result;
+	size_t result;	
 
 	if((fp = fopen(config_file, "rb" )) == NULL)
 	{
@@ -251,6 +260,12 @@ void R_ReadFogScript( char *config_file )
 			r_weather = atoi(a_string);
 			strcpy( a_string, COM_Parse( &s ) );
 			r_nosun = atoi(a_string);
+			strcpy( a_string, COM_Parse( &s ) );
+			r_sunX = atof(a_string);
+			strcpy( a_string, COM_Parse( &s ) );
+			r_sunY = atof(a_string);
+			strcpy( a_string, COM_Parse( &s ) );
+			r_sunZ = atof(a_string);
 
 			if(fog.density > 0)
 				map_fog = true;
@@ -263,6 +278,12 @@ void R_ReadFogScript( char *config_file )
 		free( buffer );
 	}
 	fclose( fp );
+	
+	if (gl_fog->integer < 1)
+	{
+		map_fog = false;
+		r_weather = false;
+	}
 
 	return;
 }
@@ -273,6 +294,7 @@ R_ReadMusicScript
 =================
 */
 
+//to do - read in secondary music location(for CTF music shift)
 void R_ReadMusicScript( char *config_file )
 {
 	FILE *fp;
@@ -323,7 +345,7 @@ qboolean R_CullBox (vec3_t mins, vec3_t maxs)
 	int		i;
 	cplane_t *p;
 
-	if (r_nocull->value)
+	if (r_nocull->integer)
 		return false;
 
 	for (i=0,p=frustum ; i<4; i++,p++)
@@ -388,12 +410,34 @@ qboolean R_CullOrigin(vec3_t origin)
 	return false;
 }
 
+qboolean R_CullSphere( const vec3_t centre, const float radius, const int clipflags )
+{
+	int		i;
+	cplane_t *p;
+
+	if (r_nocull->value)
+		return false;
+
+	for (i=0,p=frustum ; i<4; i++,p++)
+	{
+		if ( !(clipflags & (1<<i)) ) {
+			continue;
+		}
+
+		if ( DotProduct ( centre, p->normal ) - p->dist <= -radius )
+			return true;
+	}
+
+	return false;
+}
+
 void R_RotateForEntity (entity_t *e)
 {
     qglTranslatef (e->origin[0],  e->origin[1],  e->origin[2]);
 
     qglRotatef (e->angles[1],  0, 0, 1);
     qglRotatef (-e->angles[0],  0, 1, 0);
+
     qglRotatef (-e->angles[2],  1, 0, 0);
 }
 
@@ -435,43 +479,21 @@ void R_DrawNullModel (void)
 	qglEnable (GL_TEXTURE_2D);
 }
 
-/*
-=============
-LOD calculation
-
-The width and height occupied by a model on screen after it's been rendered
-will scale as the square of the FOV setting, and proportionally to the
-width/height of the screen itself. Based on the assumption that 500 units is
-an adequate LOD cutoff distance at 1920*1080 with an FOV of 90, we can scale
-the LOD cutoff distance to the lowest point where there will be no noticable
-ugliness.
-
-NOTE: Turns out the player's FOV setting goes to fov_x and not fov_y. Go
-figure.
-=============
-*/
-#define LOD_BASE_H      1920.0
-#define LOD_BASE_W      1080.0
-#define LOD_BASE_DIST   500.0
-#define LOD_BASE_FOV    90.0
-#define LOD_DIST        (LOD_BASE_DIST*\
-                        (vid.width/LOD_BASE_W)*(vid.height/LOD_BASE_H)*\
-                        (LOD_BASE_FOV/r_newrefdef.fov_x)*\
-                        (LOD_BASE_FOV/r_newrefdef.fov_x))
+#include "r_lodcalc.h"
 
 /*
 =============
 R_DrawEntitiesOnList
 =============
 */
+extern cvar_t *cl_simpleitems;
 void R_DrawEntitiesOnList (void)
 {
 	int		i;
 	rscript_t	*rs = NULL;
 	vec3_t	dist;
-	char    shortname[MAX_QPATH];
 
-	if (!r_drawentities->value)
+	if (!r_drawentities->integer)
 		return;
 
 	if ( !r_odeinit_success )
@@ -485,16 +507,15 @@ void R_DrawEntitiesOnList (void)
 		currententity = &r_newrefdef.entities[i];
 		if (currententity->flags & RF_TRANSLUCENT)
 			continue;	// transluscent
-
-		if (currententity->model && r_shaders->value)
+		
+		if (currententity->model && r_shaders->integer)
 		{
 			rs=(rscript_t *)currententity->model->script;
 
 			//custom player skin (must be done here)
 			if (currententity->skin)
 			{
-                COM_StripExtension ( currententity->skin->name, shortname );
-                rs = RS_FindScript(shortname);
+			    rs = currententity->skin->script;
                 if(rs)
                     RS_ReadyScript(rs);
             }
@@ -506,9 +527,14 @@ void R_DrawEntitiesOnList (void)
 		}
 
 		currentmodel = currententity->model;
+		
+		if (cl_simpleitems->integer && currentmodel && currentmodel->simple_texnum)
+			continue;
 
-		//get distance, set lod if available
-		VectorSubtract(r_origin, currententity->origin, dist);
+		//get distance
+		VectorSubtract(r_origin, currententity->origin, dist);		
+		
+		//set lod if available
 		if(VectorLength(dist) > LOD_DIST*2.0)
 		{
 			if(currententity->lod2)
@@ -551,15 +577,14 @@ void R_DrawEntitiesOnList (void)
 		if (!(currententity->flags & RF_TRANSLUCENT))
 			continue;	// solid
 
-		if (currententity->model && r_shaders->value)
+		if (currententity->model && r_shaders->integer)
 		{
 			rs=(rscript_t *)currententity->model->script;
 
 			//custom player skin (must be done here)
 			if (currententity->skin)
 			{
-                COM_StripExtension ( currententity->skin->name, shortname );
-                rs = RS_FindScript(shortname);
+                rs = currententity->skin->script;
                 if(rs)
                     RS_ReadyScript(rs);
             }
@@ -571,6 +596,7 @@ void R_DrawEntitiesOnList (void)
 		}
 
 		currentmodel = currententity->model;
+		fadeShadow = 1.0;
 
 		if (!currentmodel)
 		{
@@ -600,9 +626,8 @@ void R_DrawViewEntitiesOnList (void)
 {
 	int		i;
 	rscript_t	*rs = NULL;
-	char    shortname[MAX_QPATH];
 
-	if (!r_drawentities->value)
+	if (!r_drawentities->integer)
 		return;
 
 	if(r_newrefdef.rdflags & RDF_NOWORLDMODEL)
@@ -615,15 +640,14 @@ void R_DrawViewEntitiesOnList (void)
 		if (currententity->flags & RF_TRANSLUCENT)
 			continue;	// transluscent
 
-		if (currententity->model && r_shaders->value)
+		if (currententity->model && r_shaders->integer)
 		{
 			rs=(rscript_t *)currententity->model->script;
 
 			//custom player skin (must be done here)
 			if (currententity->skin)
 			{
-                COM_StripExtension ( currententity->skin->name, shortname );
-                rs = RS_FindScript(shortname);
+                rs = currententity->skin->script;
                 if(rs)
                     RS_ReadyScript(rs);
             }
@@ -664,15 +688,14 @@ void R_DrawViewEntitiesOnList (void)
 		if (!(currententity->flags & RF_TRANSLUCENT))
 			continue;	// solid
 
-		if (currententity->model && r_shaders->value)
+		if (currententity->model && r_shaders->integer)
 		{
 			rs=(rscript_t *)currententity->model->script;
 
 			//custom player skin (must be done here)
 			if (currententity->skin)
 			{
-                COM_StripExtension ( currententity->skin->name, shortname );
-                rs = RS_FindScript(shortname);
+                rs = currententity->skin->script;
                 if(rs)
                     RS_ReadyScript(rs);
             }
@@ -717,12 +740,12 @@ R_PolyBlend
 */
 void R_PolyBlend (void)
 {
-	if (!gl_polyblend->value)
+	if (!gl_polyblend->integer)
 		return;
 	if (!v_blend[3])
 		return;
 
-	if(!r_drawing_fbeffect && cl_paindist->value) {
+	if(!r_drawing_fbeffect && cl_paindist->integer) {
 		if(v_blend[0] > 2*v_blend[1] && v_blend[0] > 2*v_blend[2]) {
 			r_drawing_fbeffect = true;
 			r_fbFxType = 2; //FLASH DISTORTION
@@ -806,7 +829,7 @@ R_SetupFrame
 void R_SetupFrame (void)
 {
 	int i;
-	mleaf_t	*leaf;
+	mleaf_t *leaf;
 
 	r_framecount++;
 
@@ -820,7 +843,8 @@ void R_SetupFrame (void)
 	{
 		r_oldviewcluster = r_viewcluster;
 		r_oldviewcluster2 = r_viewcluster2;
-		leaf = Mod_PointInLeaf (r_origin, r_worldmodel);
+		r_viewleaf = leaf = Mod_PointInLeaf (r_origin, r_worldmodel);
+		r_origin_leafnum = CM_PointLeafnum (r_origin);
 		r_viewcluster = r_viewcluster2 = leaf->cluster;
 
 		// check above and below so crossing solid water doesn't draw wrong
@@ -830,7 +854,7 @@ void R_SetupFrame (void)
 
 			VectorCopy (r_origin, temp);
 			temp[2] -= 16;
-			leaf = Mod_PointInLeaf (temp, r_worldmodel);
+			r_viewleaf2 = leaf = Mod_PointInLeaf (temp, r_worldmodel);
 			if ( !(leaf->contents & CONTENTS_SOLID) &&
 				(leaf->cluster != r_viewcluster2) )
 				r_viewcluster2 = leaf->cluster;
@@ -841,7 +865,7 @@ void R_SetupFrame (void)
 
 			VectorCopy (r_origin, temp);
 			temp[2] += 16;
-			leaf = Mod_PointInLeaf (temp, r_worldmodel);
+			r_viewleaf2 = leaf = Mod_PointInLeaf (temp, r_worldmodel);
 			if ( !(leaf->contents & CONTENTS_SOLID) &&
 				(leaf->cluster != r_viewcluster2) )
 				r_viewcluster2 = leaf->cluster;
@@ -876,6 +900,29 @@ void MYgluPerspective( GLdouble fovy, GLdouble aspect,
 
 
 
+/*
+=============
+R_SetupViewport
+=============
+*/
+void R_SetupViewport (void)
+{
+	int		x, y, w, h;
+
+	// The viewport info in r_newrefdef is constructed with the upper left 
+	// corner as the origin, whereas glViewport treats the lower left corner
+	// as the origin. So we have to do some math to fix the y-coordinates.
+	
+	x = r_newrefdef.x;
+	w = r_newrefdef.width;
+	y = vid.height - r_newrefdef.y - r_newrefdef.height;
+	h = r_newrefdef.height;
+	
+	qglViewport (x, y, w, h);	// MPO : note this happens every frame interestingly enough
+}
+
+
+
 /*
 =============
 R_SetupGL
@@ -884,20 +931,8 @@ R_SetupGL
 void R_SetupGL (void)
 {
 	float	screenaspect;
-	int		x, x2, y2, y, w, h;
-
-	//
-	// set up viewport
-	//
-	x = floor(r_newrefdef.x * vid.width / vid.width);
-	x2 = ceil((r_newrefdef.x + r_newrefdef.width) * vid.width / vid.width);
-	y = floor(vid.height - r_newrefdef.y * vid.height / vid.height);
-	y2 = ceil(vid.height - (r_newrefdef.y + r_newrefdef.height) * vid.height / vid.height);
-
-	w = x2 - x;
-	h = y - y2;
-
-	qglViewport (x, y2, w, h);	// MPO : note this happens every frame interestingly enough
+	
+	R_SetupViewport ();
 
 	//
 	// set up projection matrix
@@ -932,7 +967,7 @@ void R_SetupGL (void)
 	// set drawing parms
 	//
 
-	if (gl_cull->value)
+	if (gl_cull->integer)
 		qglEnable(GL_CULL_FACE);
 
 	qglDisable(GL_BLEND);
@@ -952,9 +987,9 @@ extern qboolean have_stencil;
 void R_Clear (void)
 {
 	qglClearColor(0,0,0,1);
-	if (gl_clear->value)
+	if (gl_clear->integer)
 		qglClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-	else if (!cl_add_blend->value && info_spectator->value && (CM_PointContents(r_newrefdef.vieworg, 0) & CONTENTS_SOLID))
+	else if (!cl_add_blend->integer && info_spectator->integer && (CM_PointContents(r_newrefdef.vieworg, 0) & CONTENTS_SOLID))
 		qglClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); //out of map
 	else
 		qglClear (GL_DEPTH_BUFFER_BIT);
@@ -965,16 +1000,12 @@ void R_Clear (void)
 
 	qglDepthRange (gldepthmin, gldepthmax);
 
-	//our shadow system uses a combo of shadmaps and stencil volumes.
-	if (have_stencil && (gl_shadows->integer || gl_shadowmaps->value)) {
+	 //our shadow system uses a combo of shadmaps and stencil volumes.
+    if (have_stencil && gl_shadowmaps->integer) {
 
-		if(gl_shadowmaps->value)
-			qglClearStencil(0);
-		else
-			qglClearStencil(1);
-
-		qglClear(GL_STENCIL_BUFFER_BIT);
-	}
+        qglClearStencil(0);
+        qglClear(GL_STENCIL_BUFFER_BIT);
+    }
 }
 
 void R_Flash( void )
@@ -993,17 +1024,16 @@ r_newrefdef must be set before the first call
 void R_RenderView (refdef_t *fd)
 {
 	GLfloat colors[4] = {(GLfloat) fog.red, (GLfloat) fog.green, (GLfloat) fog.blue, (GLfloat) 0.1};
-	static int printf_throttle;
 
 	numRadarEnts = 0;
 
-	if (r_norefresh->value)
+	if (r_norefresh->integer)
 		return;
 
 	r_newrefdef = *fd;
 
 	//shadowmaps
-	if(gl_shadowmaps->value) {
+	if(gl_shadowmaps->integer) {
 
 		qglEnable(GL_DEPTH_TEST);
 		qglClearColor(0,0,0,1.0f);
@@ -1024,7 +1054,7 @@ void R_RenderView (refdef_t *fd)
 
 	if (!r_worldmodel && !( r_newrefdef.rdflags & RDF_NOWORLDMODEL ) )
 		Com_Error (ERR_DROP, "R_RenderView: NULL worldmodel");
-
+	
 	// init r_speeds counters
 	last_c_brush_polys = c_brush_polys;
 	last_c_alias_polys = c_alias_polys;
@@ -1033,20 +1063,23 @@ void R_RenderView (refdef_t *fd)
 	c_flares = 0;
 	c_grasses = 0;
 	c_beams = 0;
+	c_vbo_batches = 0;
 
 	R_PushDlights ();
 
-	if (gl_finish->value)
-		qglFinish ();
-
 	R_SetupFrame ();
 
 	R_SetFrustum ();
+	
+	R_MarkWorldSurfs ();	// done here so we know if we're in water
+	
+	if (gl_finish->integer)
+		qglFinish ();
+	
+	// OpenGL calls come after here
 
 	R_SetupGL ();
 
-	R_MarkLeaves ();	// done here so we know if we're in water
-
 	if(map_fog)
 	{
 		qglFogi(GL_FOG_MODE, GL_LINEAR);
@@ -1057,17 +1090,17 @@ void R_RenderView (refdef_t *fd)
 		qglEnable(GL_FOG);
 	}
 
-	R_DrawWorld ();
+	R_DrawWorldSurfs ();
 
-	R_DrawRSSurfaces();
-
-	if(r_lensflare->value)
+	if(r_lensflare->integer)
 		R_RenderFlares ();
 
 	R_DrawEntitiesOnList ();
-		
+			
 	R_DrawVegetationSurface ();
 
+	R_DrawSimpleItems ();
+
 	R_CastShadow();
 
 	R_RenderAllRagdolls(); //move back ahead of r_castshadow when we figure out shadow jitter bug
@@ -1076,20 +1109,29 @@ void R_RenderView (refdef_t *fd)
 
 	R_DrawAlphaSurfaces ();
 
-	R_DrawBeamSurface ();
-
-	qglLoadMatrixf (r_world_matrix); //moving trans brushes
+	if (r_lightbeam->integer)
+		R_DrawBeamSurface ();
 
 	R_DrawParticles ();
 
-	if(gl_mirror->value) {
-
-		qglBindTexture(GL_TEXTURE_2D, r_mirrortexture->texnum);
-		qglCopyTexSubImage2D(GL_TEXTURE_2D, 0,
-					0, 0, 0, 256, 512, 512);
+	if(gl_mirror->integer) 
+	{
+		int ms = Sys_Milliseconds ();
+		if (	ms < r_newrefdef.last_mirrorupdate_time || 
+				(ms-r_newrefdef.last_mirrorupdate_time) >= 16)
+		{
+			GL_SelectTexture (GL_TEXTURE0);
+			GL_Bind (r_mirrortexture->texnum);
+			qglCopyTexSubImage2D(GL_TEXTURE_2D, 0,
+						0, 0, 0, r_mirrortexture->upload_height/2, 
+						r_mirrortexture->upload_width, 
+						r_mirrortexture->upload_height);
+			r_newrefdef.last_mirrorupdate_time = ms;
+		}
 	}
 
-	R_BloomBlend( fd );//BLOOMS
+
+	R_BloomBlend( &r_newrefdef );//BLOOMS
 
 	R_RenderSun();
 
@@ -1099,27 +1141,12 @@ void R_RenderView (refdef_t *fd)
 
 	R_Flash();
 
-	if (r_speeds->value == 1 )
-	{ // display r_speeds 'wpoly' and 'epoly'counters (value==2 puts on HUD)
-		if ( c_brush_polys != last_c_brush_polys
-			|| c_alias_polys != last_c_alias_polys )
-		{
-			Com_Printf( "%5i wpoly %5i epoly\n", c_brush_polys, c_alias_polys );
-		}
-	}
-	else if( r_speeds->value == 3 )
-	{ // display other counters, of unknown utility
-		if ( !(++printf_throttle % 32) )
-		{
-			Com_Printf( "%5i flares %5i grasses %5i vlmaps %5i vtex %5i beams\n",
-			c_flares, c_grasses, c_visible_lightmaps, c_visible_textures, c_beams );
-		}
-	}
-
 	if(map_fog)
 		qglDisable(GL_FOG);
 
 	R_DrawRadar();
+	
+	*fd = r_newrefdef;
 }
 
 void	R_SetGL2D (void)
@@ -1173,7 +1200,8 @@ void R_Register( void )
 
 	con_font = Cvar_Get ("con_font", "default", CVAR_ARCHIVE);
 
-	r_lefthand = Cvar_Get( "hand", "0", CVAR_USERINFO | CVAR_ARCHIVE );
+	r_lefthand = Cvar_Get( "hand", "0", CVAR_USERINFO | CVAR_ARCHIVE | CVARDOC_INT);
+	Cvar_Describe (r_lefthand, "0 means show gun on the right, 1 means show gun on the left, 2 means hide the gun altogether.");
 	r_norefresh = Cvar_Get ("r_norefresh", "0", 0);
 	r_fullbright = Cvar_Get ("r_fullbright", "0", 0);
 	r_drawentities = Cvar_Get ("r_drawentities", "1", 0);
@@ -1181,7 +1209,6 @@ void R_Register( void )
 	r_novis = Cvar_Get ("r_novis", "0", 0);
 	r_nocull = Cvar_Get ("r_nocull", "0", 0);
 	r_lerpmodels = Cvar_Get ("r_lerpmodels", "1", 0);
-	r_speeds = Cvar_Get ("r_speeds", "0", 0);
 
 	r_wave = Cvar_Get ("r_wave", "2", CVAR_ARCHIVE); // Water waves
 
@@ -1194,87 +1221,106 @@ void R_Register( void )
 	gl_particle_att_b = Cvar_Get( "gl_particle_att_b", "0.0", CVAR_ARCHIVE );
 	gl_particle_att_c = Cvar_Get( "gl_particle_att_c", "0.01", CVAR_ARCHIVE );
 
-	gl_modulate = Cvar_Get ("gl_modulate", "2", CVAR_ARCHIVE );
+	gl_modulate = Cvar_Get ("gl_modulate", "2", CVAR_ARCHIVE|CVARDOC_INT );
+	Cvar_Describe (gl_modulate, "Brightness setting. Higher means brighter.");
 	gl_log = Cvar_Get( "gl_log", "0", 0 );
 	gl_bitdepth = Cvar_Get( "gl_bitdepth", "0", 0 );
 	gl_mode = Cvar_Get( "gl_mode", "3", CVAR_ARCHIVE );
 	gl_lightmap = Cvar_Get ("gl_lightmap", "0", 0);
-	gl_shadows = Cvar_Get ("gl_shadows", "2", CVAR_ARCHIVE );
 	gl_nobind = Cvar_Get ("gl_nobind", "0", 0);
-	gl_picmip = Cvar_Get ("gl_picmip", "0", 0);
+	gl_picmip = Cvar_Get ("gl_picmip", "0", CVAR_ARCHIVE|CVARDOC_INT);
+	Cvar_Describe (gl_picmip, "Texture detail. 0 means full detail. Each higher setting has 1/4 less detail.");
 	gl_skymip = Cvar_Get ("gl_skymip", "0", 0);
 	gl_showtris = Cvar_Get ("gl_showtris", "0", 0);
-	gl_showpolys = Cvar_Get ("gl_showpolys", "0", 0);
-	gl_finish = Cvar_Get ("gl_finish", "0", CVAR_ARCHIVE);
-	gl_clear = Cvar_Get ("gl_clear", "0", 0);
-	gl_cull = Cvar_Get ("gl_cull", "1", 0);
+	gl_showpolys = Cvar_Get ("gl_showpolys", "0", CVARDOC_INT);
+	Cvar_Describe (gl_showpolys, "Useful tool for mappers. 1 means show world polygon outlines for visible surfaces. 2 means show outlines for all surfaces in the PVS, even if they are hidden. Only works with gl_usevbo 0.");
+	gl_finish = Cvar_Get ("gl_finish", "0", CVAR_ARCHIVE|CVARDOC_BOOL);
+	Cvar_Describe (gl_finish, "Waits for graphics driver to finish drawing each frame before drawing the next one. Hurts performance but may improve smoothness on very low-end machines.");
+	gl_clear = Cvar_Get ("gl_clear", "0", CVARDOC_BOOL);
+	gl_cull = Cvar_Get ("gl_cull", "1", CVARDOC_BOOL);
+	Cvar_Describe (gl_cull, "Avoid rendering anything that's off the edge of the screen. Good for performance, recommend leaving it on.");
 	gl_polyblend = Cvar_Get ("gl_polyblend", "1", 0);
 
 // OPENGL_DRIVER defined by in config.h
-#if defined DARWIN_SPECIAL_CASE
-	// archiving gl_driver means the lib specified in config.h may be
-	//  overridden by what is in config.cfg. This defeats any attempt
-	//  by configure to specify a different location for the libGL.dylib
-	// for now make it a special case, but questionable whether it should
-	// be archived for others.
 	gl_driver = Cvar_Get( "gl_driver", OPENGL_DRIVER, 0 );
-#else
-	gl_driver = Cvar_Get( "gl_driver", OPENGL_DRIVER, CVAR_ARCHIVE );
-#endif
 
 	gl_texturemode = Cvar_Get( "gl_texturemode", "GL_LINEAR_MIPMAP_LINEAR", CVAR_ARCHIVE );
 	gl_texturealphamode = Cvar_Get( "gl_texturealphamode", "default", CVAR_ARCHIVE );
 	gl_texturesolidmode = Cvar_Get( "gl_texturesolidmode", "default", CVAR_ARCHIVE );
 	gl_lockpvs = Cvar_Get( "gl_lockpvs", "0", 0 );
 
-	gl_ext_swapinterval = Cvar_Get( "gl_ext_swapinterval", "1", CVAR_ARCHIVE );
 	gl_ext_pointparameters = Cvar_Get( "gl_ext_pointparameters", "0", CVAR_ARCHIVE );
-	gl_ext_compiled_vertex_array = Cvar_Get( "gl_ext_compiled_vertex_array", "1", CVAR_ARCHIVE );
 
 	gl_drawbuffer = Cvar_Get( "gl_drawbuffer", "GL_BACK", 0 );
-	gl_swapinterval = Cvar_Get( "gl_swapinterval", "1", CVAR_ARCHIVE );
+	gl_swapinterval = Cvar_Get( "gl_swapinterval", "1", CVAR_ARCHIVE|CVARDOC_BOOL );
+	Cvar_Describe (gl_swapinterval, "Sync to Vblank. Eliminates \"tearing\" effects, but it can hurt framerates.");
 
-	r_shaders = Cvar_Get ("r_shaders", "1", CVAR_ARCHIVE);
+	r_shaders = Cvar_Get ("r_shaders", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
 
 	r_overbrightbits = Cvar_Get( "r_overbrightbits", "2", CVAR_ARCHIVE );
 
-	gl_usevbo = Cvar_Get("gl_usevbo", "1", CVAR_ARCHIVE );
+	gl_usevbo = Cvar_Get("gl_usevbo", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	Cvar_Describe (gl_usevbo, "Use VBOs for mesh and world geometry. Leave this on unless you've got a very old graphics card.");
 
-	gl_mirror = Cvar_Get("gl_mirror", "1", CVAR_ARCHIVE);
+	gl_mirror = Cvar_Get("gl_mirror", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
 
-	vid_fullscreen = Cvar_Get( "vid_fullscreen", "1", CVAR_ARCHIVE );
+	vid_fullscreen = Cvar_Get( "vid_fullscreen", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
 	vid_gamma = Cvar_Get( "vid_gamma", "1.0", CVAR_ARCHIVE );
 	vid_contrast = Cvar_Get( "vid_contrast", "1.0", CVAR_ARCHIVE);
-	vid_ref = Cvar_Get( "vid_ref", "gl", CVAR_ARCHIVE );
+	//TODO: remove, unless we decide to add GL ES support or something.
+	vid_ref = Cvar_Get( "vid_ref", "gl", CVAR_ARCHIVE|CVAR_ROM ); 
 
 	gl_vlights = Cvar_Get("gl_vlights", "1", CVAR_ARCHIVE);
 
-	gl_normalmaps = Cvar_Get("gl_normalmaps", "0", CVAR_ARCHIVE);
-	gl_shadowmaps = Cvar_Get("gl_shadowmaps", "0", CVAR_ARCHIVE);
-	gl_glsl_postprocess = Cvar_Get("gl_glsl_postprocess", "1", CVAR_ARCHIVE);
+	gl_normalmaps = Cvar_Get("gl_normalmaps", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	gl_bspnormalmaps = Cvar_Get("gl_bspnormalmaps", "0", CVAR_ARCHIVE|CVARDOC_BOOL);
+	gl_shadowmaps = Cvar_Get("gl_shadowmaps", "0", CVAR_ARCHIVE|CVARDOC_BOOL);
+	gl_fog = Cvar_Get ("gl_fog", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	Cvar_Describe (gl_fog, "Fog and weather effects.");
 
-	r_shadowmapratio = Cvar_Get( "r_shadowmapratio", "2", CVAR_ARCHIVE );
+	r_shadowmapscale = Cvar_Get( "r_shadowmapscale", "1", CVAR_ARCHIVE );
+	r_shadowcutoff = Cvar_Get( "r_shadowcutoff", "880", CVAR_ARCHIVE );
 
-	r_lensflare = Cvar_Get( "r_lensflare", "1", CVAR_ARCHIVE );
-	r_lensflare_intens = Cvar_Get ("r_lensflare_intens", "3", CVAR_ARCHIVE);
+	r_lensflare = Cvar_Get( "r_lensflare", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	r_lensflare_intens = Cvar_Get ("r_lensflare_intens", "3", CVAR_ARCHIVE|CVARDOC_INT);
 	r_drawsun =	Cvar_Get("r_drawsun", "2", CVAR_ARCHIVE);
+	r_lightbeam = Cvar_Get ("r_lightbeam", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	r_godrays = Cvar_Get ("r_godrays", "0", CVAR_ARCHIVE|CVARDOC_BOOL);
+	r_godray_intensity = Cvar_Get ("r_godray_intensity", "1.0", CVAR_ARCHIVE);
+	r_optimize = Cvar_Get ("r_optimize", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	Cvar_Describe (r_optimize, "Skip BSP recursion unless you move. Good for performance, recommend leaving it on.");
+	
+	r_lightmapfiles = Cvar_Get("r_lightmapfiles", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	Cvar_Describe (r_lightmapfiles, "Enables the loading of .lightmap files, with more detailed light and shadow. Turn this off if video RAM is limited.");
 
 	r_minimap_size = Cvar_Get ("r_minimap_size", "256", CVAR_ARCHIVE );
 	r_minimap_zoom = Cvar_Get ("r_minimap_zoom", "1", CVAR_ARCHIVE );
 	r_minimap_style = Cvar_Get ("r_minimap_style", "1", CVAR_ARCHIVE );
-	r_minimap = Cvar_Get ("r_minimap", "0", CVAR_ARCHIVE );
-	r_ragdolls = Cvar_Get ("r_ragdolls", "1", CVAR_ARCHIVE );
-	r_ragdoll_debug = Cvar_Get("r_ragdoll_debug", "0", CVAR_ARCHIVE );
+	r_minimap = Cvar_Get ("r_minimap", "0", CVAR_ARCHIVE|CVARDOC_BOOL );
+
+	r_ragdolls = Cvar_Get ("r_ragdolls", "1", CVAR_ARCHIVE|CVARDOC_BOOL);
+	r_ragdoll_debug = Cvar_Get("r_ragdoll_debug", "0", CVAR_ARCHIVE|CVARDOC_BOOL);
 
 	sys_priority = Cvar_Get("sys_priority", "0", CVAR_ARCHIVE);
 	sys_affinity = Cvar_Get("sys_affinity", "1", CVAR_ARCHIVE);
 
-	gl_screenshot_type = Cvar_Get("gl_screenshot_type", "jpeg", CVAR_ARCHIVE);
-	gl_screenshot_jpeg_quality = Cvar_Get("gl_screenshot_jpeg_quality", "85", CVAR_ARCHIVE);
+	gl_screenshot_type = Cvar_Get("gl_screenshot_type", "jpeg", CVAR_ARCHIVE|CVARDOC_STR);
+	gl_screenshot_jpeg_quality = Cvar_Get("gl_screenshot_jpeg_quality", "85", CVAR_ARCHIVE|CVARDOC_INT);
 
-	r_firstrun = Cvar_Get("r_firstrun", "0", CVAR_ARCHIVE); //first time running the game
+	r_firstrun = Cvar_Get("r_firstrun", "0", CVAR_ARCHIVE|CVARDOC_BOOL); //first time running the game
+	Cvar_Describe (r_firstrun, "Set this to 0 if you want the game to auto detect your graphics settings next time you run it.");
 
 	r_test = Cvar_Get("r_test", "0", CVAR_ARCHIVE); //for testing things
+	
+	// FIXME HACK copied over from the video menu code. These are initialized
+	// again elsewhere. TODO: work out any complications that may arise from
+	// deleting these duplicate initializations.
+	Cvar_Get( "r_bloom", "0", CVAR_ARCHIVE );
+	Cvar_Get( "r_bloom_intensity", "0.5", CVAR_ARCHIVE);
+	Cvar_Get( "r_overbrightbits", "2", CVAR_ARCHIVE);
+	Cvar_Get( "vid_width", "640", CVAR_ARCHIVE);
+	Cvar_Get( "vid_height", "400", CVAR_ARCHIVE);
+	Cvar_Get( "gl_glsl_shaders", "1", CVAR_ARCHIVE);
 
 	Cmd_AddCommand( "imagelist", GL_ImageList_f );
 	Cmd_AddCommand( "screenshot", GL_ScreenShot_f );
@@ -1295,18 +1341,18 @@ qboolean R_SetMode (void)
 	if ( vid_fullscreen->modified && !gl_config.allow_cds )
 	{
 		Com_Printf ("R_SetMode() - CDS not allowed with this driver\n" );
-		Cvar_SetValue( "vid_fullscreen", !vid_fullscreen->value );
+		Cvar_SetValue( "vid_fullscreen", !vid_fullscreen->integer );
 		vid_fullscreen->modified = false;
 	}
 
-	fullscreen = vid_fullscreen->value;
+	fullscreen = vid_fullscreen->integer;
 
 	vid_fullscreen->modified = false;
 	gl_mode->modified = false;
 
-	if ( ( err = GLimp_SetMode( &vid.width, &vid.height, gl_mode->value, fullscreen ) ) == rserr_ok )
+	if ( ( err = GLimp_SetMode( &vid.width, &vid.height, gl_mode->integer, fullscreen ) ) == rserr_ok )
 	{
-		gl_state.prev_mode = gl_mode->value;
+		gl_state.prev_mode = gl_mode->integer;
 	}
 	else
 	{
@@ -1315,7 +1361,7 @@ qboolean R_SetMode (void)
 			Cvar_SetValue( "vid_fullscreen", 0);
 			vid_fullscreen->modified = false;
 			Com_Printf ("ref_gl::R_SetMode() - fullscreen unavailable in this mode\n" );
-			if ( ( err = GLimp_SetMode( &vid.width, &vid.height, gl_mode->value, false ) ) == rserr_ok )
+			if ( ( err = GLimp_SetMode( &vid.width, &vid.height, gl_mode->integer, false ) ) == rserr_ok )
 				return true;
 		}
 		else if ( err == rserr_invalid_mode )
@@ -1337,142 +1383,67 @@ qboolean R_SetMode (void)
 
 /*
 ===============
-R_SetLowest
+R_SetCompatibility
 ===============
 */
 
-void R_SetLowest(void)
+void R_SetCompatibility(void)
 {
-	Cvar_SetValue("r_bloom", 0);
-	Cvar_SetValue("r_bloom_intensity", 0.5);
-	Cvar_SetValue("r_overbrightbits", 2);
-	Cvar_SetValue("gl_modulate", 2);
-	Cvar_SetValue("gl_picmip", 0);
-	Cvar_SetValue("vid_gamma", 1);
-	Cvar_SetValue("vid_contrast", 1);
-	Cvar_SetValue("gl_normalmaps", 0);
-	Cvar_SetValue("gl_shadowmaps", 0);
-	Cvar_SetValue("gl_glsl_postprocess", 0);
-	Cvar_SetValue("gl_glsl_shaders", 0);
-	Cvar_SetValue("r_shaders", 0);
-	Cvar_SetValue("gl_shadows", 0);
-	Cvar_SetValue("gl_dynamic", 0);
-	Cvar_SetValue("gl_mirror", 0);
-	Cvar_SetValue("gl_vlights", 0);
-
-	Com_Printf("...autodetected LOWEST game setting\n");
+	Cmd_ExecuteString ("exec graphical_presets/compatibility.cfg");
+	Cbuf_Execute ();
+	Com_Printf("...autodetected MAX COMPATIBILITY game setting\n");
 }
 
 /*
 ===============
-R_SetLow
+R_SetMaxPerformance
 ===============
 */
 
-void R_SetLow( void )
+void R_SetMaxPerformance( void )
 {
-	Cvar_SetValue("r_bloom", 0);
-	Cvar_SetValue("r_bloom_intensity", 0.5);
-	Cvar_SetValue("r_overbrightbits", 2);
-	Cvar_SetValue("gl_modulate", 2);
-	Cvar_SetValue("gl_picmip", 0);
-	Cvar_SetValue("vid_gamma", 1);
-	Cvar_SetValue("vid_contrast", 1);
-	Cvar_SetValue("gl_normalmaps", 0);
-	Cvar_SetValue("gl_shadowmaps", 0);
-	Cvar_SetValue("gl_glsl_postprocess", 0);
-	Cvar_SetValue("gl_glsl_shaders", 0);
-	Cvar_SetValue("r_shaders", 1);
-	Cvar_SetValue("gl_shadows", 2);
-	Cvar_SetValue("gl_dynamic", 0);
-	Cvar_SetValue("gl_mirror", 1);
-	Cvar_SetValue("gl_vlights", 0);
-
-	Com_Printf("...autodetected LOW game setting\n");
+	Cmd_ExecuteString ("exec graphical_presets/maxperformance.cfg");
+	Cbuf_Execute ();
+	Com_Printf("...autodetected MAX PERFORMANCE game setting\n");
 }
 
 /*
 ===============
-R_SetMedium
+R_SetPerformance
 ===============
 */
 
-void R_SetMedium( void )
+void R_SetPerformance( void )
 {
-	Cvar_SetValue("r_bloom", 0);
-	Cvar_SetValue("r_bloom_intensity", 0.5);
-	Cvar_SetValue("r_overbrightbits", 2);
-	Cvar_SetValue("gl_modulate", 2);
-	Cvar_SetValue("gl_picmip", 0);
-	Cvar_SetValue("vid_gamma", 1);
-	Cvar_SetValue("vid_contrast", 1);
-	Cvar_SetValue("gl_normalmaps", 0);
-	Cvar_SetValue("gl_shadowmaps", 0);
-	Cvar_SetValue("gl_glsl_postprocess", 1);
-	Cvar_SetValue("gl_glsl_shaders", 1);
-	Cvar_SetValue("r_shaders", 1);
-	Cvar_SetValue("gl_shadows", 2);
-	Cvar_SetValue("gl_dynamic", 1);
-	Cvar_SetValue("gl_mirror", 1);
-	Cvar_SetValue("gl_vlights", 1);
-
-	Com_Printf("...autodetected MEDIUM game setting\n");
+	Cmd_ExecuteString ("exec graphical_presets/performance.cfg");
+	Cbuf_Execute ();
+	Com_Printf("...autodetected PERFORMANCE game setting\n");
 }
 
 /*
 ===============
-R_SetHigh
+R_SetQuality
 ===============
 */
 
-void R_SetHigh( void )
+void R_SetQuality( void )
 {
-	Cvar_SetValue("r_bloom", 1);
-	Cvar_SetValue("r_bloom_intensity", 0.5);
-	Cvar_SetValue("r_overbrightbits", 2);
-	Cvar_SetValue("gl_modulate", 2);
-	Cvar_SetValue("gl_picmip", 0);
-	Cvar_SetValue("vid_gamma", 1);
-	Cvar_SetValue("vid_contrast", 1);
-	Cvar_SetValue("gl_normalmaps", 1);
-	Cvar_SetValue("gl_shadowmaps", 0);
-	Cvar_SetValue("gl_glsl_postprocess", 1);
-	Cvar_SetValue("gl_glsl_shaders", 1);
-	Cvar_SetValue("r_shaders", 1);
-	Cvar_SetValue("gl_shadows", 2);
-	Cvar_SetValue("gl_dynamic", 1);
-	Cvar_SetValue("gl_mirror", 1);
-	Cvar_SetValue("gl_vlights", 1);
-
-	Com_Printf("...autodetected HIGH game setting\n");
+	Cmd_ExecuteString ("exec graphical_presets/quality.cfg");
+	Cbuf_Execute ();
+	Com_Printf("...autodetected QUALITY game setting\n");
 }
 
 /*
 ===============
-R_SetHighest
+R_SetMaxQuality
 ===============
 */
 
-void R_SetHighest( void )
+void R_SetMaxQuality( void )
 {
-	Cvar_SetValue("r_bloom", 1);
-	Cvar_SetValue("r_bloom_intensity", 0.5);
-	Cvar_SetValue("r_overbrightbits", 2);
-	Cvar_SetValue("gl_modulate", 2);
-	Cvar_SetValue("gl_picmip", 0);
-	Cvar_SetValue("vid_gamma", 1);
-	Cvar_SetValue("vid_contrast", 1);
-	Cvar_SetValue("gl_normalmaps", 1);
-	Cvar_SetValue("gl_shadowmaps", 1);
-	Cvar_SetValue("gl_glsl_postprocess", 1);
-	Cvar_SetValue("gl_glsl_shaders", 1);
-	Cvar_SetValue("r_shaders", 1);
-	Cvar_SetValue("gl_shadows", 0);
-	Cvar_SetValue("gl_dynamic", 1);
-	Cvar_SetValue("gl_mirror", 1);
-	Cvar_SetValue("gl_vlights", 1);
-
-	Com_Printf("...autodetected HIGHEST game setting\n");
+	Cmd_ExecuteString ("exec graphical_presets/maxquality.cfg");
+	Cbuf_Execute ();
+	Com_Printf("...autodetected MAX QUALITY game setting\n");
 }
 
 #if defined WIN32_VARIANT
@@ -1553,8 +1524,6 @@ int R_Init( void *hinstance, void *hWnd )
 		return -1;
 	}
 
-	VID_MenuInit();
-
 	/*
 	** get our various GL strings
 	*/
@@ -1599,7 +1568,7 @@ int R_Init( void *hinstance, void *hWnd )
 
 	if (strstr(gl_config.extensions_string, "GL_EXT_point_parameters"))
 	{
-		if(gl_ext_pointparameters->value)
+		if(gl_ext_pointparameters->integer)
 		{
 			qglPointParameterfEXT = (void(APIENTRY*)(GLenum, GLfloat))qwglGetProcAddress("glPointParameterfEXT");
 			qglPointParameterfvEXT = (void(APIENTRY*)(GLenum, const GLfloat*))qwglGetProcAddress("glPointParameterfvEXT");
@@ -1633,21 +1602,25 @@ int R_Init( void *hinstance, void *hWnd )
 	gl_glsl_shaders = Cvar_Get("gl_glsl_shaders", "0", CVAR_ARCHIVE);
 	gl_dynamic = Cvar_Get ("gl_dynamic", "0", CVAR_ARCHIVE);
 	Cvar_SetValue("r_firstrun", 1);
-	R_SetLow();
+	R_SetMaxPerformance();
 	Com_Printf("...Development Workaround. Low game settings forced.\n");
 #else
 
+	//always do this check for ATI drivers - they are somewhat bugged in regards to shadowmapping and use of shadow2dproj command
+	if(!strcmp(gl_config.vendor_string, "ATI Technologies Inc."))
+		gl_state.ati = true;
+
 	//load shader programs
 	R_LoadARBPrograms();
 
 	R_LoadGLSLPrograms();
 
 	//if running for the very first time, automatically set video settings
-	if(!r_firstrun->value)
+	if(!r_firstrun->integer)
 	{
 		qboolean ati_nvidia = false;
 		double CPUTotalSpeed = 4000.0; //default to this
-		int OGLVer = atoi(&gl_config.version_string[0]);
+		double OGLVer = atof(&gl_config.version_string[0]);
 		//int OGLSubVer = atoi(&gl_config.version_string[2]);
 
 #if defined WIN32_VARIANT
@@ -1697,25 +1670,25 @@ cpuinfo_exit:
 		if(!strcmp(gl_config.vendor_string, "ATI Technologies Inc.") || !strcmp(gl_config.vendor_string, "NVIDIA Corporation"))
 			ati_nvidia = true;
 
-		if(OGLVer < 2)
+		if(OGLVer < 2.1)
 		{
-			//weak GPU, set low
-			R_SetLow();
+			//weak GPU, set to maximum compatibility
+			R_SetCompatibility();
 		}
-		else if(OGLVer == 3)
+		else if(OGLVer >= 3)
 		{
 			//GPU is modern, check CPU
 			if(CPUTotalSpeed > 3800.0 && ati_nvidia)
-				R_SetHighest();
+				R_SetMaxQuality();
 			else
-				R_SetMedium();
+				R_SetQuality();
 		}
-		else
+		else 
 		{
 			if(CPUTotalSpeed > 3800.0 && ati_nvidia)
-				R_SetHigh();
+				R_SetQuality();
 			else
-				R_SetMedium();
+				R_SetPerformance();
 		}
 
 		//never run again
@@ -1724,6 +1697,8 @@ cpuinfo_exit:
 #endif
 
 	GL_SetDefaultState();
+	
+	R_CheckFBOExtensions ();
 
 	GL_InitImages ();
 	Mod_Init ();
@@ -1829,11 +1804,11 @@ void R_BeginFrame( float camera_separation )
 
 	if ( gl_log->modified )
 	{
-		GLimp_EnableLogging( gl_log->value );
+		GLimp_EnableLogging( gl_log->integer );
 		gl_log->modified = false;
 	}
 
-	if ( gl_log->value )
+	if ( gl_log->integer )
 	{
 		GLimp_LogNewFrame();
 	}
