@@ -72,6 +72,7 @@ image_t		*r_bulletnormal;
 image_t		*r_voltagetexture;
 image_t		*r_raintexture;
 image_t		*r_leaftexture;
+image_t		*r_trashtexture;
 image_t		*r_splashtexture;
 image_t		*r_splash2texture;
 image_t		*r_radarmap;
@@ -292,7 +293,7 @@ void R_InitParticleTexture (void)
 	if (!r_bubbletexture) {
 		r_bubbletexture = GL_LoadPic ("***particle***", (byte *)data, 16, 16, it_sprite, 32);
     }
-	r_reflecttexture = R_RegisterParticlePic("reflect");
+	r_reflecttexture = R_RegisterGfxPic("reflect");
 	if (!r_reflecttexture) {
 		r_reflecttexture = GL_LoadPic ("***particle***", (byte *)data, 16, 16, it_sprite, 32);
     }
@@ -351,6 +352,10 @@ void R_InitParticleTexture (void)
 	r_leaftexture = R_RegisterParticlePic("leaf");
 	if (!r_leaftexture) {
 		r_leaftexture = GL_LoadPic ("***particle***", (byte *)data, 16, 16, it_sprite, 32);
+    }
+	r_trashtexture = R_RegisterParticlePic("trash");
+	if (!r_trashtexture) {
+		r_trashtexture = GL_LoadPic ("***particle***", (byte *)data, 16, 16, it_sprite, 32);
     }
 	r_splashtexture = R_RegisterParticlePic("ripples");
 	if (!r_splashtexture) {
@@ -360,6 +365,8 @@ void R_InitParticleTexture (void)
 	if (!r_splash2texture) {
 		r_splash2texture = GL_LoadPic ("***particle***", (byte *)data, 16, 16, it_sprite, 32);
     }
+
+	//to do - move more or all of this to their appropriate places
 	r_radarmap = GL_FindImage("gfx/radar/radarmap",it_pic);
 	if (!r_radarmap) {
 		r_radarmap = GL_LoadPic ("***particle***", (byte *)data, 16, 16, it_sprite, 32);
@@ -412,7 +419,7 @@ void R_InitParticleTexture (void)
 	}
 	r_notexture = GL_LoadPic ("***r_notexture***", (byte *)data, 16, 16, it_wall, 32);
 
-	R_InitCubemapTextures ();
+	//R_InitCubemapTextures (); //not used for now - may use later for HDR, and others
 
 	//will eventually add more flaretypes
 	Com_sprintf (flares, sizeof(flares), "gfx/flares/flare0.tga");
@@ -459,6 +466,28 @@ void GL_ScreenShot_JPEG(void)
 	FILE			*f;
 	char			picname[80], checkname[MAX_OSPATH];
 	int			i, offset;
+	cvar_t		*blocking_cvars[20];
+	int			nblocking = 0;
+
+	// Yes, you can set gl_picmip to 40. No, we don't want you to show anyone.
+	if (gl_picmip->integer > 1)
+		blocking_cvars[nblocking++] = gl_picmip;
+	if (gl_fog->integer < 1)
+		blocking_cvars[nblocking++] = gl_fog;
+	// TODO: check for other avant-garde video settings here.
+	
+	if (nblocking > 0)
+	{
+		Com_Printf ("Screenshots are disabled because of your settings for the following cvar");
+		if (nblocking > 1)
+			Com_Printf ("s");
+		Com_Printf (":\n");
+		for (i = 0; i < nblocking; i++)
+		{
+			Com_Printf ("    %s: %s\n", blocking_cvars[i]->name, blocking_cvars[i]->string);
+		}
+		return;
+	}
 
 	/* Create the scrnshots directory if it doesn't exist */
 	Com_sprintf(checkname, sizeof(checkname), "%s/scrnshot/", FS_Gamedir());
@@ -515,10 +544,10 @@ void GL_ScreenShot_JPEG(void)
 	cinfo.input_components = 3;
 	jpeg_set_defaults(&cinfo);
 
-	if ((gl_screenshot_jpeg_quality->value >= 101) || (gl_screenshot_jpeg_quality->value <= 0))
+	if ((gl_screenshot_jpeg_quality->integer >= 101) || (gl_screenshot_jpeg_quality->integer <= 0))
 		Cvar_Set("gl_screenshot_jpeg_quality", "85");
 
-	jpeg_set_quality(&cinfo, gl_screenshot_jpeg_quality->value, TRUE);
+	jpeg_set_quality(&cinfo, gl_screenshot_jpeg_quality->integer, TRUE);
 
 	/* Start Compression */
 	jpeg_start_compress(&cinfo, true);
@@ -692,6 +721,15 @@ void GL_SetDefaultState( void )
 	}
 
 	GL_UpdateSwapInterval();
+
+	/* set memory alignment for glReadPixels() and glDrawPixels().
+	 * default is 4, screenshots with 1366 vid width fail with that.
+	 * setting to 1 should work with any custom vid width.
+	 * also affects glTexImage2D() and other commands, so might cause problems.
+	 */
+	qglPixelStorei(GL_PACK_ALIGNMENT, 1);
+	qglPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+
 }
 
 void GL_UpdateSwapInterval( void )
