@@ -27,6 +27,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "r_local.h"
 #include "r_iqm.h"
 #include "r_ragdoll.h"
+#include "client/sound.h"
 
 model_t	*loadmodel;
 int		modfilelen;
@@ -46,10 +47,15 @@ model_t	mod_inline[MAX_MOD_KNOWN];
 LightGroup_t LightGroups[MAX_LIGHTS];
 int r_lightgroups;
 
-
 int		registration_sequence;
 
-char map_music[128];
+#if defined WIN32_VARIANT
+char map_music[MAX_PATH];
+char map_music_sec[MAX_PATH];
+#else
+char map_music[MAX_OSPATH];
+char map_music_sec[MAX_OSPATH];
+#endif
 
 char		map_entitystring[MAX_MAP_ENTSTRING];
 int			numentitychars;
@@ -527,7 +533,7 @@ model_t *Mod_ForName (char *name, qboolean crash)
 
 	if (!name[0])
 		Com_Error (ERR_DROP, "Mod_ForName: NULL name");
-
+	
 	//
 	// inline models are grabbed only from worldmodel
 	//
@@ -555,15 +561,12 @@ model_t *Mod_ForName (char *name, qboolean crash)
 			if (mod->type == mod_alias || mod->type == mod_iqm)
 			{
 				// Make sure models scripts are definately reloaded between maps
-				char rs[MAX_OSPATH];
 				image_t *img;
 				img=mod->skins[0];
 
 				if (img != NULL)
 				{
-					strcpy(rs,mod->skins[0]->name);
-					rs[strlen(rs)-4]=0;
-					mod->script = RS_FindScript(rs);
+					mod->script = mod->skins[0]->script;
 					if (mod->script)
 						RS_ReadyScript( mod->script );
 				}
@@ -588,6 +591,8 @@ model_t *Mod_ForName (char *name, qboolean crash)
 		mod_numknown++;
 	}
 	strcpy (mod->name, name);
+	
+	R_SetSimpleTexnum (mod, name);
 
 	//
 	// load the file
@@ -828,16 +833,23 @@ void Mod_LoadEdges (lump_t *l)
 Mod_LoadTexinfo
 =================
 */
+
+int compare_unique_texinfo (const void *_a, const void *_b)
+{
+	mtexinfo_t *a = *(mtexinfo_t **)_a, *b = *(mtexinfo_t **)_b;
+	return a->image->texnum-b->image->texnum;
+}
+
 void Mod_LoadTexinfo (lump_t *l)
 {
-	texinfo_t *in;
-	mtexinfo_t *out, *step;
-	int 	i, j, count;
+	texinfo_t *in, *in_base;
+	mtexinfo_t *out, *step, **unique_temp;
+	int 	i, j, count, num_unique;
 	char	name[MAX_QPATH];
 	char	sv_name[MAX_QPATH];
 	int		next;
 
-	in = (void *)(mod_base + l->fileofs);
+	in_base = in = (void *)(mod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
 		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
 	count = l->filelen / sizeof(*in);
@@ -857,16 +869,19 @@ void Mod_LoadTexinfo (lump_t *l)
 		out->value = LittleLong (in->value);
 		out->flags = LittleLong (in->flags);
 		next = LittleLong (in->nexttexinfo);
-		if (next > 0)
+		if (next > 0 && next < loadmodel->numtexinfo)
 			out->next = loadmodel->texinfo + next;
 		else
 		    out->next = NULL;
+		
+		out->equiv = out;
 
 		Com_sprintf (name, sizeof(name), "textures/%s.wal", in->texture);
 		out->image = GL_FindImage (name, it_wall);
 
 		Com_sprintf (name, sizeof(name), "textures/%s", in->texture);
 		out->image->script = RS_FindScript(name);
+
 		if (out->image->script)
 			RS_ReadyScript(out->image->script);
 
@@ -918,6 +933,36 @@ void Mod_LoadTexinfo (lump_t *l)
 		for (step = out->next ; step && step != out ; step=step->next)
 			out->numframes++;
 	}
+	
+	//find equivalent texinfos
+	num_unique = 0;
+	unique_temp = Z_Malloc (sizeof(mtexinfo_t*)*count);
+	for (i = 0; i < count; i++)
+	{
+		for (j = 0; j < i; j++)
+		{
+			if (in_base[i].flags == in_base[j].flags && 
+				in_base[i].nexttexinfo == in_base[j].nexttexinfo && 
+				!strcmp (in_base[i].texture, in_base[j].texture))
+			{
+				loadmodel->texinfo[i].equiv = loadmodel->texinfo[j].equiv;
+				break;
+			}
+		}
+		if (j == i)
+		{
+			unique_temp[num_unique++] = loadmodel->texinfo[i].equiv;
+		}
+	}
+	
+	qsort (unique_temp, num_unique, sizeof(mtexinfo_t*), compare_unique_texinfo);
+	
+	loadmodel->unique_texinfo = Hunk_Alloc (num_unique*sizeof(mtexinfo_t*));
+	memcpy (loadmodel->unique_texinfo, unique_temp, num_unique*sizeof(mtexinfo_t*));
+	Z_Free (unique_temp);
+	loadmodel->num_unique_texinfos = num_unique;
+	
+	Com_Printf ("Condensed ^2%i texinfos into ^2%i equivalent texinfos\n", count, num_unique);
 }
 
 /*
@@ -927,10 +972,10 @@ CalcSurfaceExtents
 Fills in s->texturemins[] and s->extents[]
 ================
 */
-void CalcSurfaceExtents (msurface_t *s)
+void CalcSurfaceExtents (msurface_t *s, qboolean override, int *smax, int *tmax, float *xscale, float *yscale, int firstedge, int numedges)
 {
 	float	mins[2], maxs[2], val;
-	int		i,j, e;
+	int		i,j, e, vnum;
 	mvertex_t	*v;
 	mtexinfo_t	*tex;
 	int		bmins[2], bmaxs[2];
@@ -940,13 +985,26 @@ void CalcSurfaceExtents (msurface_t *s)
 
 	tex = s->texinfo;
 
-	for (i=0 ; i<s->numedges ; i++)
+	if (firstedge < 0 || firstedge+numedges-1 >= loadmodel->numsurfedges)
+		Com_Error (ERR_DROP,
+			"Map contains invalid value for s->firstedge!\n"
+			"The file is likely corrupted, please obtain a fresh copy.");
+	for (i=0 ; i<numedges ; i++)
 	{
-		e = loadmodel->surfedges[s->firstedge+i];
+		e = loadmodel->surfedges[firstedge+i];
+		if (abs(e) > loadmodel->numedges)
+			Com_Error (ERR_DROP,	
+				"Map contains invalid edge offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
 		if (e >= 0)
-			v = &loadmodel->vertexes[loadmodel->edges[e].v[0]];
+			vnum = loadmodel->edges[e].v[0];
 		else
-			v = &loadmodel->vertexes[loadmodel->edges[-e].v[1]];
+			vnum = loadmodel->edges[-e].v[1];
+		if (vnum < 0 || vnum >= loadmodel->numvertexes)
+			Com_Error (ERR_DROP,	
+				"Map contains invalid vertex offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
+		v = &loadmodel->vertexes[vnum];
 
 		for (j=0 ; j<2 ; j++)
 		{
@@ -961,14 +1019,33 @@ void CalcSurfaceExtents (msurface_t *s)
 		}
 	}
 
-	for (i=0 ; i<2 ; i++)
+	if (!override)
 	{
-		bmins[i] = floor(mins[i]/16);
-		bmaxs[i] = ceil(maxs[i]/16);
+#define DEFAULT_LIGHTMAP_SCALE 16 //each lightmap pixel represents 16x16 units
+	    *xscale = *yscale = DEFAULT_LIGHTMAP_SCALE;
+		for (i=0 ; i<2 ; i++)
+	    {
+		    bmins[i] = floor(mins[i]/DEFAULT_LIGHTMAP_SCALE);
+		    bmaxs[i] = ceil(maxs[i]/DEFAULT_LIGHTMAP_SCALE);
 
-		s->texturemins[i] = bmins[i] * 16;
-		s->extents[i] = (bmaxs[i] - bmins[i]) * 16;
-	}
+		    s->texturemins[i] = bmins[i] * DEFAULT_LIGHTMAP_SCALE;
+		    s->extents[i] = (bmaxs[i] - bmins[i]) * DEFAULT_LIGHTMAP_SCALE;
+	    }
+	    *smax = (s->extents[0]/DEFAULT_LIGHTMAP_SCALE)+1;
+        *tmax = (s->extents[1]/DEFAULT_LIGHTMAP_SCALE)+1;
+    }
+    else
+    {
+	    bmins[0] = floor(mins[0]/(*xscale));
+	    bmaxs[0] = ceil(maxs[0]/(*xscale));
+	    s->texturemins[0] = bmins[0] * (*xscale);
+	    s->extents[0] = (bmaxs[0] - bmins[0]) * (*xscale);
+	    
+	    bmins[1] = floor(mins[1]/(*yscale));
+	    bmaxs[1] = ceil(maxs[1]/(*yscale));
+	    s->texturemins[1] = bmins[1] * (*yscale);
+	    s->extents[1] = (bmaxs[1] - bmins[1]) * (*yscale);
+    }
 }
 
 void Mod_CalcSurfaceNormals(msurface_t *surf)
@@ -977,6 +1054,7 @@ void Mod_CalcSurfaceNormals(msurface_t *surf)
 	glpoly_t *p = surf->polys;
 	float	*v;
 	int		i;
+	float 	temp_tangentSpaceTransform[3][3];
 
 	for (; p; p = p->chain)
 	{
@@ -989,6 +1067,10 @@ void Mod_CalcSurfaceNormals(msurface_t *surf)
 		vec = p->verts[0];
 		_VectorCopy(surf->plane->normal, normal);
 
+		// FIXME: on some maps, this code doesn't always initialize v02, 
+		// leading to Valgrind complaining about use of uninitialized memory.
+		// dm-infinity and dm-zorn2k11 are two such maps. Probably not a huge
+		// deal, doesn't seem to be causing any issues.
 		for (v = p->verts[0], i = 0 ; i < p->numverts; i++, v += VERTEXSIZE)
 		{
 
@@ -1016,9 +1098,9 @@ void Mod_CalcSurfaceNormals(msurface_t *surf)
 
 		VectorNormalize( normal ); //we have the largest normal
 
-		surf->tangentSpaceTransform[ 0 ][ 2 ] = normal[ 0 ];
-		surf->tangentSpaceTransform[ 1 ][ 2 ] = normal[ 1 ];
-		surf->tangentSpaceTransform[ 2 ][ 2 ] = normal[ 2 ];
+		temp_tangentSpaceTransform[ 0 ][ 2 ] = normal[ 0 ];
+		temp_tangentSpaceTransform[ 1 ][ 2 ] = normal[ 1 ];
+		temp_tangentSpaceTransform[ 2 ][ 2 ] = normal[ 2 ];
 
 		//now get the tangent
 		s = ( p->verts[ 1 ][ 3 ] - p->verts[ 0 ][ 3 ] )
@@ -1033,9 +1115,9 @@ void Mod_CalcSurfaceNormals(msurface_t *surf)
 		VectorScale( temp3, s, tangent );
 		VectorNormalize( tangent );
 
-		surf->tangentSpaceTransform[ 0 ][ 0 ] = tangent[ 0 ];
-		surf->tangentSpaceTransform[ 1 ][ 0 ] = tangent[ 1 ];
-		surf->tangentSpaceTransform[ 2 ][ 0 ] = tangent[ 2 ];
+		temp_tangentSpaceTransform[ 0 ][ 0 ] = tangent[ 0 ];
+		temp_tangentSpaceTransform[ 1 ][ 0 ] = tangent[ 1 ];
+		temp_tangentSpaceTransform[ 2 ][ 0 ] = tangent[ 2 ];
 
 		//now get the binormal
 		VectorScale( v02, p->verts[ 1 ][ 3 ] - p->verts[ 0 ][ 3 ], temp1 );
@@ -1044,30 +1126,267 @@ void Mod_CalcSurfaceNormals(msurface_t *surf)
 		VectorScale( temp3, s, binormal );
 		VectorNormalize( binormal );
 
-		surf->tangentSpaceTransform[ 0 ][ 1 ] = binormal[ 0 ];
-		surf->tangentSpaceTransform[ 1 ][ 1 ] = binormal[ 1 ];
-		surf->tangentSpaceTransform[ 2 ][ 1 ] = binormal[ 2 ];
+		temp_tangentSpaceTransform[ 0 ][ 1 ] = binormal[ 0 ];
+		temp_tangentSpaceTransform[ 1 ][ 1 ] = binormal[ 1 ];
+		temp_tangentSpaceTransform[ 2 ][ 1 ] = binormal[ 2 ];
+		
+		//Try to find this tangentSpaceTransform in the existing array or else
+		//add it. This is not a RAM-saving measure, it's to allow comparison 
+		//of different tangentSpaceTransforms using the == operator, which is
+		//more efficient.
+		{
+			float *tst;
+			int i;
+			int j;
+			for (tst = loadmodel->tangentSpaceTransforms, i = 0; i < loadmodel->numTangentSpaceTransforms; i++, tst += 9)
+			{
+				qboolean match = true;
+				for (j = 0; j < 9; j++)
+				{
+					// If we reduce our precision to just 4 decimal places,
+					// we can cut the number of transforms by a factor of more
+					// than 5. Single-precision floating point is precise to
+					// just 6 places anyway, so we're not loosing too much.
+					// Subjectively, it still looks fine.
+					if (fabs (tst[j]-temp_tangentSpaceTransform[j/3][j%3]) > 0.0001)
+					{
+						match = false;
+						break;
+					}
+				}
+				if (match)
+					break;
+			}
+			surf->tangentSpaceTransform = tst;
+			if (i == loadmodel->numTangentSpaceTransforms)
+			{
+				for (i = 0; i < 3; i++)
+					for (j = 0; j < 3; j++)
+						*(tst++) = temp_tangentSpaceTransform[i][j];
+				loadmodel->numTangentSpaceTransforms++;
+			}
+		}
 	}
 }
 
-void BSP_BuildPolygonFromSurface(msurface_t *fa);
-void BSP_CreateSurfaceLightmap (msurface_t *surf);
+void BSP_BuildPolygonFromSurface(msurface_t *fa, float xscale, float yscale, int light_s, int light_t, int firstedge, int lnumverts);
+void BSP_CreateSurfaceLightmap (msurface_t *surf, int smax, int tmax, int *light_s, int *light_t);
 void BSP_EndBuildingLightmaps (void);
 void BSP_BeginBuildingLightmaps (model_t *m);
 
+
+// FOR REFINE: .lightmap high-detail lightmap override files
+
+ltmp_facelookup_t   lfacelookups[MAX_MAP_FACES];
+int					lightdatasize;
+byte				override_lightdata[MAX_OVERRIDE_LIGHTING];
+byte				*lightmap_header;
+
+// choose up to one override for each map face
+void Mod_LoadRefineFaceLookups (lump_t *l)
+{
+	int i, count;
+	ltmp_facelookup_t *in;
+	ltmp_facelookup_t *out;
+	
+	if (!l->filelen)
+		return;
+	in = (void *)(lightmap_header+l->fileofs);
+	printf ("%d / %d\n", l->filelen, sizeof(*in));
+	if (l->filelen % sizeof (*in))
+		Com_Error (ERR_DROP, "Mod_LoadRefineFaceLookups: funny lump size");
+	count = l->filelen / sizeof (*in);
+	
+	Com_Printf ("%f\n", in->xscale);
+	for (i = 0; i < count; i++, in++)
+	{
+		int facenum, format, offset, size, width, height;
+
+		// Silently skip over pixel formats we don't understand. We do this
+		// check first so that in the future it can be used to work around the
+		// following two for backward compatibility.
+		format = LittleLong(in->format);
+		if (format >= LTMP_NUM_SUPPORTED_PIXFMTS)
+			continue;
+	    
+		// make a federal case out of bad face offsets
+		facenum = LittleLong(in->facenum);
+		if (facenum >= MAX_MAP_FACES || facenum < 0)
+			Com_Error (ERR_DROP, "Mod_LoadRefineFaceeLookups: bad facenum");
+
+		// make a federal case out of bad pixel data offsets
+		offset = LittleLong(in->offset);
+		width = LittleLong(in->width);
+		height = LittleLong(in->height);
+		switch (format)
+		{
+			case LTMP_PIXFMT_RGB24:
+				size = 3*width*height;
+				break;
+		}
+		if (offset < 1 || offset+size > MAX_OVERRIDE_LIGHTING*4)
+			Com_Error (ERR_DROP, "Mod_LoadRefineFaceeLookups: bad offset");
+
+		out = &lfacelookups[facenum];
+		out->offset = offset;
+		out->format = format;
+		out->width = width;
+		out->height = height;
+		out->xscale = LittleFloat(in->xscale);
+		out->yscale = LittleFloat(in->yscale);
+	}
+}
+
+// load only the lightmap data that will actually be used; readjust the 
+// pixel data offsets accordingly
+void Mod_LoadRefineLighting (lump_t *l)
+{
+	int					i;
+	byte				*in_buffer;
+	qboolean			overflowed = false;
+	
+	if (!l->filelen)
+		return;
+	if (l->filelen > MAX_OVERRIDE_LIGHTING*4)
+		Com_Error (ERR_DROP, "Mod_LoadRefineLighting: too much light data");
+	
+	lightdatasize = 0;
+	in_buffer = lightmap_header + l->fileofs;
+	for (i = 0; i < loadmodel->numsurfaces; i++)
+	{
+		int in_offset = lfacelookups[i].offset;
+		if (in_offset)
+		{
+			int size = 0;
+			switch (lfacelookups[i].format)
+			{
+				case LTMP_PIXFMT_RGB24:
+					size = 3*lfacelookups[i].width*lfacelookups[i].height;
+					break;
+			}
+			if (lightdatasize + size >= MAX_OVERRIDE_LIGHTING)
+			{
+				overflowed = true;
+				break;
+			}
+			memcpy (override_lightdata+lightdatasize, in_buffer+in_offset-1, size);
+			lfacelookups[i].offset = lightdatasize+1;
+			lightdatasize += size;
+		}
+	}
+	
+	if (overflowed)
+	{
+		Com_Printf ("Mod_LoadRefineLighting: MAX_OVERRIDE_LIGHTING overflow!\n");
+		Com_Printf ("Disabling HD lightmaps for this map.\n");
+		memset (lfacelookups, 0, sizeof(lfacelookups));
+	}
+}
+
+void Mod_LoadRefineLightmap (char *bsp_name)
+{
+	byte 				*buf, *uncompressed_buf;
+	lightmapheader_t	header;
+	int					length;
+	char				name[MAX_OSPATH];
+	char				*extension;
+	int					lightmap_file_lump_order[LTMP_LUMPS] = {
+		LTMP_LUMP_FACELOOKUP, LTMP_LUMP_LIGHTING
+	};
+	
+	sizebuf_t			in, out;
+
+#ifndef HAVE_ZLIB
+	Com_Printf ("Zlib support must be enabled to use HD lightmaps!\n");
+	Com_Printf ("Please recompile with Zlib support.\n");
+	return;
+#endif
+	
+	strncpy (name, bsp_name, MAX_OSPATH-1-strlen(".lightmap")+strlen(".bsp"));
+	extension = strstr (name, ".bsp");
+	if (extension)
+		*extension = 0;
+	strcat (name, ".lightmap"); 
+	
+	length = FS_LoadFile (name, (void **)&buf);
+	if (!buf)
+	{
+		Com_Printf ("Could not load %s\n", name);
+		return;
+	}
+	else
+		Com_Printf ("Loaded %s\n", name);
+	
+	SZ_Init (&in, buf, length);
+	in.cursize = length;
+	uncompressed_buf = malloc (LTMP_MAX_UNCOMPRESSED_DATA);
+	if (!uncompressed_buf)
+	{
+		Com_Printf ("Mod_LoadRefineLightmap: unable to allocate %d bytes!\n", LTMP_MAX_UNCOMPRESSED_DATA);
+		return;
+	}
+	SZ_Init (&out, (byte *)uncompressed_buf, LTMP_MAX_UNCOMPRESSED_DATA);
+	qdecompress (&in, &out, compression_zlib_header);
+	FS_FreeFile (buf);
+	
+	if (!out.cursize)
+	{
+		Com_Printf ("Mod_LoadRefineLightmap: unable to decompress data in %s!\n",name);
+		free (uncompressed_buf);
+		return;
+	}
+	
+	lightmap_header = (byte *)uncompressed_buf;
+	header = *(lightmapheader_t *)uncompressed_buf;
+	
+	if (header.ident != IDLIGHTMAPHEADER)
+	{
+		Com_Printf ("Mod_LoadRefineLightmap: invalid magic number in %s!\n"
+					"The file is likely corrupt, please obtain a fresh copy.\n", name);
+		free (uncompressed_buf);			
+		return;
+	}
+	if (header.version != LTMPVERSION)
+	{
+		Com_Printf ("Mod_LoadRefineLightmap: invalid major version number in %s!\n"
+					"Version %d of the format is not supported. Current major version number is %d\n", 
+					name, header.version, LTMPVERSION);
+		free (uncompressed_buf);
+		return;
+	}
+	
+	if (checkLumps (header.lumps, sizeof(int)*3, lightmap_file_lump_order, uncompressed_buf, LTMP_LUMPS, out.cursize))
+	{
+		Com_Printf ("Mod_LoadRefineLightmap: lumps in %s don't add up right!\n"
+					"The file is likely corrupt, please obtain a fresh copy.\n",name);
+		free (uncompressed_buf);
+		return;
+	}
+	
+	Mod_LoadRefineFaceLookups (&header.lumps[LTMP_LUMP_FACELOOKUP]);
+	Mod_LoadRefineLighting (&header.lumps[LTMP_LUMP_LIGHTING]);
+	
+	free (uncompressed_buf);
+}
+
 /*
 =================
 Mod_LoadFaces
 =================
 */
 
-void Mod_LoadFaces (lump_t *l)
+// We need access to the lighting lump so we can check the bounds of lightmap
+// offsets. Otherwise, crafted or corrupted BSPs could crash the client.
+void Mod_LoadFaces (lump_t *l, lump_t *lighting)
 {
 	dface_t		*in;
 	msurface_t 	*out;
 	int			i, count, surfnum;
 	int			planenum, side;
 	int			ti;
+	int			smax, tmax;
+	int			light_s, light_t;
+	float		xscale, yscale;
 	rscript_t	*rs;
 	vec3_t		color;
 
@@ -1079,8 +1398,15 @@ void Mod_LoadFaces (lump_t *l)
 
 	loadmodel->surfaces = out;
 	loadmodel->numsurfaces = count;
+	
+	//we are guaranteed not to need more than this
+	loadmodel->tangentSpaceTransforms = Hunk_Alloc (sizeof(float)*9*count);
 
 	currentmodel = loadmodel;
+	
+	memset (lfacelookups, 0, sizeof(lfacelookups));
+	if (r_lightmapfiles->integer)
+		Mod_LoadRefineLightmap (loadmodel->name);
 
 	BSP_BeginBuildingLightmaps (loadmodel);
 
@@ -1088,24 +1414,41 @@ void Mod_LoadFaces (lump_t *l)
 
 	for ( surfnum=0 ; surfnum<count ; surfnum++, in++, out++)
 	{
-		out->firstedge = LittleLong(in->firstedge);
-		out->numedges = LittleShort(in->numedges);
-		out->flags = 0;
+		int firstedge = LittleLong(in->firstedge);
+		int numedges = LittleShort(in->numedges);
+		out->iflags = 0;
 		out->polys = NULL;
 
-		planenum = LittleShort(in->planenum);
+		planenum = (unsigned short)LittleShort(in->planenum);
 		side = LittleShort(in->side);
 		if (side)
-			out->flags |= SURF_PLANEBACK;
+			out->iflags |= ISURF_PLANEBACK;
 
+		if (planenum >= loadmodel->numplanes)
+			Com_Error (ERR_DROP, 
+				"Map has invalid plane offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
 		out->plane = loadmodel->planes + planenum;
 
 		ti = LittleShort (in->texinfo);
 		if (ti < 0 || ti >= loadmodel->numtexinfo)
-			Com_Error (ERR_DROP, "MOD_LoadBmodel: bad texinfo number");
+			Com_Error (ERR_DROP, 
+				"Map has invalid texinfo offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
 		out->texinfo = loadmodel->texinfo + ti;
 
-		CalcSurfaceExtents (out);
+		if (lfacelookups[surfnum].offset)
+		{
+			smax=lfacelookups[surfnum].width;
+			tmax=lfacelookups[surfnum].height;
+			xscale=lfacelookups[surfnum].xscale; 
+			yscale=lfacelookups[surfnum].yscale;
+			CalcSurfaceExtents (out, true, &smax, &tmax, &xscale, &yscale, firstedge, numedges);
+		}
+		else
+		{
+			CalcSurfaceExtents (out, false, &smax, &tmax, &xscale, &yscale, firstedge, numedges);
+		}
 
 	// lighting info
 
@@ -1113,32 +1456,35 @@ void Mod_LoadFaces (lump_t *l)
 			out->styles[i] = in->styles[i];
 		i = LittleLong(in->lightofs);
 
-		if (i == -1)
+		if (i < 0 || i >= lighting->filelen)
 			out->samples = NULL;
+		else if (lfacelookups[surfnum].offset > 0 && lfacelookups[surfnum].offset <= lightdatasize)
+			out->samples = override_lightdata + lfacelookups[surfnum].offset-1;
 		else
 			out->samples = loadmodel->lightdata + i;
 
 		// set the drawing flags
 		if (out->texinfo->flags & SURF_WARP)
 		{
-			out->flags |= SURF_DRAWTURB;
+			out->iflags |= ISURF_DRAWTURB;
 			for (i=0 ; i<2 ; i++)
 			{
 				out->extents[i] = 16384;
 				out->texturemins[i] = -8192;
 			}
 			if(!(gl_state.glsl_shaders && gl_glsl_shaders->value) || !strcmp(out->texinfo->normalMap->name, out->texinfo->image->name))
-				R_SubdivideSurface (out);	// cut up polygon for warps
+				R_SubdivideSurface (out, firstedge, numedges);	// cut up polygon for warps
 		}
 
 		// create lightmaps and polygons
+		light_s = light_t = 0;
 		if ( !SurfaceHasNoLightmap(out) )
 		{
-			BSP_CreateSurfaceLightmap (out);
+			BSP_CreateSurfaceLightmap (out, smax, tmax, &light_s, &light_t);
 		}
 
 		if ( (! (out->texinfo->flags & SURF_WARP)) || (gl_state.glsl_shaders && gl_glsl_shaders->value && strcmp(out->texinfo->normalMap->name, out->texinfo->image->name)))
-			BSP_BuildPolygonFromSurface(out);
+			BSP_BuildPolygonFromSurface(out, xscale, yscale, light_s, light_t, firstedge, numedges);
 
 		rs = (rscript_t *)out->texinfo->image->script;
 
@@ -1167,6 +1513,8 @@ void Mod_LoadFaces (lump_t *l)
 					Mod_AddBeamSurface(out, stage->texture->texnum, color, stage->scale.scaleX, stage->texture->bare_name, stage->beamtype,
 						stage->xang, stage->yang, stage->rotating);
 				}
+				if (stage->cube)
+					out->texinfo->flags |= SURF_SHINY;
 			} while ( (stage = stage->next) );
 		}
 		Mod_CalcSurfaceNormals(out);
@@ -1223,19 +1571,35 @@ void Mod_LoadNodes (lump_t *l)
 		}
 
 		p = LittleLong(in->planenum);
+		if (p < 0 || p >= loadmodel->numplanes)
+			Com_Error (ERR_DROP,
+				"Map has invalid plane offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
 		out->plane = loadmodel->planes + p;
 
-		out->firstsurface = LittleShort (in->firstface);
-		out->numsurfaces = LittleShort (in->numfaces);
+		out->firstsurface = (unsigned short)LittleShort (in->firstface);
+		out->numsurfaces = (unsigned short)LittleShort (in->numfaces);
 		out->contents = -1;	// differentiate from leafs
 
 		for (j=0 ; j<2 ; j++)
 		{
 			p = LittleLong (in->children[j]);
 			if (p >= 0)
+			{
+				if (p >= loadmodel->numnodes)
+					Com_Error (ERR_DROP, 
+						"Map file has invalid node offsets!\n"
+						"The file is likely corrupted, please obtain a fresh copy.");
 				out->children[j] = loadmodel->nodes + p;
+			}
 			else
+			{
+				if (-1-p >= loadmodel->numleafs)
+					Com_Error (ERR_DROP,
+						"Map file has invalid leaf offsets!\n"
+						"The file is likely corrupted, please obtain a fresh copy.");
 				out->children[j] = (mnode_t *)(loadmodel->leafs + (-1 - p));
+			}
 		}
 
 	}
@@ -1253,6 +1617,7 @@ void Mod_LoadLeafs (lump_t *l)
 	dleaf_t 	*in;
 	mleaf_t 	*out;
 	int			i, j, count, p;
+	int			to_subtract = 0; //for removing SURF_NODRAW surfaces
 
 	in = (void *)(mod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
@@ -1278,18 +1643,46 @@ void Mod_LoadLeafs (lump_t *l)
 		out->area = LittleShort(in->area);
 
 		out->firstmarksurface = loadmodel->marksurfaces +
-			LittleShort(in->firstleafface);
-		out->nummarksurfaces = LittleShort(in->numleaffaces);
-
-		// gl underwater warp
-		if (out->contents & MASK_WATER )
+			(unsigned short)(LittleShort(in->firstleafface));
+		out->nummarksurfaces = (unsigned short)LittleShort(in->numleaffaces);
+		if (out->firstmarksurface < loadmodel->marksurfaces || 
+			out->nummarksurfaces < 0 ||
+			(unsigned short)LittleShort(in->firstleafface) > 
+			loadmodel->nummarksurfaces)
+			Com_Error (ERR_DROP,
+				"Map file has invalid leaf surface offsets!\n"
+				"The file is likely corrupted, please obtain a fresh copy.");
+		
+		out->firstmarksurface -= to_subtract;
+		
+		// Remove SURF_NODRAW surfaces. We do this here instead of in 
+		// Mod_LoadMarkSurfaces so we can correct the offsets in each leaf.
+		for (j = 0; j < out->nummarksurfaces; j++)
 		{
-			for (j=0 ; j<out->nummarksurfaces ; j++)
+			msurface_t *s;
+			out->firstmarksurface[j] = out->firstmarksurface[j+to_subtract];
+			s = out->firstmarksurface[j];
+			if (s->texinfo->flags & SURF_NODRAW)
 			{
-				out->firstmarksurface[j]->flags |= SURF_UNDERWATER;
+				to_subtract++;
+				j--;
+				out->nummarksurfaces--;
 			}
 		}
+
+		// gl underwater warp
+		for (j=0 ; j<out->nummarksurfaces ; j++)
+		{
+			msurface_t *s = out->firstmarksurface[j];
+			if	(	(out->contents & MASK_WATER) || 
+					(s->texinfo->flags & SURF_UNDERWATER))
+			out->firstmarksurface[j]->iflags |= ISURF_UNDERWATER;
+		}
 	}
+	
+	loadmodel->nummarksurfaces -= to_subtract;
+	
+	Com_Printf ("Eliminated ^2%i invisible surfaces.\n", to_subtract);
 }
 
 /*
@@ -1331,6 +1724,7 @@ void Mod_LoadSurfedges (lump_t *l)
 	int		i, count;
 	int		*in, *out;
 
+
 	in = (void *)(mod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
 		Com_Error (ERR_DROP, "MOD_LoadBmodel: funny lump size in %s",loadmodel->name);
@@ -1387,6 +1781,79 @@ void Mod_LoadPlanes (lump_t *l)
 	}
 }
 
+/*
+=================
+Mod_SummarizePVS
+
+Gather some statistics from the PVS data which allows the renderer to take 
+some shortcuts.
+=================
+*/
+void Mod_SummarizePVS (void)
+{
+	byte		*vis;
+	mleaf_t		*leaf, *leaf2; 
+	int			cluster;
+	int			i;
+	
+	loadmodel->num_areas = 0;
+	for (i = 0; i < MAX_MAP_AREAS; i++)
+	{
+		loadmodel->area_min_leaf[i] = loadmodel->numleafs-1;
+		loadmodel->area_max_leaf[i] = 0;
+	}
+	
+	for (i=0,leaf=loadmodel->leafs ; i<loadmodel->numleafs ; i++, leaf++)
+	{
+		cluster = leaf->cluster;
+		if (cluster == -1)
+			continue;
+		
+		vis = Mod_ClusterPVS (cluster, loadmodel);
+		
+		if (loadmodel->area_min_leaf[leaf->area] > i)
+			loadmodel->area_min_leaf[leaf->area] = i;
+		if (loadmodel->area_max_leaf[leaf->area] < i)
+			loadmodel->area_max_leaf[leaf->area] = i;
+		if (leaf->area > loadmodel->num_areas)
+			loadmodel->num_areas = leaf->area;
+		
+		// Two separate loops, one for minPVSleaf and one for maxPVSleaf,
+		// each coming from opposite directions, for greater efficiency.
+		
+		for (	leaf->minPVSleaf = 0, leaf2 = loadmodel->leafs;
+				leaf->minPVSleaf < loadmodel->numleafs;
+				leaf->minPVSleaf++, leaf2++
+			)
+		{
+			cluster = leaf2->cluster;
+			if ((leaf2->contents & CONTENTS_SOLID) || cluster == -1)
+				continue;
+			if (leaf2->nummarksurfaces == 0)
+				continue;
+			
+			if (vis[cluster>>3] & (1<<(cluster&7)))
+				break;
+		}
+		
+		for (	leaf->maxPVSleaf = loadmodel->numleafs-1, leaf2 = &loadmodel->leafs[leaf->maxPVSleaf];
+				leaf->maxPVSleaf >= leaf->minPVSleaf;
+				leaf->maxPVSleaf--, leaf2--
+			)
+		{
+			cluster = leaf2->cluster;
+			if ((leaf2->contents & CONTENTS_SOLID) || cluster == -1)
+				continue;
+			if (leaf2->nummarksurfaces == 0)
+				continue;
+			
+			if (vis[cluster>>3] & (1<<(cluster&7)))
+				break;
+		}
+	}
+	loadmodel->num_areas++;
+}
+
 /*
 =================
 Mod_LoadBrushModel
@@ -1448,11 +1915,12 @@ void Mod_LoadBrushModel (model_t *mod, void *buffer)
 	Mod_LoadLighting (&header->lumps[LUMP_LIGHTING]);
 	Mod_LoadPlanes (&header->lumps[LUMP_PLANES]);
 	Mod_LoadTexinfo (&header->lumps[LUMP_TEXINFO]);
-	Mod_LoadFaces (&header->lumps[LUMP_FACES]);
+	Mod_LoadFaces (&header->lumps[LUMP_FACES], &header->lumps[LUMP_LIGHTING]);
 	Mod_LoadMarksurfaces (&header->lumps[LUMP_LEAFFACES]);
 	Mod_LoadVisibility (&header->lumps[LUMP_VISIBILITY]);
 	Mod_LoadLeafs (&header->lumps[LUMP_LEAFS]);
 	Mod_LoadNodes (&header->lumps[LUMP_NODES]);
+	Mod_SummarizePVS ();
 	Mod_LoadSubmodels (&header->lumps[LUMP_MODELS]);
 	mod->num_frames = 2;		// regular and alternate animation
 
@@ -1486,6 +1954,7 @@ void Mod_LoadBrushModel (model_t *mod, void *buffer)
 
 	R_ParseLightEntities();
 	R_FindSunEntity();
+	R_FinalizeGrass(loadmodel);
 }
 
 //=============================================================================
@@ -1515,6 +1984,8 @@ PModelList_t BasePModels[] =
 {
 	{ "martianenforcer" },
 	{ "martiancyborg" },
+	{ "martianoverlord" },
+	{ "martianwarrior" },
     { "enforcer" },
 	{ "commander" },
 	{ "lauren" },
@@ -1573,6 +2044,9 @@ void R_RegisterBasePlayerModels( void )
 			Com_sprintf( mod_filename, sizeof(mod_filename), "players/%s/%s", BasePModels[i].name, BaseWModels[j]);
 			R_RegisterModel(mod_filename);
 		}
+		
+		//register standard sounds
+		S_RegisterSoundsForPlayer (BasePModels[i].name);
 
 		//register all skins
 		Com_sprintf( scratch, sizeof(scratch), "players/%s/*.jpg", BasePModels[i].name);
@@ -1633,6 +2107,9 @@ void R_RegisterCustomPlayerModels( void )
 			if(FS_FileExists(mod_filename))
 				R_RegisterModel(mod_filename);
 		}
+		
+		//register standard sounds
+		S_RegisterSoundsForPlayer (dirnames[i]);
 
 		//register all skins
 		strcpy( scratch, dirnames[i] );
@@ -1679,6 +2156,11 @@ void R_BeginRegistration (char *model)
 	// check for fog file, using file system search path
 	path = NULL;
 	map_fog = false;
+	r_weather = false;
+	r_nosun = false;
+	r_sunX = 0;
+	r_sunY = 0;
+	r_sunZ = 0;
 	for(;;)
 	{
 		path = FS_NextPath( path );
@@ -1698,7 +2180,10 @@ void R_BeginRegistration (char *model)
 	}
 
 	// check for background music file, , using file system search path
-	strcpy(map_music, "music/none.wav");
+	//defaults below
+	strcpy(map_music, "music/menumusic.ogg");
+	strcpy(map_music_sec, "music/adrenaline.ogg");
+	S_RegisterSound (map_music_sec);
 	path = NULL;
 	for(;;)
 	{
@@ -1710,13 +2195,18 @@ void R_BeginRegistration (char *model)
 		Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.mus", path, model);
 		i = 0;
 		R_FindFile( fullname, &file ); //does a music file exist?
-		if(file) {
+		if(file) 
+		{
 			//read the file, get music information
 			fclose( file );
-			R_ReadMusicScript( fullname );
+			R_ReadMusicScript( fullname );						
 			break;
 		}
 	}
+	//set ctf flags
+	r_gotFlag = false;
+	r_lostFlag = false;
+	Cvar_Set("rs_hasflag", "0");
 
 	Com_sprintf (fullname, sizeof(fullname), "maps/%s.bsp", model);
 
