@@ -48,6 +48,25 @@ typedef struct
 
 worldLight_t r_worldLights[MAX_LIGHTS];
 int r_numWorldLights;
+
+/*
+==============================================================================
+
+SUN LIGHTS
+
+==============================================================================
+*/
+
+typedef struct
+{
+	char targetname[128];
+	vec3_t	origin;
+	vec3_t	target;
+	qboolean has_Sun;
+} sunLight_t;
+
+sunLight_t *r_sunLight;
+
 /*
 ==============================================================================
 
@@ -93,7 +112,7 @@ typedef struct
 //base player models and weapons for prechache
 typedef struct PModelList_s {
 
-    const char *name;
+    char *name;
 
 } PModelList_t;
 
@@ -102,7 +121,7 @@ extern int PModelsCount;
 
 typedef struct WModelList_s {
 
-    const char *name;
+    char *name;
 
 } WModelList_t;
 
@@ -150,13 +169,18 @@ typedef struct
 #define	SIDE_BACK	1
 #define	SIDE_ON		2
 
+//Internal surface flags, applied by the engine
+#define	ISURF_PLANEBACK		0x1
+#define ISURF_DRAWTURB		0x2
+// same effect as SURF_UNDERWATER, but has to be separate because
+// SURF_UNDERWATER applies to texinfos, whereas we have to work with
+// individual surfaces.
+#define ISURF_UNDERWATER	0x4 
 
-#define	SURF_PLANEBACK		1
-#define SURF_DRAWTURB		2
-#define SURF_UNDERWATER		0x80
-
-#define SurfaceIsTranslucent(surf) ((surf)->texinfo->flags & (SURF_TRANS33|SURF_TRANS66))
-#define SurfaceIsAlphaBlended(surf) ((surf)->texinfo->flags & SURF_TRANS33 && (surf)->texinfo->flags & SURF_TRANS66)
+#define TexinfoIsTranslucent(texinfo) ((texinfo)->flags & (SURF_TRANS33|SURF_TRANS66))
+#define SurfaceIsTranslucent(surf) (TexinfoIsTranslucent((surf)->texinfo))
+#define TexinfoIsAlphaBlended(texinfo) ((texinfo)->flags & SURF_TRANS33 && (texinfo)->flags & SURF_TRANS66)
+#define SurfaceIsAlphaBlended(surf) (TexinfoIsAlphaBlended ((surf)->texinfo))
 #define SurfaceHasNoLightmap(surf) ((surf)->texinfo->flags & (SURF_SKY|SURF_TRANS33|SURF_TRANS66|SURF_WARP) && !SurfaceIsAlphaBlended((surf)))
 
 // !!! if this is changed, it must be changed in asm_draw.h too !!!
@@ -179,11 +203,24 @@ typedef struct mtexinfo_s
 	int			flags;
 	int			numframes;
 	struct mtexinfo_s	*next;		// animation chain
+	struct mtexinfo_s	*equiv;		// equivalent texinfo
 	image_t		*image;
 	image_t		*normalMap;
 	image_t		*heightMap;
+	qboolean	has_normalmap;
+	qboolean	has_heightmap;
 	struct		rscript_s	*script;
 	int			value;
+	
+	//for BSP rendering-- not always cleared each frame
+	struct 		msurface_s	*w_glsl_surfaces, 
+							*w_glsl_dynamic_surfaces, 
+							*w_lightmap_surfaces;
+	
+	//for entity rendering-- cleared for each brush model drawn
+	struct 		msurface_s	*e_glsl_surfaces, 
+							*e_glsl_dynamic_surfaces, 
+							*e_lightmap_surfaces;
 } mtexinfo_t;
 
 #define	VERTEXSIZE	10
@@ -197,38 +234,42 @@ typedef struct glpoly_s
 
 } glpoly_t;
 
+// FIXME: We really need a smaller version of this struct with less data in
+// it, for use by BSP_RecursiveWorldNode. This one is 136 bytes, which is huge
+// and killing our cache locality.
 typedef struct msurface_s
 {
 	int			visframe;		// should be drawn when node is crossed
+	
+	int			iflags;			// internal flags, applied by the engine
+	
+	mtexinfo_t	*texinfo;
 
 	cplane_t	*plane;
-	int			flags;
-
-	int			firstedge;	// look up in model->surfedges[], negative numbers
-	int			numedges;	// are backwards edges
+	
+	//texture chains for batching
+	struct	msurface_s	*texturechain;
+	struct	msurface_s	*rscriptchain;
+	
+	//texture chain for lightstyle updating
+	struct	msurface_s	*flickerchain;
+	
+	vec3_t mins;
+	vec3_t maxs;
 
 	int			texturemins[2];
 	int			extents[2];
 
-	int			light_s, light_t;	// gl lightmap coordinates
-	int			dlight_s, dlight_t; // gl lightmap coordinates for dynamic lightmaps
-
 	glpoly_t	*polys;				// multiple if warped
 
-	//texture chains for batching
-	struct	msurface_s	*texturechain;
-	struct	msurface_s	*glslchain;
-	struct  msurface_s	*glsldynamicchain;
-	struct  msurface_s	*lightmapchain;
-	struct	msurface_s	*rscriptchain;
-	struct  msurface_s	*normalchain;
-
-	mtexinfo_t	*texinfo;
 	float		c_s, c_t;
 
 // lighting info
 	int			dlightframe;
 	int			dlightbits;
+	
+	int			lightmins[2];
+	int			lightmaxs[2];
 
 	int			lightmaptexturenum;
 	byte		styles[MAXLIGHTMAPS];
@@ -237,17 +278,15 @@ typedef struct msurface_s
 
 	entity_t	*entity;
 
-	float	tangentSpaceTransform[3][3];
-
-	vec3_t mins;
-	vec3_t maxs;
+	float		*tangentSpaceTransform;
 
 	//vbo
-	size_t vbo_pos;
-	int	xyz_size;
-	int st_size;
-	int lm_size;
 	int has_vbo;
+	int vbo_first_vert;
+	int vbo_num_verts;
+	
+	// XXX: for future reference, the glDrawRangeElements code was last seen
+	// here at revision 3246.
 
 } msurface_t;
 
@@ -287,6 +326,8 @@ typedef struct mleaf_s
 
 	msurface_t	**firstmarksurface;
 	int			nummarksurfaces;
+	
+	int			minPVSleaf, maxPVSleaf;
 } mleaf_t;
 
 
@@ -310,6 +351,9 @@ typedef struct {
 
 typedef enum {mod_bad, mod_brush, mod_sprite, mod_alias, mod_iqm } modtype_t;
 
+typedef enum	{	simplecolor_white, simplecolor_green, simplecolor_blue, 
+					simplecolor_purple	} simplecolor_t;
+
 typedef struct model_s
 {
 	char		name[MAX_QPATH];
@@ -353,9 +397,14 @@ typedef struct model_s
 
 	int			numtexinfo;
 	mtexinfo_t	*texinfo;
+	int			num_unique_texinfos;
+	mtexinfo_t	**unique_texinfo;
 
 	int			numsurfaces;
 	msurface_t	*surfaces;
+	
+	int			numTangentSpaceTransforms;
+	float		*tangentSpaceTransforms;
 
 	int			numsurfedges;
 	int			*surfedges;
@@ -366,6 +415,11 @@ typedef struct model_s
 	dvis_t		*vis;
 
 	byte		*lightdata;
+	
+	//minimum and maximum leaf indexes for each area
+	int			area_min_leaf[MAX_MAP_AREAS];
+	int			area_max_leaf[MAX_MAP_AREAS];
+	int			num_areas;
 
 	// for alias models and skins
 	image_t		*skins[MAX_MD2SKINS];
@@ -378,9 +432,11 @@ typedef struct model_s
 	int			num_frames;
 
 	//iqm skeletal model info
+	unsigned int	version;
 	mvertex_t		*animatevertexes;
 	int				num_joints;
 	iqmjoint_t		*joints;
+	iqmjoint2_t		*joints2;
 	matrix3x4_t		*frames;
 	matrix3x4_t		*outframe;
 	matrix3x4_t		*baseframe;
@@ -406,6 +462,10 @@ typedef struct model_s
 	//ragdoll info
 	int hasRagDoll;
 	mragdoll_t ragdoll;
+	
+	//simple item texnum
+	int					simple_texnum;
+	simplecolor_t		simple_color;
 
 } model_t;
 
@@ -425,4 +485,4 @@ int		Hunk_End (void);
 void	Hunk_Free (void *base);
 
 void	Mod_Free (model_t *mod);
-void    Mod_FreeAll (void);
\ No newline at end of file
+void    Mod_FreeAll (void);
