@@ -132,12 +132,28 @@ void RS_ResetScript (rscript_t *rs)
 	rs->ready = false;
 }
 
+void rs_free_if_subexpr (rs_cond_val_t *expr)
+{
+	if (expr->lval.string)
+		Z_Free (expr->lval.string);
+	if (expr->subexpr1)
+		rs_free_if_subexpr (expr->subexpr1);
+	if (expr->subexpr2)
+		rs_free_if_subexpr (expr->subexpr2);
+	Z_Free (expr);
+}
+
 void RS_ClearStage (rs_stage_t *stage)
 {
 	anim_stage_t	*anim = stage->anim_stage, *tmp_anim;
 	random_stage_t	*randStage = stage->rand_stage, *tmp_rand;
 
 
+	if (stage->condv != NULL)
+	{
+		rs_free_if_subexpr (stage->condv);
+		stage->condv = NULL;
+	}
 	while (anim != NULL)
 	{
 		tmp_anim = anim;
@@ -188,21 +204,14 @@ void RS_ClearStage (rs_stage_t *stage)
 	stage->scroll.typeX = 0;
 	stage->scroll.typeY = 0;
 
-	stage->frames.enabled = false;
-	stage->frames.start = 0;
-	stage->frames.end = 0;
-
 	stage->rot_speed = 0;
 
-	VectorClear(stage->origin);
-	VectorClear(stage->angle);
-
 	stage->texture = NULL;
 	stage->texture2 = NULL;
+	stage->texture3 = NULL;
 
-	stage->depthhack = 0;
+	stage->depthhack = false;
 	stage->envmap = false;
-	stage->dynamic = false;
 	stage->lensflare = false;
 	stage->flaretype = 0;
 	stage->normalmap = false;
@@ -215,21 +224,30 @@ void RS_ClearStage (rs_stage_t *stage)
 	stage->rotating = false;
 	stage->fx = false;
 	stage->glow = false;
+	stage->cube = false;
 
 	stage->lightmap = true;
 
 	stage->next = NULL;
 }
 
-rscript_t *RS_NewScript (char *name)
+// Create a new script with the given name. Reuse the "old" struct if possible
+// to avoid breaking old pointers.
+rscript_t *RS_NewScript (char *name, rscript_t *old)
 {
 	rscript_t	*rs;
 	unsigned int	i;
 
-	if (!rs_rootscript)
+	if (old != NULL)
+	{
+		RS_ResetScript (old);
+		rs = old;
+	}
+	else if (!rs_rootscript)
 	{
 		rs_rootscript = (rscript_t *)malloc(sizeof(rscript_t));
 		rs = rs_rootscript;
+		rs->next = NULL;
 	}
 	else
 	{
@@ -240,13 +258,13 @@ rscript_t *RS_NewScript (char *name)
 
 		rs->next = (rscript_t *)malloc(sizeof(rscript_t));
 		rs = rs->next;
+		rs->next = NULL;
 	}
 
 	COMPUTE_HASH_KEY(rs->hash_key, name, i);
 	strncpy (rs->name, name, sizeof(rs->name));
 
 	rs->stage = NULL;
-	rs->next = NULL;
 	rs->dontflush = false;	
 	rs->ready = false;	
 
@@ -274,11 +292,13 @@ rs_stage_t *RS_NewStage (rscript_t *rs)
 
 	strncpy (stage->name, "***r_notexture***", sizeof(stage->name));
 	strncpy (stage->name2, "***r_notexture***", sizeof(stage->name2));
+	strncpy (stage->name3, "***r_notexture***", sizeof(stage->name3));
 
 	stage->rand_stage = NULL;
 	stage->anim_stage = NULL;
 	stage->next = NULL;
 	stage->last_anim = NULL;
+	stage->condv = NULL;
 
 	RS_ClearStage (stage);
 
@@ -301,10 +321,14 @@ void RS_FreeAllScripts (void)
 void RS_ReloadImageScriptLinks (void)
 {
 	image_t		*image;
-	int	i;
+	int			i;
+	char		shortname[MAX_QPATH];
 
 	for (i=0, image=gltextures ; i<numgltextures ; i++,image++)
-		image->script = RS_FindScript(image->bare_name);
+	{
+		COM_StripExtension (image->name, shortname);
+		image->script = RS_FindScript (shortname);
+	}
 
 }
 
@@ -335,8 +359,16 @@ void RS_FreeScript(rscript_t *rs)
 
 void RS_FreeUnmarked (void)
 {
+	image_t		*image;
+	int			i;
 	rscript_t	*rs = rs_rootscript, *tmp_rs;
 
+	for (i=0, image=gltextures ; i<numgltextures ; i++,image++)
+	{
+		if (image->script && !((rscript_t *)image->script)->dontflush)
+			image->script = NULL;
+	}
+	
 	while (rs != NULL)
 	{
 		tmp_rs = rs->next;
@@ -420,6 +452,10 @@ void RS_ReadyScript (rscript_t *rs)
 			stage->texture2 = GL_FindImage (stage->name2, mode);
 		if (!stage->texture2)
 			stage->texture2 = r_notexture;
+		if (stage->name3[0])
+			stage->texture3 = GL_FindImage (stage->name3, mode);
+		if (!stage->texture3)
+			stage->texture3 = r_notexture;
 
 		//check alpha
 		if (stage->blendfunc.blend)
@@ -483,7 +519,8 @@ scriptname
 	safe
 	{
 		map <texturename>
-		map2 <texturename>
+		map2 <texturename> - specific to "fx"
+		map3 <texturename> - specific to "cube"
 		scroll <xtype> <xspeed> <ytype> <yspeed>
 		blendfunc <source> <dest>
 		alphashift <speed> <min> <max>
@@ -503,6 +540,7 @@ scriptname
 		rotating
 		fx
 		glow
+		cube
 	}
 }
 */
@@ -521,11 +559,11 @@ void rs_stage_map2 (rs_stage_t *stage, char **token)
 	strncpy (stage->name2, *token, sizeof(stage->name2));
 }
 
-void rs_stage_model (rs_stage_t *stage, char **token)
+void rs_stage_map3 (rs_stage_t *stage, char **token)
 {
 	*token = strtok (NULL, TOK_DELIMINATORS);
 
-	strncpy (stage->model, *token, sizeof(stage->model));
+	strncpy (stage->name3, *token, sizeof(stage->name3));
 }
 
 void rs_stage_colormap (rs_stage_t *stage, char **token)
@@ -542,20 +580,6 @@ void rs_stage_colormap (rs_stage_t *stage, char **token)
 	stage->colormap.blue = atof(*token);
 }
 
-void rs_stage_frames (rs_stage_t *stage, char **token)
-{
-	stage->frames.enabled = true;
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->frames.speed = atof(*token);
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->frames.start = atoi(*token);
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->frames.end = atoi(*token);
-}
-
 void rs_stage_scroll (rs_stage_t *stage, char **token)
 {
 	*token = strtok (NULL, TOK_DELIMINATORS);
@@ -676,8 +700,7 @@ void rs_stage_anim (rs_stage_t *stage, char **token)
 
 void rs_stage_depthhack (rs_stage_t *stage, char **token)
 {
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->depthhack = (float)atof(*token);
+	stage->depthhack = true;
 }
 
 void rs_stage_envmap (rs_stage_t *stage, char **token)
@@ -685,11 +708,6 @@ void rs_stage_envmap (rs_stage_t *stage, char **token)
 	stage->envmap = true;
 }
 
-void rs_stage_dynamic (rs_stage_t *stage, char **token)
-{
-	stage->dynamic = true;
-}
-
 void rs_stage_nolightmap (rs_stage_t *stage, char **token)
 {
 	stage->lightmap = false;
@@ -706,30 +724,6 @@ void rs_stage_rotate (rs_stage_t *stage, char **token)
 	stage->rot_speed = (float)atof(*token);
 }
 
-void rs_stage_origin (rs_stage_t *stage, char **token)
-{
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->origin[0] = (float)atof(*token);
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->origin[1] = (float)atof(*token);
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->origin[2] = (float)atof(*token);
-}
-
-void rs_stage_angle (rs_stage_t *stage, char **token)
-{
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->angle[0] = (float)atof(*token);
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->angle[1] = (float)atof(*token);
-
-	*token = strtok (NULL, TOK_DELIMINATORS);
-	stage->angle[2] = (float)atof(*token);
-}
-
 void rs_stage_scale (rs_stage_t *stage, char **token)
 {
 	*token = strtok (NULL, TOK_DELIMINATORS);
@@ -814,14 +808,162 @@ void rs_stage_glow (rs_stage_t *stage, char **token)
 {
 	stage->glow = true;
 }
+void rs_stage_cube (rs_stage_t *stage, char **token)
+{
+	stage->cube = true;
+}
+
+typedef struct 
+{
+	char			*opname;
+	rs_cond_op_t	op;
+} rs_cond_op_key_t;
+static rs_cond_op_key_t rs_cond_op_keys[] = 
+{
+	{	"==",	rs_cond_eq		},
+	{	"!=",	rs_cond_neq		},
+	{	">",	rs_cond_gt		},
+	{	"<=",	rs_cond_ngt		},
+	{	"<",	rs_cond_lt		},
+	{	">=",	rs_cond_nlt		},
+	{	"&&",	rs_cond_and		},
+	{	"||",	rs_cond_or		},
+	
+	{	NULL,	0				}
+};
+rs_cond_val_t *rs_stage_if_subexpr (char **token)
+{
+	int i;
+	rs_cond_val_t *res = Z_Malloc (sizeof(rs_cond_val_t));
+	*token = strtok (NULL, TOK_DELIMINATORS);
+	if (!(*token)) 
+	{
+		Z_Free (res);
+		return NULL;
+	}
+	if (!Q_strcasecmp (*token, "(")) 
+	{
+		res->subexpr1 = rs_stage_if_subexpr (token);
+		if (!res->subexpr1) //there was an error somewhere
+		{
+			Z_Free (res);
+			return NULL;
+		}
+		*token = strtok (NULL, TOK_DELIMINATORS);
+		if (!(*token)) {
+			Com_Printf ("Ran out of tokens in stage conditional!\n");
+			rs_free_if_subexpr (res->subexpr1);
+			Z_Free (res);
+			return NULL;
+		}
+		if (!Q_strcasecmp (*token, ")"))
+		{
+			res->optype = rs_cond_is;
+			return res;
+		}
+		for (i = 0; rs_cond_op_keys[i].opname; i++)
+			if (!Q_strcasecmp (*token, rs_cond_op_keys[i].opname))
+				break;
+		if (!rs_cond_op_keys[i].opname) 
+		{
+			Com_Printf ("Invalid stage conditional operation %s\n", *token);
+			rs_free_if_subexpr (res->subexpr1);
+			Z_Free (res);
+			return NULL;
+		}
+		res->optype = rs_cond_op_keys[i].op;
+		res->subexpr2 = rs_stage_if_subexpr (token);
+		if (!res->subexpr2) //there was an error somewhere
+		{
+			rs_free_if_subexpr (res->subexpr1);
+			Z_Free (res);
+			return NULL;
+		}
+		*token = strtok (NULL, TOK_DELIMINATORS);
+		if (!(*token) || Q_strcasecmp (*token, ")")) 
+		{
+			Com_Printf ("Missing ) in stage conditional!\n");
+			if (*token)
+				Com_Printf ("Instead got %s\n", *token);
+			rs_free_if_subexpr (res->subexpr1);
+			rs_free_if_subexpr (res->subexpr2);
+			Z_Free (res);
+			return NULL;
+		}
+		return res;
+	} 
+	else if (!Q_strcasecmp (*token, "!"))
+	{
+		res->optype = rs_cond_lnot;
+		res->subexpr1 = rs_stage_if_subexpr (token);
+		if (!res->subexpr1) // there was an error somewhere
+		{
+			Z_Free (res);
+			return NULL;
+		}
+		return res;
+	}
+	else if (!Q_strcasecmp (*token, "$"))
+	{
+		res->optype = rs_cond_none;
+		*token = strtok (NULL, TOK_DELIMINATORS);
+		if (!(*token)) {
+			Com_Printf ("Missing cvar name in stage conditional!\n");
+			rs_free_if_subexpr (res->subexpr1);
+			Z_Free (res);
+			return NULL;
+		}
+		res->val = Cvar_Get (*token, "0", 0);
+		return res;
+	}
+	else
+	{
+		res->optype = rs_cond_none;
+		res->val = &(res->lval);
+		Anon_Cvar_Set (res->val, *token);
+		return res;
+	}
+}
+void rs_stage_if (rs_stage_t *stage, char **token)
+{
+	stage->condv = rs_stage_if_subexpr (token);
+	if (!stage->condv)
+		Com_Printf ("ERROR in stage conditional!\n");
+}
+
+// For legacy origin and angle commands that aren't actually used in the code.
+// Some old rscripts still have the origin command in them, so we should parse
+// it anyway. Can't find any angle commands, but may as well handle those too.
+void rs_stage_consume3 (rs_stage_t *stage, char **token)
+{
+	Com_Printf ("WARN: depreciated Rscript command: %s\n", *token); 
+	*token = strtok (NULL, TOK_DELIMINATORS);
+	*token = strtok (NULL, TOK_DELIMINATORS);
+	*token = strtok (NULL, TOK_DELIMINATORS);
+}
+
+// Used for the old "model" command, even though I can't find any rscripts 
+// that have that command.
+void rs_stage_consume1 (rs_stage_t *stage, char **token)
+{
+	Com_Printf ("WARN: depreciated Rscript command: %s\n", *token); 
+	*token = strtok (NULL, TOK_DELIMINATORS);
+}
+
+// Used for the old "dynamic" command, even though I can't find any rscripts
+// that have that command.
+void rs_stage_consume0 (rs_stage_t *stage, char **token)
+{
+	Com_Printf ("WARN: depreciated Rscript command: %s\n", *token);
+}
+
 static rs_stagekey_t rs_stagekeys[] =
 {
 	{	"colormap",		&rs_stage_colormap		},
 	{	"map",			&rs_stage_map			},
 	{	"map2",			&rs_stage_map2			},
-	{	"model",		&rs_stage_model			},
+	{	"map3",			&rs_stage_map3			},
 	{	"scroll",		&rs_stage_scroll		},
-	{	"frames",		&rs_stage_frames		},
 	{	"blendfunc",	&rs_stage_blendfunc		},
 	{	"alphashift",	&rs_stage_alphashift	},
 	{	"rand",			&rs_stage_random		},
@@ -831,10 +973,7 @@ static rs_stagekey_t rs_stagekeys[] =
 	{	"nolightmap",	&rs_stage_nolightmap	},
 	{	"alphamask",	&rs_stage_alphamask		},
 	{	"rotate",		&rs_stage_rotate		},
-	{	"origin",		&rs_stage_origin		},
-	{	"angle",		&rs_stage_angle			},
 	{	"scale",		&rs_stage_scale			},
-	{	"dynamic",		&rs_stage_dynamic		},
 	{	"alphafunc",	&rs_stage_alphafunc		},
 	{	"lensflare",	&rs_stage_lensflare		},
 	{	"flaretype",	&rs_stage_flaretype		},
@@ -848,6 +987,15 @@ static rs_stagekey_t rs_stagekeys[] =
 	{	"rotating",		&rs_stage_rotating		},
 	{	"fx",			&rs_stage_fx			},
 	{	"glow",			&rs_stage_glow			},
+	{	"cube",			&rs_stage_cube			},
+	{	"if",			&rs_stage_if			},
+	
+	// Depreciated stuff
+	{	"model",		&rs_stage_consume1		},
+	{	"frames",		&rs_stage_consume3		},
+	{	"origin",		&rs_stage_consume3		},
+	{	"angle",		&rs_stage_consume3		},
+	{	"dynamic",		&rs_stage_consume1		},
 
 	{	NULL,			NULL					}
 };
@@ -903,10 +1051,7 @@ void RS_LoadScript(char *script)
 			{
 				rs = RS_FindScript(token);
 
-				if (rs)
-					RS_FreeScript(rs);
-
-				rs = RS_NewScript(token);
+				rs = RS_NewScript (token, rs);
 			}
 		}
 		else if (inscript && !ignored)
@@ -969,6 +1114,21 @@ void RS_ScanPathForScripts (void)
 
 		FS_FreeFileList(script_list, script_count);
 	}
+	
+	script_count = 0;
+	//TODO: gl_interactivescripts cvar
+	script_list = FS_ListFilesInFS("scripts/interactive/*.rscript", &script_count, 0, SFF_SUBDIR | SFF_HIDDEN | SFF_SYSTEM);
+
+	if(script_list) {
+		for (i = 0; i < script_count; i++)
+		{
+			c = COM_SkipPath(script_list[i]);
+			Com_sprintf(script, MAX_OSPATH, "scripts/interactive/%s", c);
+			RS_LoadScript(script);
+		}
+
+		FS_FreeFileList(script_list, script_count);
+	}
 
 	script_count = 0;
 	if(gl_normalmaps->value) { //search for normal map scripts ONLY if we are using normal mapping, do last to overide anything
@@ -1180,27 +1340,6 @@ endalpha:
 	return alpha;
 }
 
-float RS_AlphaFuncAlias (int alphafunc, float alpha, vec3_t normal, vec3_t org)
-{
-	// float oldalpha = alpha; // unused
-
-	if (!abs(alphafunc))
-		goto endalpha;
-
-	if (alphafunc == ALPHAFUNC_GLOSS)
-	{
-		//glossmap stuff here...
-	}
-
-endalpha:
-
-	if (alpha<0) alpha = 0;
-	if (alpha>1) alpha = 1;
-	if (alphafunc<0) alpha = 1-alpha;
-
-	return alpha;
-}
-
 image_t *BSP_TextureAnimation (mtexinfo_t *tex);
 rscript_t	*surfaceScript(msurface_t *surf)
 {
@@ -1306,6 +1445,69 @@ void ToggleLightmap (qboolean toggle)
 	}
 }
 
+static cvar_t *rs_eval_if_subexpr (rs_cond_val_t *expr)
+{
+	int resv;
+	switch (expr->optype)
+	{
+		case rs_cond_none:
+			return expr->val;
+		case rs_cond_is:
+			return rs_eval_if_subexpr (expr->subexpr1);
+		case rs_cond_lnot:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value == 0);
+			break;
+		case rs_cond_eq:
+			resv = Q_strcasecmp (
+					rs_eval_if_subexpr (expr->subexpr1)->string,
+					rs_eval_if_subexpr (expr->subexpr2)->string
+				) == 0;
+			break;
+		case rs_cond_neq:
+			resv = Q_strcasecmp (
+					rs_eval_if_subexpr (expr->subexpr1)->string,
+					rs_eval_if_subexpr (expr->subexpr2)->string
+				) != 0;
+			break;
+		case rs_cond_gt:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value >
+					rs_eval_if_subexpr (expr->subexpr2)->value
+				);
+			break;
+		case rs_cond_ngt:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value <=
+					rs_eval_if_subexpr (expr->subexpr2)->value
+				);
+			break;
+		case rs_cond_lt:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value <
+					rs_eval_if_subexpr (expr->subexpr2)->value
+				);
+			break;
+		case rs_cond_nlt:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value >=
+					rs_eval_if_subexpr (expr->subexpr2)->value
+				);
+			break;
+		case rs_cond_and:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value &&
+					rs_eval_if_subexpr (expr->subexpr2)->value
+				);
+			break;
+		case rs_cond_or:
+			resv = (rs_eval_if_subexpr (expr->subexpr1)->value ||
+					rs_eval_if_subexpr (expr->subexpr2)->value
+				);
+			break;
+		default:
+			Com_Printf ("Unknown optype %d! (Can't happen)\n", expr->optype);
+			resv = 0;
+			break;
+	}
+	Anon_Cvar_Set (&(expr->lval), va ("%d", resv));
+	return &(expr->lval);
+}
+
 //This is the shader drawing routine for bsp surfaces - it will draw on top of the
 //existing texture.
 void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
@@ -1333,8 +1535,10 @@ void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
 	lightmaptoggle = true;
 	do
 	{
-		if (stage->lensflare || stage->grass || stage->beam)
+		if (stage->lensflare || stage->grass || stage->beam || stage->cube)
 			break; //handled elsewhere
+		if (stage->condv && !(rs_eval_if_subexpr(stage->condv)->value))
+			continue; //stage should not execute
 
 		if(stage->lightmap)
 		{
@@ -1516,7 +1720,7 @@ void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
 			VertexCounter++;		
 		}
 
-		R_DrawVarrays(GL_POLYGON, 0, VertexCounter, false);
+		R_DrawVarrays(GL_POLYGON, 0, VertexCounter);
 					
 		qglColor4f(1,1,1,1);
 		if (stage->colormap.enabled)
@@ -1560,7 +1764,7 @@ void RS_Surface (msurface_t *surf)
 
 	//Underwater Caustics
 	if(rs_caustics)
-		if (surf->flags & SURF_UNDERWATER )
+		if (surf->iflags & ISURF_UNDERWATER )
 				RS_DrawSurfaceTexture(surf, rs_caustics);
 
 	//all other textures shaders
