@@ -272,6 +272,18 @@ void R_KillVArrays (void)
 	if (KillFlags & KILL_RGBA_POINTER)
 		qglDisableClientState (GL_COLOR_ARRAY);
 
+	if (KillFlags & KILL_TMU5_POINTER)
+	{
+		qglClientActiveTextureARB (GL_TEXTURE5);
+		qglDisableClientState (GL_TEXTURE_COORD_ARRAY);
+	}
+
+	if (KillFlags & KILL_TMU4_POINTER)
+	{
+		qglClientActiveTextureARB (GL_TEXTURE4);
+		qglDisableClientState (GL_TEXTURE_COORD_ARRAY);
+	}
+
 	if (KillFlags & KILL_TMU3_POINTER)
 	{
 		qglClientActiveTextureARB (GL_TEXTURE3);
@@ -300,8 +312,7 @@ void R_KillVArrays (void)
 	qglDisableClientState (GL_VERTEX_ARRAY);
 }
 
-void R_DrawVarrays(GLenum mode, GLint first, GLsizei count, qboolean vbo)
-
+void R_DrawVarrays(GLenum mode, GLint first, GLsizei count)
 {
 	if(count < 1)
 		return; //do not send arrays of zero size to GPU!
@@ -309,18 +320,7 @@ void R_DrawVarrays(GLenum mode, GLint first, GLsizei count, qboolean vbo)
 	if(gl_state.vbo)
 		GL_BindVBO(NULL); //make sure that we aren't using an invalid buffer
 
-	if(!vbo)
-	{
-		if(qglLockArraysEXT)
-			qglLockArraysEXT(0, count);
-
-		qglDrawArrays (mode, first, count);
-
-		if(qglUnlockArraysEXT)
-			qglUnlockArraysEXT();
-	}
-	else
-		qglDrawArrays (mode, first, count);
+	qglDrawArrays (mode, first, count);
 }
 
 void R_AddSurfToVArray (msurface_t *surf)
@@ -349,10 +349,99 @@ void R_AddSurfToVArray (msurface_t *surf)
 		}
 
 		// draw the poly
-		R_DrawVarrays(GL_POLYGON, 0, VertexCounter, false);
+		R_DrawVarrays(GL_POLYGON, 0, VertexCounter);
 	}
 }
 
+extern int SignbitsForPlane (cplane_t *out);
+void R_AddShadowSurfToVArray (msurface_t *surf, vec3_t origin)
+{
+	glpoly_t *p = surf->polys;
+    float    *v, *v2;
+    int i;
+    float  sq_len;
+    vec3_t vDist;
+    vec3_t tmp, mins, maxs;
+ 
+    qboolean renderPoly;
+    
+    VectorAdd (currentmodel->maxs, origin, maxs);
+    VectorAdd (currentmodel->mins, origin, mins);
+ 
+    for (; p; p = p->chain)
+    {
+		// reset pointer and counter
+        VArray = &VArrayVerts[0];
+        VertexCounter = 0;
+ 
+        VectorSubtract(currentmodel->maxs, currentmodel->mins, tmp);
+        
+        /* lengths used for comparison only, so squared lengths may be used.*/
+        sq_len = tmp[0]*tmp[0] + tmp[1]*tmp[1] + tmp[2]*tmp[2] ;
+        if ( sq_len <  4096.0f )
+            sq_len = 4096.0f; /* 64^2 */
+        else if ( sq_len > 65536.0f )
+            sq_len = 65536.0f; /* 256^2 */
+         
+        renderPoly = false;
+        for ( v = p->verts[0], i = 0; i < p->numverts; i++, v += VERTEXSIZE )
+        {
+         	// Generate plane out of the triangle between v, v+1, and the 
+		    // light point. This plane will be one of the borders of the
+		    // 3D volume within which an object may cast a shadow on the
+		    // world polygon.
+		    vec3_t plane_line_1, plane_line_2;
+		    cplane_t plane;
+		     	
+		    // generate two vectors representing two sides of the triangle
+		    VectorSubtract (v, statLightPosition, plane_line_1);
+		    v2 = p->verts[(i+1)%p->numverts];
+		    VectorSubtract (v2, v, plane_line_2);
+		     	
+		    // generate the actual plane
+		    CrossProduct (plane_line_1, plane_line_2, plane.normal);
+		    VectorNormalize (plane.normal);
+		    plane.type = PLANE_ANYZ;
+		    plane.dist = DotProduct (v, plane.normal);
+		    plane.signbits = SignbitsForPlane (&plane);
+		     	
+		    // CullBox-type operation
+	     	if (BoxOnPlaneSide (mins, maxs, &plane) == 2)
+	     	{
+	     		//completely clipped; we can skip this surface
+	     		return;
+	     	}
+	     	         		
+             if ( !renderPoly )
+             { /* once set, no need for calculation */
+                 float vsq_len;
+                 
+                 VectorSubtract( origin, v, vDist );
+                 vsq_len = vDist[0]*vDist[0] + vDist[1]*vDist[1] + vDist[2]*vDist[2];
+                 if ( vsq_len < sq_len )
+                 {
+                     renderPoly = true;
+                 }
+             }
+ 
+            // copy in vertex data
+             VArray[0] = v[0];
+             VArray[1] = v[1];
+             VArray[2] = v[2];
+ 
+            // nothing else is needed
+             // increment pointer and counter
+             VArray += VertexSizes[VERT_NO_TEXTURE];
+             VertexCounter++;
+        }
+ 
+        // draw the poly
+         if(renderPoly)
+             R_DrawVarrays(GL_POLYGON, 0, VertexCounter);
+     }
+ }
+
+
 /*
 =================
 R_AddTexturedSurfToVArray
@@ -393,7 +482,7 @@ void R_AddTexturedSurfToVArray (msurface_t *surf, float scroll)
 		}
 
 		// draw the poly
-		R_DrawVarrays(GL_POLYGON, 0, VertexCounter, false);
+		R_DrawVarrays(GL_POLYGON, 0, VertexCounter);
 	}
 }
 
@@ -444,10 +533,10 @@ void R_AddLightMappedSurfToVArray (msurface_t *surf, float scroll)
 	 *  gl_showpolys == 1 : perform depth test.
 	 *  gl_showpolys == 2 : disable depth test. everything in "visible set"
 	 */
-	if (gl_showpolys->value) // restricted cvar, maxclients = 1
+	if (gl_showpolys->integer) // restricted cvar, maxclients = 1
 	{
 		qglDisable (GL_TEXTURE_2D);
-		if (gl_showpolys->value > 1.9f)
+		if (gl_showpolys->integer >= 2)
 		{ // lots of lines so make them narrower
 			qglDisable(GL_DEPTH_TEST);
 			qglLineWidth (2.0f);
@@ -467,7 +556,7 @@ void R_AddLightMappedSurfToVArray (msurface_t *surf, float scroll)
 	}
 	
 	// draw the polys
-	R_DrawVarrays(GL_POLYGON, 0, VertexCounter, false);
+	R_DrawVarrays(GL_POLYGON, 0, VertexCounter);
 }
 
 void R_AddGLSLShadedWarpSurfToVArray (msurface_t *surf, float scroll)
@@ -493,19 +582,15 @@ void R_AddGLSLShadedWarpSurfToVArray (msurface_t *surf, float scroll)
 			VArray[3] = v[3] + scroll;
 			VArray[4] = v[4];
 
-			VectorCopy(surf->plane->normal, NormalsArray[VertexCounter]);
-
 			// nothing else is needed
 			// increment pointer and counter
-			VArray += VertexSizes[VERT_NORMAL_COLOURED_TEXTURED];
+			VArray += VertexSizes[VERT_COLOURED_TEXTURED];
 			VertexCounter++;
 		}				
 	}
 
-	qglNormalPointer(GL_FLOAT, 0, NormalsArray);
-
 	// draw the polys
-	R_DrawVarrays(GL_POLYGON, 0, VertexCounter, false);
+	R_DrawVarrays(GL_POLYGON, 0, VertexCounter);
 }
 
 
