@@ -165,7 +165,7 @@ boundaries so that turbulent and sky warps
 can be done reasonably.
 ================
 */
-void R_SubdivideSurface (msurface_t *fa)
+void R_SubdivideSurface (msurface_t *fa, int firstedge, int numedges)
 {
 	vec3_t		verts[64];
 	int			numverts;
@@ -179,9 +179,9 @@ void R_SubdivideSurface (msurface_t *fa)
 	// convert edges back to a normal polygon
 	//
 	numverts = 0;
-	for (i=0 ; i<fa->numedges ; i++)
+	for (i=0 ; i<numedges ; i++)
 	{
-		lindex = loadmodel->surfedges[fa->firstedge + i];
+		lindex = loadmodel->surfedges[firstedge + i];
 
 		if (lindex > 0)
 			vec = loadmodel->vertexes[loadmodel->edges[lindex].v[0]].position;
@@ -221,7 +221,7 @@ void R_RenderWaterPolys (msurface_t *fa, int texnum, float scaleX, float scaleY)
 	float		s, t, os, ot;
 	float		scroll;
 	float		rdt = r_newrefdef.time;
-	vec3_t		nv, tangent;
+	vec3_t		nv;
 
 	if (fa->texinfo->flags & SURF_FLOWING)
 		scroll = -64.0f * ((r_newrefdef.time * 0.5f) - (int)(r_newrefdef.time * 0.5f));
@@ -229,12 +229,12 @@ void R_RenderWaterPolys (msurface_t *fa, int texnum, float scaleX, float scaleY)
 		scroll = 0.0f;
 	  
 	if(gl_state.glsl_shaders && gl_glsl_shaders->value
-		&& fa->texinfo->has_normalmap) {
-
+		&& fa->texinfo->has_normalmap)
+	{
 		if (SurfaceIsAlphaBlended(fa))
 			qglEnable( GL_ALPHA_TEST );
 
-		R_InitVArrays (VERT_NORMAL_COLOURED_TEXTURED);
+		R_InitVArrays (VERT_COLOURED_TEXTURED);
 
 		glUseProgramObjectARB( g_waterprogramObj );
 
@@ -244,33 +244,34 @@ void R_RenderWaterPolys (msurface_t *fa, int texnum, float scaleX, float scaleY)
 		qglBindTexture (GL_TEXTURE_2D, fa->texinfo->image->texnum);
 		glUniform1iARB( g_location_baseTexture, 0);
 
-		//note - moving this to tmu2 fixed a very odd, obsure bug.  It isn't clear yet why it fixes it, but it does
 		qglActiveTextureARB(GL_TEXTURE1);
 		qglBindTexture(GL_TEXTURE_2D, fa->texinfo->normalMap->texnum);
 		glUniform1iARB( g_location_normTexture, 1);
 
+        if (texnum)
+        {
+			qglActiveTextureARB(GL_TEXTURE2);
+			qglBindTexture(GL_TEXTURE_2D, texnum);
+			glUniform1iARB( g_location_refTexture, 2);
+		}
+		else
+		{
+			glUniform1iARB( g_location_refTexture, 0);
+		}
+
 		if(fa->texinfo->flags &(SURF_TRANS33|SURF_TRANS66))
-			glUniform1iARB( g_location_trans, 1);
+			glUniform1fARB( g_location_trans, 0.5);
 		else
-			glUniform1iARB( g_location_trans, 0);
-
-		qglActiveTextureARB(GL_TEXTURE2);
-		qglBindTexture(GL_TEXTURE_2D, texnum);
-		glUniform1iARB( g_location_refTexture, 2);
-
+			glUniform1fARB( g_location_trans, 0.75);
+		
 		if(texnum)
-		{
 			glUniform1iARB( g_location_reflect, 1);
-		}
 		else
-		{
 			glUniform1iARB( g_location_reflect, 0);
-		}
-			
-		AngleVectors(fa->plane->normal, NULL, tangent, NULL);
 
 		//send these to the shader program	
-		glUniform3fARB( g_location_tangent, tangent[0], tangent[1], tangent[2]);
+		glUniformMatrix3fvARB( g_location_tangentSpaceTransform, 1, GL_FALSE, (const GLfloat *) fa->tangentSpaceTransform );
+		glUniform3fARB( g_location_waterEyePos, r_origin[0], r_origin[1], r_origin[2] );
 		glUniform3fARB( g_location_lightPos, r_worldLightVec[0], r_worldLightVec[1], r_worldLightVec[2]);
 		glUniform1iARB( g_location_fogamount, map_fog);
 		glUniform1fARB( g_location_time, rs_realtime);
@@ -866,9 +867,8 @@ void R_SetSky (char *name, float rotate, vec3_t axis)
 
 	for (i=0 ; i<6 ; i++)
 	{
-		// chop down rotating skies for less memory
-		if (gl_skymip->value || skyrotate)
-			gl_picmip->value++;
+		if (gl_skymip->integer)
+			gl_picmip->integer += gl_skymip->integer;
 
 		Com_sprintf (pathname, sizeof(pathname), "env/%s%s.tga", skyname, suf[i]);
 
@@ -879,32 +879,40 @@ void R_SetSky (char *name, float rotate, vec3_t axis)
 			if (r_shaders->value) {
 				strcpy(pathname,sky_images[i]->name);
 				pathname[strlen(pathname)-4]=0;
-				sky_images[i]->script = RS_FindScript(pathname);
 				if(sky_images[i]->script)
 					RS_ReadyScript(sky_images[i]->script);
 			}
 		}
 
-		if (strstr(pathname, "space")) {
-			VectorSet(sun_origin, -5000, -100000, 115000);
-			spacebox = true;
+		//set only if not set by fog script
+		if(r_sunX == 0.0 && r_sunY == 0.0 && r_sunZ == 0.0)
+		{
+			if (strstr(pathname, "space")) {
+				VectorSet(sun_origin, -5000, -100000, 115000);
+				spacebox = true;
+			}
+			else if (strstr(pathname, "sea")) {
+				VectorSet(sun_origin, 140000, -80000, 125000);
+				spacebox = false;
+			}
+			else if (strstr(pathname, "hell")) {
+				VectorSet(sun_origin, 140000, 160000, 85000);
+				spacebox = false;
+			}
+			else {
+				VectorSet(sun_origin, 140000, -80000, 45000);
+				spacebox = false;
+			}
 		}
-		else if (strstr(pathname, "sea")) {
-			VectorSet(sun_origin, 140000, -80000, 125000);
-			spacebox = false;
-		}
-		else if (strstr(pathname, "hell")) {
-			VectorSet(sun_origin, 140000, 160000, 85000);
-			spacebox = false;
-		}
-		else {
-			VectorSet(sun_origin, 140000, -80000, 45000);
-			spacebox = false;
+		else
+		{
+			VectorSet(sun_origin, r_sunX, r_sunY, r_sunZ);
+			spacebox = false; //we will change this to sun vs star
 		}
 
-		if (gl_skymip->value || skyrotate)
+		if (gl_skymip->integer)
 		{	// take less memory
-			gl_picmip->value--;
+			gl_picmip->integer -= gl_skymip->integer;
 			sky_min = 1.0/256;
 			sky_max = 255.0/256;
 		}
