@@ -47,6 +47,8 @@ cvar_t	*allow_download_maps;
 cvar_t	*sv_airaccelerate;
 
 cvar_t	*sv_joustmode;
+cvar_t	*sv_tactical;
+cvar_t  *sv_excessive;
 
 cvar_t	*sv_noreload;			// don't reload level state when reentering
 
@@ -152,106 +154,88 @@ SV_StatusString
 Builds the string that is sent as heartbeats and status replies
 ===============
 */
-char	*SV_StatusString (void)
+char *SV_StatusString (void)
 {
-	char	player[1024];
-	static char	status[MAX_MSGLEN - 16];
-	int		i;
-#ifdef NOTUSED
-	int j, k;
-#endif
-	client_t	*cl;
-	int		statusLength;
-	int		playerLength;
-	char *name;
-	//char *s;
-	char nametxt[MAX_INFO_STRING];
+	static char status[MAX_MSGLEN - 16]; // static buffer for the status string
+	qboolean msg_overflow = false;
 
-	strcpy (status, Cvar_Serverinfo());
-	strcat (status, "\n");
-	statusLength = strlen(status);
+	char      player[MAX_INFO_STRING];
+	client_t *cl;
+	size_t    count;
 
-	for (i=0 ; i<maxclients->value ; i++)
+	// server info string. MAX_INFO_STRING is always < sizeof(status)
+	strcpy( status, Cvar_Serverinfo() );
+	strcat( status, "\n" );
+
+	// real player score info
+	for ( cl=svs.clients, count=maxclients->integer ; count-- ; cl++ )
 	{
-		cl = &svs.clients[i];
 		if (cl->state == cs_connected || cl->state == cs_spawned )
 		{
-			name = cl->name;
-#ifdef NOTUSED
-			//handle color chars
-			nametxt[0] = 0;
-			k = 0;
-			while(j = *name) {
-				if ( Q_IsColorString( name ) ) {
-					name +=2;
-					continue;
-				}
-				nametxt[k] = j;
-				name++;
-				k++;
-				//failsafe - break at 32 chars
-				if(k>30)
-					break;
+			/* send score of 0 for spectators and players not yet in game
+			 *  statistics program uses this data when polling for scores
+			 */
+			int cl_score = 0;
+			if ( cl->state == cs_spawned
+					&& cl->edict->client->ps.stats[STAT_SPECTATOR] == 0 )
+			{
+				cl_score = cl->edict->client->ps.stats[STAT_FRAGS];;
+			}
+			/*
+			 * send color characters
+			 * do not send actual ip addresses for security/privacy reasons
+			 */
+			Com_sprintf( player, sizeof(player),
+					"%i %i \"%s\" \"127.0.0.1\"\n",
+					cl_score, cl->ping, cl->name);
+			if ( (strlen(status) + strlen(player) + 1) < sizeof(status) )
+			{
+				strncat( status, player, strlen(player) );
+			}
+			else
+			{
+				msg_overflow = true;
+				break;
 			}
-			nametxt[k]=0;
-#else
-			//allow color chars to be sent
-			Q_strncpyz2( nametxt, name, sizeof(nametxt) );
-#endif
-			//s = NET_AdrToString ( cl->netchan.remote_address); //fuck you Luigi
-			Com_sprintf (player, sizeof(player), "%i %i \"%s\" \"127.0.0.1\"\n",
-				cl->edict->client->ps.stats[STAT_FRAGS], cl->ping, nametxt);
-			playerLength = strlen(player);
-			if (statusLength + playerLength >= sizeof(status) )
-				break;		// can't hold any more
-			strcpy (status + statusLength, player);
-			statusLength += playerLength;
-
 		}
 	}
-	//bot score info
-	for (i=0 ; i<maxclients->value ; i++)
+
+	// bot score info
+	if ( !msg_overflow )
 	{
-		cl = &svs.clients[i];
-
-		if(cl->edict->client->ps.botnum) {
-			for(i = 0; i < cl->edict->client->ps.botnum; i++) {
-
-				name = cl->edict->client->ps.bots[i].name;
-#ifdef NOTUSED
-				//handle color chars
-				nametxt[0] = 0;
-				k = 0;
-				while(j = *name) {
-					if ( Q_IsColorString( name ) ) {
-						name +=2;
-						continue;
+		for ( cl=svs.clients, count=maxclients->integer ; count-- ; cl++ )
+		{
+			size_t bot_count;
+			bot_count = cl->edict->client->ps.botnum;
+			if ( bot_count )
+			{ // normally, first client contains the bot names and scores
+				bot_t* ps_bot;
+				for ( ps_bot = cl->edict->client->ps.bots ; bot_count-- ; ps_bot++ )
+				{
+					int bot_score = ps_bot->score;
+					Com_sprintf( player, sizeof(player),
+							"%i %i \"%s\" \"127.0.0.1\"\n",
+							bot_score,
+							0, // bot ping
+							ps_bot->name );
+					if ( (strlen(status) + strlen(player) + 1) < sizeof(status) )
+					{
+						strncat( status, player, strlen(player) );
 					}
-					nametxt[k] = j;
-					name++;
-					k++;
-					//failsafe - break at 32 chars
-					if(k>30)
+					else
+					{
+						msg_overflow = true;
 						break;
+					}
 				}
-				nametxt[k]=0;
-#else
-				//allow color chars to be sent
-				Q_strncpyz2( nametxt, name, sizeof(nametxt) );
-#endif
-
-				Com_sprintf (player, sizeof(player), "%i %i \"%s\" \"127.0.0.1\"\n",
-					cl->edict->client->ps.bots[i].score, 0, nametxt);
-				playerLength = strlen(player);
-				if (statusLength + playerLength >= sizeof(status) )
-					break;		// can't hold any more
-				strcpy (status + statusLength, player);
-				statusLength += playerLength;
 			}
 			break;
 		}
 	}
-	//end bot score info
+	if ( msg_overflow )
+	{
+		Com_DPrintf("SV_StatusString overflowed\n");
+	}
 
 	return status;
 }
@@ -362,7 +346,7 @@ void SVC_Info (void)
 	int		version;
 	client_t	*cl;
 
-	if (maxclients->value == 1)
+	if (maxclients->integer == 1)
 		return;		// ignore in single player
 
 	version = atoi (Cmd_Argv(1));
@@ -376,12 +360,12 @@ void SVC_Info (void)
 	else
 	{
 		count = 0;
-		for (i=0 ; i<maxclients->value ; i++)
+		for (i=0 ; i<maxclients->integer ; i++)
 			if (svs.clients[i].state >= cs_connected)
 				count++;
 
 		//bot score info
-		for (i=0 ; i<maxclients->value ; i++)
+		for (i=0 ; i<maxclients->integer ; i++)
 		{
 			cl = &svs.clients[i];
 			if(cl->edict->client->ps.botnum > 0)
@@ -494,8 +478,8 @@ void SVC_DirectConnect (void)
 	version = atoi(Cmd_Argv(1));
 	if (version != PROTOCOL_VERSION)
 	{
-		Netchan_OutOfBandPrint (NS_SERVER, adr, "print\nServer is version %s\n", VERSION);
-		Com_DPrintf ("    rejected connect from version %i\n", version);
+		Netchan_OutOfBandPrint (NS_SERVER, adr, "print\nServer is protocol version %s\n", PROTOCOL_VERSION);
+		Com_DPrintf ("    rejected connect from protocol version %i\n", version);
 		SV_LogEvent( adr , "RVR" , NULL );
 		return;
 	}
@@ -508,7 +492,7 @@ void SVC_DirectConnect (void)
 
 	//limit connections from a single IP
 	previousclients = 0;
-	for (i=0,cl=svs.clients ; i<maxclients->value ; i++,cl++)
+	for (i=0,cl=svs.clients ; i<maxclients->integer ; i++,cl++)
 	{
 		if (cl->state == cs_free)
 			continue;
@@ -612,7 +596,7 @@ void SVC_DirectConnect (void)
 	memset (newcl, 0, sizeof(client_t));
 
 	// if there is already a slot for this ip, reuse it
-	for (i=0,cl=svs.clients ; i<maxclients->value ; i++,cl++)
+	for (i=0,cl=svs.clients ; i<maxclients->integer ; i++,cl++)
 	{
 		if (cl->state == cs_free)
 			continue;
@@ -647,7 +631,7 @@ void SVC_DirectConnect (void)
 	// find a client slot
 
 	//get number of bots
-	for (i=botnum=0 ; i<maxclients->value ; i++)
+	for (i=botnum=0 ; i<maxclients->integer ; i++)
 	{
 		cl = &svs.clients[i];
 		botnum = cl->edict->client->ps.botnum;
@@ -670,7 +654,7 @@ void SVC_DirectConnect (void)
 			botnum = sv_numbots;
 	}
 
-	for (i=0,cl=svs.clients ; i<maxclients->value-botnum; i++,cl++)
+	for (i=0,cl=svs.clients ; i<maxclients->integer-botnum; i++,cl++)
 	{
 		if (cl->state == cs_free)
 		{
@@ -732,6 +716,9 @@ gotnewcl:
 	newcl->datagram.allowoverflow = true;
 	newcl->lastmessage = svs.realtime;	// don't timeout
 	newcl->lastconnect = svs.realtime;
+	
+	ge->ForceExitIntermission ();
+	Cbuf_Execute ();
 }
 
 int Rcon_Validate (void)
@@ -874,7 +861,7 @@ void SV_CalcPings (void)
 	client_t	*cl;
 	int			total, count;
 
-	for (i=0 ; i<maxclients->value ; i++)
+	for (i=0 ; i<maxclients->integer ; i++)
 	{
 		cl = &svs.clients[i];
 		if (cl->state != cs_spawned )
@@ -928,7 +915,7 @@ void SV_GiveMsec (void)
 	if (sv.framenum & 15)
 		return;
 
-	for (i=0 ; i<maxclients->value ; i++)
+	for (i=0 ; i<maxclients->integer ; i++)
 	{
 		cl = &svs.clients[i];
 		if (cl->state == cs_free )
@@ -969,7 +956,7 @@ void SV_ReadPackets (void)
 		qport = MSG_ReadShort (&net_message) & 0xffff;
 
 		// check for packets from connected clients
-		for (i=0, cl=svs.clients ; i<maxclients->value ; i++,cl++)
+		for (i=0, cl=svs.clients ; i<maxclients->integer ; i++,cl++)
 		{
 			if (cl->state == cs_free)
 				continue;
@@ -995,7 +982,7 @@ void SV_ReadPackets (void)
 			break;
 		}
 
-		if (i != maxclients->value)
+		if (i != maxclients->integer)
 			continue;
 	}
 }
@@ -1023,7 +1010,7 @@ void SV_CheckTimeouts (void)
 	droppoint = svs.realtime - 1000*timeout->value;
 	zombiepoint = svs.realtime - 1000*zombietime->value;
 
-	for (i=0,cl=svs.clients ; i<maxclients->value ; i++,cl++)
+	for (i=0,cl=svs.clients ; i<maxclients->integer ; i++,cl++)
 	{
 
 
@@ -1077,7 +1064,7 @@ SV_RunGameFrame
 */
 void SV_RunGameFrame (void)
 {
-	if (host_speeds->value)
+	if (host_speeds->integer)
 		time_before_game = Sys_Milliseconds ();
 
 	// we always need to bump framenum, even if we
@@ -1088,20 +1075,20 @@ void SV_RunGameFrame (void)
 	sv.time = sv.framenum*100;
 
 	// don't run if paused
-	if (!sv_paused->value || maxclients->value > 1)
+	if (!sv_paused->integer || maxclients->integer > 1)
 	{
 		ge->RunFrame ();
 
 		// never get more than one tic behind
 		if (sv.time < svs.realtime)
 		{
-			if (sv_showclamp->value)
+			if (sv_showclamp->integer)
 				Com_Printf ("sv highclamp\n");
 			svs.realtime = sv.time;
 		}
 	}
 
-	if (host_speeds->value)
+	if (host_speeds->integer)
 		time_after_game = Sys_Milliseconds ();
 
 }
@@ -1137,12 +1124,12 @@ void SV_Frame (int msec)
 	SV_ReadPackets ();
 
 	// move autonomous things around if enough time has passed
-	if (!sv_timedemo->value && svs.realtime < sv.time)
+	if (!sv_timedemo->integer && svs.realtime < sv.time)
 	{
 		// never let the time get too far off
 		if (sv.time - svs.realtime > 100)
 		{
-			if (sv_showclamp->value)
+			if (sv_showclamp->integer)
 				Com_Printf ("sv lowclamp\n");
 			svs.realtime = sv.time - 100;
 		}
@@ -1197,7 +1184,7 @@ void Master_Heartbeat (void)
 
 	// pgm post3.19 change, cvar pointer not validated before dereferencing
 
-	if(!public_server || !public_server->value)
+	if(!public_server || !public_server->integer)
 		return;
 
 	// check for time wraparound
@@ -1225,7 +1212,7 @@ void Master_Shutdown (void)
 {
 
 	// pgm post3.19 change, cvar pointer not validated before dereferencing
-	if (!public_server || !public_server->value)
+	if (!public_server || !public_server->integer)
 		return;		// a private dedicated game
 
 	SV_HandleMasters ("shutdown", "shutdown");
@@ -1246,7 +1233,7 @@ void SV_HandleMasters (const char *message, const char *console_message)
 	qboolean	updated_master;
 
 	// if the server is not dedicated, we need to check cl_master
-	if ( !( dedicated && dedicated->value ) )
+	if ( !( dedicated && dedicated->integer ) )
 	{
 		if ( !sv_master )
 		{
@@ -1412,7 +1399,7 @@ void SV_Init (void)
 	allow_download_models = Cvar_Get ("allow_download_models", "1", CVAR_ARCHIVE);
 	allow_download_sounds = Cvar_Get ("allow_download_sounds", "1", CVAR_ARCHIVE);
 	allow_download_maps	  = Cvar_Get ("allow_download_maps", "1", CVAR_ARCHIVE);
-	sv_downloadurl = Cvar_Get("sv_downloadurl", "http://icculus.org/alienarena/sv_downloadurl", CVAR_SERVERINFO);
+	sv_downloadurl = Cvar_Get("sv_downloadurl", "http://red.planetarena.org/sv_downloadurl", CVAR_SERVERINFO);
 
 	sv_iplogfile = Cvar_Get("sv_iplogfile" , "" , CVAR_ARCHIVE);
 
@@ -1421,6 +1408,8 @@ void SV_Init (void)
 	sv_airaccelerate = Cvar_Get("sv_airaccelerate", "0", CVAR_LATCH);
 
 	sv_joustmode = Cvar_Get("sv_joustmode", "0", CVAR_SERVERINFO);
+	sv_tactical = Cvar_Get("g_tactical", "0", CVAR_LATCH | CVAR_GAMEINFO);
+	sv_excessive = Cvar_Get("excessive", "0", CVAR_LATCH | CVAR_GAMEINFO);
 
 	public_server = Cvar_Get ("sv_public", "1", 0);
 
@@ -1432,6 +1421,8 @@ void SV_Init (void)
 
 	SZ_Init (&net_message, net_message_buffer, sizeof(net_message_buffer));
 	SZ_SetName (&net_message, "Net message buffer", true);
+
+	remoteserver_runspeed = 300; //default
 }
 
 /*
@@ -1462,13 +1453,13 @@ void SV_FinalMessage (char *message, qboolean reconnect)
 	// send it twice
 	// stagger the packets to crutch operating system limited buffers
 
-	for (i=0, cl = svs.clients ; i<maxclients->value ; i++, cl++)
+	for (i=0, cl = svs.clients ; i<maxclients->integer ; i++, cl++)
 
 		if (cl->state >= cs_connected)
 			Netchan_Transmit (&cl->netchan, net_message.cursize
 			, net_message.data);
 
-	for (i=0, cl = svs.clients ; i<maxclients->value ; i++, cl++)
+	for (i=0, cl = svs.clients ; i<maxclients->integer ; i++, cl++)
 
 		if (cl->state >= cs_connected)
 			Netchan_Transmit (&cl->netchan, net_message.cursize
