@@ -18,13 +18,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
-#ifndef __unix__
-#error You shouldnt be including this file on non-Unix platforms
-#endif
-
-/*#ifndef __GLW_LINUX_H__
-#define __GLW_LINUX_H__
-*/
 typedef struct
 {
 	void *OpenGLLib; // instance of OpenGL library
@@ -34,6 +27,5 @@ typedef struct
 
 extern glwstate_t glw_state;
 
-//#endif
 
 
