@@ -23,19 +23,24 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <windows.h>
 
-#include <dsound.h>
+// not used
+// #include <dsound.h>
 
 #define	WINDOW_STYLE	(WS_OVERLAPPED|WS_BORDER|WS_CAPTION|WS_VISIBLE)
 
 extern	HINSTANCE	global_hInstance;
 
+/* old sound stuff (2010-08)
 extern LPDIRECTSOUND pDS;
 extern LPDIRECTSOUNDBUFFER pDSBuf;
 
 extern DWORD gSndBufSize;
+*/
 
 extern HWND			cl_hwnd;
-extern qboolean		ActiveApp, Minimized;
+extern int      ActiveApp;
+extern qboolean Minimized;
+
 
 void IN_Activate (qboolean active);
 void IN_MouseEvent (int mstate);
